@@ -31,9 +31,9 @@
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Revision: 26796 $
- * $Author: rjongbloed $
- * $Date: 2011-12-13 03:26:25 -0600 (Tue, 13 Dec 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_CODEC_OPALPLUGIN_H
@@ -85,6 +85,7 @@ extern "C" {
 #define  PLUGIN_CODEC_VERSION_FAX       4    // added fax
 #define  PLUGIN_CODEC_VERSION_OPTIONS   5    // added options handling
 #define  PLUGIN_CODEC_VERSION_INTERSECT 6    // added media option intersection merge functionality
+#define  PLUGIN_CODEC_VERSION_H245_DEF_GEN_PARAM 7 // added suppression of H.245 generic parameters via default
 
 #define  PLUGIN_CODEC_VERSION PLUGIN_CODEC_VERSION_INTERSECT // Always latest version
 
@@ -272,6 +273,8 @@ enum PluginCodec_OptionMerge {
 #define PluginCodec_H245_OLC           0x02000000
 #define PluginCodec_H245_ReqMode       0x01000000
 #define PluginCodec_H245_OrdinalMask   0x0000ffff
+#define PluginCodec_H245_PositionMask  0x00ff0000
+#define PluginCodec_H245_PositionShift 16
 
 typedef int (*PluginCodec_MergeFunction)(char ** result, const char * dest, const char * src);
 typedef void (*PluginCodec_FreeFunction)(char * string);
@@ -290,6 +293,7 @@ struct PluginCodec_Option {
   const char *                 m_maximum;
   PluginCodec_MergeFunction    m_mergeFunction; // Used if m_merge==PluginCodec_CustomMerge
   PluginCodec_FreeFunction     m_freeFunction;
+  const char *                 m_H245Default;
 };
 
 
@@ -299,9 +303,10 @@ struct PluginCodec_Option {
 #define PLUGINCODEC_OPTION_CHANNELS                   "Channels"
 #define PLUGINCODEC_OPTION_FRAME_TIME                 "Frame Time"
 #define PLUGINCODEC_OPTION_MAX_FRAME_SIZE             "Max Frame Size"
-#define PLUGINCODEC_OPTION_MAX_TX_PACKET_SIZE         "Max Tx Packet Size"
+#define PLUGINCODEC_OPTION_MAX_TX_PACKET_SIZE         "Max Tx Packet Size"   /* Really max PAYLOAD size */
 #define PLUGINCODEC_OPTION_MAX_BIT_RATE               "Max Bit Rate"
 #define PLUGINCODEC_OPTION_TARGET_BIT_RATE            "Target Bit Rate"
+#define PLUGINCODEC_OPTION_RATE_CONTROL_PERIOD        "Rate Control Period"
 #define PLUGINCODEC_OPTION_RX_FRAMES_PER_PACKET       "Rx Frames Per Packet"
 #define PLUGINCODEC_OPTION_TX_FRAMES_PER_PACKET       "Tx Frames Per Packet"
 #define PLUGINCODEC_OPTION_FRAME_WIDTH                "Frame Width"
@@ -313,6 +318,9 @@ struct PluginCodec_Option {
 #define PLUGINCODEC_OPTION_TEMPORAL_SPATIAL_TRADE_OFF "Temporal Spatial Trade Off"
 #define PLUGINCODEC_OPTION_TX_KEY_FRAME_PERIOD        "Tx Key Frame Period"
 
+#define PLUGINCODEC_OPTION_PROTOCOL      "Protocol"
+#define PLUGINCODEC_OPTION_PROTOCOL_H323 "H.323"
+#define PLUGINCODEC_OPTION_PROTOCOL_SIP  "SIP"
 
 
 // Full definition of the codec
