@@ -31,9 +31,9 @@
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Revision: 26787 $
- * $Author: rjongbloed $
- * $Date: 2011-12-08 18:49:50 -0600 (Thu, 08 Dec 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_CODEC_OPALPLUGIN_HPP
@@ -175,7 +175,8 @@ class PluginCodec_MediaFormat
                        OptionMap  & changed,
                        const char * option)
     {
-      if (original[option] != value)
+      OptionMap::iterator it = original.find(option);
+      if (it != original.end() && it->second != value)
         changed[option] = value;
     }
 
