@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22817 $
- * $Author: rjongbloed $
- * $Date: 2009-06-05 02:37:23 +0000 (Fri, 05 Jun 2009) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_CODEC_OPALPLUGINMGR_H
@@ -73,14 +73,19 @@ typedef PFactory<H323StaticPluginCodec> H323StaticPluginCodecFactory;
 
 class OpalPluginCodecManager;
 
+PFACTORY_LOAD(OpalPluginCodecManager);
+
 class OpalPluginCodecHandler : public PObject
 {
   PCLASSINFO(OpalPluginCodecHandler, PObject);
   public:
     OpalPluginCodecHandler();
 
+    static int GetChannelCount(const PluginCodec_Definition * codeDefn);
+
     virtual OpalMediaFormatInternal * OnCreateAudioFormat(OpalPluginCodecManager & mgr,
                                             const PluginCodec_Definition * codecDefn,
+                                                              const char * fmtName,
                                                               const char * rtpEncodingName,
                                                                   unsigned frameTime,
                                                                   unsigned timeUnits,
@@ -89,6 +94,7 @@ class OpalPluginCodecHandler : public PObject
 #if OPAL_VIDEO
     virtual OpalMediaFormatInternal * OnCreateVideoFormat(OpalPluginCodecManager & mgr,
                                             const PluginCodec_Definition * codecDefn,
+                                                              const char * fmtName,
                                                               const char * rtpEncodingName,
                                                                     time_t timeStamp);
     virtual void RegisterVideoTranscoder(const PString & src, const PString & dst, const PluginCodec_Definition * codec, bool v);
@@ -97,6 +103,7 @@ class OpalPluginCodecHandler : public PObject
 #if OPAL_T38_CAPABILITY
     virtual OpalMediaFormatInternal * OnCreateFaxFormat(OpalPluginCodecManager & mgr,
                                           const PluginCodec_Definition * codecDefn,
+                                                            const char * fmtName,
                                                             const char * rtpEncodingName,
                                                                 unsigned frameTime,
                                                                 unsigned timeUnits,
@@ -112,6 +119,8 @@ class OpalPluginCodecManager : public PPluginModuleManager
     OpalPluginCodecManager(PPluginManager * pluginMgr = NULL);
     ~OpalPluginCodecManager();
 
+    PFACTORY_GET_SINGLETON(PFactory<PPluginModuleManager>, OpalPluginCodecManager);
+
     void RegisterStaticCodec(const H323StaticPluginCodecFactory::Key_T & name,
                              PluginCodec_GetAPIVersionFunction getApiVerFn,
                              PluginCodec_GetCodecFunction getCodecFn);
@@ -120,8 +129,6 @@ class OpalPluginCodecManager : public PPluginModuleManager
 
     virtual void OnShutdown();
 
-    static void Bootstrap();
-
 #if OPAL_H323
     H323Capability * CreateCapability(
           const PString & _mediaFormat, 
@@ -138,6 +145,7 @@ class OpalPluginCodecManager : public PPluginModuleManager
     void RegisterCodecPlugins  (unsigned int count, const PluginCodec_Definition * codecList, OpalPluginCodecHandler * handler);
     void UnregisterCodecPlugins(unsigned int count, const PluginCodec_Definition * codecList, OpalPluginCodecHandler * handler);
 
+    bool AddMediaFormat(OpalPluginCodecHandler * handler, const PTime & timeNow, const PluginCodec_Definition * codecDefn, const char * fmtName);
 #if OPAL_H323
     void RegisterCapability(const PluginCodec_Definition * codecDefn);
 #endif
@@ -215,6 +223,7 @@ class OpalPluginTranscoder
     ~OpalPluginTranscoder();
 
     bool UpdateOptions(const OpalMediaFormat & fmt);
+    bool ExecuteCommand(const OpalMediaCommand & command);
     bool Transcode(const void * from, unsigned * fromLen, void * to, unsigned * toLen, unsigned * flags) const
     {
       return codecDef != NULL && codecDef->codecFunction != NULL &&
@@ -240,10 +249,11 @@ class OpalPluginAudioFormatInternal : public OpalAudioFormatInternal, public Opa
 
     OpalPluginAudioFormatInternal(
       const PluginCodec_Definition * codecDefn,
-      const char * rtpEncodingName, /// rtp encoding name
-      unsigned frameTime,           /// Time for frame in RTP units (if applicable)
-      unsigned /*timeUnits*/,       /// RTP units for frameTime (if applicable)
-      time_t timeStamp              /// timestamp (for versioning)
+      const char * fmtName,
+      const char * rtpEncodingName,
+      unsigned frameTime,
+      unsigned timeUnits,
+      time_t timeStamp
     );
     virtual PObject * Clone() const;
     virtual bool IsValidForProtocol(const PString & protocol) const;
@@ -258,6 +268,7 @@ class OpalPluginFramedAudioTranscoder : public OpalFramedTranscoder, public Opal
   public:
     OpalPluginFramedAudioTranscoder(const PluginCodec_Definition * codecDefn, bool isEncoder);
     bool UpdateMediaFormats(const OpalMediaFormat & input, const OpalMediaFormat & output);
+    PBoolean ExecuteCommand(const OpalMediaCommand & command);
     PBoolean ConvertFrame(const BYTE * input, PINDEX & consumed, BYTE * output, PINDEX & created);
     virtual PBoolean ConvertSilentFrame(BYTE * buffer);
     virtual bool AcceptComfortNoise() const { return comfortNoise; }
@@ -272,6 +283,7 @@ class OpalPluginStreamedAudioTranscoder : public OpalStreamedTranscoder, public
   public:
     OpalPluginStreamedAudioTranscoder(const PluginCodec_Definition * codec, bool isEncoder);
     bool UpdateMediaFormats(const OpalMediaFormat & input, const OpalMediaFormat & output);
+    PBoolean ExecuteCommand(const OpalMediaCommand & command);
     virtual bool AcceptComfortNoise() const { return comfortNoise; }
     virtual int ConvertOne(int from) const;
   protected:
@@ -288,6 +300,7 @@ class OpalPluginVideoFormatInternal : public OpalVideoFormatInternal, public Opa
   public:
     OpalPluginVideoFormatInternal(
       const PluginCodec_Definition * codec,
+      const char * fmtName,
       const char * rtpEncodingName, /// rtp encoding name
       time_t timeStamp              /// timestamp (for versioning)
     );
@@ -307,10 +320,19 @@ class OpalPluginVideoTranscoder : public OpalVideoTranscoder, public OpalPluginT
 
     PBoolean ConvertFrames(const RTP_DataFrame & src, RTP_DataFrameList & dstList);
     bool UpdateMediaFormats(const OpalMediaFormat & input, const OpalMediaFormat & output);
+    PBoolean ExecuteCommand(const OpalMediaCommand & command);
 
   protected:
+    bool EncodeFrames(const RTP_DataFrame & src, RTP_DataFrameList & dstList);
+    bool DecodeFrames(const RTP_DataFrame & src, RTP_DataFrameList & dstList);
+    bool DecodeFrame(const RTP_DataFrame & src, RTP_DataFrameList & dstList);
+
     RTP_DataFrame * m_bufferRTP;
-    PTimeInterval   m_lastVideoFastUpdate;
+    DWORD           m_lastSequenceNumber;
+    DWORD           m_lastDecodedTimestamp; // For missing marker bit detection
+    DWORD           m_lastMarkerTimestamp;  // For continuous marker bit detection
+    bool            m_badMarkers;
+    PSimpleTimer    m_videoPictureLossTimer;
 
 #if PTRACING
     unsigned m_consecutiveIntraFrames;
@@ -321,13 +343,14 @@ class OpalPluginVideoTranscoder : public OpalVideoTranscoder, public OpalPluginT
 
 ///////////////////////////////////////////////////////////////////////////////
 
-#if OPAL_T38_CAPABILITY
+#if OPAL_FAX
 
 class OpalPluginFaxFormatInternal : public OpalMediaFormatInternal, public OpalPluginMediaFormatInternal
 {
   public:
     OpalPluginFaxFormatInternal(
       const PluginCodec_Definition * codec,
+      const char * fmtName,
       const char * rtpEncodingName, /// rtp encoding name
       unsigned frameTime,
       unsigned /*timeUnits*/,           /// RTP units for frameTime (if applicable)
@@ -337,12 +360,12 @@ class OpalPluginFaxFormatInternal : public OpalMediaFormatInternal, public OpalP
     virtual bool IsValidForProtocol(const PString & protocol) const;
 };
 
-#endif // OPAL_T38_CAPABILITY
+#endif // OPAL_FAX
 
 
 //////////////////////////////////////////////////////
 //
-//  this is the base class for codecs accesible via the abstract factory functions
+//  this is the base class for codecs accessible via the abstract factory functions
 //
 
 /**Class for codcs which is accessible via the abstract factory functions.
@@ -431,9 +454,9 @@ class OpalPluginTranscoderFactory : public OpalTranscoderFactory
 class H323PluginCapabilityInfo
 {
   public:
-    H323PluginCapabilityInfo(const PluginCodec_Definition * codecDefn);
+    H323PluginCapabilityInfo(const PluginCodec_Definition * codecDefn, const OpalMediaFormat & mediaFormat);
 
-    H323PluginCapabilityInfo(const PString & _baseName);
+    //H323PluginCapabilityInfo(const PString & _baseName);
 
     const PString & GetFormatName() const
     { return m_capabilityFormatName; }
@@ -456,13 +479,9 @@ class H323AudioPluginCapability : public H323AudioCapability,
   PCLASSINFO(H323AudioPluginCapability, H323AudioCapability);
   public:
     H323AudioPluginCapability(const PluginCodec_Definition * codecDefn,
+                              const OpalMediaFormat & mediaFormat,
                               unsigned pluginSubType);
 
-    // this constructor is only used when creating a capability without a codec
-    H323AudioPluginCapability(const PString & _mediaFormat,
-                              const PString & _baseName,
-                              unsigned _pluginSubType);
-
     virtual PObject * Clone() const;
 
     virtual PString GetFormatName() const;
@@ -492,17 +511,12 @@ class H323PluginG7231Capability : public H323AudioPluginCapability
 {
   PCLASSINFO(H323PluginG7231Capability, H323AudioPluginCapability);
   public:
-    H323PluginG7231Capability(const PluginCodec_Definition * codecDefn, bool annexA = true);
-
-    // this constructor is used for creating empty codecs
-    H323PluginG7231Capability(const OpalMediaFormat & fmt, bool annexA = true);
+    H323PluginG7231Capability(const PluginCodec_Definition * codecDefn,
+                              const OpalMediaFormat & mediaFormat);
 
     virtual PObject * Clone() const;
     virtual PBoolean OnSendingPDU(H245_AudioCapability & cap, unsigned packetSize) const;
     virtual PBoolean OnReceivedPDU(const H245_AudioCapability & cap,  unsigned & packetSize);
-
-  protected:
-    bool m_annexA;
 };
 
 #define OPAL_DECLARE_EMPTY_G7231_CAPABILITY(fmt, annex) \
@@ -526,10 +540,12 @@ class H323CodecPluginNonStandardAudioCapability : public H323NonStandardAudioCap
   PCLASSINFO(H323CodecPluginNonStandardAudioCapability, H323NonStandardAudioCapability);
   public:
     H323CodecPluginNonStandardAudioCapability(const PluginCodec_Definition * codecDefn,
+                                              const OpalMediaFormat & mediaFormat,
                                               H323NonStandardCapabilityInfo::CompareFuncType compareFunc,
                                               const unsigned char * data, unsigned dataLen);
 
     H323CodecPluginNonStandardAudioCapability(const PluginCodec_Definition * codecDefn,
+                                              const OpalMediaFormat & mediaFormat,
                                               const unsigned char * data, unsigned dataLen);
 
     virtual PObject * Clone() const;
@@ -548,7 +564,8 @@ class H323CodecPluginGenericAudioCapability : public H323GenericAudioCapability,
   PCLASSINFO(H323CodecPluginGenericAudioCapability, H323GenericAudioCapability);
   public:
     H323CodecPluginGenericAudioCapability(const PluginCodec_Definition * codecDefn,
-				          const PluginCodec_H323GenericCodecData * data);
+                                          const OpalMediaFormat & mediaFormat,
+                                          const PluginCodec_H323GenericCodecData * data);
 
     virtual PObject * Clone() const;
     virtual PString GetFormatName() const;
@@ -557,32 +574,6 @@ class H323CodecPluginGenericAudioCapability : public H323GenericAudioCapability,
 
 #if OPAL_VIDEO
 
-//////////////////////////////////////////////////////////////////////////////
-//
-// Class for handling most audio plugin capabilities
-//
-
-class H323VideoPluginCapability : public H323VideoCapability,
-                             public H323PluginCapabilityInfo
-{
-  PCLASSINFO(H323VideoPluginCapability, H323VideoCapability);
-  public:
-    H323VideoPluginCapability(const PluginCodec_Definition * codecDefn,
-                              unsigned _pluginSubType);
-
-    virtual PString GetFormatName() const;
-
-    virtual unsigned GetSubType() const;
-
-    static bool SetOptionsFromMPI(OpalMediaFormat & mediaFormat, int frameWidth, int frameHeight, int frameRate);
-
-    virtual void PrintOn(std::ostream & strm) const;
-
-  protected:
-    unsigned pluginSubType;
-    unsigned h323subType;   // only set if using capability without codec
-};
-
 //////////////////////////////////////////////////////////////////////////////
 //
 // Class for handling non standard video capabilities
@@ -594,10 +585,12 @@ class H323CodecPluginNonStandardVideoCapability : public H323NonStandardVideoCap
   PCLASSINFO(H323CodecPluginNonStandardVideoCapability, H323NonStandardVideoCapability);
   public:
     H323CodecPluginNonStandardVideoCapability(const PluginCodec_Definition * codecDefn,
+                                              const OpalMediaFormat & mediaFormat,
                                               H323NonStandardCapabilityInfo::CompareFuncType compareFunc,
                                               const unsigned char * data, unsigned dataLen);
 
     H323CodecPluginNonStandardVideoCapability(const PluginCodec_Definition * codecDefn,
+                                              const OpalMediaFormat & mediaFormat,
                                               const unsigned char * data, unsigned dataLen);
 
     virtual PObject * Clone() const;
@@ -616,7 +609,8 @@ class H323CodecPluginGenericVideoCapability : public H323GenericVideoCapability,
   PCLASSINFO(H323CodecPluginGenericVideoCapability, H323GenericVideoCapability);
   public:
     H323CodecPluginGenericVideoCapability(const PluginCodec_Definition * codecDefn,
-				          const PluginCodec_H323GenericCodecData * data);
+                                          const OpalMediaFormat & mediaFormat,
+                                          const PluginCodec_H323GenericCodecData * data);
 
     virtual PObject * Clone() const;
 
@@ -628,16 +622,20 @@ class H323CodecPluginGenericVideoCapability : public H323GenericVideoCapability,
 // Class for handling H.261 plugin capabilities
 //
 
-class H323H261PluginCapability : public H323VideoPluginCapability
+class H323H261PluginCapability : public H323VideoCapability,
+                                 public H323PluginCapabilityInfo
 {
-  PCLASSINFO(H323H261PluginCapability, H323VideoPluginCapability);
+    PCLASSINFO(H323H261PluginCapability, H323VideoCapability);
   public:
-    H323H261PluginCapability(const PluginCodec_Definition * codecDefn);
+    H323H261PluginCapability(const PluginCodec_Definition * codecDefn, const OpalMediaFormat & mediaFormat);
 
     Comparison Compare(const PObject & obj) const;
 
     virtual PObject * Clone() const;
 
+    virtual PString GetFormatName() const;
+    virtual unsigned GetSubType() const;
+
     virtual PBoolean OnSendingPDU(
       H245_VideoCapability & pdu  /// PDU to set information on
     ) const;
@@ -656,16 +654,20 @@ class H323H261PluginCapability : public H323VideoPluginCapability
 // Class for handling H.263 plugin capabilities
 //
 
-class H323H263PluginCapability : public H323VideoPluginCapability
+class H323H263PluginCapability : public H323VideoCapability,
+                                 public H323PluginCapabilityInfo
 {
-  PCLASSINFO(H323H263PluginCapability, H323VideoPluginCapability);
+  PCLASSINFO(H323H263PluginCapability, H323VideoCapability);
   public:
-    H323H263PluginCapability(const PluginCodec_Definition * codecDefn);
+    H323H263PluginCapability(const PluginCodec_Definition * codecDefn, const OpalMediaFormat & mediaFormat);
 
     Comparison Compare(const PObject & obj) const;
 
     virtual PObject * Clone() const;
 
+    virtual PString GetFormatName() const;
+    virtual unsigned GetSubType() const;
+
     virtual PBoolean OnSendingPDU(
       H245_VideoCapability & pdu  /// PDU to set information on
     ) const;
@@ -677,7 +679,10 @@ class H323H263PluginCapability : public H323VideoPluginCapability
     virtual PBoolean OnReceivedPDU(
       const H245_VideoCapability & pdu  /// PDU to get information from
     );
-    virtual PBoolean IsMatch(const PASN_Choice & subTypePDU) const;
+    virtual PBoolean IsMatch(
+      const PASN_Choice & subTypePDU,     ///<  sub-type PDU of H323Capability
+      const PString & mediaPacketization  ///< Media packetization used
+    ) const;
 };
 
 #endif // OPAL_VIDEO
