@@ -21,13 +21,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21004 $
- * $Author: rjongbloed $
- * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
-#ifndef __OPALWAVFILE_H
-#define __OPALWAVFILE_H
+#ifndef OPAL_CODEC_OPALWAVFILE_H
+#define OPAL_CODEC_OPALWAVFILE_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -37,16 +37,14 @@
 
 #include <ptclib/pwavfile.h>
 
-namespace PWLibStupidLinkerHacks {
-  extern int opalwavfileLoader;
-};
+
+class OpalMediaFormat;
+
 
 /**This class is similar to the PWavFile class found in the PWlib
-   components library. However, it will tranparently convert all data
+   components library. However, it will transparently convert all data
    to/from PCM format, allowing applications to be unconcerned with 
-   the underlying data format. Note that this will only work with
-   sample-based formats that can be converted to/from PCM data, such as
-   uLaw and aLaw
+   the underlying data format.
   */
 
 class OpalWAVFile : public PWAVFile
@@ -64,14 +62,14 @@ class OpalWAVFile : public PWAVFile
 
        If a WAV file is being created, the type parameter can be used
        to create a PCM Wave file or a G.723.1 Wave file by using
-       #WaveType enum#
+       <code>WaveType</code> enum
 
-       The #PChannel::IsOpen()# function may be used after object
+       The <code>PChannel::IsOpen()</code> function may be used after object
        construction to determine if the file was successfully opened.
      */
     OpalWAVFile(
       OpenMode mode,            ///<  Mode in which to open the file.
-      int opts = ModeDefault,   ///<  #OpenOptions enum# for open operation.
+      int opts = ModeDefault,   ///<  <code>OpenOptions</code> enum for open operation.
       unsigned format = fmt_PCM ///<  Type of WAV File to create
     );
 
@@ -79,20 +77,29 @@ class OpalWAVFile : public PWAVFile
        the specified mode and with the specified options.
        If a WAV file is being created, the type parameter can be used
        to create a PCM Wave file or a G.723.1 Wave file by using
-       #WaveType enum#
+       <code>WaveType</code> enum.
 
-       The #PChannel::IsOpen()# function may be used after object
+       The <code>PChannel::IsOpen()</code> function may be used after object
        construction to determine if the file was successfully opened.
      */
     OpalWAVFile(
       const PFilePath & name,     ///<  Name of file to open.
       OpenMode mode = ReadWrite,  ///<  Mode in which to open the file.
-      int opts = ModeDefault,     ///<  #OpenOptions enum# for open operation.
-      unsigned format = fmt_PCM ///<  Type of WAV File to create
+      int opts = ModeDefault,     ///<  <code>OpenOptions</code> enum for open operation.
+      unsigned format = fmt_PCM   ///<  Type of WAV File to create
+    );
+
+
+    static bool AddMediaFormat(
+      const OpalMediaFormat & mediaFormat
     );
 };
 
-#endif // __OPALWAVFILE_H
+
+PFACTORY_LOAD(PWAVFileConverterULaw);
+
+
+#endif // OPAL_CODEC_OPALWAVFILE_H
 
 
 // End of File ///////////////////////////////////////////////////////////////
