@@ -19,9 +19,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22655 $
- * $Author: rjongbloed $
- * $Date: 2009-05-19 02:10:43 -0500 (Tue, 19 May 2009) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_H224_H323H224_H
@@ -31,19 +31,17 @@
 #pragma interface
 #endif
 
-#ifndef _PTLIB_H
-#include <ptlib.h>
-#endif
-
 #include <opal/buildopts.h>
 
 #if OPAL_HAS_H224
+#if OPAL_H323
 
 #include <h323/h323caps.h>
 
 #include <h224/h224.h>
 #include <h224/h224handler.h>
 
+
 /** This class describes the H.224 capability, using the H.323 Annex Q mode
 */
 class H323_H224_AnnexQCapability : public H323DataCapability
@@ -73,6 +71,7 @@ public:
   virtual PBoolean OnReceivedPDU(const H245_DataApplicationCapability & pdu);
 };
 
+
 /** This class describes the H.224 capability, using the HDLC tunneling mode
  */
 class H323_H224_HDLCTunnelingCapability : public H323DataCapability
@@ -108,6 +107,8 @@ public:
   H323_REGISTER_CAPABILITY(H323_H224_HDLCTunnelingCapability, GetOpalH224_HDLCTunneling().GetName()); \
 
 
+#endif // OPAL_H323
+
 #endif // OPAL_HAS_H224
 
 #endif // OPAL_H224_H323H224_H
