@@ -23,9 +23,9 @@
  *
  * Contributor(s): Frbass Franz <franz.fuerbass@infonova.at>
  *
- * $Revision: 21283 $
- * $Author: rjongbloed $
- * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_H323_H235AUTH_H
@@ -37,6 +37,11 @@
 
 #include <opal/buildopts.h>
 
+#if OPAL_H323
+
+#include <ptlib/pfactory.h>
+
+
 class H323TransactionPDU;
 class H225_CryptoH323Token;
 class H225_ArrayOf_AuthenticationMechanism;
@@ -47,9 +52,6 @@ class PASN_ObjectId;
 class PASN_Sequence;
 class PASN_Array;
 
-namespace PWLibStupidLinkerHacks {
-extern int h235AuthLoader;
-};
 
 /** This abtract class embodies an H.235 authentication mechanism.
     NOTE: descendants must have a Clone() function for correct operation.
@@ -72,7 +74,7 @@ class H235Authenticator : public PObject
     );
 
     virtual H235_ClearToken * CreateClearToken();
-    virtual H225_CryptoH323Token * CreateCryptoToken();
+    virtual H225_CryptoH323Token * CreateCryptoToken(bool digits);
 
     virtual PBoolean Finalise(
       PBYTEArray & rawPDU
@@ -123,9 +125,9 @@ class H235Authenticator : public PObject
     virtual PBoolean IsActive() const;
 
     void Enable(
-      PBoolean enab = PTrue
+      PBoolean enab = true
     ) { enabled = enab; }
-    void Disable() { enabled = PFalse; }
+    void Disable() { enabled = false; }
 
     const PString & GetRemoteId() const { return remoteId; }
     void SetRemoteId(const PString & id) { remoteId = id; }
@@ -157,6 +159,9 @@ class H235Authenticator : public PObject
     int      timestampGracePeriod;
 
     PMutex mutex;
+
+  private:
+    P_REMOVE_VIRTUAL(H225_CryptoH323Token *,CreateCryptoToken(),NULL);
 };
 
 
@@ -197,7 +202,7 @@ class H235AuthSimpleMD5 : public H235Authenticator
 
     virtual const char * GetName() const;
 
-    virtual H225_CryptoH323Token * CreateCryptoToken();
+    virtual H225_CryptoH323Token * CreateCryptoToken(bool digits);
 
     virtual ValidationResult ValidateCryptoToken(
       const H225_CryptoH323Token & cryptoToken,
@@ -220,6 +225,8 @@ class H235AuthSimpleMD5 : public H235Authenticator
     ) const;
 };
 
+PFACTORY_LOAD(H235AuthSimpleMD5);
+
 
 /** This class embodies a RADIUS compatible based authentication (aka Cisco
     Access Token or CAT).
@@ -259,13 +266,11 @@ class H235AuthCAT : public H235Authenticator
     ) const;
 };
 
+PFACTORY_LOAD(H235AuthCAT);
+
 
 #if OPAL_PTLIB_SSL
 
-namespace PWLibStupidLinkerHacks {
-  extern int h235AuthProcedure1Loader;
-};
-
 /** This class embodies the H.235 "base line Procedure 1" from Annex D.
 */
 class H235AuthProcedure1 : public H235Authenticator
@@ -278,7 +283,7 @@ class H235AuthProcedure1 : public H235Authenticator
 
     virtual const char * GetName() const;
 
-    virtual H225_CryptoH323Token * CreateCryptoToken();
+    virtual H225_CryptoH323Token * CreateCryptoToken(bool digits);
 
     virtual PBoolean Finalise(
       PBYTEArray & rawPDU
@@ -302,8 +307,11 @@ class H235AuthProcedure1 : public H235Authenticator
     virtual PBoolean UseGkAndEpIdentifiers() const;
 };
 
-#endif
+PFACTORY_LOAD(H235AuthProcedure1);
 
+#endif // OPAL_PTLIB_SSL
+
+#endif // OPAL_H323
 
 #endif //OPAL_H323_H235AUTH_H
 
