@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26128 $
- * $Author: rjongbloed $
- * $Date: 2011-07-07 21:36:57 -0500 (Thu, 07 Jul 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_H323_H323CAPS_H
@@ -280,7 +280,8 @@ class H323Capability : public PObject
     /**Compare the PDU part of the capability.
       */
     virtual PBoolean IsMatch(
-      const PASN_Choice & subTypePDU  ///<  sub-type PDU of H323Capability
+      const PASN_Choice & subTypePDU,     ///<  sub-type PDU of H323Capability
+      const PString & mediaPacketization  ///< Media packetization used
     ) const;
 
     /**Validate that the capability is usable given the connection.
@@ -355,6 +356,7 @@ class H323Capability : public PObject
 
   private:
     mutable OpalMediaFormat m_mediaFormat;
+    P_REMOVE_VIRTUAL(PBoolean, IsMatch(const PASN_Choice &) const, false);
 
   friend class H323Capabilities;
 };
@@ -839,7 +841,8 @@ class H323NonStandardAudioCapability : public H323AudioCapability,
     /**Compare the nonStandardData part of the capability, if applicable.
       */
     virtual PBoolean IsMatch(
-      const PASN_Choice & subTypePDU  ///<  sub-type PDU of H323Capability
+      const PASN_Choice & subTypePDU,     ///<  sub-type PDU of H323Capability
+      const PString & mediaPacketization  ///< Media packetization used
     ) const;
   //@}
 };
@@ -930,7 +933,8 @@ class H323GenericAudioCapability : public H323AudioCapability,
     /**Compare the generic part of the capability, if applicable.
      */
     virtual PBoolean IsMatch(
-      const PASN_Choice & subTypePDU  ///<  sub-type PDU of H323Capability
+      const PASN_Choice & subTypePDU,     ///<  sub-type PDU of H323Capability
+      const PString & mediaPacketization  ///< Media packetization used
     ) const;
   //@}
 };
@@ -1190,7 +1194,8 @@ class H323NonStandardVideoCapability : public H323VideoCapability,
     /**Compare the nonStandardData part of the capability, if applicable.
       */
     virtual PBoolean IsMatch(
-      const PASN_Choice & subTypePDU  ///<  sub-type PDU of H323Capability
+      const PASN_Choice & subTypePDU,     ///<  sub-type PDU of H323Capability
+      const PString & mediaPacketization  ///< Media packetization used
     ) const;
   //@}
 };
@@ -1279,7 +1284,8 @@ class H323GenericVideoCapability : public H323VideoCapability,
     /**Compare the generic part of the capability, if applicable.
      */
     virtual PBoolean IsMatch(
-      const PASN_Choice & subTypePDU  ///<  sub-type PDU of H323Capability
+      const PASN_Choice & subTypePDU,     ///<  sub-type PDU of H323Capability
+      const PString & mediaPacketization  ///< Media packetization used
     ) const;
   //@}
 };
@@ -1356,7 +1362,8 @@ class H323ExtendedVideoCapability : public H323GenericVideoCapability
     /**Compare the generic part of the capability, if applicable.
      */
     virtual PBoolean IsMatch(
-      const PASN_Choice & subTypePDU  ///<  sub-type PDU of H323Capability
+      const PASN_Choice & subTypePDU,     ///<  sub-type PDU of H323Capability
+      const PString & mediaPacketization  ///< Media packetization used
     ) const;
   //@}
 
@@ -1440,7 +1447,8 @@ class H323GenericControlCapability : public H323Capability,
     /**Compare the generic part of the capability, if applicable.
      */
     virtual PBoolean IsMatch(
-      const PASN_Choice & subTypePDU  ///<  sub-type PDU of H323Capability
+      const PASN_Choice & subTypePDU,     ///<  sub-type PDU of H323Capability
+      const PString & mediaPacketization  ///< Media packetization used
     ) const;
   //@}
 
@@ -1828,7 +1836,8 @@ class H323NonStandardDataCapability : public H323DataCapability,
     /**Compare the nonStandardData part of the capability, if applicable.
       */
     virtual PBoolean IsMatch(
-      const PASN_Choice & subTypePDU  ///<  sub-type PDU of H323Capability
+      const PASN_Choice & subTypePDU,     ///<  sub-type PDU of H323Capability
+      const PString & mediaPacketization  ///< Media packetization used
     ) const;
   //@}
 };
@@ -2310,7 +2319,8 @@ class H323Capabilities : public PObject
        NULL if no capability meeting the criteria was found
       */
     H323Capability * FindCapability(
-      const H245_DataType & dataType  ///<  H245 data type of codec
+      const H245_DataType & dataType,  ///<  H245 data type of codec
+      const PString & mediaPacketization = PString::Empty()  ///< Media packetization, if present
     ) const;
 
     /**Find the capability given the H.245 data type PDU.
@@ -2319,7 +2329,8 @@ class H323Capabilities : public PObject
        NULL if no capability meeting the criteria was found
       */
     H323Capability * FindCapability(
-      const H245_ModeElement & modeElement  ///<  H245 data type of codec
+      const H245_ModeElement & modeElement,  ///<  H245 data type of codec
+      const PString & mediaPacketization = PString::Empty()  ///< Media packetization, if present
     ) const;
 
     /**Find the capability given the type codecs.
@@ -2385,15 +2396,13 @@ class H323Capabilities : public PObject
       */
     OpalMediaFormatList GetMediaFormats() const;
 
-    const PStringSet & GetMediaPacketizations() const { return mediaPacketizations; }
+    const PStringSet & GetMediaPacketizations() const { return m_mediaPacketizations; }
   //@}
 
-  protected:
-    void SetMediaPacketizations(const PStringSet & packetizations) { mediaPacketizations = packetizations; }
-    
+  protected:    
     H323CapabilitiesList table;
     H323CapabilitiesSet  set;
-    PStringSet           mediaPacketizations;
+    PStringSet           m_mediaPacketizations;
 };
 
 
