@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26728 $
- * $Author: rjongbloed $
- * $Date: 2011-12-01 22:59:14 -0600 (Thu, 01 Dec 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_H323_H323CON_H
@@ -296,7 +296,7 @@ class H323Connection : public OpalRTPConnection
     /**Switch to/from FAX mode.
       */
     virtual bool SwitchFaxMediaStreams(
-      bool enableFax  ///< Enable FAX or return to audio mode
+      bool toT38  ///< Enable T.38 or return to audio mode
     );
 #endif
 
@@ -687,47 +687,6 @@ class H323Connection : public OpalRTPConnection
       H323Connection & secondaryCall  ///<  Secondary call for consultation
     );
 
-    /**Place the call on hold, suspending all media channels (H.450.4).  Note it is the responsibility 
-       of the application layer to delete the MOH Channel if music on hold is provided to the remote
-       endpoint.  So far only Local Hold has been implemented. 
-     */
-    bool HoldCall(
-      PBoolean localHold   ///<  true for Local Hold, false for Remote Hold
-    );
-
-    /**Retrieve the call from hold, activating all media channels (H.450.4).
-       This method examines the call hold state and performs the necessary
-       actions required to retrieve a Near-end or Remote-end call on hold.
-       NOTE: Only Local Hold is implemented so far. 
-    */
-    bool RetrieveCall();
-
-    /**Set the alternative media channel.  This channel can be used to provide
-       Media On Hold (MOH) for a near end call hold operation or to provide
-       Recorded Voice Anouncements (RVAs).  If this method is not called before
-       a call hold operation is attempted, no media on hold will be provided
-       for the held endpoint.
-      */
-    void SetHoldMedia(
-      PChannel * audioChannel
-    );
-
-    /**Determine if Meadia On Hold is enabled.
-      */
-    PBoolean IsMediaOnHold() const;
-
-    /**Determine if held.
-      */
-    PBoolean IsLocalHold() const;
-
-    /**Determine if held.
-      */
-    PBoolean IsRemoteHold() const;
-
-    /**Determine if the current call is held or in the process of being held.
-      */
-    PBoolean IsCallOnHold() const;
-
     /**Begin a call intrusion request.
        Calls h45011handler->IntrudeCall where SS pdu is added to Call Setup
        message.
@@ -1916,12 +1875,6 @@ class H323Connection : public OpalRTPConnection
       const H225_EndpointType & pdu ///<  PDU from which to extract application info.
     );
     
-    /**Get the remote party address.
-       This will return the "best guess" at an address to use in a
-       to call the user again later.
-      */
-    PString GetRemotePartyURL() const;
-    
     /**Get the remotes capability table for this connection.
      */
     const H323Capabilities & GetLocalCapabilities() const { return localCapabilities; }
@@ -2058,6 +2011,7 @@ class H323Connection : public OpalRTPConnection
     virtual void InternalEstablishedConnectionCheck();
     PBoolean InternalEndSessionCheck(PPER_Stream & strm);
     void SetRemoteVersions(const H225_ProtocolIdentifier & id);
+    void SetBearerCapabilities(H323SignalPDU & pdu) const;
     void MonitorCallStatus();
     PDECLARE_NOTIFIER(PThread, H323Connection, StartOutgoing);
     PDECLARE_NOTIFIER(PThread, H323Connection, NewOutgoingControlChannel);
@@ -2067,7 +2021,7 @@ class H323Connection : public OpalRTPConnection
 
     H323TransportAddress m_remoteConnectAddress;
     int                  remoteCallWaiting; // Number of call's waiting at the remote endpoint
-    PBoolean                 gatekeeperRouted;
+    bool                 gatekeeperRouted;
     unsigned             distinctiveRing;
     unsigned             callReference;
     unsigned             m_progressIndicator;
@@ -2081,16 +2035,15 @@ class H323Connection : public OpalRTPConnection
     H323Capabilities   remoteCapabilities; // Capabilities remote system supports
     unsigned           remoteMaxAudioDelayJitter;
     PTimer             roundTripDelayTimer;
-    WORD               maxAudioDelayJitter;
     unsigned           uuiesRequested;
     PString            gkAccessTokenOID;
     PBYTEArray         gkAccessTokenData;
-    PBoolean               addAccessTokenToSetup;
+    bool               addAccessTokenToSetup;
 
     H323Transport * signallingChannel;
     H323Transport * controlChannel;
     OpalListener  * controlListener;
-    PBoolean            h245Tunneling;
+    bool            h245Tunneling;
     H323SignalPDU * h245TunnelRxPDU;
     H323SignalPDU * h245TunnelTxPDU;
     H323SignalPDU * setupPDU;
@@ -2112,27 +2065,24 @@ class H323Connection : public OpalRTPConnection
 
     unsigned   h225version;
     unsigned   h245version;
-    PBoolean       h245versionSet;
-    PBoolean doH245inSETUP;
-    PBoolean lastPDUWasH245inSETUP;
+    bool       h245versionSet;
+    bool       doH245inSETUP;
+    bool       lastPDUWasH245inSETUP;
 
-    PBoolean mustSendDRQ;
-    PBoolean mediaWaitForConnect;
-    PBoolean transmitterSidePaused;
-    bool     remoteTransmitPaused;
-    PBoolean earlyStart;
+    bool       mustSendDRQ;
+    bool       mediaWaitForConnect;
+    bool       transmitterSidePaused;
+    bool       remoteTransmitPaused;
+    bool       earlyStart;
     PString    t38ModeChangeCapabilities;
     PSyncPoint digitsWaitFlag;
-    PBoolean       endSessionNeeded;
+    bool       endSessionNeeded;
     PSyncPoint endSessionReceived;
     PTimer     enforcedDurationLimit;
-
-    // Used as part of a local call hold operation involving MOH
-    PChannel * holdMediaChannel;
-    PBoolean       isConsultationTransfer;
+    bool       isConsultationTransfer;
 
     /** Call Intrusion flag and parameters */
-    PBoolean     isCallIntrusion;
+    bool     isCallIntrusion;
     unsigned callIntrusionProtectionLevel;
 
     enum FastStartStates {
@@ -2180,8 +2130,6 @@ class H323Connection : public OpalRTPConnection
 #endif
 
   private:
-    PChannel * SwapHoldMediaChannels(PChannel * newChannel);
-
     P_REMOVE_VIRTUAL_VOID(CleanUpOnCallEnd());
     P_REMOVE_VIRTUAL_VOID(OnCleared());
 };
