@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25678 $
- * $Author: rjongbloed $
- * $Date: 2011-05-05 17:46:37 -0500 (Thu, 05 May 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_H323_H323EP_H
@@ -145,7 +145,7 @@ class H323EndPoint : public OpalRTPEndPoint
       OpalCall & call,                  ///<  Owner of connection
       const PString & party,            ///<  Remote party to call
       void * userData  = NULL,          ///<  Arbitrary data to pass to connection
-      unsigned int options = NULL,      ///<  options to pass to conneciton
+      unsigned int options = 0,         ///<  options to pass to connection
       OpalConnection::StringOptions * stringOptions = NULL
     );
   //@}
@@ -494,14 +494,6 @@ class H323EndPoint : public OpalRTPEndPoint
       const PString & secondaryCallToken  ///<  Token of secondary call
     );
 
-    /**Place the call on hold, suspending all media channels (H.450.4)
-    * NOTE: Only Local Hold is implemented so far. 
-    */
-    void HoldCall(
-      const PString & token,        ///<  Existing connection to be transferred
-      PBoolean localHold   ///<  true for Local Hold, false for Remote Hold
-    );
-
     /** Initiate Call intrusion
         Designed similar to MakeCall function
       */
