@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21847 $
- * $Author: csoutheren $
- * $Date: 2008-12-22 00:38:20 +0000 (Mon, 22 Dec 2008) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_H323_H323PDU_H
@@ -214,11 +214,11 @@ class H323SignalPDU : public H225_H323_UserInformation
        incoming connection.
       */
     PString GetDestinationAlias(
-      PBoolean firstAliasOnly = PFalse   ///<  Only return the first possible alias
+      PBoolean firstAliasOnly = false   ///<  Only return the first possible alias
     ) const;
 
     /**Get the source endpoints identification as a phone number.
-       This returns PFalse if the remote never provided any alias or Q.931
+       This returns false if the remote never provided any alias or Q.931
        field that indicated a valid e.164 telephone number.
       */
     PBoolean GetSourceE164(
@@ -226,7 +226,7 @@ class H323SignalPDU : public H225_H323_UserInformation
     ) const;
 
     /**Get the destiation  phone number.
-       This returns PFalse if the remote never provided any alias or Q.931
+       This returns false if the remote never provided any alias or Q.931
        field that indicated a valid e.164 telephone number.
       */
     PBoolean GetDestinationE164(
@@ -244,11 +244,7 @@ class H323SignalPDU : public H225_H323_UserInformation
       */
     void SetQ931Fields(
       const H323Connection & connection,
-      PBoolean insertPartyNumbers = PFalse,
-      unsigned plan = 1,
-      unsigned type = 0,
-      int presentation = -1,
-      int screening = -1
+      bool insertPartyNumbers = false
     );
 
 #ifdef OPAL_H460
@@ -384,6 +380,23 @@ class H323ControlPDU : public H245_MultimediaSystemControlMessage
       unsigned type
     );
 
+    H245_GenericMessage & BuildGenericRequest(
+      const PString & identifier,
+      unsigned subMsgId
+    );
+    H245_GenericMessage & BuildGenericResponse(
+      const PString & identifier,
+      unsigned subMsgId
+    );
+    H245_GenericMessage & BuildGenericCommand(
+      const PString & identifier,
+      unsigned subMsgId
+    );
+    H245_GenericMessage & BuildGenericIndication(
+      const PString & identifier,
+      unsigned subMsgId
+    );
+
     H245_FunctionNotUnderstood & BuildFunctionNotUnderstood(
       const H323ControlPDU & pdu
     );
@@ -500,9 +513,53 @@ bool H323SetRTPPacketization(
 PString H323GetRTPPacketization(
   const H245_RTPPayloadType & rtpPacketization
 );
-bool H323GetRTPPacketization(
-  OpalMediaFormat & mediaFormat,
-  const H245_RTPPayloadType & rtpPacketization
+
+PString H323GetCapabilityIdentifier(
+  const H245_CapabilityIdentifier & capId
+);
+bool H323SetCapabilityIdentifier(
+  const PString & str,
+  H245_CapabilityIdentifier & capId
+);
+
+const H245_ParameterValue * H323GetGenericParameter(
+  const H245_ArrayOf_GenericParameter & params,
+  unsigned ordinal
+);
+bool H323GetGenericParameterBoolean(
+  const H245_ArrayOf_GenericParameter & params,
+  unsigned ordinal
+);
+unsigned H323GetGenericParameterInteger(
+  const H245_ArrayOf_GenericParameter & params,
+  unsigned ordinal,
+  unsigned defValue = 0,
+  H245_ParameterValue::Choices subType = H245_ParameterValue::e_unsignedMin
+);
+H245_ParameterValue * H323AddGenericParameter(
+  H245_ArrayOf_GenericParameter & params,
+  unsigned ordinal
+);
+void H323AddGenericParameterBoolean(
+  H245_ArrayOf_GenericParameter & params,
+  unsigned ordinal,
+  bool value
+);
+void H323AddGenericParameterInteger(
+  H245_ArrayOf_GenericParameter & params,
+  unsigned ordinal,
+  unsigned value,
+  H245_ParameterValue::Choices subType
+);
+void H323AddGenericParameterString(
+  H245_ArrayOf_GenericParameter & params,
+  unsigned ordinal,
+  const PString & value
+);
+void H323AddGenericParameterOctets(
+  H245_ArrayOf_GenericParameter & params,
+  unsigned ordinal,
+  const PBYTEArray & value
 );
 
 #if PTRACING
