@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21293 $
- * $Author: rjongbloed $
- * $Date: 2008-10-12 23:24:41 +0000 (Sun, 12 Oct 2008) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_H323_H450PDU_H
@@ -498,9 +498,9 @@ class H4502Handler : public H450xHandler
      */
     const PString& getTransferringCallToken() const { return transferringCallToken; }
 
-    /**Set the 'consultationTransfer' member to PTrue (indicating a successful transfer)
+    /**Set the 'consultationTransfer' member to true (indicating a successful transfer)
      */
-    void SetConsultationTransferSuccess() { consultationTransfer = PTrue; }
+    void SetConsultationTransferSuccess() { consultationTransfer = true; }
 
     /**Was the transfer through consultation successful.
      */
