@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21293 $
- * $Author: rjongbloed $
- * $Date: 2008-10-12 23:24:41 +0000 (Sun, 12 Oct 2008) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_H323_Q931_H
@@ -149,17 +149,20 @@ class Q931 : public PObject
     };
 
     void SetBearerCapabilities(
-      InformationTransferCapability capability,
+      InformationTransferCapability capability, ///< Bearer cability enum
       unsigned transferRate,        ///<  Number of 64k B channels
       unsigned codingStandard = 0,  ///<  0 = ITU-T standardized coding
       unsigned userInfoLayer1 = 5   ///<  5 = Recommendations H.221 and H.242
     );
+    void SetBearerCapabilities(
+      const PString & caps  ///< String of comma separated integers or hex for all cap bytes
+    );
 
     PBoolean GetBearerCapabilities(
-      InformationTransferCapability & capability,
-      unsigned & transferRate,        ///<  Number of 64k B channels
-      unsigned * codingStandard = NULL,
-      unsigned * userInfoLayer1 = NULL
+      InformationTransferCapability & capability, ///< Bearer cability enum
+      unsigned & transferRate,          ///<  Number of 64k B channels
+      unsigned * codingStandard = NULL, ///<  0 = ITU-T standardized coding
+      unsigned * userInfoLayer1 = NULL  ///<  5 = Recommendations H.221 and H.242
     );
 
     enum CauseValues {
@@ -204,7 +207,7 @@ class Q931 : public PObject
     friend ostream & operator<<(ostream & strm, CauseValues cause);
 
     void SetCause(
-      CauseValues value,
+      CauseValues value,      ///<  Cause code
       unsigned standard = 0,  ///<  0 = ITU-T standardized coding
       unsigned location = 0   ///<  0 = User
     );
@@ -233,7 +236,7 @@ class Q931 : public PObject
       CallState_ErrorInIE             = 0x100
     };
     void SetCallState(
-      CallStates value,
+      CallStates value,      ///< Call state code
       unsigned standard = 0  ///<  0 = ITU-T standardized coding
     );
     CallStates GetCallState(
@@ -316,10 +319,10 @@ class Q931 : public PObject
 
     void SetCallingPartyNumber(
       const PString & number, ///<  Number string
-      unsigned plan = 1,      ///<  1 = ISDN/Telephony numbering system
+      unsigned plan = 1,      ///<  1 = ISDN/Telephony numbering system, see Q.931 Table 4-11 for more
       unsigned type = 0,      ///<  0 = Unknown number type
-      int presentation = -1,  ///<  0 = presentation allowed, -1 = no octet3a
-      int screening = -1      ///<   0 = user provided, not screened
+      int presentation = -1,  ///<  0 = presentation allowed, 1 = presentation restricted, -1 = no octet3a
+      int screening = -1      ///<  0 = user provided, not screened, -1 = no octet3a
     );
     PBoolean GetCallingPartyNumber(
       PString & number,               ///<  Number string
