@@ -24,78 +24,13 @@
  *
  * The author of this code is Derek J Smithies
  *
- *
- *  $Log: callprocessor.h,v $
- *  Revision 1.9  2007/04/19 06:17:21  csoutheren
- *  Fixes for precompiled headers with gcc
- *
- *  Revision 1.8  2007/01/18 04:45:16  csoutheren
- *  Messy, but simple change to add additional options argument to OpalConnection constructor
- *  This allows the provision of non-trivial arguments for connections
- *
- *  Revision 1.7  2007/01/17 22:27:52  dereksmithies
- *  Correctly sends DTMF to remote node. Tidy up string handling.
- *
- *  Revision 1.6  2007/01/17 03:48:13  dereksmithies
- *  Tidy up comments, remove leaks, improve reporting of packet types.
- *
- *  Revision 1.5  2007/01/16 03:17:42  dereksmithies
- *  tidyup of comments. Remove unused variables.
- *  Guarantee that media frames are sent with a monotonically increasing timestamp
- *
- *  Revision 1.4  2007/01/11 03:02:15  dereksmithies
- *  Remove the previous audio buffering code, and switch to using the jitter
- *  buffer provided in Opal. Reduce the verbosity of the log mesasges.
- *
- *  Revision 1.3  2006/09/22 00:33:19  csoutheren
- *  Changed PAtomicInteger to PBoolean
- *
- *  Revision 1.2  2006/09/11 03:08:51  dereksmithies
- *  Add fixes from Stephen Cook (sitiveni@gmail.com) for new patches to
- *  improve call handling. Notably, IAX2 call transfer. Many thanks.
- *  Thanks also to the Google summer of code for sponsoring this work.
- *
- *  Revision 1.1  2006/08/09 03:46:39  dereksmithies
- *  Add ability to register to a remote Asterisk box. The iaxProcessor class is split
- *  into a callProcessor and a regProcessor class.
- *  Big thanks to Stephen Cook, (sitiveni@gmail.com) for this work.
- *
- *  Revision 1.7  2005/09/05 01:19:43  dereksmithies
- *  add patches from Adrian Sietsma to avoid multiple hangup packets at call end,
- *  and stop the sending of ping/lagrq packets at call end. Many thanks.
- *
- *  Revision 1.6  2005/08/26 03:07:38  dereksmithies
- *  Change naming convention, so all class names contain the string "IAX2"
- *
- *  Revision 1.5  2005/08/25 03:26:06  dereksmithies
- *  Add patch from Adrian Sietsma to correctly set the packet timestamps under windows.
- *  Many thanks.
- *
- *  Revision 1.4  2005/08/24 04:56:25  dereksmithies
- *  Add code from Adrian Sietsma to send FullFrameTexts and FullFrameDtmfs to
- *  the remote end.  Many Thanks.
- *
- *  Revision 1.3  2005/08/24 01:38:38  dereksmithies
- *  Add encryption, iax2 style. Numerous tidy ups. Use the label iax2, not iax
- *
- *  Revision 1.2  2005/08/04 08:14:17  rjongbloed
- *  Fixed Windows build under DevStudio 2003 of IAX2 code
- *
- *  Revision 1.1  2005/07/30 07:01:32  csoutheren
- *  Added implementation of IAX2 (Inter Asterisk Exchange 2) protocol
- *  Thanks to Derek Smithies of Indranet Technologies Ltd. for
- *  writing and contributing this code
- *
- *
- *
- *
- *
- *
- *
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
-#ifndef CALLPROCESSOR_H
-#define CALLPROCESSOR_H
+#ifndef OPAL_IAX2_CALLPROCESSOR_H
+#define OPAL_IAX2_CALLPROCESSOR_H
 
 #ifndef _PTLIB_H
 #include <ptlib.h>
@@ -103,6 +38,8 @@
 
 #include <opal/buildopts.h>
 
+#if OPAL_IAX2
+
 #include <opal/connection.h>
 
 #include <iax2/processor.h>
@@ -139,9 +76,6 @@ class IAX2CallProcessor : public IAX2Processor
   /**Get the iax2 encryption info */
   IAX2Encryption & GetEncryptionInfo() { return encryption; }
 
-  /**Call back from the IAX2Connection class */
-  virtual void Release(OpalConnection::CallEndReason releaseReason = OpalConnection::EndedByLocalUser);
-
   /**From the IAX2Connection class. CAlling this sends a hangup frame */
   void ClearCall(OpalConnection::CallEndReason releaseReason = OpalConnection::EndedByLocalUser);
 
@@ -169,7 +103,7 @@ class IAX2CallProcessor : public IAX2Processor
   */
   virtual PBoolean SetUpConnection();
 
-  /**Return PTrue if the remote info in the frame matches the remote info in
+  /**Return true if the remote info in the frame matches the remote info in
      this connection */
   PBoolean Matches(IAX2Frame *frame) { return remote == (frame->GetRemoteInfo()); }
   
@@ -181,7 +115,7 @@ class IAX2CallProcessor : public IAX2Processor
      in/out packets) to be printed*/
   void ReportStatistics();  
 
-  /**Return PTrue if the arg matches the source call number for this connection */
+  /**Return true if the arg matches the source call number for this connection */
   PBoolean MatchingLocalCallNumber(PINDEX compare) { return (compare == remote.SourceCallNumber()); }  
   
   /**Get the bit pattern of the selected codec */
@@ -234,6 +168,10 @@ class IAX2CallProcessor : public IAX2Processor
   /**Get the username*/
   PString GetUserName() const;
   
+  /**Report the calling name - which we send to the remote endpoint
+     when making a call */
+  PString GetCallingName() const { return callingName; }
+
   /**Set the password for when we connect to a remote node
      we use it as authentication.  Note this must only be
      used before the main thread is started.  This is optional
@@ -319,8 +257,9 @@ class IAX2CallProcessor : public IAX2Processor
   /**Internal method to process an incoming network frame of type
      IAX2FullFrameProtocol.
      
-  A frame of FullFrameProtocol type is labelled as AST_FRAME_IAX in the asterisk souces,
-  It will contain 0, 1, 2 or more Information Elements (Ie) in the data section.*/
+  A frame of FullFrameProtocol type is labelled as AST_FRAME_IAX in
+  the asterisk souces, It will contain 0, 1, 2 or more Information
+  Elements (Ie) in the data section.*/
   virtual PBoolean ProcessNetworkFrame(IAX2FullFrameProtocol * src);
   
   /**Internal method to process an incoming network frame of type
@@ -356,7 +295,7 @@ class IAX2CallProcessor : public IAX2Processor
      The data in the array is already compressed. */
   void SendSoundMessage(PBYTEArray *sound);
   
-  /**Sends a transfer message ONLY if doTransfer is PTrue*/
+  /**Sends a transfer message ONLY if doTransfer is true*/
   void SendTransferMessage();
   
   /**Send a message to put the remote connection on hold*/
@@ -498,13 +437,28 @@ class IAX2CallProcessor : public IAX2Processor
      Provision device  */
   void ProcessIaxCmdProvision(IAX2FullFrameProtocol *src);
   
-  /** Process a FullFrameProtocol class, where the sub Class value is Download
-     firmware  */
+  /** Process a FullFrameProtocol class, where the sub Class value is
+     Download firmware  */
   void ProcessIaxCmdFwDownl(IAX2FullFrameProtocol *src);
   
-  /** Process a FullFrameProtocol class, where the sub Class value is Firmware
-     Data  */
+  /** Process a FullFrameProtocol class, where the sub Class value is
+     Firmware Data  */
   void ProcessIaxCmdFwData(IAX2FullFrameProtocol *src);
+
+  /** Process a FullFrameProtocol class, where the sub Class value is
+     CallToken. This is an authentication like packet that IAX2 added
+     in sep 2009 to prevent a DOS attack from too many incoming
+     callers.  
+
+     Reply with a NEW packet and the calltoken from supplied frame.*/
+  void ProcessIaxCmdCallToken(IAX2FullFrameProtocol *src);
+
+  /**We have to send a new frame to the remote host. This builds the
+     frame, and adds many of the information elements required. does
+     not add the calltoken ie - that is added elsewhere. This method
+     assumes this class already holds the relevant information on the
+     destination box */
+  IAX2FullFrameProtocol *BuildNewFrameForSending(IAX2FullFrameProtocol *inReplyTo = NULL);
   
   /**Count of the number of sound frames sent */
   PAtomicInteger audioFramesSent;
@@ -590,35 +544,35 @@ class IAX2CallProcessor : public IAX2Processor
   unsigned short callStatus;
   
   /** Mark call status as having sent a iaxcmdRinging packet */
-  void SetCallSentRinging(PBoolean newValue = PTrue) 
+  void SetCallSentRinging(PBoolean newValue = true) 
     { if (newValue) callStatus |= callSentRinging; else callStatus &= ~callSentRinging; }
   
   /** Mark call status as having received a new packet */
-  void SetCallNewed(PBoolean newValue = PTrue) 
+  void SetCallNewed(PBoolean newValue = true) 
     { if (newValue) callStatus |= callNewed; else callStatus &= ~callNewed; }
   
   /** Mark call status Registered (argument determines flag status) */
-  void SetCallRegistered(PBoolean newValue = PTrue) 
+  void SetCallRegistered(PBoolean newValue = true) 
     { if (newValue) callStatus |= callRegistered; else callStatus &= ~callRegistered; }
   
   /** Mark call status Authorised (argument determines flag status) */
-  void SetCallAuthorised(PBoolean newValue = PTrue) 
+  void SetCallAuthorised(PBoolean newValue = true) 
     { if (newValue) callStatus |= callAuthorised; else callStatus &= ~callAuthorised; }
   
   /** Mark call status Accepted (argument determines flag status) */
-  void SetCallAccepted(PBoolean newValue = PTrue) 
+  void SetCallAccepted(PBoolean newValue = true) 
     { if (newValue) callStatus |= callAccepted; else callStatus &= ~callAccepted; }
   
   /** Mark call status Ringing (argument determines flag status) */
-  void SetCallRinging(PBoolean newValue = PTrue) 
+  void SetCallRinging(PBoolean newValue = true) 
     { if (newValue) callStatus |= callRinging; else callStatus &= ~callRinging; }
   
   /** Mark call status Answered (argument determines flag status) */
-  void SetCallAnswered(PBoolean newValue = PTrue) 
+  void SetCallAnswered(PBoolean newValue = true) 
     { if (newValue) callStatus |= callAnswered; else callStatus &= ~callAnswered; }
 
   /** Mark call status as terminated (is processing IAX2 hangup packets etc)*/
-  void SetCallTerminating(PBoolean newValue = PTrue) 
+  void SetCallTerminating(PBoolean newValue = true) 
     { if (newValue) callStatus |= callTerminating; else callStatus &= ~callTerminating; }
   
   /** See if any of the flag bits are on, which indicate this call is actually active */
@@ -718,6 +672,21 @@ class IAX2CallProcessor : public IAX2Processor
   /**Process a full frame and respond accordingly to it*/
   virtual void ProcessFullFrame(IAX2FullFrame & fullFrame);
   
+  /**The calling Name - used when we send a new packet to the remote
+     host - this name is put the new packet */
+  PString callingName;
+
+  /**The calling extension - derived from the destination of the
+     call */
+  PString callingExtension;
+
+  /**The Dnid - used in creating a cmdNew packet for creating call */
+  PString callingDnid;
+
+  /**the context index - used in creating a cmdNew packet for creating
+     call */
+  PString callingContext;
+
   /**Optional username for when we connect to a remote node
      we use it as authentication.  Note this must only be
      set before the main thread is started.*/
@@ -739,14 +708,19 @@ class IAX2CallProcessor : public IAX2Processor
   
   /**The context to call for a transfer -- this is optional*/
   PString transferCalledContext;    
+
+  /**Flag to indicate we should not send the hangup frame on call
+     termination.  Normally, when a call ends, we send a hangup
+     frame. However, in the case where the remote endpoint rejects our
+     call, we are not supposed to send a hangup frame */
+  PBoolean suppressHangupFrame;
 };
 
 /////////////////////////////////////////////////////////////////////////
 
-/* The comment below is magic for those who use emacs to edit this file. */
-/* With the comment below, the tab key does auto indent to 4 spaces.     */
-
-/*
+/* The comment below is magic for those who use emacs to edit this file.
+ * With the comment below, the tab key does auto indent to 2 spaces.    
+ *
  * Local Variables:
  * mode:c
  * c-basic-offset:2
@@ -754,4 +728,6 @@ class IAX2CallProcessor : public IAX2Processor
  */
 
 
-#endif // CALLPROCESSOR_H
+#endif // OPAL_IAX2
+
+#endif // OPAL_IAX2_CALLPROCESSOR_H
