@@ -25,9 +25,9 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 24606 $
- * $Author: dereksmithies $
- * $Date: 2010-07-28 22:51:05 -0500 (Wed, 28 Jul 2010) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_IAX2_MEDIASTRM_H
@@ -89,12 +89,6 @@ class OpalIAX2MediaStream : public OpalMediaStream
     */
     virtual PBoolean Start();
 
-    /**Close the media stream.
- 
-       The default does nothing.
-      */
-    virtual PBoolean Close();
- 
     /**
        Goes to the IAX2Connection class, and removes a packet from the connection. The connection class turned the media 
        packet into a RTP_DataFrame class, and jitter buffered it.
@@ -122,6 +116,8 @@ class OpalIAX2MediaStream : public OpalMediaStream
   //@}
 
   protected:
+    virtual void InternalClose() { }
+
     /**The connection is the source/sink of our data packets */
     IAX2Connection & connection;
 
