@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26270 $
- * $Author: rjongbloed $
- * $Date: 2011-08-03 03:29:01 -0500 (Wed, 03 Aug 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_IM_IM_H
@@ -411,6 +411,9 @@ class OpalIMMediaStream : public OpalMediaStream
 
     bool ReadPacket(RTP_DataFrame & packet);
     bool WritePacket(RTP_DataFrame & packet);
+
+  protected:
+    virtual void InternalClose() { }
 };
 
 #endif // OPAL_HAS_IM
