@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22675 $
- * $Author: rjongbloed $
- * $Date: 2009-05-19 23:23:29 -0500 (Tue, 19 May 2009) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_IM_T140_H
@@ -71,6 +71,8 @@ class T140String : public PBYTEArray
     PINDEX AppendUnicode16(WORD c);
     PINDEX AppendUTF(const BYTE * utf, PINDEX utfLen);
 
+    bool AsString(PString & str);
+
   protected:
     PINDEX SetUTF(BYTE * ptr, WORD c);
     PINDEX length;
