@@ -26,9 +26,9 @@
  * 
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23898 $
- * $Author: rjongbloed $
- * $Date: 2009-12-23 19:57:27 -0600 (Wed, 23 Dec 2009) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_LIDS_LID_H
@@ -112,13 +112,13 @@ class OpalLineInterfaceDevice : public PObject
       */
     virtual PBoolean IsLinePresent(
       unsigned line,      ///<  Number of line
-      PBoolean force = PFalse  ///<  Force test, do not optimise
+      PBoolean force = false  ///<  Force test, do not optimise
     );
 
 
     /**Determine if line is currently off hook.
        This function implies that the state is debounced and that a return
-       value of PTrue indicates that the phone is really off hook. That is
+       value of true indicates that the phone is really off hook. That is
        hook flashes and winks are masked out.
       */
     virtual PBoolean IsLineOffHook(
@@ -132,7 +132,7 @@ class OpalLineInterfaceDevice : public PObject
       */
     virtual PBoolean SetLineOffHook(
       unsigned line,        ///<  Number of line
-      PBoolean newState = PTrue  ///<  New state to set
+      PBoolean newState = true  ///<  New state to set
     ) = 0;
 
     /**Set the hook state of the line.
@@ -140,7 +140,7 @@ class OpalLineInterfaceDevice : public PObject
       */
     virtual PBoolean SetLineOnHook(
       unsigned line        ///<  Number of line
-    ) { return SetLineOffHook(line, PFalse); }
+    ) { return SetLineOffHook(line, false); }
 
     /**Set the hook state off then straight back on again.
        This will only operate if the line is currently off hook.
@@ -150,14 +150,14 @@ class OpalLineInterfaceDevice : public PObject
       unsigned flashTime = 200    ///<  Time for hook flash in milliseconds
     );
 
-    /**Return PTrue if a hook flash has been detected
+    /**Return true if a hook flash has been detected
       */
     virtual PBoolean HasHookFlash(unsigned line);
 
 
     /**Determine if line is ringing.
        This function implies that the state is "debounced" and that a return
-       value of PTrue indicates that the phone is still ringing and it is not
+       value of true indicates that the phone is still ringing and it is not
        simply a pause in the ring cadence.
 
        If cadence is not NULL then it is set with the bit pattern for the
@@ -237,7 +237,7 @@ class OpalLineInterfaceDevice : public PObject
       */
     virtual PBoolean IsLineDisconnected(
       unsigned line,   ///<  Number of line
-      PBoolean checkForWink = PTrue
+      PBoolean checkForWink = true ///< Flag to check for remote hang up
     );
 
 
@@ -384,14 +384,14 @@ class OpalLineInterfaceDevice : public PObject
       */
     virtual PBoolean EnableAudio(
       unsigned line,      ///<  Number of line
-      PBoolean enable = PTrue
+      PBoolean enable = true ///< Enable/disable audio
     );
 
     /**Disable audio for the line.
       */
     PBoolean DisableAudio(
       unsigned line   ///<  Number of line
-    ) { return EnableAudio(line, PFalse); }
+    ) { return EnableAudio(line, false); }
 
     /**Determine if audio for the line is enabled.
       */
@@ -498,7 +498,7 @@ class OpalLineInterfaceDevice : public PObject
     virtual PBoolean GetCallerID(
       unsigned line,      ///<  Number of line
       PString & idString, ///<  ID string returned
-      PBoolean full = PFalse   ///<  Get full information in idString
+      PBoolean full = false   ///<  Get full information in idString
     );
 
     /**Set Caller ID information.
@@ -531,7 +531,7 @@ class OpalLineInterfaceDevice : public PObject
       */
     virtual PBoolean SendVisualMessageWaitingIndicator(
       unsigned line,            ///<  Number of line
-      PBoolean on
+      PBoolean on               ///< Flag for VMWI on/off
     );
 
 
@@ -710,7 +710,7 @@ class OpalLineInterfaceDevice : public PObject
        The takes the line off hook, waits for dial tone, and transmits the
        specified number as DTMF tones.
 
-       If the requireTones flag is PTrue the call is aborted of the call
+       If the requireTones flag is true the call is aborted of the call
        progress tones are not detected. Otherwise the call proceeds with short
        delays while it tries to detect the call progress tones.
 
@@ -831,8 +831,8 @@ class OpalLineInterfaceDevice : public PObject
       * start recording audio
       */
     virtual PBoolean RecordAudioStart(
-      unsigned line,            /// line
-      const PString & filename  /// File Name
+      unsigned line,            ///< line
+      const PString & filename  ///< File Name
     );
     
     /**
@@ -840,7 +840,7 @@ class OpalLineInterfaceDevice : public PObject
      */
         
     virtual PBoolean RecordAudioStop(
-      unsigned line            /// line
+      unsigned line            ///< line
     );
     
 
@@ -943,13 +943,13 @@ class OpalLine : public PObject
     /**Determine if a physical line is present on the logical line.
       */
     virtual PBoolean IsPresent(
-      PBoolean force = PFalse  ///<  Force test, do not optimise
+      PBoolean force = false  ///<  Force test, do not optimise
     ) { return device.IsLinePresent(lineNumber, force); }
 
 
     /**Determine if line is currently off hook.
        This function implies that the state is debounced and that a return
-       value of PTrue indicates that the phone is really off hook. That is
+       value of true indicates that the phone is really off hook. That is
        hook flashes and winks are masked out.
       */
     virtual PBoolean IsOffHook() { return device.IsLineOffHook(lineNumber); }
@@ -959,12 +959,12 @@ class OpalLine : public PObject
        a standard telephone handset. The hook state is determined by external
        hardware and cannot be changed by the software.
       */
-    virtual PBoolean SetOffHook() { return device.SetLineOffHook(lineNumber, PTrue); }
+    virtual PBoolean SetOffHook() { return device.SetLineOffHook(lineNumber, true); }
 
     /**Set the hook state of the line.
        This is the complement of SetLineOffHook().
       */
-    virtual PBoolean SetOnHook() { return device.SetLineOffHook(lineNumber, PFalse); }
+    virtual PBoolean SetOnHook() { return device.SetLineOffHook(lineNumber, false); }
 
     /**Set the hook state off then straight back on again.
        This will only operate if the line is currently off hook.
@@ -973,14 +973,14 @@ class OpalLine : public PObject
       unsigned flashTime = 200    ///<  Time for hook flash in milliseconds
     ) { return device.HookFlash(lineNumber, flashTime); }
 
-    /**Return PTrue if a hook flash has been detected
+    /**Return true if a hook flash has been detected
       */
     virtual PBoolean HasHookFlash() { return device.HasHookFlash(lineNumber); }
 
 
     /**Determine if line is ringing.
        This function implies that the state is "debounced" and that a return
-       value of PTrue indicates that the phone is still ringing and it is not
+       value of true indicates that the phone is still ringing and it is not
        simply a pause in the ring cadence.
 
        If cadence is not NULL then it is set with the bit pattern for the
@@ -1145,12 +1145,12 @@ class OpalLine : public PObject
     /**Enable audio for the line.
       */
     virtual PBoolean EnableAudio(
-      PBoolean enable = PTrue
+      PBoolean enable = true
     ) { return device.EnableAudio(lineNumber, enable); }
 
     /**Disable audio for the line.
       */
-    PBoolean DisableAudio() { return EnableAudio(PFalse); }
+    PBoolean DisableAudio() { return EnableAudio(false); }
 
     /**Determine if audio is ebabled for the line.
       */
@@ -1231,7 +1231,7 @@ class OpalLine : public PObject
       */
     virtual PBoolean GetCallerID(
       PString & idString, ///<  ID string returned
-      PBoolean full = PFalse   ///<  Get full information in idString
+      PBoolean full = false   ///<  Get full information in idString
     ) { return device.GetCallerID(lineNumber, idString, full); }
 
     /**Set Caller ID information.
@@ -1337,7 +1337,7 @@ class OpalLine : public PObject
        The takes the line off hook, waits for dial tone, and transmits the
        specified number as DTMF tones.
 
-       If the requireTones flag is PTrue the call is aborted of the call
+       If the requireTones flag is true the call is aborted of the call
        progress tones are not detected. Otherwise the call proceeds with short
        delays while it tries to detect the call progress tones.
 
