@@ -26,9 +26,9 @@
  * 
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24766 $
- * $Author: rjongbloed $
- * $Date: 2010-09-30 02:18:52 -0500 (Thu, 30 Sep 2010) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_LIDS_LIDEP_H
@@ -581,12 +581,6 @@ class OpalLineMediaStream : public OpalMediaStream
       */
     virtual PBoolean Open();
 
-    /**Close the media stream.
-
-       The default does nothing.
-      */
-    virtual PBoolean Close();
-
     /**Read an RTP frame of data from the source media stream.
        The default behaviour simply calls ReadData() on the data portion of the
        RTP_DataFrame and sets the frames timestamp and marker from the internal
@@ -661,6 +655,8 @@ class OpalLineMediaStream : public OpalMediaStream
   //@}
 
   protected:
+    virtual void InternalClose();
+
     OpalLine & line;
     bool       notUsingRTP;
     bool       useDeblocking;
