@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21293 $
- * $Author: rjongbloed $
- * $Date: 2008-10-12 18:24:41 -0500 (Sun, 12 Oct 2008) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_LIDS_LIDPLUGINMGR_H
@@ -131,13 +131,13 @@ class OpalPluginLID : public OpalLineInterfaceDevice
       */
     virtual PBoolean IsLinePresent(
       unsigned line,      ///<  Number of line
-      PBoolean force = PFalse  ///<  Force test, do not optimise
+      PBoolean force = false  ///<  Force test, do not optimise
     );
 
 
     /**Determine if line is currently off hook.
        This function implies that the state is debounced and that a return
-       value of PTrue indicates that the phone is really off hook. That is
+       value of true indicates that the phone is really off hook. That is
        hook flashes and winks are masked out.
       */
     virtual PBoolean IsLineOffHook(
@@ -151,7 +151,7 @@ class OpalPluginLID : public OpalLineInterfaceDevice
       */
     virtual PBoolean SetLineOffHook(
       unsigned line,        ///<  Number of line
-      PBoolean newState = PTrue  ///<  New state to set
+      PBoolean newState = true  ///<  New state to set
     );
 
     /**Set the hook state off then straight back on again.
@@ -162,14 +162,14 @@ class OpalPluginLID : public OpalLineInterfaceDevice
       unsigned flashTime = 200    ///<  Time for hook flash in milliseconds
     );
 
-    /**Return PTrue if a hook flash has been detected
+    /**Return true if a hook flash has been detected
       */
     virtual PBoolean HasHookFlash(unsigned line);
 
 
     /**Determine if line is ringing.
        This function implies that the state is "debounced" and that a return
-       value of PTrue indicates that the phone is still ringing and it is not
+       value of true indicates that the phone is still ringing and it is not
        simply a pause in the ring cadence.
 
        If cadence is not NULL then it is set with the bit pattern for the
@@ -238,7 +238,7 @@ class OpalPluginLID : public OpalLineInterfaceDevice
       */
     virtual PBoolean IsLineDisconnected(
       unsigned line,   ///<  Number of line
-      PBoolean checkForWink = PTrue
+      PBoolean checkForWink = true   ///< Check for remote hang up signal
     );
 
 
@@ -362,7 +362,7 @@ class OpalPluginLID : public OpalLineInterfaceDevice
       */
     virtual PBoolean EnableAudio(
       unsigned line,      ///<  Number of line
-      PBoolean enable = PTrue
+      PBoolean enable = true   ///< Flag for enable/disable audio on line
     );
 
     /**Determine if audio for the line is enabled.
@@ -445,7 +445,7 @@ class OpalPluginLID : public OpalLineInterfaceDevice
        ID data, or if full is true, all of the fields in the caller ID data.
 
        The full data of the caller ID string consists fields separated by tab
-       characters ('\t'), the first three are always the Calling Line Identity
+       characters ('\\t'), the first three are always the Calling Line Identity
        (CLI or calling number), the date and the Calling Line Name field. Other
        fields may follow and are the of the form name=value. The values are
        LID dependent.
@@ -456,7 +456,7 @@ class OpalPluginLID : public OpalLineInterfaceDevice
     virtual PBoolean GetCallerID(
       unsigned line,      ///<  Number of line
       PString & idString, ///<  ID string returned
-      PBoolean full = PFalse   ///<  Get full information in idString
+      PBoolean full = false   ///<  Get full information in idString
     );
 
     /**Set Caller ID information.
@@ -464,7 +464,7 @@ class OpalPluginLID : public OpalLineInterfaceDevice
        Identity.
 
        The full data of the caller ID string consists fields separated by tab
-       characters ('\t'), the first three are always the Calling Line Identity
+       characters ('\\t'), the first three are always the Calling Line Identity
        (CLI or calling number), the date and the Calling Line Name field. Other
        fields may follow and are the of the form name=value. The values are
        LID dependent.
@@ -489,7 +489,7 @@ class OpalPluginLID : public OpalLineInterfaceDevice
       */
     virtual PBoolean SendVisualMessageWaitingIndicator(
       unsigned line,            ///<  Number of line
-      PBoolean on
+      PBoolean on               ///< Flag for VMWI on/off
     );
 
 
@@ -504,7 +504,7 @@ class OpalPluginLID : public OpalLineInterfaceDevice
     );
 
     /**Read a DTMF digit detected.
-       This may be characters from the set 0-9, A-D, * or #. A null ('\0')
+       This may be characters from the set 0-9, A-D, * or #. A null ('\\0')
        character indicates that there are no tones in the queue.
        Characters E through P indicate the following tones:
 
@@ -592,7 +592,7 @@ class OpalPluginLID : public OpalLineInterfaceDevice
        The takes the line off hook, waits for dial tone, and transmits the
        specified number as DTMF tones.
 
-       If the requireTones flag is PTrue the call is aborted of the call
+       If the requireTones flag is true the call is aborted of the call
        progress tones are not detected. Otherwise the call proceeds with short
        delays while it tries to detect the call progress tones.
 
@@ -663,4 +663,7 @@ class OpalPluginLID : public OpalLineInterfaceDevice
 };
 
 
+PFACTORY_LOAD(OpalPluginLIDManager);
+
+
 #endif // OPAL_LIDS_LIDPLUGINMGR_H
