@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25336 $
- * $Author: rjongbloed $
- * $Date: 2011-03-16 03:51:39 -0500 (Wed, 16 Mar 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_OPAL_CALL_H
@@ -584,9 +584,9 @@ class OpalCall : public PSafeObject
 
     void InternalOnClear();
 
-  protected:
     void SetPartyNames();
 
+  protected:
     bool EnumerateConnections(
       PSafePtr<OpalConnection> & connection,
       PSafetyMode mode,
@@ -602,9 +602,10 @@ class OpalCall : public PSafeObject
     PTime   startTime;
     bool    isEstablished;
     bool    isClearing;
+    PAtomicBoolean m_isCleared;
 
     OpalConnection::CallEndReason callEndReason;
-    PSyncPoint                  * endCallSyncPoint;
+    std::list<PSyncPoint *> m_endCallSyncPoint;
 
     PSafeList<OpalConnection> connectionsActive;
 
