@@ -27,9 +27,9 @@
  *     US Joint Forces Command Joint Concept Development & Experimentation (J9)
  *     http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision: 26939 $
- * $Author: rjongbloed $
- * $Date: 2012-02-07 01:17:49 -0600 (Tue, 07 Feb 2012) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_OPAL_CONNECTION_H
@@ -586,12 +586,17 @@ class OpalConnection : public PSafeObject
 
        Note that this function will return quickly as the release and
        disposal of the connections is done by another thread.
+
+       If \p sync is not NULL then it is signalled when the calls are cleared.
       */
     void ClearCall(
-      CallEndReason reason = EndedByLocalUser ///<  Reason for call clearing
+      CallEndReason reason = EndedByLocalUser, ///<  Reason for call clearing
+      PSyncPoint * sync = NULL
     );
 
     /**Clear a current connection, synchronously.
+       This hangs up the connection to a remote endpoint. Note that this function
+       is always synchronous. If \p sync is NULL then a local PSyncPoint is used.
       */
     virtual void ClearCallSynchronous(
       PSyncPoint * sync,                       ///<  Synchronisation object to signal
@@ -871,7 +876,8 @@ class OpalConnection : public PSafeObject
        disposal of the connections is done by another thread.
       */
     virtual void Release(
-      CallEndReason reason = EndedByLocalUser ///<  Reason for call release
+      CallEndReason reason = EndedByLocalUser, ///<  Reason for call release
+      bool synchronous = false                 ///< If true spawns thread to release connection
     );
 
     /**Clean up the termination of the connection.
@@ -992,18 +998,27 @@ class OpalConnection : public PSafeObject
     );
 
 #if OPAL_FAX
-    /**Switch to/from FAX mode.
+    /**Switch to/from T.38 mode.
       */
     virtual bool SwitchFaxMediaStreams(
-      bool enableFax  ///< Enable FAX or return to audio mode
+      bool toT38  ///< Enable T.38 or return to audio mode
     );
 
-    /**Indicate status of switch to/from FAX mode.
+    /**Indicate status of switch to/from T.38 mode.
 
        Default behaviour does nothing.
       */
     virtual void OnSwitchedFaxMediaStreams(
-      bool enabledFax  ///< Enabled FAX or audio mode
+      bool toT38,       ///< Were switching to T.38 or audio mode
+      bool success      ///< Switch succeeded or failed
+    );
+
+    /**Indicate remote is requesting switch to/from T.38 fax mode.
+
+       Default behaviour calls the same function on the other connection.
+      */
+    virtual bool OnSwitchingFaxMediaStreams(
+      bool toT38   ///< Is switching to T.38 or audio mode
     );
 #endif
 
@@ -1218,11 +1233,34 @@ class OpalConnection : public PSafeObject
     ) const;
 #endif
 
-    /**Set the volume (gain) for the audio media channel to the specified percentage.
+    /**Set the volume (gain) for the audio media channel.
+       The volume range is 0 == muted, 100 == LOUDEST.
       */
     virtual PBoolean SetAudioVolume(
-      PBoolean source,                  ///< true for source (microphone), false for sink (speaker)
-      unsigned percentage           ///< Gain, 0=silent, 100=maximun
+      PBoolean source,        ///< true for source (microphone), false for sink (speaker)
+      unsigned percentage     ///< Gain, 0=silent, 100=maximun
+    );
+
+    /**Get  the volume (gain) for the audio media channel.
+       The volume range is 0 == muted, 100 == LOUDEST.
+      */
+    virtual PBoolean GetAudioVolume(
+      PBoolean source,        ///< true for source (microphone), false for sink (speaker)
+      unsigned & percentage   ///< Gain, 0=silent, 100=maximun
+    );
+
+    /**Set the mute state for the audio media channel.
+      */
+    virtual bool SetAudioMute(
+      bool source,        ///< true for source (microphone), false for sink (speaker)
+      bool mute           ///< Flag for muted audio
+    );
+
+    /**Get the mute state for the audio media channel.
+      */
+    virtual bool GetAudioMute(
+      bool source,        ///< true for source (microphone), false for sink (speaker)
+      bool & mute         ///< Flag for muted audio
     );
 
     /**Get the average signal level (0..32767) for the audio media channel.
@@ -1766,7 +1804,6 @@ class OpalConnection : public PSafeObject
     PString              remotePartyAddress;
     PString              m_redirectingParty;
     CallEndReason        callEndReason;
-    bool                 synchronousOnRelease;
     PString              m_calledPartyNumber;
     PString              m_calledPartyName;
 
@@ -1888,6 +1925,7 @@ class OpalConnection : public PSafeObject
     P_REMOVE_VIRTUAL(bool, IsConnectionOnHold(bool), false);
     P_REMOVE_VIRTUAL_VOID(ApplyStringOptions(OpalConnection::StringOptions &));
     P_REMOVE_VIRTUAL(bool, OnTransferNotify(const PStringToString &), false);
+    P_REMOVE_VIRTUAL_VOID(OnSwitchedFaxMediaStreams(bool));
 };
 
 #endif // OPAL_OPAL_CONNECTION_H
