@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26937 $
- * $Author: rjongbloed $
- * $Date: 2012-02-06 19:54:19 -0600 (Mon, 06 Feb 2012) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_OPAL_ENDPOINT_H
@@ -476,6 +476,8 @@ class OpalEndPoint : public PObject
        call disposed of. Note that this function returns quickly and the
        disposal happens at some later time by a background thread. This it is
        safe to call this function from anywhere.
+
+       If \p sync is not NULL then it is signalled when the calls are cleared.
       */
     virtual PBoolean ClearCall(
       const PString & token,    ///<  Token for identifying connection
@@ -484,8 +486,8 @@ class OpalEndPoint : public PObject
     );
 
     /**Clear a current connection.
-       This hangs up the connection to a remote endpoint. Note that these functions
-       are synchronous
+       This hangs up the connection to a remote endpoint. Note that this function
+       is always synchronous. If \p sync is NULL then a local PSyncPoint is used.
       */
     virtual PBoolean ClearCallSynchronous(
       const PString & token,    ///<  Token for identifying connection
@@ -826,7 +828,7 @@ class OpalEndPoint : public PObject
     PMutex inUseFlag;
 
     friend void OpalManager::GarbageCollection();
-    friend void OpalConnection::Release(CallEndReason reason);
+    friend void OpalConnection::Release(CallEndReason,bool);
 
   private:
     P_REMOVE_VIRTUAL(PBoolean, OnIncomingConnection(OpalConnection &, unsigned), false);
