@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26927 $
- * $Author: rjongbloed $
- * $Date: 2012-02-01 16:09:34 -0600 (Wed, 01 Feb 2012) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_OPAL_IVR_H
@@ -353,12 +353,6 @@ class OpalIVRMediaStream : public OpalRawMediaStream
       */
     virtual PBoolean Open();
 
-    /**Close the media stream.
-
-       Closes the associated PChannel.
-      */
-    virtual PBoolean Close();
-
     /**Indicate if the media stream is synchronous.
        Returns false for IVR streams.
       */
@@ -366,6 +360,8 @@ class OpalIVRMediaStream : public OpalRawMediaStream
   //@}
 
   protected:
+    virtual void InternalClose();
+
     PVXMLSession & m_vxmlSession;
 };
 
