@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26939 $
- * $Author: rjongbloed $
- * $Date: 2012-02-07 01:17:49 -0600 (Tue, 07 Feb 2012) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_OPAL_LOCALEP_H
@@ -140,6 +140,15 @@ class OpalLocalEndPoint : public OpalEndPoint
       OpalConnection::StringOptions * stringOptions ///< Options to pass to connection
     );
 
+    /**Call back just before remote is contacted.
+       If false is returned the call is aborted with EndedByNoAccept.
+
+       The default implementation returns true.
+      */
+    virtual bool OnOutgoingSetUp(
+      const OpalLocalConnection & connection ///<  Connection having event
+    );
+
     /**Call back to indicate that remote is ringing.
        If false is returned the call is aborted.
 
@@ -201,6 +210,9 @@ class OpalLocalEndPoint : public OpalEndPoint
     /**Call back to get media data for transmission.
        If false is returned then OnReadMediaData() is called.
 
+       Care with the handling of real time is rqeuired, see GetSynchronicity
+       for more details.
+
        The default implementation returns false.
       */
     virtual bool OnReadMediaFrame(
@@ -212,7 +224,9 @@ class OpalLocalEndPoint : public OpalEndPoint
     /**Call back to handle received media data.
        If false is returned then OnWriteMediaData() is called.
 
-       The default implementation .
+       Care with the handling of real time is rqeuired, see GetSynchronicity
+       for more details.
+
        The default implementation returns false.
       */
     virtual bool OnWriteMediaFrame(
@@ -224,6 +238,9 @@ class OpalLocalEndPoint : public OpalEndPoint
     /**Call back to get media data for transmission.
        If false is returned the media stream will be closed.
 
+       Care with the handling of real time is rqeuired, see GetSynchronicity
+       for more details.
+
        The default implementation returns false.
       */
     virtual bool OnReadMediaData(
@@ -237,6 +254,9 @@ class OpalLocalEndPoint : public OpalEndPoint
     /**Call back to handle received media data.
        If false is returned the media stream will be closed.
 
+       Care with the handling of real time is rqeuired, see GetSynchronicity
+       for more details.
+
        The default implementation returns false.
       */
     virtual bool OnWriteMediaData(
@@ -247,19 +267,61 @@ class OpalLocalEndPoint : public OpalEndPoint
       PINDEX & written                        ///<  Amount of data written
     );
 
-    /**Indicate that I/O is synchronous.
+    /**Indicate the synchronous mode for I/O.
        This indicates that the OnReadMediaXXX and OnWriteMediaXXX functions
-       will execute blocking to the correct real time synchonisation. So if
-       for example OnWriteMediaData() is sent 320 bytes of PCM data then it
-       will, on average, block for 20 milliseconds per call.
+       will execute blocking to the correct real time synchronisation.
+       If GetSynchronicity() returns e_Synchronous, then, for example, when
+       OnWriteMediaData() is sent 320 bytes of PCM data, it will block for
+       20 milliseconds.
 
-       If the function returns false, then the system will try and simulate
-       the correct timing using the operating system sleep function. However
-       this is not desirable as this function is notoriously inaccurate.
+       If GetSynchronicity() returns e_SimulateSyncronous, then the system will try and
+       simulate the correct timing using the operating system sleep function.
+       This is not desirable as this function is notoriously inaccurate, and
+       OPAL does it's best to compensate, but very often there is no other
+       choice.
 
-       Default returns true.
+       Note, it is important for the correct oeprating of the jitter buffer
+       that one of the above two modes is used for audio.
+
+       If GetSynchronicity() returns e_Asynchronous, then the system will indicate
+       that blocking is not required in any way. For example, when playing video,
+       this is done as fast as data comes in from the network and there is no
+       real time pacing required.
       */
-    virtual bool IsSynchronous() const;
+    enum Synchronicity {
+      e_Synchronous,        ///< Functions will block for correct real time
+      e_Asynchronous,       ///< Functions will not block, and do not require any real time handling.
+      e_SimulateSyncronous  ///< Functions wlll not block, but do require real time handling.
+    };
+
+    /**Indicate the I/O synchronous mode.
+       See Synchronicity for more details.
+
+       Default behaviour returns m_defaultAudioSynchronicity (initially
+       e_Synchronous) when is audio source or sink,
+       m_defaultVideoSourceSynchronicity (initially e_Synchronous) when a
+       video source, e_Asynchronous in all other cases.
+      */
+    virtual Synchronicity GetSynchronicity(
+      const OpalMediaFormat & mediaFormat,  ///< Media format for stream being opened.
+      bool isSource                         ///< Stream is a a source
+    ) const;
+
+    /**Get default synchronous mode for audio sources and sinks.
+      */
+    Synchronicity GetDefaultAudioSynchronicity() const { return m_defaultAudioSynchronicity; }
+
+    /**Set default synchronous mode for audio sources and sinks.
+      */
+    void SetDefaultAudioSynchronicity(Synchronicity sync) { m_defaultAudioSynchronicity = sync; }
+
+    /**Get default synchronous mode for video sources.
+      */
+    Synchronicity GetDefaultVideoSourceSynchronicity() const { return m_defaultVideoSourceSynchronicity; }
+
+    /**Set default synchronous mode for video sources.
+      */
+    void SetDefaultVideoSourceSynchronicity(Synchronicity sync) { m_defaultVideoSourceSynchronicity = sync; }
 
     /**Indicate OnAlerting() is be deferred or immediate.
       */
@@ -282,8 +344,12 @@ class OpalLocalEndPoint : public OpalEndPoint
     bool m_deferredAlerting;
     bool m_deferredAnswer;
 
+    Synchronicity m_defaultAudioSynchronicity;
+    Synchronicity m_defaultVideoSourceSynchronicity;
+
   private:
     P_REMOVE_VIRTUAL(OpalLocalConnection *, CreateConnection(OpalCall &, void *), 0);
+    P_REMOVE_VIRTUAL(bool, IsSynchronous() const, false);
 };
 
 
@@ -332,7 +398,10 @@ class OpalLocalConnection : public OpalConnection
        This function will initiate the connection to the remote entity, for
        example in H.323 it sends a SETUP, in SIP it sends an INVITE etc.
 
-       The default behaviour does.
+       The default behaviour determines if this si incoming or outgoing call
+       by checking if we are party A or B, then does approriate setting up
+       of the conenction, including calling OnOutgoing() or OnIncoming() as
+       appropriate.
       */
     virtual PBoolean SetUpConnection();
 
@@ -405,6 +474,32 @@ class OpalLocalConnection : public OpalConnection
 
   /**@name New operations */
   //@{
+    /**Call back just before remote is contacted.
+
+       The default implementation call OpalLocalEndPoint::OnOutgoingSetUp().
+
+       @return false if the call is to be aborted with EndedByNoAccept.
+      */
+    virtual bool OnOutgoingSetUp();
+
+    /**Call back to indicate that remote is ringing.
+
+       The default implementation call OpalLocalEndPoint::OnOutgoingCall().
+
+       @return false if the call is to be aborted with EndedByNoAccept.
+      */
+    virtual bool OnOutgoing();
+
+    /**Call back to indicate that there is an incoming call.
+       Note this function should not block or it will impede the operation of
+       the stack.
+
+       The default implementation call OpalLocalEndPoint::OnIncomingCall().
+
+       @return false if the call is to be aborted with status of EndedByLocalBusy.
+      */
+    virtual bool OnIncoming();
+
     /**Indicate alerting for the incoming connection.
       */
     virtual void AlertingIncoming();
@@ -417,15 +512,15 @@ class OpalLocalConnection : public OpalConnection
   /**@name Member variable access */
   //@{
     /// Get user data pointer.
-    void * GetUserData() const  { return userData; }
+    void * GetUserData() const  { return m_userData; }
 
     /// Set user data pointer.
-    void SetUserData(void * v)  { userData = v; }
+    void SetUserData(void * v)  { m_userData = v; }
   //@}
 
   protected:
     OpalLocalEndPoint & endpoint;
-    void * userData;
+    void * m_userData;
 };
 
 
@@ -446,7 +541,7 @@ class OpalLocalMediaStream : public OpalMediaStream, public OpalMediaStreamPacin
       const OpalMediaFormat & mediaFormat, ///<  Media format for stream
       unsigned sessionID,                  ///<  Session number for stream
       bool isSource,                       ///<  Is a source stream
-      bool isSynchronous                   ///<  Can accept data and block accordingly
+      OpalLocalEndPoint::Synchronicity synchronicity ///< Synchronous mode
     );
   //@}
 
@@ -495,7 +590,9 @@ class OpalLocalMediaStream : public OpalMediaStream, public OpalMediaStreamPacin
   //@}
 
   protected:
-    bool m_isSynchronous;
+    virtual void InternalClose() { }
+
+    OpalLocalEndPoint::Synchronicity m_synchronicity;
 };
 
 
