@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26937 $
- * $Author: rjongbloed $
- * $Date: 2012-02-06 19:54:19 -0600 (Mon, 06 Feb 2012) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_OPAL_MANAGER_H
@@ -222,6 +222,8 @@ class OpalManager : public PObject
        call are disposed of. Note that this function returns quickly and the
        disposal happens at some later time in a background thread. It is safe
        to call this function from anywhere.
+
+       If \p sync is not NULL then it is signalled when the calls are cleared.
       */
     virtual PBoolean ClearCall(
       const PString & token,    ///<  Token for identifying connection
@@ -825,14 +827,6 @@ class OpalManager : public PObject
       PBoolean requiresPatchThread = true  ///< The patch requires a thread
     );
 
-    /**Destroy a OpalMediaPatch instance.
-
-       The default behaviour simply calls delete patch.
-      */
-    virtual void DestroyMediaPatch(
-      OpalMediaPatch * patch
-    );
-
     /**Call back for a media patch thread starting.
        This function is called within the context of the thread associated
        with the media patch.
@@ -1288,7 +1282,7 @@ class OpalManager : public PObject
      */
     WORD GetUDPPortMax() const { return udpPorts.max; }
 
-    /**Set the TCP port number base and max for H.245 channels.
+    /**Set the UDP port number base and max for RAS channels.
      */
     void SetUDPPorts(unsigned udpBase, unsigned udpMax);
 
