@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26920 $
- * $Author: rjongbloed $
- * $Date: 2012-01-31 23:02:10 -0600 (Tue, 31 Jan 2012) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_OPAL_MEDIAFMT_H
@@ -313,21 +313,35 @@ class OpalMediaOption : public PObject
 
 #if OPAL_H323
     struct H245GenericInfo {
-      H245GenericInfo() { memset(this, 0, sizeof(*this)); }
-      unsigned ordinal:16;
+      H245GenericInfo()
+        : ordinal(0)
+        , mode(None)
+        , integerType(UnsignedInt)
+        , excludeTCS(false)
+        , excludeOLC(false)
+        , excludeReqMode(false)
+        , position(-1)
+      { }
+
+      unsigned ordinal;
+
       enum Modes {
         None,
         Collapsing,
         NonCollapsing
-      } mode:3;
+      } mode;
+
       enum IntegerTypes {
         UnsignedInt,
         Unsigned32,
         BooleanArray
-      } integerType:3;
-      bool excludeTCS:1;
-      bool excludeOLC:1;
-      bool excludeReqMode:1;
+      } integerType;
+
+      bool    excludeTCS;
+      bool    excludeOLC;
+      bool    excludeReqMode;
+      int     position;     // Position in sequence for parameter
+      PString defaultValue; // Do not include parameter if this value
     };
 
     const H245GenericInfo & GetH245Generic() const { return m_H245Generic; }
@@ -1207,6 +1221,8 @@ class OpalMediaFormat : public PContainer
 #if OPAL_H323
     static const PString & MediaPacketizationOption();
     static const PString & MediaPacketizationsOption();
+    PStringSet GetMediaPacketizations() const;
+    void SetMediaPacketizations(const PStringSet & packetizations);
 #endif
 
   private:
@@ -1313,9 +1329,8 @@ class OpalVideoFormat : public OpalMediaFormat
     static const PString & MaxRxFrameHeightOption();
     static const PString & TemporalSpatialTradeOffOption();
     static const PString & TxKeyFramePeriodOption();
-    static const PString & RateControlEnableOption();
-    static const PString & RateControllerOption();
-    static const PString & RateControllerBitRateScalerOption();
+    static const PString & RateControlPeriodOption(); // Period over which the rate controller maintains the target bit rate.
+    static const PString & RateControllerOption(); // String for controller algorithm. Empty is none.
 
     /**The "role" of the content in the video stream based on this media
        format. This is based on RFC4796 and H.239 semantics and is an
