@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26728 $
- * $Author: rjongbloed $
- * $Date: 2011-12-01 22:59:14 -0600 (Thu, 01 Dec 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_OPAL_MEDIASTRM_H
@@ -196,7 +196,8 @@ class OpalMediaStream : public PSafeObject
 
     /**Close the media stream.
 
-       The default does nothing.
+       The default marks the stream as closed and calls
+       OpalConnection::OnClosedMediaStream().
       */
     virtual PBoolean Close();
 
@@ -380,7 +381,7 @@ class OpalMediaStream : public PSafeObject
 
     /**Get the patch thread that is using the stream.
       */
-    OpalMediaPatch * GetPatch() const { return mediaPatch; }
+    OpalMediaPatch * GetPatch() const { return m_mediaPatch; }
 
     /**Add a filter to the owning patch safely.
       */
@@ -405,6 +406,13 @@ class OpalMediaStream : public PSafeObject
     void IncrementTimestamp(PINDEX size);
     bool InternalWriteData(const BYTE * data, PINDEX length, PINDEX & written);
 
+    /**Close any internal components of the stream.
+       This should be used in preference to overriding the Close() function as
+       it is guaranteed to be called exactly once and avoids race conditions in
+       the shut down sequence of a media stream.
+      */
+    virtual void InternalClose() = 0;
+
     OpalConnection & connection;
     unsigned         sessionID;
     PString          identifier;
@@ -417,7 +425,8 @@ class OpalMediaStream : public PSafeObject
     bool             marker;
     unsigned         mismatchedPayloadTypes;
 
-    OpalMediaPatch * mediaPatch;
+    typedef PSafePtr<OpalMediaPatch, PSafePtrMultiThreaded> PatchPtr;
+    PatchPtr m_mediaPatch;
 
     RTP_DataFrame::PayloadTypes m_payloadType;
     unsigned                    m_frameTime;
@@ -505,6 +514,15 @@ class OpalNullMediaStream : public OpalMediaStream, public OpalMediaStreamPacing
       PINDEX & written     ///<  Length of data actually written
     );
 	
+    /**Set the paused state for stream.
+       This will stop reading/writing data from the stream.
+       Returns true if the pause state was changed
+      */
+    virtual bool SetPaused(
+      bool pause,             ///< Indicate that the stream should be paused
+      bool fromPatch = false  ///<  Is being called from OpalMediaPatch
+    );
+
     /**Indicate if the media stream requires a OpalMediaPatch thread (active patch).
        The default behaviour returns the value of m_isSynchronous.
       */
@@ -517,6 +535,8 @@ class OpalNullMediaStream : public OpalMediaStream, public OpalMediaStreamPacing
   //@}
 
   protected:
+    virtual void InternalClose() { }
+
     bool m_isSynchronous;
     bool m_requiresPatchThread;
 };
@@ -557,12 +577,6 @@ class OpalRTPMediaStream : public OpalMediaStream
       */
     virtual PBoolean Open();
 
-    /**Close the media stream.
-
-       The default does nothing.
-      */
-    virtual PBoolean Close();
-
     /**Set the paused state for stream.
        This will stop reading/writing data from the stream.
       */
@@ -632,6 +646,8 @@ class OpalRTPMediaStream : public OpalMediaStream
   //@}
 
   protected:
+    virtual void InternalClose();
+
     RTP_Session & rtpSession;
     unsigned      minAudioJitterDelay;
     unsigned      maxAudioJitterDelay;
@@ -695,18 +711,14 @@ class OpalRawMediaStream : public OpalMediaStream
       bool autoDelete = true  ///< Auto delete channel on exit or replacement
     );
 
-    /**Close the media stream.
-
-       Closes the associated PChannel.
-      */
-    virtual PBoolean Close();
-
     /**Get average signal level in last frame.
       */
     virtual unsigned GetAverageSignalLevel();
   //@}
 
   protected:
+    virtual void InternalClose();
+
     PChannel * m_channel;
     bool       m_autoDelete;
     PMutex     m_channelMutex;
@@ -894,12 +906,6 @@ class OpalVideoMediaStream : public OpalMediaStream
       */
     virtual PBoolean Open();
 
-    /**Close the media stream.
-
-       The default does nothing.
-      */
-    virtual PBoolean Close();
-
     /**Read raw media data from the source media stream.
        The default behaviour simply calls ReadPacket() on the data portion of the
        RTP_DataFrame and sets the frames timestamp and marker from the internal
@@ -951,6 +957,8 @@ class OpalVideoMediaStream : public OpalMediaStream
   //@}
 
   protected:
+    virtual void InternalClose();
+
     PVideoInputDevice  * m_inputDevice;
     PVideoOutputDevice * m_outputDevice;
     bool                 m_autoDeleteInput;
@@ -1004,15 +1012,11 @@ class OpalUDPMediaStream : public OpalMediaStream
        Returns false.
       */
     virtual PBoolean IsSynchronous() const;
-
-    /**Close the media stream.
-       Closes the associated OpalTransportUDP.
-      */
-    virtual PBoolean Close();
-
   //@}
 
   private:
+    virtual void InternalClose();
+
     OpalTransportUDP & udpTransport;
 };
 
