@@ -25,9 +25,9 @@
  * Contributor(s): Craig Southeren (craigs@postincrement.com)
  *                 Robert Jongbloed (robertj@voxlucida.com.au)
  *
- * $Revision: 26969 $
- * $Author: rjongbloed $
- * $Date: 2012-02-10 18:45:16 -0600 (Fri, 10 Feb 2012) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 
@@ -44,10 +44,12 @@
 
 #include <opal/localep.h>
 #include <codec/vidcodec.h>
+#include <ptclib/threadpool.h>
 
 
 class RTP_DataFrame;
 class OpalJitterBuffer;
+class OpalMixerConnection;
 
 
 //#define OPAL_MIXER_AUDIO_DEBUG 1
@@ -514,18 +516,42 @@ class OpalMixerNodeManager : public PObject
     void RemoveNodeNames(
       PStringList names   ///< list of alias names for nodes
     );
+
+    /**Queue user input for braodcast
+      */
+    void QueueUserInput(
+      const PSafePtr<OpalMixerNode> & node,     ///< Node to qhich user input is broadcast
+      const OpalMixerConnection * connection,   ///<  Connection NOT to send to
+      const PString & value                     ///<  String value of indication
+    );
   //@}
 
   protected:
     PSafeDictionary<PGloballyUniqueID, OpalMixerNode> m_nodesByUID;
     PDictionary<PString, OpalMixerNode>               m_nodesByName;
+
+    struct UserInput {
+      UserInput(
+        const PSafePtr<OpalMixerNode> & node,
+        const OpalMixerConnection * connection,
+        const PString & value
+      ) : m_node(node)
+        , m_connection(connection)
+        , m_value(value)
+      { }
+
+      PSafePtr<OpalMixerNode> m_node;
+      const OpalMixerConnection * m_connection;
+      PString m_value;
+
+      void Work();
+    };
+    PQueuedThreadPool<UserInput> m_userInputPool;
 };
 
 
 ///////////////////////////////////////////////////////////////////////////////
 
-class OpalMixerConnection;
-
 /** Mixer EndPoint.
     This class represents an endpoint that mixes media. It can be used as the
     basis for a Multipoint Conferencing Unit.
@@ -899,10 +925,6 @@ class OpalMixerMediaStream : public OpalMediaStream
       */
     virtual PBoolean Open();
 
-    /**Close the media stream.
-      */
-    virtual PBoolean Close();
-
     /**Write an RTP frame of data to the sink media stream.
        The default behaviour simply calls WriteData() on the data portion of the
        RTP_DataFrame and and sets the internal timestamp and marker from the
@@ -947,6 +969,8 @@ class OpalMixerMediaStream : public OpalMediaStream
   //@}
 
   protected:
+    virtual void InternalClose();
+
     PSafePtr<OpalMixerNode> m_node;
     bool m_listenOnly;
 #if OPAL_VIDEO
