@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26730 $
- * $Author: rjongbloed $
- * $Date: 2011-12-02 01:12:02 -0600 (Fri, 02 Dec 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_OPAL_PATCH_H
@@ -57,7 +57,7 @@ class OpalTranscoder;
    the Start() function is called on the patch when the creator code is
    ready for it to begin. For example all sink streams have been added.
   */
-class OpalMediaPatch : public PObject
+class OpalMediaPatch : public PSafeObject
 {
     PCLASSINFO(OpalMediaPatch, PObject);
   public:
@@ -239,6 +239,7 @@ class OpalMediaPatch : public PObject
                 
     /**Called from the associated patch thread */
     virtual void Main();
+    void StopThread();
     bool DispatchFrame(RTP_DataFrame & frame);
     bool EnableJitterBuffer();
 
@@ -299,7 +300,6 @@ class OpalMediaPatch : public PObject
 
     Thread * patchThread;
     PMutex patchThreadMutex;
-    mutable PReadWriteMutex inUse;
 
   private:
     P_REMOVE_VIRTUAL(bool, OnPatchStart(), false);
@@ -322,6 +322,9 @@ class OpalPassiveMediaPatch : public OpalMediaPatch
     );
 
     virtual void Start();
+
+  protected:
+    bool m_started;
 };
 
 
