@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26069 $
- * $Author: rjongbloed $
- * $Date: 2011-06-23 19:56:50 -0500 (Thu, 23 Jun 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_OPAL_PCSS_H
@@ -356,11 +356,34 @@ class OpalPCSSConnection : public OpalLocalConnection
       PBoolean isSource                        ///<  Is a source stream
     );
 
-    /**Set  the volume (gain) for the audio media channel to the specified percentage.
+    /**Set  the volume (gain) for the audio media channel.
+       The volume range is 0 == muted, 100 == LOUDEST.
       */
     virtual PBoolean SetAudioVolume(
-      PBoolean source,                  ///< true for source (microphone), false for sink (speaker)
-      unsigned percentage           ///< Gain, 0=silent, 100=maximun
+      PBoolean source,        ///< true for source (microphone), false for sink (speaker)
+      unsigned percentage     ///< Gain, 0=silent, 100=maximun
+    );
+
+    /**Get  the volume (gain) for the audio media channel.
+       The volume range is 0 == muted, 100 == LOUDEST.
+      */
+    virtual PBoolean GetAudioVolume(
+      PBoolean source,        ///< true for source (microphone), false for sink (speaker)
+      unsigned & percentage   ///< Gain, 0=silent, 100=maximun
+    );
+
+    /**Set the mute state for the audio media channel.
+      */
+    virtual bool SetAudioMute(
+      bool source,        ///< true for source (microphone), false for sink (speaker)
+      bool mute           ///< Flag for muted audio
+    );
+
+    /**Get the mute state for the audio media channel.
+      */
+    virtual bool GetAudioMute(
+      bool source,        ///< true for source (microphone), false for sink (speaker)
+      bool & mute         ///< Flag for muted audio
     );
 
     /**Get the average signal level (0..32767) for the audio media channel.
