@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26283 $
- * $Author: rjongbloed $
- * $Date: 2011-08-08 02:41:14 -0500 (Mon, 08 Aug 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_IM_PRES_ENT_H
@@ -166,15 +166,15 @@ class OpalPresentity : public PSafeObject
         presentity has successfully been indicated as "present" on the server
         only that the underlying system can do so at some time.
       */
-    virtual bool Open() = 0;
+    virtual bool Open();
 
     /** Indicate if the presentity has been successfully opened.
       */
-    virtual bool IsOpen() const = 0;
+    virtual bool IsOpen() const { return m_open; }
 
     /**Close the presentity.
       */
-    virtual bool Close() = 0;
+    virtual bool Close();
   //@}
 
   /**@name Attributes */
@@ -519,8 +519,6 @@ class OpalPresentity : public PSafeObject
 
     void Internal_SendMessageToCommand(const OpalSendMessageToCommand & cmd);
 
-    virtual PString GetID() const;
-
   protected:
     OpalPresentityCommand * InternalCreateCommand(const char * cmdName);
 
@@ -533,9 +531,8 @@ class OpalPresentity : public PSafeObject
     PresenceChangeNotifier       m_onPresenceChangeNotifier;
     ReceivedMessageNotifier      m_onReceivedMessageNotifier;
 
+    PAtomicBoolean m_open;
     PMutex m_notificationMutex;
-    PAtomicInteger::IntegerType m_idNumber;
-
     bool m_temporarilyUnavailable;
     OpalPresenceInfo::State m_localState;      ///< our presentity state
     PString m_localStateNote;                  ///< Additional note attached to the 
