@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25336 $
- * $Author: rjongbloed $
- * $Date: 2011-03-16 03:51:39 -0500 (Wed, 16 Mar 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_OPAL_RTPCONN_H
@@ -486,10 +486,15 @@ class OpalRTPConnection : public OpalConnection
       OpalMediaPatch & patch    ///<  New patch
     );
 
-    /** Notifier function for OpalVideoUpdatePicture.
+    /** Callback for media commands.
         Calls the SendIntraFrameRequest on the rtp session
+
+       @returns true if command is handled.
       */
-    void OnMediaCommand(OpalMediaCommand & command, INT extra);
+    virtual bool OnMediaCommand(
+      OpalMediaStream & stream,         ///< Stream command executed on
+      const OpalMediaCommand & command  ///< Media command being executed
+    );
   //@}
 
     virtual void SessionFailing(RTP_Session & session);
