@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25780 $
- * $Author: rjongbloed $
- * $Date: 2011-05-16 21:05:05 -0500 (Mon, 16 May 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_OPAL_TRANSCODERS_H
@@ -173,6 +173,8 @@ class OpalTranscoder : public OpalMediaFormatPair
        apply to a video transcoder.
 
        The default behaviour simply returns false.
+
+       @returns true if command is handled.
       */
     virtual PBoolean ExecuteCommand(
       const OpalMediaCommand & command    ///<  Command to execute.
