@@ -28,9 +28,9 @@
  *     US Joint Forces Command Joint Concept Development & Experimentation (J9)
  *     http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision: 26231 $
- * $Author: rjongbloed $
- * $Date: 2011-07-19 21:23:34 -0500 (Tue, 19 Jul 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_OPAL_TRANSPORT_H
@@ -206,7 +206,9 @@ class OpalTransportAddress : public PCaselessString
     /**Translate the transport address to a more human readable form.
        Returns the hostname if using IP.
       */
-    virtual PString GetHostName() const;
+    virtual PString GetHostName(
+      bool includeService = false
+    ) const;
 
     enum BindOptions {
       NoBinding,
@@ -1231,7 +1233,8 @@ class OpalInternalTransport : public PObject
     PCLASSINFO(OpalInternalTransport, PObject);
   public:
     virtual PString GetHostName(
-      const OpalTransportAddress & address
+      const OpalTransportAddress & address,
+      bool includeService
     ) const;
 
     virtual PBoolean GetIpAndPort(
@@ -1261,7 +1264,8 @@ class OpalInternalIPTransport : public OpalInternalTransport
     PCLASSINFO(OpalInternalIPTransport, OpalInternalTransport);
   public:
     virtual PString GetHostName(
-      const OpalTransportAddress & address
+      const OpalTransportAddress & address,
+      bool includeService
     ) const;
     virtual PBoolean GetIpAndPort(
       const OpalTransportAddress & address,
