@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25990 $
- * $Author: rjongbloed $
- * $Date: 2011-06-08 22:17:09 -0500 (Wed, 08 Jun 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_RTP_JITTER_H
@@ -170,6 +170,7 @@ class OpalJitterBuffer : public PSafeObject
 
     DWORD    m_averageFrameTime;
     DWORD    m_lastTimestamp;
+    DWORD    m_lastSyncSource;
     DWORD    m_bufferFilledTime;
     DWORD    m_bufferLowTime;
     DWORD    m_bufferEmptiedTime;
@@ -223,12 +224,11 @@ class OpalJitterBufferThread : public OpalJitterBuffer
       RTP_DataFrame & frame   ///<  Frame read from the RTP session
     ) = 0;
 
+    void StartThread();
+
   protected:
     PDECLARE_NOTIFIER(PThread, OpalJitterBufferThread, JitterThreadMain);
 
-    /// Internal function to be called from derived class constructor
-    void StartThread();
-
     /// Internal function to be called from derived class destructor
     void WaitForThreadTermination();
 
