@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26716 $
- * $Author: rjongbloed $
- * $Date: 2011-11-30 17:51:43 -0600 (Wed, 30 Nov 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_RTP_RTP_H
@@ -45,11 +45,11 @@
 
 #include <list>
 
-#include <rtp/metrics.h>
 
 class RTP_JitterBuffer;
 class PNatMethod;
 class OpalSecurityMode;
+class RTCP_XR_Metrics;
 
 ///////////////////////////////////////////////////////////////////////////////
 // 
@@ -585,6 +585,10 @@ class RTP_Session : public PObject
       RTP_DataFrame & frame   ///<  Frame read from the RTP session
     ) = 0;
 
+    /**Flush incoming data.
+      */
+    virtual void FlushData();
+
     /**Write a data frame from the RTP channel.
       */
     virtual PBoolean WriteData(
@@ -605,10 +609,6 @@ class RTP_Session : public PObject
       RTP_ControlFrame & frame    ///<  Frame to write to the RTP session
     ) = 0;
 
-    /**Write the RTCP reports.
-      */
-    virtual PBoolean SendReport();
-
     /**Close down the RTP session.
       */
     virtual bool Close(
@@ -720,9 +720,13 @@ class RTP_Session : public PObject
     };
     PARRAY(ExtendedReportArray, ExtendedReport);
 
-    virtual void OnRxExtendedReport(DWORD src,
-                                    const ExtendedReportArray & reports);                             
-#endif
+    virtual void OnRxExtendedReport(
+      DWORD src,
+      const ExtendedReportArray & reports
+    );
+
+    RTCP_XR_Metrics * GetExtendedMetrics() const { return m_metrics; }
+#endif // OPAL_RTCP_XR
   //@}
 
   /**@name Member variable access */
@@ -794,18 +798,13 @@ class RTP_Session : public PObject
 
     /**Get the time interval for sending RTCP reports in the session.
       */
-    const PTimeInterval & GetReportTimeInterval() { return reportTimeInterval; }
+    const PTimeInterval & GetReportTimeInterval() { return m_reportTimer.GetResetTime(); }
 
     /**Set the time interval for sending RTCP reports in the session.
       */
     void SetReportTimeInterval(
       const PTimeInterval & interval ///<  New time interval for reports.
-    )  { reportTimeInterval = interval; }
-
-    /**Get the current report timer
-     */
-    PTimeInterval GetReportTimer()
-    { return reportTimer; }
+    )  { m_reportTimer.RunContinuous(interval); }
 
     /**Get the interval for transmitter statistics in the session.
       */
@@ -983,22 +982,14 @@ class RTP_Session : public PObject
 
     void AddFilter(const FilterNotifier & filter);
 
-#if OPAL_RTCP_XR
-    const RTCP_XR_Metrics & GetMetrics() const { return m_metrics; }
-#endif
-
     virtual void SendBYE();
 
   protected:
+    RTP_Session::ReceiverReportArray BuildReceiverReportArray(const RTP_ControlFrame & frame, PINDEX offset);
     void AddReceiverReport(RTP_ControlFrame::ReceiverReport & receiver);
 
     PBoolean InsertReportPacket(RTP_ControlFrame & report);
     
-#if OPAL_RTCP_XR
-    void InsertExtendedReportPacket(RTP_ControlFrame & report);
-    void OnRxSenderReportToMetrics(const RTP_ControlFrame & frame, PINDEX offset);    
-#endif
-
     PString             m_encoding;
     PMutex              m_encodingMutex;
     RTP_Encoding      * m_encodingHandler;
@@ -1021,7 +1012,6 @@ class RTP_Session : public PObject
     bool          allowOneSyncSourceChange;
     PBoolean      allowRemoteTransmitAddressChange;
     PBoolean      allowSequenceChange;
-    PTimeInterval reportTimeInterval;
     unsigned      txStatisticsInterval;
     unsigned      rxStatisticsInterval;
     WORD          lastSentSequenceNumber;
@@ -1076,7 +1066,8 @@ class RTP_Session : public PObject
     
 #if OPAL_RTCP_XR
     // Calculate the VoIP Metrics for RTCP-XR
-    RTCP_XR_Metrics m_metrics;
+    RTCP_XR_Metrics * m_metrics;
+    friend class RTCP_XR_Metrics;
 #endif
 
     DWORD    averageSendTimeAccum;
@@ -1091,8 +1082,9 @@ class RTP_Session : public PObject
     RTP_DataFrame::PayloadTypes lastReceivedPayloadType;
     PBoolean ignorePayloadTypeChanges;
 
-    PMutex reportMutex;
-    PTimer reportTimer;
+    PMutex m_reportMutex;
+    PTimer m_reportTimer;
+    PDECLARE_NOTIFIER(PTimer, RTP_Session, SendReport);
 
     PBoolean closeOnBye;
     PBoolean byeSent;
@@ -1130,6 +1122,10 @@ class RTP_UDP : public RTP_Session
     virtual PBoolean ReadData(RTP_DataFrame & frame);
     virtual PBoolean Internal_ReadData(RTP_DataFrame & frame);
 
+    /**Flush incoming data.
+      */
+    virtual void FlushData();
+
     /** Write a data frame to the RTP channel.
       */
     virtual PBoolean WriteData(RTP_DataFrame & frame);
@@ -1286,7 +1282,6 @@ class RTP_UDP : public RTP_Session
     bool appliedQOS;
     bool remoteIsNAT;
     bool localHasNAT;
-    bool m_firstData;
     bool m_firstControl;
     int  badTransmitCounter;
     PTime badTransmitStart;
