@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21293 $
- * $Author: rjongbloed $
- * $Date: 2008-10-12 18:24:41 -0500 (Sun, 12 Oct 2008) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_RTP_SRTP_H
@@ -49,9 +49,6 @@
 
 #if OPAL_SRTP
 
-namespace PWLibStupidLinkerHacks {
-  extern int libSRTPLoader;
-};
 
 ////////////////////////////////////////////////////////////////////
 //
@@ -139,6 +136,8 @@ class LibSRTP_UDP : public OpalSRTP_UDP
     virtual SendReceiveStatus OnReceiveControl(RTP_ControlFrame & frame);
 };
 
+PFACTORY_LOAD(LibSRTPSecurityMode_STRONGHOLD);
+
 
 #endif // OPAL_SRTP
 
