@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26833 $
- * $Author: rjongbloed $
- * $Date: 2011-12-22 23:24:13 -0600 (Thu, 22 Dec 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_SIP_HANDLERS_H
@@ -88,7 +88,7 @@ public:
   virtual OpalTransport * GetTransport();
 
   virtual SIPAuthentication * GetAuthentication()
-  { return authentication; }
+  { return m_authentication; }
 
   virtual const SIPURL & GetAddressOfRecord()
     { return m_addressOfRecord; }
@@ -115,6 +115,7 @@ public:
   virtual void OnReceivedResponse(SIPTransaction & transaction, SIP_PDU & response);
   virtual void OnReceivedIntervalTooBrief(SIPTransaction & transaction, SIP_PDU & response);
   virtual void OnReceivedTemporarilyUnavailable(SIPTransaction & transaction, SIP_PDU & response);
+  virtual void OnReceivedForbidden(SIPTransaction & transaction, SIP_PDU & response);
   virtual void OnReceivedAuthenticationRequired(SIPTransaction & transaction, SIP_PDU & response);
   virtual void OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response);
   virtual void OnTransactionFailed(SIPTransaction & transaction);
@@ -146,7 +147,8 @@ protected:
 
   SIPEndPoint               & endpoint;
 
-  SIPAuthentication         * authentication;
+  SIPAuthentication         * m_authentication;
+  unsigned                    m_authenticateErrors;
   PString                     m_username;
   PString                     m_password;
   PString                     m_realm;
@@ -168,6 +170,7 @@ protected:
   PTimer                      m_expireTimer; 
   SIPURL                      m_proxy;
   OpalProductInfo             m_productInfo;
+  bool                        m_retry403;
 
   // Keep a copy of the keys used for easy removal on destruction
   typedef std::map<PString, PSafePtr<SIPHandler> > IndexMap;
