@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26927 $
- * $Author: rjongbloed $
- * $Date: 2012-02-01 16:09:34 -0600 (Wed, 01 Feb 2012) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_SIP_SIPCON_H
@@ -625,6 +625,8 @@ class SIPConnection : public OpalRTPConnection
 
   protected:
     virtual bool GarbageCollection();
+    void OnUserInputInlineRFC2833(OpalRFC2833Info & info, INT type);
+
     PDECLARE_NOTIFIER(PTimer, SIPConnection, OnSessionTimeout);
     PDECLARE_NOTIFIER(PTimer, SIPConnection, OnInviteResponseRetry);
     PDECLARE_NOTIFIER(PTimer, SIPConnection, OnInviteResponseTimeout);
@@ -670,7 +672,7 @@ class SIPConnection : public OpalRTPConnection
     );
 
     bool SendReINVITE(PTRACE_PARAM(const char * msg));
-    void StartPendingReINVITE();
+    bool StartPendingReINVITE();
 
     friend class SIPInvite;
     static PBoolean WriteINVITE(OpalTransport & transport, void * param);
@@ -721,13 +723,14 @@ class SIPConnection : public OpalRTPConnection
     unsigned              m_sdpVersion; // Really a sequence number
     bool                  m_needReINVITE;
     bool                  m_handlingINVITE;
+    bool                  m_resolveMultipleFormatReINVITE;
     bool                  m_symmetricOpenStream;
     SIPDialogContext      m_dialog;
     OpalGloballyUniqueID  m_dialogNotifyId;
     int                   m_appearanceCode;
     PString               m_alertInfo;
     SIPAuthentication   * m_authentication;
-    unsigned              m_authenticatedCseq;
+    unsigned              m_authenticateErrors;
     PTimer                sessionTimer;
 
     std::map<SIP_PDU::Methods, unsigned> m_lastRxCSeq;
@@ -746,7 +749,7 @@ class SIPConnection : public OpalRTPConnection
     PSafeList<SIPTransaction> m_pendingTransactions;
 
 #if OPAL_FAX
-    bool m_switchedToFaxMode;
+    bool m_switchedToT38;
 #endif
 
     enum {
@@ -766,6 +769,12 @@ class SIPConnection : public OpalRTPConnection
     PSafePtr<OpalSIPIMContext> m_imContext;
 #endif
 
+    enum {
+      UserInputMethodUnknown,
+      ReceivedRFC2833,
+      ReceivedINFO
+    } m_receivedUserInputMethod;
+
   private:
     P_REMOVE_VIRTUAL_VOID(OnCreatingINVITE(SIP_PDU&));
     P_REMOVE_VIRTUAL_VOID(OnReceivedTrying(SIP_PDU &));
