@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26937 $
- * $Author: rjongbloed $
- * $Date: 2012-02-06 19:54:19 -0600 (Mon, 06 Feb 2012) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_SIP_SIPPDU_H
@@ -192,10 +192,14 @@ class SIPURL : public PURL
         @return FALSE if DNS is available but entry is larger than last SRV record entry,
                 TRUE if DNS lookup fails or no DNS is available
       */
-    PBoolean AdjustToDNS(
-      PINDEX entry = 0  /// Entry in the SRV record to adjust to
+    PBoolean AdjustToDNSe(
+      PINDEX entry  /// Entry in the SRV record to adjust to
     );
 
+    PBoolean AdjustToDNS(SIPEndPoint & ep);
+
+    PBoolean TryNextSRV(SIPEndPoint & ep);
+
     /// Generate a unique string suitable as a dialog tag
     static PString GenerateTag();
 
@@ -228,8 +232,13 @@ class SIPURL : public PURL
 class SIPURLList : public std::list<SIPURL>
 {
   public:
-    bool FromString(const PString & str, bool reversed = false);
+    bool FromString(
+      const PString & str,
+      SIPURL::UsageContext context = SIPURL::RouteURI,
+      bool reversed = false
+    );
     PString ToString() const;
+    friend ostream & operator<<(ostream & strm, const SIPURLList & urls);
 };
 
 
@@ -777,7 +786,7 @@ class SIPDialogContext
              !m_remoteTag.IsEmpty();
     }
 
-    OpalTransportAddress GetRemoteTransportAddress() const;
+    OpalTransportAddress GetRemoteTransportAddress(SIPEndPoint &) const;
 
     void SetForking(bool f) { m_forking = f; }
 
@@ -816,6 +825,7 @@ struct SIPParameters
   PCaselessString m_proxyAddress;
   PCaselessString m_addressOfRecord;
   PCaselessString m_contactAddress;
+  PCaselessString m_interface;
   SIPMIMEInfo     m_mime;
   PString         m_authID;
   PString         m_password;
@@ -824,6 +834,7 @@ struct SIPParameters
   unsigned        m_restoreTime;
   PTimeInterval   m_minRetryTime;
   PTimeInterval   m_maxRetryTime;
+  bool            m_retry403;
   void          * m_userData;
 };
 
@@ -932,8 +943,9 @@ class SIPTransaction : public SIP_PDU
 };
 
 
-#define OPAL_PROXY_PARAM    "OPAL-proxy"
-#define OPAL_LOCAL_ID_PARAM "OPAL-local-id"
+#define OPAL_PROXY_PARAM     "OPAL-proxy"
+#define OPAL_LOCAL_ID_PARAM  "OPAL-local-id"
+#define OPAL_INTERFACE_PARAM "OPAL-interface"
 
 
 /////////////////////////////////////////////////////////////////////////
