@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26102 $
- * $Author: rjongbloed $
- * $Date: 2011-06-30 00:00:16 -0500 (Thu, 30 Jun 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef OPAL_T38_T38PROTO_H
@@ -215,8 +215,9 @@ class OpalFaxConnection : public OpalLocalConnection
     virtual void OnStopMediaPatch(OpalMediaPatch & patch);
     virtual PBoolean SendUserInputTone(char tone, unsigned duration);
     virtual void OnUserInputTone(char tone, unsigned duration);
-    virtual bool SwitchFaxMediaStreams(bool enableFax);
-    virtual void OnSwitchedFaxMediaStreams(bool enabledFax);
+    virtual bool SwitchFaxMediaStreams(bool toT38);
+    virtual void OnSwitchedFaxMediaStreams(bool toT38, bool success);
+    virtual bool OnSwitchingFaxMediaStreams(bool toT38);
   //@}
 
   /**@name New operations */
