@@ -23,20 +23,15 @@
  * 2550 Garcia Avenue
  * Mountain View, California  94043
  */
-/* 16kbps version created, used 24kbps code and changing as little as possible.
- * G.726 specs are available from ITU's gopher or WWW site (http://www.itu.ch)
- * If any errors are found, please contact me at mrand@tamu.edu
- *      -Marc Randolph
- */
 
 /*
- * g726_16.c
+ * g726_24.c
  *
  * Description:
  *
- * g723_16_encoder(), g723_16_decoder()
+ * g723_24_encoder(), g723_24_decoder()
  *
- * These routines comprise an implementation of the CCITT G.726 16 Kbps
+ * These routines comprise an implementation of the CCITT G.723 24 Kbps
  * ADPCM coding algorithm.  Essentially, this implementation is identical to
  * the bit level description except for a few deviations which take advantage
  * of workstation attributes, such as hardware 2's complement arithmetic.
@@ -51,55 +46,46 @@
  * 2:1 over the G.711 format. For 24 kbps 40 kbps operation, the quantized
  * difference signal is 3 bits and 5 bits, respectively.
  *
- * $Revision: 22675 $
- * $Author: rjongbloed $
- * $Date: 2009-05-19 23:23:29 -0500 (Tue, 19 May 2009) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 #include "g72x.h"
 #include "private.h"
 
 /*
- * Maps G.723_16 code word to reconstructed scale factor normalized log
- * magnitude values.  Comes from Table 11/G.726
+ * Maps G.723_24 code word to reconstructed scale factor normalized log
+ * magnitude values.
  */
-static short	_dqlntab[4] = { 116, 365, 365, 116}; 
+static short	_dqlntab[8] = {-2048, 135, 273, 373, 373, 273, 135, -2048};
 
-/* Maps G.723_16 code word to log of scale factor multiplier.
- *
- * _witab[4] is actually {-22 , 439, 439, -22}, but FILTD wants it
- * as WI << 5  (multiplied by 32), so we'll do that here 
- */
-static short	_witab[4] = {-704, 14048, 14048, -704};
+/* Maps G.723_24 code word to log of scale factor multiplier. */
+static short	_witab[8] = {-128, 960, 4384, 18624, 18624, 4384, 960, -128};
 
 /*
- * Maps G.723_16 code words to a set of values whose long and short
+ * Maps G.723_24 code words to a set of values whose long and short
  * term averages are computed and then compared to give an indication
  * how stationary (steady state) the signal is.
  */
+static short	_fitab[8] = {0, 0x200, 0x400, 0xE00, 0xE00, 0x400, 0x200, 0};
 
-/* Comes from FUNCTF */
-static short	_fitab[4] = {0, 0xE00, 0xE00, 0};
-
-/* Comes from quantizer decision level tables (Table 7/G.726)
- */
-static int qtab_723_16[1] = {261};
-
+static int qtab_723_24[3] = {8, 218, 331};
 
 /*
- * g723_16_encoder()
+ * g723_24_encoder()
  *
- * Encodes a linear PCM, A-law or u-law input sample and returns its 2-bit code.
+ * Encodes a linear PCM, A-law or u-law input sample and returns its 3-bit code.
  * Returns -1 if invalid input coding value.
  */
 int
-g726_16_encoder(
+g726_24_encoder(
 	int		sl,
 	int		in_coding,
 	g726_state *state_ptr)
 {
 	int		sezi;
-	int		sez;			/* ACCUM */
 	int		sei;
+	int		sez;			/* ACCUM */
 	int		se;
 	int		d;				/* SUBTA */
 	int		y;				/* MIX */
@@ -131,35 +117,27 @@ g726_16_encoder(
 
 	/* quantize prediction difference d */
 	y = step_size(state_ptr);	/* quantizer step size */
-	i = quantize(d, y, qtab_723_16, 1);  /* i = ADPCM code */
-
-	      /* Since quantize() only produces a three level output
-	       * (1, 2, or 3), we must create the fourth one on our own
-	       */
-	if (i == 3)                          /* i code for the zero region */
-	  if ((d & 0x8000) == 0)             /* If d > 0, i=3 isn't right... */
-	    i = 0;
-	    
-	dq = reconstruct(i & 2, _dqlntab[i], y); /* quantized diff. */
+	i = quantize(d, y, qtab_723_24, 3);	/* i = ADPCM code */
+	dq = reconstruct(i & 4, _dqlntab[i], y); /* quantized diff. */
 
 	sr = (dq < 0) ? se - (dq & 0x3FFF) : se + dq; /* reconstructed signal */
 
 	dqsez = sr + sez - se;		/* pole prediction diff. */
 
-	update(2, y, _witab[i], _fitab[i], dq, sr, dqsez, state_ptr);
+	update(3, y, _witab[i], _fitab[i], dq, sr, dqsez, state_ptr);
 
 	return (i);
 }
 
 /*
- * g723_16_decoder()
+ * g723_24_decoder()
  *
- * Decodes a 2-bit CCITT G.723_16 ADPCM code and returns
+ * Decodes a 3-bit CCITT G.723_24 ADPCM code and returns
  * the resulting 16-bit linear PCM, A-law or u-law sample value.
  * -1 is returned if the output coding is unknown.
  */
 int
-g726_16_decoder(
+g726_24_decoder(
 	int		i,
 	int		out_coding,
 	g726_state *state_ptr)
@@ -173,26 +151,26 @@ g726_16_decoder(
 	int		sr;				/* ADDB */
 	int		dqsez;
 
-	i &= 0x03;			/* mask to get proper bits */
+	i &= 0x07;				/* mask to get proper bits */
 	sezi = predictor_zero(state_ptr);
 	sez = sezi >> 1;
 	sei = sezi + predictor_pole(state_ptr);
 	se = sei >> 1;			/* se = estimated signal */
 
 	y = step_size(state_ptr);	/* adaptive quantizer step size */
-	dq = reconstruct(i & 0x02, _dqlntab[i], y); /* unquantize pred diff */
+	dq = reconstruct(i & 0x04, _dqlntab[i], y); /* unquantize pred diff */
 
 	sr = (dq < 0) ? (se - (dq & 0x3FFF)) : (se + dq); /* reconst. signal */
 
 	dqsez = sr - se + sez;			/* pole prediction diff. */
 
-	update(2, y, _witab[i], _fitab[i], dq, sr, dqsez, state_ptr);
+	update(3, y, _witab[i], _fitab[i], dq, sr, dqsez, state_ptr);
 
 	switch (out_coding) {
 	case AUDIO_ENCODING_ALAW:
-		return (tandem_adjust_alaw(sr, se, y, i, 2, qtab_723_16));
+		return (tandem_adjust_alaw(sr, se, y, i, 4, qtab_723_24));
 	case AUDIO_ENCODING_ULAW:
-		return (tandem_adjust_ulaw(sr, se, y, i, 2, qtab_723_16));
+		return (tandem_adjust_ulaw(sr, se, y, i, 4, qtab_723_24));
 	case AUDIO_ENCODING_LINEAR:
 		return (sr << 2);	/* sr was of 14-bit dynamic range */
 	default:
