@@ -19,9 +19,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22746 $
- * $Author: rjongbloed $
- * $Date: 2009-05-27 20:34:50 -0500 (Wed, 27 May 2009) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #define _CRT_NONSTDC_NO_DEPRECATE 1
@@ -37,7 +37,7 @@
 
 #include <codec/opalplugin.h>
 
-#include "./inc/gsm.h"
+#include <gsm.h>
 
 #define	BITS_PER_SECOND         13200
 
