@@ -25,9 +25,9 @@
  * Contributor(s): Michele Piccini (michele@piccini.com)
  *                 Derek Smithies (derek@indranet.co.nz)
  *
- * $Revision: 25138 $
- * $Author: rjongbloed $
- * $Date: 2011-02-20 20:31:59 -0600 (Sun, 20 Feb 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 /*
@@ -560,12 +560,8 @@ static struct PluginCodec_information licenseInfo = {
 };
 
 static const char YUV420PDesc[]  = { "YUV420P" };
-
-static const char h261QCIFDesc[]  = { "H.261-QCIF" };
-static const char h261CIFDesc[]   = { "H.261-CIF" };
-static const char h261Desc[]      = { "H.261" };
-
-static const char sdpH261[]   = { "h261" };
+static const char h261Desc[]     = { "H.261" };
+static const char sdpH261[]      = { "h261" };
 
 static const char QCIF_MPI[]  = PLUGINCODEC_QCIF_MPI;
 static const char  CIF_MPI[]  =  PLUGINCODEC_CIF_MPI;
@@ -985,152 +981,6 @@ static struct PluginCodec_Option const * const xcifOptionTable[] = {
 
 static struct PluginCodec_Definition h261CodecDefn[] =
 {
-  { 
-    // CIF only encoder
-    PLUGIN_CODEC_VERSION_OPTIONS,       // codec API version
-    &licenseInfo,                       // license information
-
-    PluginCodec_MediaTypeVideo |        // audio codec
-    PluginCodec_RTPTypeExplicit,        // specified RTP type
-
-    h261CIFDesc,                        // text decription
-    YUV420PDesc,                        // source format
-    h261CIFDesc,                        // destination format
-
-    cifOptionTable,                     // user data 
-
-    H261_CLOCKRATE,                     // samples per second
-    H261_BITRATE,                       // raw bits per second
-    20000,                              // nanoseconds per frame
-
-    {{
-      CIF_WIDTH,                          // frame width
-      CIF_HEIGHT,                         // frame height
-      10,                                 // recommended frame rate
-      60,                                 // maximum frame rate
-    }},
-    
-    RTP_RFC2032_PAYLOAD,                // IANA RTP payload code
-    sdpH261,                            // RTP payload name
-
-    create_encoder,                     // create codec function
-    destroy_encoder,                    // destroy codec
-    codec_encoder,                      // encode/decode
-    h323EncoderControls,                // codec controls
-
-    PluginCodec_H323VideoCodec_h261,    // h323CapabilityType 
-    NULL                                // h323CapabilityData
-  },
-  { 
-    // CIF only decoder
-    PLUGIN_CODEC_VERSION_OPTIONS,       // codec API version
-    &licenseInfo,                       // license information
-
-    PluginCodec_MediaTypeVideo |        // audio codec
-    PluginCodec_RTPTypeExplicit,        // specified RTP type
-
-    h261CIFDesc,                        // text decription
-    h261CIFDesc,                        // source format
-    YUV420PDesc,                        // destination format
-
-    cifOptionTable,                     // user data 
-
-    H261_CLOCKRATE,                     // samples per second
-    H261_BITRATE,                       // raw bits per second
-    20000,                              // nanoseconds per frame
-
-    {{
-      CIF_WIDTH,                        // frame width
-      CIF_HEIGHT,                       // frame height
-      10,                               // recommended frame rate
-      60,                               // maximum frame rate
-    }},
-    
-    RTP_RFC2032_PAYLOAD,                // IANA RTP payload code
-    sdpH261,                            // RTP payload name
-
-    create_decoder,                     // create codec function
-    destroy_decoder,                    // destroy codec
-    codec_decoder,                      // encode/decode
-    h323DecoderControls,                // codec controls
-
-    PluginCodec_H323VideoCodec_h261,    // h323CapabilityType 
-    NULL                                // h323CapabilityData
-  },
-
-  { 
-    // QCIF only encoder
-    PLUGIN_CODEC_VERSION_OPTIONS,       // codec API version
-    &licenseInfo,                       // license information
-
-    PluginCodec_MediaTypeVideo |        // audio codec
-    PluginCodec_RTPTypeExplicit,        // specified RTP type
-
-    h261QCIFDesc,                       // text decription
-    YUV420PDesc,                        // source format
-    h261QCIFDesc,                       // destination format
-
-    qcifOptionTable,                    // user data 
-
-    H261_CLOCKRATE,                     // samples per second
-    H261_BITRATE,                       // raw bits per second
-    20000,                              // nanoseconds per frame
-
-    {{
-      QCIF_WIDTH,                         // frame width
-      QCIF_HEIGHT,                        // frame height
-      10,                                 // recommended frame rate
-      60,                                 // maximum frame rate
-    }},
-    
-    RTP_RFC2032_PAYLOAD,                // IANA RTP payload code
-    sdpH261,                            // RTP payload name
-
-    create_encoder,                     // create codec function
-    destroy_encoder,                    // destroy codec
-    codec_encoder,                      // encode/decode
-    h323EncoderControls,                // codec controls
-
-    PluginCodec_H323VideoCodec_h261,    // h323CapabilityType 
-    NULL                                // h323CapabilityData
-  },
-  { 
-    // QCIF only decoder
-    PLUGIN_CODEC_VERSION_OPTIONS,       // codec API version
-    &licenseInfo,                       // license information
-
-    PluginCodec_MediaTypeVideo |        // audio codec
-    PluginCodec_RTPTypeExplicit,        // specified RTP type
-
-    h261QCIFDesc,                       // text decription
-    h261QCIFDesc,                       // source format
-    YUV420PDesc,                        // destination format
-
-    qcifOptionTable,                    // user data 
-
-    H261_CLOCKRATE,                     // samples per second
-    H261_BITRATE,                       // raw bits per second
-    20000,                              // nanoseconds per frame
-
-    {{
-      QCIF_WIDTH,                       // frame width
-      QCIF_HEIGHT,                      // frame height
-      10,                               // recommended frame rate
-      60,                               // maximum frame rate
-    }},
-    
-    RTP_RFC2032_PAYLOAD,                // IANA RTP payload code
-    sdpH261,                            // RTP payload name
-
-    create_decoder,                     // create codec function
-    destroy_decoder,                    // destroy codec
-    codec_decoder,                      // encode/decode
-    h323DecoderControls,                // codec controls
-
-    PluginCodec_H323VideoCodec_h261,    // h323CapabilityType 
-    NULL                                // h323CapabilityData
-  },
-
   { 
     // Both QCIF and CIF (dynamic) encoder
     PLUGIN_CODEC_VERSION_OPTIONS,       // codec API version
