@@ -38,12 +38,9 @@
  *                 Guilhem Tardy (gtardy@salyens.com)
  *                 Craig Southeren (craigs@postincrement.com)
  *
- */
-
-/*
-  Notes
-  -----
-
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef PLUGIN_CODEC_DLL_EXPORTS
@@ -59,21 +56,17 @@
 
 #include "../common/mpi.h"
 #include "../common/dyna.h"
+#include "../common/ffmpeg.h"
 #include "rfc2190.h"
 #include "rfc2429.h"
 
 
-extern "C" {
-#include LIBAVCODEC_HEADER
-};
-
-
 static const char YUV420PDesc[]  = { "YUV420P" };
-static const char h263PDesc[]    = { "H.263P" };
-static const char sdpH263P[]     = { "h263-1998" };
+static const char h263PDesc[]    = { "H.263plus" };
+static const char sdpH263P[]     = { "H263-1998" };
 
 static const char h263Desc[]     = { "H.263" };
-static const char sdpH263[]      = { "h263" };
+static const char sdpH263[]      = { "H263" };
 
 
 #define MAX_H263_CUSTOM_SIZES 10
@@ -258,7 +251,7 @@ bool H263_Base_EncoderContext::Init(CodecID codecId)
     m_context->debug |= FF_DEBUG_BUGS | FF_DEBUG_BUFFERS;
 #endif
 
-  PTRACE(4, m_prefix, "Encoder created");
+  PTRACE(4, m_prefix, "Encoder created (SVN $Revision$)");
 
   return true;
 }
@@ -281,6 +274,7 @@ bool H263_Base_EncoderContext::SetOptions(const char * const * options)
 void H263_Base_EncoderContext::SetOption(const char * option, const char * value)
 {
   if (STRCMPI(option, PLUGINCODEC_OPTION_FRAME_TIME) == 0) {
+    m_context->time_base.den = 2997;
     m_context->time_base.num = atoi(value)*m_context->time_base.den/VIDEO_CLOCKRATE;
     return;
   }
@@ -297,6 +291,7 @@ void H263_Base_EncoderContext::SetOption(const char * option, const char * value
 
   if (STRCMPI(option, PLUGINCODEC_OPTION_MAX_TX_PACKET_SIZE) == 0) {
     m_context->rtp_payload_size = atoi(value);
+    m_packetizer->SetMaxPayloadSize(m_context->rtp_payload_size);
     return;
   }
 
@@ -307,6 +302,8 @@ void H263_Base_EncoderContext::SetOption(const char * option, const char * value
 
   if (STRCMPI(option, PLUGINCODEC_OPTION_TEMPORAL_SPATIAL_TRADE_OFF) == 0) {
     m_context->qmax = atoi(value);
+    if (m_context->qmax <= m_context->qmin)
+      m_context->qmax = m_context->qmin+1;
     return;
   }
 
@@ -362,7 +359,7 @@ void H263_Base_EncoderContext::SetOption(const char * option, const char * value
   if (STRCMPI(option, H263_ANNEX_K) == 0) {
     // Annex K: Slice Structure
     // does not work with eyeBeam
-    if (atoi(value) == 1)
+    if (atoi(value) != 0)
       m_context->flags |= CODEC_FLAG_H263P_SLICE_STRUCT; 
     else
       m_context->flags &= ~CODEC_FLAG_H263P_SLICE_STRUCT; 
@@ -378,6 +375,18 @@ void H263_Base_EncoderContext::SetOption(const char * option, const char * value
       m_context->flags &= ~CODEC_FLAG_H263P_AIV; 
     return;
   }
+
+  if (STRCMPI(option, PLUGINCODEC_MEDIA_PACKETIZATION) == 0 ||
+      STRCMPI(option, PLUGINCODEC_MEDIA_PACKETIZATIONS) == 0) {
+    if (strstr(value, m_packetizer->GetName()) == NULL) {
+      PTRACE(4, m_prefix, "Packetisation changed to " << value);
+      delete m_packetizer;
+      if (STRCMPI(value, "RFC2429") == 0)
+        m_packetizer = new RFC2429Frame;
+      else
+        m_packetizer = new RFC2190Packetizer;
+    }
+  }
 }
 
 
@@ -392,6 +401,14 @@ bool H263_Base_EncoderContext::OpenCodec()
   m_context->rc_max_rate = m_context->bit_rate;
   m_context->rc_buffer_size = m_context->bit_rate*4; // Enough bits for 4 seconds
   m_context->bit_rate_tolerance = m_context->bit_rate/10;
+  
+  // FFMPEG requires bit rate tolerance to be at least one frame size
+  int oneFrameBits = (int)((int64_t)m_context->bit_rate*m_context->time_base.num/m_context->time_base.den) + 1;
+  if (m_context->bit_rate_tolerance < oneFrameBits) {
+    PTRACE(4, m_prefix, "Limited bit_rate_tolerance "
+           "(" << m_context->bit_rate_tolerance << ") to size of one frame (" << oneFrameBits << ')');
+    m_context->bit_rate_tolerance = oneFrameBits;
+  }
 
   m_context->i_quant_factor = (float)-0.6;  // qscale factor between p and i frames
   m_context->i_quant_offset = (float)0.0;   // qscale offset between p and i frames
@@ -417,15 +434,21 @@ bool H263_Base_EncoderContext::OpenCodec()
   m_inputFrame->data[1] = m_inputFrame->data[0] + planeSize;
   m_inputFrame->data[2] = m_inputFrame->data[1] + (planeSize / 4);
 
+  if (m_context->width > 352)
+    m_context->flags &= ~CODEC_FLAG_EMU_EDGE; // Totally bizarre! FFMPEG crashes if on for CIF4
+
   // Dump info
   PTRACE(5, m_prefix, "Size is " << m_context->width << "x" << m_context->height);
   PTRACE(5, m_prefix, "GOP is " << m_context->gop_size);
+  PTRACE(5, m_prefix, "time_base set to " << m_context->time_base.num << '/' << m_context->time_base.den
+         << " (" << std::setprecision(2) << ((double)m_context->time_base.den/m_context->time_base.num) << "fps)");
   PTRACE(5, m_prefix, "bit_rate set to " << m_context->bit_rate);
   PTRACE(5, m_prefix, "rc_max_rate is " <<  m_context->rc_max_rate);
   PTRACE(5, m_prefix, "rc_min_rate set to " << m_context->rc_min_rate);
   PTRACE(5, m_prefix, "bit_rate_tolerance set to " <<m_context->bit_rate_tolerance);
   PTRACE(5, m_prefix, "qmin set to " << m_context->qmin);
   PTRACE(5, m_prefix, "qmax set to " << m_context->qmax);
+  PTRACE(5, m_prefix, "payload size set to " << m_context->rtp_payload_size);
 
   #define CODEC_TRACER_FLAG(tracer, flag) \
     PTRACE(4, m_prefix, #flag " is " << ((m_context->flags & flag) ? "enabled" : "disabled"));
@@ -522,10 +545,9 @@ bool H263_Base_EncoderContext::EncodeFrames(const BYTE * src, unsigned & srcLen,
     }
 
     if (encodedLen == 0) {
-      PTRACE(1, m_prefix, "Encoder returned no data");
-      dstRTP.SetPayloadSize(0);
-      dstLen = dstRTP.GetHeaderSize();
+      PTRACE(3, m_prefix, "Encoder returned no data");
       flags |= PluginCodec_ReturnCoderLastFrame;
+      dstLen = 0;
       return true;
     }
 
@@ -537,8 +559,10 @@ bool H263_Base_EncoderContext::EncodeFrames(const BYTE * src, unsigned & srcLen,
 
     // return the first encoded block of data
     if (!m_packetizer->GetPacket(dstRTP, flags)) {
-      PTRACE(1, m_prefix, "Packetizer failed");
-      return false; // Huh?
+      PTRACE(3, m_prefix, "No packets encoded for frame");
+      flags |= PluginCodec_ReturnCoderLastFrame;
+      dstLen = 0;
+      return true;
     }
   }
 
@@ -591,12 +615,6 @@ bool H263_RFC2190_EncoderContext::Init()
 }
 
 
-void H263_RFC2190_EncoderContext::SetMaxRTPFrameSize(unsigned size)
-{
-  m_context->rtp_payload_size = size;
-}
-
-
 /////////////////////////////////////////////////////////////////////////////
 
 H263_RFC2429_EncoderContext::H263_RFC2429_EncoderContext()
@@ -615,9 +633,11 @@ bool H263_RFC2429_EncoderContext::Init()
 }
 
 
-void H263_RFC2429_EncoderContext::SetMaxRTPFrameSize (unsigned size)
+/////////////////////////////////////////////////////////////////////////////
+
+Packetizer::Packetizer()
+  : m_maxPayloadSize(PluginCodec_RTP_MaxPayloadSize)
 {
-  ((RFC2429Frame *)m_packetizer)->SetMaxPayloadSize((uint16_t)size);
 }
 
 
@@ -625,6 +645,9 @@ void H263_RFC2429_EncoderContext::SetMaxRTPFrameSize (unsigned size)
 
 H263_Base_DecoderContext::H263_Base_DecoderContext(const char * prefix, Depacketizer * depacketizer)
   : m_prefix(prefix)
+  , m_codec(NULL)
+  , m_context(NULL)
+  , m_outputFrame(NULL)
   , m_depacketizer(depacketizer)
 {
   if (!FFMPEGLibraryInstance.Load())
@@ -659,7 +682,7 @@ H263_Base_DecoderContext::H263_Base_DecoderContext(const char * prefix, Depacket
 
   m_depacketizer->NewFrame();
 
-  PTRACE(4, m_prefix, "Decoder created");
+  PTRACE(4, m_prefix, "Decoder created (SVN $Revision$)");
 }
 
 H263_Base_DecoderContext::~H263_Base_DecoderContext()
@@ -701,6 +724,24 @@ void H263_Base_DecoderContext::CloseCodec()
   }
 }
 
+bool H263_Base_DecoderContext::SetOptions(const char * const * options)
+{
+  for (const char * const * option = options; *option != NULL; option += 2) {
+    if (STRCMPI(option[0], PLUGINCODEC_MEDIA_PACKETIZATION) == 0 ||
+        STRCMPI(option[0], PLUGINCODEC_MEDIA_PACKETIZATIONS) == 0) {
+      if (strstr(option[1], m_depacketizer->GetName()) == NULL) {
+        PTRACE(4, m_prefix, "Packetisation changed to " << option[1]);
+        delete m_depacketizer;
+        if (STRCMPI(option[1], "RFC2429") == 0)
+          m_depacketizer = new RFC2429Frame;
+        else
+          m_depacketizer = new RFC2190Depacketizer;
+      }
+    }
+  }
+  return true;
+}
+
 bool H263_Base_DecoderContext::DecodeFrames(const BYTE * src, unsigned & srcLen, BYTE * dst, unsigned & dstLen, unsigned int & flags)
 {
   // create RTP frame from source buffer
@@ -731,8 +772,8 @@ bool H263_Base_DecoderContext::DecodeFrames(const BYTE * src, unsigned & srcLen,
   if (m_depacketizer->IsIntraFrame())
     flags |= PluginCodec_ReturnCoderIFrame;
 
-#if FFMPEG_HAS_DECODE_ERROR_COUNT
-  unsigned error_before = m_context->decode_error_count;
+#ifdef FFMPEG_HAS_DECODE_ERROR_COUNT
+  int error_before = m_context->decode_error_count;
 #endif
 
   PTRACE(5, m_prefix, "Decoding " << m_depacketizer->GetLength()  << " bytes");
@@ -745,22 +786,18 @@ bool H263_Base_DecoderContext::DecodeFrames(const BYTE * src, unsigned & srcLen,
 
   m_depacketizer->NewFrame();
 
+#ifdef FFMPEG_HAS_DECODE_ERROR_COUNT
   // if error occurred, tell the other end to send another I-frame and hopefully we can resync
-  if (bytesDecoded < 0
-#if FFMPEG_HAS_DECODE_ERROR_COUNT
-      || error_before != m_context->decode_error_count
-#endif
-  ) {
+  if (error_before != m_context->decode_error_count)
     flags = PluginCodec_ReturnCoderRequestIFrame;
-    return true;
-  }
+#endif
 
-  if (!gotPicture || bytesDecoded == 0) {
+  if (!gotPicture) {
     PTRACE(3, m_prefix, "Decoded "<< bytesDecoded << " bytes without getting a Picture"); 
     return true;
   }
 
-  PTRACE(5, m_prefix, "Decoded " << bytesDecoded << " bytes"<< ", Resolution: " << m_context->width << "x" << m_context->height);
+  PTRACE(5, m_prefix, "Decoded " << bytesDecoded << " bytes" << ", Resolution: " << m_context->width << "x" << m_context->height);
 
   // if decoded frame size is not legal, request an I-Frame
   if (m_context->width <= 0 || m_context->height <= 0) {
@@ -859,13 +896,6 @@ static int free_codec_options ( const struct PluginCodec_Definition *, void *, c
   return 1;
 }
 
-static int valid_for_protocol ( const struct PluginCodec_Definition *, void *, const char *, void * parm, unsigned * parmLen)
-{
-  if (parmLen == NULL || parm == NULL || *parmLen != sizeof(char *))
-    return 0;
-
-  return (STRCMPI((const char *)parm, "sip") == 0) ? 1 : 0;
-}
 
 /////////////////////////////////////////////////////////////////////////////
 
@@ -1268,6 +1298,18 @@ static int decoder_get_output_data_size(const PluginCodec_Definition * codec, vo
   return sizeof(PluginCodec_Video_FrameHeader) + ((codec->parm.video.maxFrameWidth * codec->parm.video.maxFrameHeight * 3) / 2);
 }
 
+static int decoder_set_options(const PluginCodec_Definition *, 
+                               void * context,
+                               const char * , 
+                               void * parm, 
+                               unsigned * parmLen)
+{
+  if (parmLen == NULL || *parmLen != sizeof(const char **) || parm == NULL)
+    return 0;
+
+  return ((H263_Base_DecoderContext *)context)->SetOptions((const char * const *)parm);
+}
+
 PLUGINCODEC_CONTROL_LOG_FUNCTION_DEF
 
 /////////////////////////////////////////////////////////////////////////////
@@ -1306,7 +1348,6 @@ static const char CIF4_MPI[]   = PLUGINCODEC_CIF4_MPI;
 static const char CIF16_MPI[]  = PLUGINCODEC_CIF16_MPI;
 
 static PluginCodec_ControlDefn EncoderControls[] = {
-  { PLUGINCODEC_CONTROL_VALID_FOR_PROTOCOL,    valid_for_protocol },
   { PLUGINCODEC_CONTROL_GET_CODEC_OPTIONS,     get_codec_options },
   { PLUGINCODEC_CONTROL_FREE_CODEC_OPTIONS,    free_codec_options },
   { PLUGINCODEC_CONTROL_TO_NORMALISED_OPTIONS, to_normalised_options },
@@ -1319,6 +1360,7 @@ static PluginCodec_ControlDefn EncoderControls[] = {
 
 static PluginCodec_ControlDefn DecoderControls[] = {
   { PLUGINCODEC_CONTROL_GET_CODEC_OPTIONS,     get_codec_options },
+  { PLUGINCODEC_CONTROL_SET_CODEC_OPTIONS,     decoder_set_options },
   { PLUGINCODEC_CONTROL_GET_OUTPUT_DATA_SIZE,  decoder_get_output_data_size },
   { NULL }
 };
@@ -1416,6 +1458,15 @@ static struct PluginCodec_Option const mediaPacketization =
   "RFC2190"                           // Initial value
 };
 
+static struct PluginCodec_Option const mediaPacketizationPlus =
+{
+  PluginCodec_StringOption,           // Option type
+  PLUGINCODEC_MEDIA_PACKETIZATIONS,   // User visible name
+  true,                               // User Read/Only flag
+  PluginCodec_IntersectionMerge,      // Merge mode
+  "RFC2429,RFC2190"                   // Initial value
+};
+
 static struct PluginCodec_Option const customMPI =
 {
   PluginCodec_StringOption,           // Option type
@@ -1424,7 +1475,7 @@ static struct PluginCodec_Option const customMPI =
   PluginCodec_CustomMerge,            // Merge mode
   DEFAULT_CUSTOM_MPI,                 // Initial value
   "CUSTOM",                           // FMTP option name
-  NULL,                               // FMTP default value
+  DEFAULT_CUSTOM_MPI,                 // FMTP default value
   0,                                  // H.245 generic capability code and bit mask
   NULL,                               // Minimum value
   NULL,                               // Maximum value
@@ -1433,16 +1484,16 @@ static struct PluginCodec_Option const customMPI =
 };
 
 static struct PluginCodec_Option const annexF =
-  { PluginCodec_BoolOption,    H263_ANNEX_F,   false,  PluginCodec_MinMerge, "1", "F", "0" };
+  { PluginCodec_BoolOption,    H263_ANNEX_F,   false, PluginCodec_AndMerge, "1", "F", "0" };
 
 static struct PluginCodec_Option const annexI =
-  { PluginCodec_BoolOption,    H263_ANNEX_I,   false,  PluginCodec_MinMerge, "1", "I", "0" };
+  { PluginCodec_BoolOption,    H263_ANNEX_I,   false, PluginCodec_AndMerge, "1", "I", "0" };
 
 static struct PluginCodec_Option const annexJ =
-  { PluginCodec_BoolOption,    H263_ANNEX_J,   true,  PluginCodec_MinMerge, "1", "J", "0" };
+  { PluginCodec_BoolOption,    H263_ANNEX_J,   false, PluginCodec_AndMerge, "1", "J", "0" };
 
 static struct PluginCodec_Option const annexK =
-  { PluginCodec_IntegerOption, H263_ANNEX_K,   true,  PluginCodec_EqualMerge, "0", "K", "0", 0, "0", "4" };
+  { PluginCodec_IntegerOption, H263_ANNEX_K,   false, PluginCodec_MinMerge, "0", "K", "0", 0, "0", "4" };
 
 static struct PluginCodec_Option const annexN =
   { PluginCodec_BoolOption,    H263_ANNEX_N,   true,  PluginCodec_AndMerge, "0", "N", "0" };
@@ -1451,9 +1502,25 @@ static struct PluginCodec_Option const annexT =
   { PluginCodec_BoolOption,    H263_ANNEX_T,   true,  PluginCodec_AndMerge, "0", "T", "0" };
 
 static struct PluginCodec_Option const annexD =
-  { PluginCodec_BoolOption,    H263_ANNEX_D,   true,  PluginCodec_MinMerge, "1", "D", "0" };
+  { PluginCodec_BoolOption,    H263_ANNEX_D,   false, PluginCodec_AndMerge, "1", "D", "0" };
+
+static struct PluginCodec_Option const TemporalSpatialTradeOff =
+{
+  PluginCodec_IntegerOption,          // Option type
+  PLUGINCODEC_OPTION_TEMPORAL_SPATIAL_TRADE_OFF, // User visible name
+  false,                              // User Read/Only flag
+  PluginCodec_AlwaysMerge,            // Merge mode
+  "31",                               // Initial value
+  NULL,                               // FMTP option name
+  NULL,                               // FMTP default value
+  0,                                  // H.245 generic capability code and bit mask
+  "1",                                // Minimum value
+  "31"                                // Maximum value
+};
 
 static struct PluginCodec_Option const * const h263POptionTable[] = {
+  &mediaPacketizationPlus,
+  &maxBR,
   &qcifMPI,
   &cifMPI,
   &sqcifMPI,
@@ -1467,6 +1534,7 @@ static struct PluginCodec_Option const * const h263POptionTable[] = {
   &annexN,
   &annexT,
   &annexD,
+  &TemporalSpatialTradeOff,
   NULL
 };
 
@@ -1505,10 +1573,13 @@ static struct PluginCodec_Definition h263CodecDefn[] = {
   H263_BITRATE,                       // raw bits per second
   20000,                              // nanoseconds per frame
 
+  {{
   CIF4_WIDTH,                         // frame width
   CIF4_HEIGHT,                        // frame height
   10,                                 // recommended frame rate
   60,                                 // maximum frame rate
+  }},
+
   0,                                  // IANA RTP payload code
   sdpH263P,                           // RTP payload name
 
@@ -1517,7 +1588,7 @@ static struct PluginCodec_Definition h263CodecDefn[] = {
   codec_encoder,                      // encode/decode
   EncoderControls,                    // codec controls
 
-  PluginCodec_H323Codec_NoH323,    // h323CapabilityType 
+  PluginCodec_H323VideoCodec_h263,    // h323CapabilityType 
   NULL                                // h323CapabilityData
 },
 { 
@@ -1539,10 +1610,13 @@ static struct PluginCodec_Definition h263CodecDefn[] = {
   H263_BITRATE,                       // raw bits per second
   20000,                              // nanoseconds per frame
 
+  {{
   CIF4_WIDTH,                         // frame width
   CIF4_HEIGHT,                        // frame height
   10,                                 // recommended frame rate
   60,                                 // maximum frame rate
+  }},
+
   0,                                  // IANA RTP payload code
   sdpH263P,                           // RTP payload name
 
@@ -1551,7 +1625,7 @@ static struct PluginCodec_Definition h263CodecDefn[] = {
   codec_decoder,                      // encode/decode
   DecoderControls,                    // codec controls
 
-  PluginCodec_H323Codec_NoH323,    // h323CapabilityType 
+  PluginCodec_H323VideoCodec_h263,    // h323CapabilityType 
   NULL                                // h323CapabilityData
 },
 
