@@ -31,20 +31,20 @@
  *                 Derek Smithies (derek@indranet.co.nz)
  *                 Robert Jongbloed (robertj@voxlucida.com.au)
  *
- * $Revision: 26869 $
- * $Author: rjongbloed $
- * $Date: 2012-01-19 06:36:13 -0600 (Thu, 19 Jan 2012) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef PLUGIN_CODEC_DLL_EXPORTS
 #include "plugin-config.h"
 #endif
 
-#include "ffmpeg.h"
-#include "dyna.h"
-
 #include <codec/opalplugin.hpp>
 
+#include "../common/ffmpeg.h"
+#include "../common/dyna.h"
+
 #include "shared/h264frame.h"
 #include "shared/x264wrap.h"
 
@@ -54,7 +54,9 @@
 
 class MY_CODEC { };
 
-#ifdef PLUGIN_CODEC_VERSION_INTERSECT
+#if defined(PLUGIN_CODEC_VERSION_H245_DEF_GEN_PARAM)
+static unsigned MyVersion = PLUGIN_CODEC_VERSION_H245_DEF_GEN_PARAM;
+#elif defined(PLUGIN_CODEC_VERSION_INTERSECT)
 static unsigned MyVersion = PLUGIN_CODEC_VERSION_INTERSECT;
 #else
 static unsigned MyVersion = PLUGIN_CODEC_VERSION_OPTIONS;
@@ -64,7 +66,7 @@ static const char MyDescription[] = "x264 Video Codec";     // Human readable de
 static const char FormatNameH323[] = "H.264";               // OpalMediaFormat name string to generate
 static const char FormatNameSIP0[] = "H.264-0";             // OpalMediaFormat name string to generate
 static const char FormatNameSIP1[] = "H.264-1";             // OpalMediaFormat name string to generate
-static const char MyPayloadName[] = "h264";                 // RTP payload name (IANA approved)
+static const char MyPayloadName[] = "H264";                 // RTP payload name (IANA approved)
 static unsigned   MyClockRate = 90000;                      // RTP dictates 90000
 static unsigned   MyMaxFrameRate = 60;                      // Maximum frame rate (per second)
 static unsigned   MyMaxWidth = 2816;                        // Maximum width of frame
@@ -110,7 +112,7 @@ PLUGINCODEC_CONTROL_LOG_FUNCTION_DEF
 
 // Level 3 allows 4CIF at 25fps
 #define DefaultProfileStr          H264_PROFILE_STR_BASELINE
-#define DefaultProfileInt          66
+#define DefaultProfileInt          H264_PROFILE_INT_BASELINE
 #define DefaultProfileH241         64
 #define DefaultLevelStr            H264_LEVEL_STR_3
 #define DefaultLevelInt            30
@@ -125,6 +127,10 @@ PLUGINCODEC_CONTROL_LOG_FUNCTION_DEF
 #define H264_PROFILE_STR_MAIN      "Main"
 #define H264_PROFILE_STR_EXTENDED  "Extended"
 
+#define H264_PROFILE_INT_BASELINE  66
+#define H264_PROFILE_INT_MAIN      77
+#define H264_PROFILE_INT_EXTENDED  88
+
 #define H264_LEVEL_STR_1    "1"
 #define H264_LEVEL_STR_1_b  "1.b"
 #define H264_LEVEL_STR_1_1  "1.1"
@@ -145,8 +151,8 @@ PLUGINCODEC_CONTROL_LOG_FUNCTION_DEF
 
 enum
 {
-    H241_PROFILES                      = 41 | PluginCodec_H245_Collapsing | PluginCodec_H245_TCS | PluginCodec_H245_OLC | PluginCodec_H245_ReqMode | PluginCodec_H245_BooleanArray,
-    H241_LEVEL                         = 42 | PluginCodec_H245_Collapsing | PluginCodec_H245_TCS | PluginCodec_H245_OLC | PluginCodec_H245_ReqMode,
+    H241_PROFILES                      = 41 | PluginCodec_H245_Collapsing | PluginCodec_H245_TCS | PluginCodec_H245_OLC | PluginCodec_H245_ReqMode | PluginCodec_H245_BooleanArray | (1 << PluginCodec_H245_PositionShift),
+    H241_LEVEL                         = 42 | PluginCodec_H245_Collapsing | PluginCodec_H245_TCS | PluginCodec_H245_OLC | PluginCodec_H245_ReqMode                                 | (2 << PluginCodec_H245_PositionShift),
     H241_CustomMaxMBPS                 =  3 | PluginCodec_H245_Collapsing | PluginCodec_H245_TCS | PluginCodec_H245_OLC | PluginCodec_H245_ReqMode,
     H241_CustomMaxFS                   =  4 | PluginCodec_H245_Collapsing | PluginCodec_H245_TCS | PluginCodec_H245_OLC | PluginCodec_H245_ReqMode,
     H241_CustomMaxDPB                  =  5 | PluginCodec_H245_Collapsing | PluginCodec_H245_TCS | PluginCodec_H245_OLC | PluginCodec_H245_ReqMode,
@@ -243,38 +249,78 @@ static struct PluginCodec_Option const SDPProfileAndLevel =
   "42800A"                            // FMTP default value (as per RFC)
 };
 
-static struct PluginCodec_Option const MaxMBPS =
+static struct PluginCodec_Option const MaxMBPS_SDP =
 {
   PluginCodec_IntegerOption,          // Option type
-  "Max MBPS",                         // User visible name
-  false,                              // User Read/Only flag
+  "SIP/SDP Max MBPS",                 // User visible name
+  true,                               // User Read/Only flag
   PluginCodec_MinMerge,               // Merge mode
   "0",                                // Initial value
   "max-mbps",                         // FMTP option name
   "0",                                // FMTP default value
-  H241_CustomMaxMBPS,                 // H.245 generic capability code and bit mask
+  0,                                  // H.245 generic capability code and bit mask
   "0",                                // Minimum value
   "983040"                            // Maximum value
 };
 
-static struct PluginCodec_Option const MaxFS =
+static struct PluginCodec_Option const MaxMBPS_H241 =
 {
   PluginCodec_IntegerOption,          // Option type
-  "Max FS",                           // User visible name
-  false,                              // User Read/Only flag
+  "H.241 Max MBPS",                   // User visible name
+  true,                               // User Read/Only flag
+  PluginCodec_MinMerge,               // Merge mode
+  "0",                                // Initial value
+  NULL,                               // FMTP option name
+  NULL,                               // FMTP default value
+  H241_CustomMaxMBPS,                 // H.245 generic capability code and bit mask
+  "0",                                // Minimum value
+  "1966"                              // Maximum value
+#ifdef PLUGIN_CODEC_VERSION_H245_DEF_GEN_PARAM
+  ,
+  NULL,
+  NULL,
+  "0"                                 // H.245 default value
+#endif
+};
+
+static struct PluginCodec_Option const MaxFS_SDP =
+{
+  PluginCodec_IntegerOption,          // Option type
+  "SIP/SDP Max FS",                   // User visible name
+  true,                               // User Read/Only flag
   PluginCodec_MinMerge,               // Merge mode
   "0",                                // Initial value
   "max-fs",                           // FMTP option name
   "0",                                // FMTP default value
-  H241_CustomMaxFS,                   // H.245 generic capability code and bit mask
+  0,                                  // H.245 generic capability code and bit mask
   "0",                                // Minimum value
   "36864"                             // Maximum value
 };
 
-static struct PluginCodec_Option const MaxBR =
+static struct PluginCodec_Option const MaxFS_H241 =
 {
   PluginCodec_IntegerOption,          // Option type
-  "Max BR",                           // User visible name
+  "H.241 Max FS",                     // User visible name
+  true,                               // User Read/Only flag
+  PluginCodec_MinMerge,               // Merge mode
+  "0",                                // Initial value
+  NULL,                               // FMTP option name
+  NULL,                               // FMTP default value
+  H241_CustomMaxFS,                   // H.245 generic capability code and bit mask
+  "0",                                // Minimum value
+  "144"                               // Maximum value
+#ifdef PLUGIN_CODEC_VERSION_H245_DEF_GEN_PARAM
+  ,
+  NULL,
+  NULL,
+  "0"                                 // H.245 default value
+#endif
+};
+
+static struct PluginCodec_Option const MaxBR_SDP =
+{
+  PluginCodec_IntegerOption,          // Option type
+  "SIP/SDP Max BR",                   // User visible name
   true,                               // User Read/Only flag
   PluginCodec_MinMerge,               // Merge mode
   "0",                                // Initial value
@@ -285,6 +331,26 @@ static struct PluginCodec_Option const MaxBR =
   "240000"                            // Maximum value
 };
 
+static struct PluginCodec_Option const MaxBR_H241 =
+{
+  PluginCodec_IntegerOption,          // Option type
+  "H.241 Max BR",                     // User visible name
+  true,                               // User Read/Only flag
+  PluginCodec_MinMerge,               // Merge mode
+  "0",                                // Initial value
+  NULL,                               // FMTP option name
+  NULL,                               // FMTP default value
+  H241_CustomMaxBRandCPB,             // H.245 generic capability code and bit mask
+  "0",                                // Minimum value
+  "9600"                              // Maximum value
+#ifdef PLUGIN_CODEC_VERSION_H245_DEF_GEN_PARAM
+  ,
+  NULL,
+  NULL,
+  "0"                                 // H.245 default value
+#endif
+};
+
 static struct PluginCodec_Option const MaxNaluSize =
 {
   PluginCodec_IntegerOption,          // Option type
@@ -295,22 +361,19 @@ static struct PluginCodec_Option const MaxNaluSize =
   "max-rcmd-nalu-size",               // FMTP option name
   STRINGIZE(H241_MAX_NALU_SIZE),      // FMTP default value
   H241_Max_NAL_unit_size,             // H.245 generic capability code and bit mask
-  STRINGIZE(UNCOMPRESSED_MB_SIZE),    // Minimum value
+  "396",                              // Minimum value - uncompressed macro block size 16*16*3+12
   "65535"                             // Maximum value
 };
 
-#define OpalPluginCodec_Identifer_H264_Truncated "0.0.8.241.0.0.0" // Some stupid endpoints (e.g. Polycom) use this, one zero short!
-
 #ifdef PLUGIN_CODEC_VERSION_INTERSECT
 static struct PluginCodec_Option const MediaPacketizations =
 {
   PluginCodec_StringOption,           // Option type
   PLUGINCODEC_MEDIA_PACKETIZATIONS,   // User visible name
-  true,                               // User Read/Only flag
+  false,                              // User Read/Only flag
   PluginCodec_IntersectionMerge,      // Merge mode
   OpalPluginCodec_Identifer_H264_Aligned "," // Initial value
-  OpalPluginCodec_Identifer_H264_NonInterleaved ","
-  OpalPluginCodec_Identifer_H264_Truncated
+  OpalPluginCodec_Identifer_H264_NonInterleaved
 };
 #endif
 
@@ -371,11 +434,10 @@ static struct PluginCodec_Option const * OptionTable[] = {
   &Level,
   &H241Profiles,
   &H241Level,
-  &SDPProfileAndLevel,
   &MaxNaluSize,
-  &MaxMBPS,
-  &MaxFS,
-  &MaxBR,
+  &MaxMBPS_H241,
+  &MaxFS_H241,
+  &MaxBR_H241,
   &TemporalSpatialTradeOff,
   &SendAccessUnitDelimiters,
 #ifdef PLUGIN_CODEC_VERSION_INTERSECT
@@ -389,9 +451,9 @@ static struct PluginCodec_Option const * const OptionTable_0[] = {
   &Level,
   &SDPProfileAndLevel,
   &MaxNaluSize,
-  &MaxMBPS,
-  &MaxFS,
-  &MaxBR,
+  &MaxMBPS_SDP,
+  &MaxFS_SDP,
+  &MaxBR_SDP,
   &PacketizationMode_0,
   &TemporalSpatialTradeOff,
   &SendAccessUnitDelimiters,
@@ -403,9 +465,9 @@ static struct PluginCodec_Option const * const OptionTable_1[] = {
   &Level,
   &SDPProfileAndLevel,
   &MaxNaluSize,
-  &MaxMBPS,
-  &MaxFS,
-  &MaxBR,
+  &MaxMBPS_SDP,
+  &MaxFS_SDP,
+  &MaxBR_SDP,
   &PacketizationMode_1,
   &TemporalSpatialTradeOff,
   &SendAccessUnitDelimiters,
@@ -424,9 +486,9 @@ static struct {
   unsigned m_H241;
   unsigned m_Constraints;
 } const ProfileInfo[] = {
-  { H264_PROFILE_STR_BASELINE, 66, 64, 0x80 },
-  { H264_PROFILE_STR_MAIN,     77, 32, 0x40 },
-  { H264_PROFILE_STR_EXTENDED, 88, 16, 0x20 }
+  { H264_PROFILE_STR_BASELINE, H264_PROFILE_INT_BASELINE, 64, 0x80 },
+  { H264_PROFILE_STR_MAIN,     H264_PROFILE_INT_MAIN,     32, 0x40 },
+  { H264_PROFILE_STR_EXTENDED, H264_PROFILE_INT_EXTENDED, 16, 0x20 }
 };
 
 static struct LevelInfoStruct {
@@ -560,8 +622,11 @@ public:
   {
     size_t levelIndex = 0;
     size_t profileIndex = sizeof(ProfileInfo)/sizeof(ProfileInfo[0]);
+    unsigned maxMBPS;
+    unsigned maxFrameSizeInMB;
+    unsigned maxBitRate;
 
-    if (original["Protocol"] == "H.323") {
+    if (original[PLUGINCODEC_OPTION_PROTOCOL] == PLUGINCODEC_OPTION_PROTOCOL_H323) {
       unsigned h241profiles = String2Unsigned(original[H241Profiles.m_name]);
       while (--profileIndex > 0) {
         if ((h241profiles&ProfileInfo[profileIndex].m_H241) != 0)
@@ -573,8 +638,12 @@ public:
         if (h241level <= LevelInfo[levelIndex].m_H241)
           break;
       }
+
+      maxMBPS = String2Unsigned(original[MaxMBPS_H241.m_name])*500;
+      maxFrameSizeInMB = String2Unsigned(original[MaxFS_H241.m_name])*256;
+      maxBitRate = String2Unsigned(original[MaxBR_H241.m_name])*25000;
     }
-    else {
+    else if (original[PLUGINCODEC_OPTION_PROTOCOL] == PLUGINCODEC_OPTION_PROTOCOL_SIP) {
       std::string sdpProfLevel = original[SDPProfileAndLevel.m_name];
       if (sdpProfLevel.length() < 6) {
         PTRACE(1, MY_CODEC_LOG, "SDP profile-level-id field illegal.");
@@ -605,12 +674,34 @@ public:
         if ((sdpConstraints & 0x10) == LevelInfo[levelIndex].m_constraints)
           break;
       }
+
+      maxMBPS = String2Unsigned(original[MaxMBPS_SDP.m_name]);
+      maxFrameSizeInMB = String2Unsigned(original[MaxFS_SDP.m_name]);
+      maxBitRate = String2Unsigned(original[MaxBR_SDP.m_name])*1000;
+    }
+    else {
+      std::string profileName = original[Profile.m_name];
+      while (--profileIndex > 0) {
+        if (profileName == ProfileInfo[profileIndex].m_Name)
+          break;
+      }
+
+      std::string levelName = original[Level.m_name];
+      for (; levelIndex < sizeof(LevelInfo)/sizeof(LevelInfo[0])-1; ++levelIndex) {
+        if (levelName == LevelInfo[levelIndex].m_Name)
+          break;
+      }
+
+      maxMBPS = 0;
+      maxFrameSizeInMB = 0;
+      maxBitRate = 0;
     }
 
     Change(ProfileInfo[profileIndex].m_Name, original, changed, Profile.m_name); 
     Change(LevelInfo[levelIndex].m_Name, original, changed, Level.m_name);
 
-    unsigned maxFrameSizeInMB = std::max(LevelInfo[levelIndex].m_MaxFrameSize, String2Unsigned(original[MaxFS.m_name]));
+    if (maxFrameSizeInMB < LevelInfo[levelIndex].m_MaxFrameSize)
+      maxFrameSizeInMB = LevelInfo[levelIndex].m_MaxFrameSize;
     ClampSizes(LevelInfo[levelIndex],
                String2Unsigned(original[PLUGINCODEC_OPTION_MAX_RX_FRAME_WIDTH]),
                String2Unsigned(original[PLUGINCODEC_OPTION_MAX_RX_FRAME_HEIGHT]),
@@ -618,13 +709,15 @@ public:
                original, changed);
 
     // Frame rate
-    unsigned maxMBPS = std::max(LevelInfo[levelIndex].m_MaxMBPS, String2Unsigned(original[MaxMBPS.m_name]));
+    if (maxMBPS < LevelInfo[levelIndex].m_MaxMBPS)
+      maxMBPS = LevelInfo[levelIndex].m_MaxMBPS;
     ClampMin(GetMacroBlocks(String2Unsigned(original[PLUGINCODEC_OPTION_MIN_RX_FRAME_WIDTH]),
                             String2Unsigned(original[PLUGINCODEC_OPTION_MIN_RX_FRAME_HEIGHT]))*MyClockRate/maxMBPS,
              original, changed, PLUGINCODEC_OPTION_FRAME_TIME);
 
     // Bit rate
-    unsigned maxBitRate = std::max(LevelInfo[levelIndex].m_MaxBitRate, String2Unsigned(original[MaxBR.m_name])*1000);
+    if (maxBitRate < LevelInfo[levelIndex].m_MaxBitRate)
+      maxBitRate = LevelInfo[levelIndex].m_MaxBitRate;
     ClampMax(maxBitRate, original, changed, PLUGINCODEC_OPTION_MAX_BIT_RATE);
     ClampMax(maxBitRate, original, changed, PLUGINCODEC_OPTION_TARGET_BIT_RATE);
     return true;
@@ -687,18 +780,24 @@ public:
     ClampSizes(LevelInfo[levelIndex], maxWidth, maxHeight, maxFrameSizeInMB, original, changed);
 
     // Do this afer the clamping, maxFrameSizeInMB may change
-    if (maxFrameSizeInMB > LevelInfo[levelIndex].m_MaxFrameSize)
-      Change(maxFrameSizeInMB, original, changed, MaxFS.m_name);
+    if (maxFrameSizeInMB > LevelInfo[levelIndex].m_MaxFrameSize) {
+      Change(maxFrameSizeInMB, original, changed, MaxFS_SDP.m_name);
+      Change((maxFrameSizeInMB+255)/256, original, changed, MaxFS_H241.m_name);
+    }
 
     // Set exception to bit rate if necessary
     unsigned bitRate = String2Unsigned(original[PLUGINCODEC_OPTION_MAX_BIT_RATE]);
-    if (bitRate > LevelInfo[levelIndex].m_MaxBitRate)
-      Change(bitRate/1000, original, changed, MaxBR.m_name);
+    if (bitRate > LevelInfo[levelIndex].m_MaxBitRate) {
+      Change((bitRate+999)/1000, original, changed, MaxBR_SDP.m_name);
+      Change((bitRate+24999)/25000, original, changed, MaxBR_H241.m_name);
+    }
 
     // Set exception to frame rate if necessary
     unsigned mbps = maxFrameSizeInMB*MyClockRate/String2Unsigned(original[PLUGINCODEC_OPTION_FRAME_TIME]);
-    if (mbps > LevelInfo[levelIndex].m_MaxMBPS)
-      Change(mbps, original, changed, MaxMBPS.m_name);
+    if (mbps > LevelInfo[levelIndex].m_MaxMBPS) {
+      Change(mbps, original, changed, MaxMBPS_SDP.m_name);
+      Change((mbps+499)/500, original, changed, MaxMBPS_H241.m_name);
+    }
 
     return true;
   }
@@ -706,7 +805,7 @@ public:
 
   virtual bool IsValidForProtocol(const char * protocol)
   {
-    return (strcasecmp(protocol, "SIP") == 0) == m_sipOnly;
+    return (strcasecmp(protocol, PLUGINCODEC_OPTION_PROTOCOL_SIP) == 0) == m_sipOnly;
   }
 }; 
 
@@ -734,6 +833,7 @@ class MyEncoder : public PluginCodec<MY_CODEC>
     unsigned m_maxNALUSize;
     unsigned m_tsto;
     unsigned m_keyFramePeriod;
+    unsigned m_rateControlPeriod;
 
     H264Encoder m_encoder;
 
@@ -748,10 +848,11 @@ class MyEncoder : public PluginCodec<MY_CODEC>
       , m_level(DefaultLevelInt)
       , m_constraints(0)
       , m_packetisationMode(1)
-      , m_maxRTPSize(PluginCodec_RTP_MaxPacketSize)
+      , m_maxRTPSize(PluginCodec_RTP_MaxPayloadSize)
       , m_maxNALUSize(1400)
       , m_tsto(31)
       , m_keyFramePeriod(0)
+      , m_rateControlPeriod(1000)
     {
     }
 
@@ -763,8 +864,10 @@ class MyEncoder : public PluginCodec<MY_CODEC>
          be done in the normal C++ constructor. */
 
       // ABR with bit rate tolerance = 1 is CBR...
-      if (FFMPEGLibraryInstance.Load() && m_encoder.Load(this))
+      if (FFMPEGLibraryInstance.Load() && m_encoder.Load(this)) {
+        PTRACE(4, MY_CODEC_LOG, "Opened encoder (SVN $Revision$)");
         return true;
+      }
 
       PTRACE(1, MY_CODEC_LOG, "Could not open encoder.");
       return false;
@@ -802,6 +905,9 @@ class MyEncoder : public PluginCodec<MY_CODEC>
       if (strcasecmp(optionName, PLUGINCODEC_OPTION_TX_KEY_FRAME_PERIOD) == 0)
         return SetOptionUnsigned(m_keyFramePeriod, optionValue, 0);
 
+      if (strcasecmp(optionName, PLUGINCODEC_OPTION_RATE_CONTROL_PERIOD) == 0)
+        return SetOptionUnsigned(m_rateControlPeriod, optionValue, 100, 60000);
+
       if (strcasecmp(optionName, Level.m_name) == 0) {
         for (size_t i = 0; i < sizeof(LevelInfo)/sizeof(LevelInfo[0]); i++) {
           if (strcasecmp(optionValue, LevelInfo[i].m_Name) == 0) {
@@ -824,18 +930,16 @@ class MyEncoder : public PluginCodec<MY_CODEC>
 
       if (
 #ifdef PLUGIN_CODEC_VERSION_INTERSECT
-         strcasecmp(optionName, PLUGINCODEC_MEDIA_PACKETIZATIONS) == 0 ||
+          strcasecmp(optionName, PLUGINCODEC_MEDIA_PACKETIZATIONS) == 0 ||
 #endif
-          strcasecmp(optionName, PLUGINCODEC_MEDIA_PACKETIZATION) == 0) {
+          strcasecmp(optionName, PLUGINCODEC_MEDIA_PACKETIZATION ) == 0) {
         if (strstr(optionValue, OpalPluginCodec_Identifer_H264_Interleaved) != NULL)
           return SetPacketisationMode(2);
         if (strstr(optionValue, OpalPluginCodec_Identifer_H264_NonInterleaved) != NULL)
           return SetPacketisationMode(1);
-        if (strstr(optionValue, OpalPluginCodec_Identifer_H264_Aligned) != NULL ||
-            strstr(optionValue, OpalPluginCodec_Identifer_H264_Truncated) != NULL)
-          return SetPacketisationMode(0);
-        PTRACE(2, MY_CODEC_LOG, "Unknown packetisation mode: \"" << optionValue << '"');
-        return false; // Unknown/unsupported media packetization
+        if (*optionValue != '\0' && strstr(optionValue, OpalPluginCodec_Identifer_H264_Aligned) == NULL)
+          PTRACE(2, MY_CODEC_LOG, "Unknown packetisation mode: \"" << optionValue << '"');
+        return SetPacketisationMode(0);
       }
 
       if (strcasecmp(optionName, NamePacketizationMode) == 0)
@@ -873,9 +977,20 @@ class MyEncoder : public PluginCodec<MY_CODEC>
       m_encoder.SetFrameHeight(m_height);
       m_encoder.SetFrameRate(m_frameRate);
       m_encoder.SetTargetBitrate(m_bitRate/1000);
-      m_encoder.SetMaxRTPPayloadSize(std::min(m_maxRTPSize-PluginCodec_RTP_MinHeaderSize, m_maxNALUSize));
+      m_encoder.SetRateControlPeriod(m_rateControlPeriod);
       m_encoder.SetTSTO(m_tsto);
       m_encoder.SetMaxKeyFramePeriod(m_keyFramePeriod);
+
+      if (m_packetisationMode == 0) {
+        unsigned size = std::min(m_maxRTPSize, m_maxNALUSize);
+        m_encoder.SetMaxRTPPayloadSize(size);
+        m_encoder.SetMaxNALUSize(size);
+      }
+      else {
+        m_encoder.SetMaxRTPPayloadSize(m_maxRTPSize);
+        m_encoder.SetMaxNALUSize(m_maxNALUSize);
+      }
+
       m_encoder.ApplyOptions();
 
       PTRACE(3, MY_CODEC_LOG, "Applied options: "
@@ -884,7 +999,9 @@ class MyEncoder : public PluginCodec<MY_CODEC>
                               "res=" << m_width << 'x' << m_height << " "
                               "fps=" << m_frameRate << " "
                               "bps=" << m_bitRate << " "
+                              "period=" << m_rateControlPeriod << " "
                               "RTP=" << m_maxRTPSize << " "
+                              "NALU=" << m_maxNALUSize << " "
                               "TSTO=" << m_tsto);
       return true;
     }
@@ -970,6 +1087,7 @@ class MyDecoder : public PluginCodec<MY_CODEC>
       if (FFMPEGLibraryInstance.AvcodecOpen(m_context, m_codec) < 0)
         return false;
 
+      PTRACE(4, MY_CODEC_LOG, "Opened decoder (SVN $Revision$)");
       return true;
     }
 
@@ -1005,6 +1123,9 @@ class MyDecoder : public PluginCodec<MY_CODEC>
 
       int bytesToDecode = m_fullFrame.GetFrameSize();
 
+      if (m_fullFrame.GetProfile() == H264_PROFILE_INT_BASELINE)
+        m_context->has_b_frames = 0;
+
       /* Do conversion of RTP packet. Note that typically many are received
          before an output frame is generated. If no output fram is available
          yet, simply return true and more will be sent. */
@@ -1026,6 +1147,7 @@ class MyDecoder : public PluginCodec<MY_CODEC>
       if (!gotPicture) {
         PTRACE(3, MY_CODEC_LOG, "Decoded " << bytesDecoded<< " of " << bytesToDecode 
                << " bytes without a picture.");
+        flags |= PluginCodec_ReturnCoderRequestIFrame;
         return true;
       }
 
