@@ -38,9 +38,9 @@
  * Initial implementation of MPEG4 codec plugin using ffmpeg.
  * Untested under Windows or H.323
  *
- * $Revision: 26694 $
- * $Author: rjongbloed $
- * $Date: 2011-11-25 00:10:08 -0600 (Fri, 25 Nov 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 /*
@@ -102,7 +102,7 @@ extern "C" {
 #include <libavcodec/mpegvideo.h>
 
 #else /* LIBAVCODEC_HAVE_SOURCE_DIR */
-#include LIBAVCODEC_HEADER
+#include "../common/ffmpeg.h"
 #endif /* LIBAVCODEC_HAVE_SOURCE_DIR */
 }
 
