@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26521 $
- * $Author: rjongbloed $
- * $Date: 2011-10-04 23:57:06 -0500 (Tue, 04 Oct 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include "precompile.h"
@@ -117,7 +117,7 @@ void CodecTest::Main()
       PluginCodec_GetCodecFunction getCodecs;
       if (plugin.GetFunction(PLUGIN_CODEC_GET_CODEC_FN_STR, (PDynaLink::Function &)getCodecs)) {
         unsigned count = 0;
-        PluginCodec_Definition * codecs = (*getCodecs)(&count, PLUGIN_CODEC_VERSION);
+        const PluginCodec_Definition * codecs = (*getCodecs)(&count, PLUGIN_CODEC_VERSION);
         while (count > 0) {
           PString name(codecs->descr);
           if (codecNames.GetStringsIndex(name) == P_MAX_INDEX)
@@ -634,7 +634,27 @@ bool VideoThread::Initialise(PArgList & args)
     cout << "driver \"" << driverName << "\" and ";
   cout << "device \"" << m_display->GetDeviceName() << "\" opened." << endl;
 
-  // Configure sizes/speeds
+  // Configure formats/sizes/speeds
+  cout << "Grabber colour format set to " << m_grabber->GetColourFormat() << " (";
+  if (m_grabber->GetColourFormat() == "YUV420P")
+    cout << "native";
+  else
+    cout << "converted to YUV420P";
+  cout << ')' << endl;
+
+  if (!m_display->SetColourFormatConverter("YUV420P")) {
+    cerr << "Video display device could not be set to colour format YUV420P" << endl;
+    return false;
+  }
+
+  cout << "Display colour format set to " << m_display->GetColourFormat() << " (";
+  if (m_display->GetColourFormat() == "YUV420P")
+    cout << "native";
+  else
+    cout << "converted from YUV420P";
+  cout << ')' << endl;
+
+
   unsigned width, height;
   if (args.HasOption("frame-size")) {
     PString sizeString = args.GetOptionString("frame-size");
@@ -670,26 +690,6 @@ bool VideoThread::Initialise(PArgList & args)
     return false;
   }
 
-  cout << "Grabber colour format set to " << m_grabber->GetColourFormat() << " (";
-  if (m_grabber->GetColourFormat() == "YUV420P")
-    cout << "native";
-  else
-    cout << "converted to YUV420P";
-  cout << ')' << endl;
-
-  if (!m_display->SetColourFormatConverter("YUV420P")) {
-    cerr << "Video display device could not be set to colour format YUV420P" << endl;
-    return false;
-  }
-
-  cout << "Display colour format set to " << m_display->GetColourFormat() << " (";
-  if (m_display->GetColourFormat() == "YUV420P")
-    cout << "native";
-  else
-    cout << "converted from YUV420P";
-  cout << ')' << endl;
-
-
   if (args.HasOption("bit-rate")) {
     PString str = args.GetOptionString("bit-rate");
     double bitrate = str.AsReal();
@@ -730,8 +730,11 @@ bool VideoThread::Initialise(PArgList & args)
 
   m_frameTime = mediaFormat.GetFrameTime();
   if (m_encoder != NULL) {
-    if (args.HasOption('p'))
-      m_encoder->SetMaxOutputSize(args.GetOptionString('p').AsUnsigned());
+    if (args.HasOption('p')) {
+      unsigned bytes = args.GetOptionString('p').AsUnsigned();
+      m_encoder->SetMaxOutputSize(bytes);
+      mediaFormat.SetOptionInteger(OpalMediaFormat::MaxTxPacketSizeOption(), bytes);
+    }
     m_encoder->UpdateMediaFormats(OpalMediaFormat(), mediaFormat);
   }
 
@@ -927,7 +930,7 @@ void TranscoderThread::Main()
   // main loop
   //
 
-  RTP_DataFrame * srcFrame_ = NULL;
+  RTP_DataFrame * srcFramePtr = NULL;
 
   while ((m_running && m_framesToTranscode < 0) || (m_framesToTranscode-- > 0)) {
 
@@ -935,10 +938,9 @@ void TranscoderThread::Main()
     //
     //  acquire and format source frame
     //
-    if (srcFrame_ != NULL)
-      delete srcFrame_;
-    srcFrame_ = new RTP_DataFrame(0);
-    RTP_DataFrame & srcFrame = *srcFrame_;
+    delete srcFramePtr;
+    srcFramePtr = new RTP_DataFrame(0);
+    RTP_DataFrame & srcFrame = *srcFramePtr;
 
     {
       bool state = Read(srcFrame);
@@ -1047,8 +1049,8 @@ void TranscoderThread::Main()
       totalEncodedPacketCount += encFrames.GetSize();
 
       if (isVideo && m_calcSNR) {
-        ((VideoThread *)this)->SaveSNRFrame(srcFrame_);
-        srcFrame_ = NULL;
+        ((VideoThread *)this)->SaveSNRFrame(srcFramePtr);
+        srcFramePtr = NULL;
       }
 
       //////////////////////////////////////////////
@@ -1056,14 +1058,14 @@ void TranscoderThread::Main()
       //  drop encoded frames if required
       //
       if (m_dropPercent > 0) {
-        PINDEX i = 0;
-        while (i < encFrames.GetSize()) {
+        RTP_DataFrameList::iterator it = encFrames.begin();
+        while (it != encFrames.end()) {
           int n = PRandom::Number() % 100; 
           if (n >= m_dropPercent) 
-            i++;
+            ++it;
           else {
             ++totalDroppedPacketCount;
-            encFrames.RemoveAt(i);
+            encFrames.erase(it++);
           }
         }
       }
@@ -1198,6 +1200,8 @@ void TranscoderThread::Main()
     }
   }
 
+  delete srcFramePtr;
+
   //
   //  end of main loop
   //
