@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26157 $
- * $Author: rjongbloed $
- * $Date: 2011-07-10 21:23:41 -0500 (Sun, 10 Jul 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include "precompile.h"
@@ -112,19 +112,6 @@ void FaxOPAL::Main()
   static char const * FormatMask[] = { "!G.711*", "!@fax" };
   m_manager->SetMediaFormatMask(PStringArray(PARRAYSIZE(FormatMask), FormatMask));
 
-  if (!m_manager->Initialise(args, !args.HasOption('q')))
-    return;
-
-  // Create audio or T.38 fax endpoint.
-  MyFaxEndPoint * fax  = new MyFaxEndPoint(*m_manager);
-  if (args.HasOption('d'))
-    fax->SetDefaultDirectory(args.GetOptionString('d'));
-
-  if (!fax->IsAvailable()) {
-    cerr << "No fax codecs, SpanDSP plug-in probably not installed." << endl;
-    return;
-  }
-
   PString prefix;
 
   if (args.HasOption('q'))
@@ -147,8 +134,18 @@ void FaxOPAL::Main()
   }
   cout << '\n';
 
-  m_manager->AddRouteEntry("sip.*:.* = " + prefix + ":" + args[0] + ";receive");
-  m_manager->AddRouteEntry("h323.*:.* = " + prefix + ":" + args[0] + ";receive");
+  if (!m_manager->Initialise(args, !args.HasOption('q'), prefix + ":" + args[0] + ";receive"))
+    return;
+
+  // Create audio or T.38 fax endpoint.
+  MyFaxEndPoint * fax  = new MyFaxEndPoint(*m_manager);
+  if (args.HasOption('d'))
+    fax->SetDefaultDirectory(args.GetOptionString('d'));
+
+  if (!fax->IsAvailable()) {
+    cerr << "No fax codecs, SpanDSP plug-in probably not installed." << endl;
+    return;
+  }
 
   OpalConnection::StringOptions stringOptions;
 
@@ -165,23 +162,27 @@ void FaxOPAL::Main()
   }
 
   if (args.HasOption('F')) {
-    stringOptions.SetAt(OPAL_NO_G111_FAX, "true");
+    stringOptions.SetBoolean(OPAL_NO_G111_FAX, true);
     cout << "Disabled fallback to audio (G.711) mode on T.38 switch failure\n";
   }
 
   if (args.HasOption('e')) {
-    stringOptions.SetAt(OPAL_SWITCH_ON_CED, "true");
+    stringOptions.SetBoolean(OPAL_SWITCH_ON_CED, true);
     cout << "Enabled switch to T.38 on receipt of CED\n";
   }
 
   if (args.HasOption('X')) {
     unsigned seconds = args.GetOptionString('X').AsUnsigned();
-    stringOptions.SetAt(OPAL_T38_SWITCH_TIME, seconds);
+    stringOptions.SetInteger(OPAL_T38_SWITCH_TIME, seconds);
     cout << "Switch to T.38 after " << seconds << "seconds\n";
   }
   else
     cout << "No T.38 switch timeout set\n";
 
+  // Wait for call to come in and finish (default one year)
+  PSimpleTimer timeout(args.GetOptionString('T', "365:0:0:0"));
+  cout << "Completion timeout is " << timeout.AsString(0, PTimeInterval::IncludeDays) << '\n';
+
   if (args.GetCount() == 1)
     cout << "Receive directory: " << fax->GetDefaultDirectory() << "\n"
             "\n"
@@ -201,8 +202,6 @@ void FaxOPAL::Main()
 
   map<PString, OpalMediaStatistics> lastStatisticsByToken;
 
-  // Wait for call to come in and finish (default one year)
-  PSimpleTimer timeout(0, args.GetOptionString('T', "31536000").AsUnsigned());
   while (!m_manager->m_completed.Wait(1000)) {
     if (timeout.HasExpired()) {
       cout << " no call";
