@@ -21,16 +21,17 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20534 $
- * $Author: rjongbloed $
- * $Date: 2008-07-04 03:47:05 +0000 (Fri, 04 Jul 2008) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
-#include <opal/manager.h>
+#include <opal/console_mgr.h>
 #include <t38/t38proto.h>
 #include <sip/sipep.h>
 #include <h323/h323ep.h>
+#include <lids/lidep.h>
 
 
 // End of File ///////////////////////////////////////////////////////////////
