@@ -32,24 +32,27 @@
 #include "MobileOPAL.h"
 #include "MobileOpalDlg.h"
 #include "OptionsGeneral.h"
+#include "OptionsCodecs.h"
 #include "OptionsH323.h"
 #include "OptionsSIP.h"
 
 #include <winsock2.h>
 #include <iphlpapi.h>
+#include <pmpolicy.h>
+#include <pm.h>
 
 
 #ifdef _DEBUG
 #define new DEBUG_NEW
-#define TRACE_OPTIONS "TraceLevel=4 TraceFile=\\MobileOpalLog.txt"
-//#define TRACE_OPTIONS "TraceLevel=4"
-#else
-#define TRACE_OPTIONS "TraceLevel=4 TraceAppend TraceFile=\\MobileOpalLog.txt"
 #endif
 
 #define SPEAKERMODE_METHOD 3
 
 
+#define TRACE_OPTIONS "TraceLevel=4 TraceFile=\\MobileOpalLog.txt"
+//#define TRACE_OPTIONS "TraceLevel=3 TraceAppend TraceFile=\\MobileOpalLog.txt"
+
+
 static UINT TimerID = 1;
 
 static TCHAR const RecentCallsSection[]  = L"RecentCalls";
@@ -57,6 +60,9 @@ static TCHAR const OptionsSection[]      = L"Options";
 static TCHAR const UserNameKey[]         = L"UserName";
 static TCHAR const DisplayNameKey[]      = L"DisplayName";
 static TCHAR const STUNServerKey[]       = L"STUNServer";
+static TCHAR const MediaOrderKey[]       = L"MediaOrder";
+static TCHAR const MediaMaskKey[]        = L"MediaMask";
+static TCHAR const MediaOptionsKey[]     = L"MediaOptions";
 static TCHAR const InterfaceAddressKey[] = L"InterfaceAddress";
 static TCHAR const AutoStartTxVideoKey[] = L"AutoStartTxVideo";
 static TCHAR const GkTypeKey[]           = L"GatekeeperType";
@@ -71,6 +77,9 @@ static TCHAR const RegistrarUserKey[]    = L"RegistrarUser";
 static TCHAR const RegistrarPassKey[]    = L"RegistrarPassword";
 static TCHAR const RegistrarRealmKey[]   = L"RegistrarRealm";
 
+static TCHAR const DefaultMediaOrder[]   = L"G.722.2\nGSM-AMR\nG.729\nG.729A\nG.723.1\nG.728\nG.711-uLaw-64k\nG.711-ALaw-64k\nH.264\nH.263\nH.261";
+static TCHAR const DefaultMediaMask[]    = L"RFC4175_RGB\nRFC4175_YCbCr-4\nMPEG4";
+static TCHAR const DefaultMediaOptions[] = L"H.264:Level=1b";
 
 
 static CString GetOptionString(LPCTSTR szKey, LPCTSTR szDefault = NULL)
@@ -158,7 +167,7 @@ void GetNetworkInterfaces(CStringArray & interfaces, bool includeNames)
     }
   }
 
-#elif SPEAKERMODE_METHOD==2
+#elif SPEAKERMODE_METHOD==2 || SPEAKERMODE_METHOD==5
 
   /* Stuff from Wavedev.h */
   #define IOCTL_WAV_MESSAGE 0x001d000c
@@ -199,12 +208,12 @@ void GetNetworkInterfaces(CStringArray & interfaces, bool includeNames)
         if (hWavDev == INVALID_HANDLE_VALUE)
           return false;
 
-        DWORD dwRet, dwOut; 
+        DWORD dwRet = 0, dwOut; 
         MMDRV_MESSAGE_PARAMS mp; 
         memset(&mp,0,sizeof(mp));
         mp.uMsg = uMsg;
         mp.dwParam1 = dwParam1;
-        mp.dwParam2 = dwParam1;
+        mp.dwParam2 = dwParam2;
         if (DeviceIoControl(hWavDev, IOCTL_WAV_MESSAGE, &mp, sizeof(mp), &dwOut, sizeof(dwOut), &dwRet, 0))
           return true;
 
@@ -219,6 +228,7 @@ void GetNetworkInterfaces(CStringArray & interfaces, bool includeNames)
   static void SetSpeakerMode(bool speakerphone)
   {
     WavDevice wd;
+#if SPEAKERMODE_METHOD==2
     if (!wd.SendMessage(1002, speakerphone ? 0 : 2))
       return;
     if (!wd.SendMessage(1012, speakerphone ? 0 : 1))
@@ -226,6 +236,9 @@ void GetNetworkInterfaces(CStringArray & interfaces, bool includeNames)
     if (speakerphone && !wd.SendMessage(1013))
       return;
     wd.SendMessage(1000, speakerphone ? 2 : 4, speakerphone ? 0 : 7);
+#else
+    wd.SendMessage(102, speakerphone);
+#endif
   }
 
 #elif SPEAKERMODE_METHOD==3
@@ -283,6 +296,55 @@ void GetNetworkInterfaces(CStringArray & interfaces, bool includeNames)
     dll.SetSpeakerMode(speakerphone ? 1 : 0);
   }
 
+#elif SPEAKERMODE_METHOD==4
+
+  #include "ril.h"
+  #pragma comment(lib, "ril.lib")
+
+  static void CALLBACK ResultCallback(DWORD dwCode,           // @parm result code
+                                      HRESULT hrCmdID,        // @parm ID returned by the command that originated this response
+                                      const void* lpData,     // @parm data associated with the notification
+                                      DWORD cbData,           // @parm size of the strcuture pointed to lpData
+                                      DWORD dwParam           // @parm parameter passed to <f RIL_Initialize>
+                                     )
+  {
+  }
+
+  static void CALLBACK NotifyCallback(DWORD dwCode,           // @parm notification code
+                                      const void* lpData,     // @parm data associated with the notification
+                                      DWORD cbData,           // @parm size of the strcuture pointed to lpData
+                                      DWORD dwParam           // @parm parameter passed to <f RIL_Initialize>
+                                     )
+  {
+  }
+
+  static void SetSpeakerMode(bool speakerphone)
+  {
+    HRIL hRIL;
+    HRESULT result = RIL_Initialize(1, ResultCallback, NotifyCallback, RIL_NCLASS_MISC, 0, &hRIL);
+    if (result != S_OK) {
+      wchar_t buffer[200];
+      wsprintf(buffer, L"Could not open RIL, error=0x%x", result);
+      AfxMessageBox(buffer);
+      return;
+    }
+
+    RILAUDIODEVICEINFO info;
+    info.cbSize = sizeof(info);
+    info.dwParams = RIL_PARAM_ADI_ALL; // RIL_PARAM_ADI_TXDEVICE;
+    info.dwRxDevice = RIL_AUDIO_HANDSET;
+    info.dwTxDevice = speakerphone ? RIL_AUDIO_SPEAKERPHONE : RIL_AUDIO_HANDSET;
+
+    result = RIL_SetAudioDevices(hRIL, &info);
+    if (result != S_OK) {
+      wchar_t buffer[200];
+      wsprintf(buffer, L"Could not open RIL, error=0x%x", result);
+      AfxMessageBox(buffer);
+    }
+
+    RIL_Deinitialize(hRIL);
+  }
+
 #else
 
   static void SetSpeakerMode(bool /*speakerphone*/)
@@ -301,6 +363,7 @@ CMobileOpalDlg::CMobileOpalDlg(CWnd* pParent /*=NULL*/)
   , m_opal(NULL)
   , m_opalVersion(OPAL_C_API_VERSION)
   , m_speakerphone(false)
+  , m_hPowerRequirement(NULL)
 {
   m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
 }
@@ -330,6 +393,7 @@ BEGIN_MESSAGE_MAP(CMobileOpalDlg, CDialog)
   ON_CBN_EDITCHANGE(IDC_ADDRESS, &CMobileOpalDlg::OnChangedAddress)
   ON_CBN_SELENDOK(IDC_ADDRESS, &CMobileOpalDlg::OnSelectedAddress)
   ON_COMMAND(IDM_OPTIONS_GENERAL, &CMobileOpalDlg::OnMenuOptionsGeneral)
+  ON_COMMAND(IDM_OPTIONS_CODECS, &CMobileOpalDlg::OnMenuOptionsCodecs)
   ON_COMMAND(IDM_OPTIONS_H323, &CMobileOpalDlg::OnMenuOptionsH323)
   ON_COMMAND(IDM_OPTIONS_SIP, &CMobileOpalDlg::OnMenuOptionsSIP)
   ON_COMMAND(IDM_EXIT, &CMobileOpalDlg::OnExit)
@@ -475,30 +539,33 @@ void CMobileOpalDlg::InitialiseOPAL()
                                videoWidth+2, box.Height()-previewHeight,
                                previewWidth, previewHeight);
 
-  CStringA strMediaOptions;
-  strMediaOptions.Format("Video:Max Bit Rate=128000\n"
-                         "Video:Target Bit Rate=128000\n"
-                         "Video:Frame Time=9000\n" // 10 frames/second
-                         "Video:Frame Width=%u\n"
-                         "Video:Frame Height=%u\n"
-                         "Video:Max Rx Frame Width=%u\n"
-                         "Video:Max Rx Frame Height=%u\n"
-                         "H.264:Level=1b\n",
-                          videoWidth, videoHeight,
-                          videoWidth, videoHeight);
+  CStringA strMediaOptions = GetOptionStringA(MediaOptionsKey, DefaultMediaOptions);
+  strMediaOptions.AppendFormat("Video:Max Bit Rate=128000\n"
+                               "Video:Target Bit Rate=128000\n"
+                               "Video:Frame Time=9000\n" // 10 frames/second
+                               "Video:Frame Width=%u\n"
+                               "Video:Frame Height=%u\n"
+                               "Video:Max Rx Frame Width=%u\n"
+                               "Video:Max Rx Frame Height=%u\n",
+                                videoWidth, videoHeight,
+                                videoWidth, videoHeight);
+
+  CStringA strMediaOrder = GetOptionStringA(MediaOrderKey, DefaultMediaOrder);
+  CStringA strMediaMask = GetOptionStringA(MediaMaskKey, DefaultMediaMask);
+  CStringA strStunServer = GetOptionStringA(STUNServerKey);
 
   // General options
   memset(&command, 0, sizeof(command));
   command.m_type = OpalCmdSetGeneralParameters;
 
-  CStringA strStunServer = GetOptionStringA(STUNServerKey);
   command.m_param.m_general.m_stunServer = strStunServer;
   command.m_param.m_general.m_audioPlayerDevice = "Audio Output";
   command.m_param.m_general.m_audioRecordDevice = "Audio Input";
   command.m_param.m_general.m_videoInputDevice = "CAM1:";
   command.m_param.m_general.m_videoOutputDevice = strVideoOutputDevice;
   command.m_param.m_general.m_videoPreviewDevice = strVideoPreviewDevice;
-  command.m_param.m_general.m_mediaMask = "RFC4175*\nMPEG4";
+  command.m_param.m_general.m_mediaOrder = strMediaOrder;
+  command.m_param.m_general.m_mediaMask = strMediaMask;
   command.m_param.m_general.m_mediaOptions = strMediaOptions;
   command.m_param.m_general.m_autoTxMedia = GetOptionInt(AutoStartTxVideoKey, true) != 0 ? "audio video" : "audio";
   command.m_param.m_general.m_rtpMaxPayloadSize = 1400;
@@ -580,19 +647,15 @@ void CMobileOpalDlg::InitialiseOPAL()
     if ((response = OpalSendMessage(m_opal, &command)) != NULL && response->m_type != OpalIndCommandError) {
       m_currentAOR = strAOR;
       m_currentHost = strHost;
-
-      OpalFreeMessage(response);
-      command.m_param.m_registrationInfo.m_eventPackage = OPAL_MWI_EVENT_PACKAGE;
-      response = OpalSendMessage(m_opal, &command);
     }
-
-    if (response == NULL || response->m_type == OpalIndCommandError) {
+    else {
       ErrorBox(IDS_REGISTRATION_FAIL, response);
       SetStatusText(IDS_READY);
     }
-
     OpalFreeMessage(response);
   }
+
+  PowerPolicyNotify(PPN_UNATTENDEDMODE, TRUE);
 }
 
 
@@ -628,7 +691,7 @@ void CMobileOpalDlg::SetStatusText(UINT ids, const char * str)
     }
 
     if (text.IsEmpty())
-    text = str;
+      text = str;
   }
   else {
     if (ids == 0)
@@ -647,7 +710,7 @@ void CMobileOpalDlg::SetCallButton(bool enabled, UINT strId)
   memset (&tbi, 0, sizeof (tbi));
   tbi.cbSize = sizeof (tbi);
   tbi.dwMask = TBIF_STATE;
-  if (!::SendMessage(m_dlgCommandBar.m_hCommandBar, TB_GETBUTTONINFO, ID_CALL_ANSWER, (LPARAM)&tbi)) 
+  if (!::SendMessageW(m_dlgCommandBar.m_hCommandBar, TB_GETBUTTONINFO, ID_CALL_ANSWER, (LPARAM)&tbi)) 
     return;
 
   if (enabled)
@@ -660,7 +723,7 @@ void CMobileOpalDlg::SetCallButton(bool enabled, UINT strId)
     tbi.dwMask |= TBIF_TEXT;
     tbi.pszText = (LPWSTR)(const TCHAR *)text;
   }
-  ::SendMessage(m_dlgCommandBar.m_hCommandBar, TB_SETBUTTONINFO, ID_CALL_ANSWER, (LPARAM)&tbi);
+  ::SendMessageW(m_dlgCommandBar.m_hCommandBar, TB_SETBUTTONINFO, ID_CALL_ANSWER, (LPARAM)&tbi);
 }
 
 
@@ -686,6 +749,15 @@ void CMobileOpalDlg::AddRecentCall(const CString & uri)
 }
 
 
+void CMobileOpalDlg::EnableFullPower()
+{
+  m_hPowerRequirement = SetPowerRequirement(_T("WAV1:"), D0, POWER_FORCE|POWER_NAME, NULL, 0);
+  if (m_hPowerRequirement == NULL)
+    MessageBox(_T("Could not set power mode!"));
+  SetSystemPowerState(NULL, POWER_STATE_ON, POWER_FORCE);
+}
+
+
 void CMobileOpalDlg::OnTimer(UINT_PTR nIDEvent)
 {
   if (m_opal == NULL)
@@ -739,11 +811,22 @@ void CMobileOpalDlg::HandleMessage(OpalMessage & message)
     case OpalIndIncomingCall :
       if (m_incomingCallToken.IsEmpty() && m_currentCallToken.IsEmpty()) {
         m_incomingCallToken = message.m_param.m_incomingCall.m_callToken;
-        SetStatusText(IDS_INCOMING_CALL);
+
+        CString text;
+        text.LoadString(IDS_INCOMING_CALL);
+        text += "\r\n";
+        if (*message.m_param.m_incomingCall.m_remoteDisplayName != '\0')
+          text += message.m_param.m_incomingCall.m_remoteDisplayName;
+        else if (*message.m_param.m_incomingCall.m_remoteAddress != '\0')
+          text += message.m_param.m_incomingCall.m_remoteAddress;
+        m_ctrlStatus.SetWindowText(text);
+        m_ctrlStatus.UpdateWindow();
+
         SetCallButton(true, IDS_ANSWER);
         m_ctrlCallAddress.EnableWindow(false);
         ShowWindow(true);
         BringWindowToTop();
+        EnableFullPower();
       }
       else {
         OpalMessage command;
@@ -781,6 +864,12 @@ void CMobileOpalDlg::HandleMessage(OpalMessage & message)
         m_ctrlCallAddress.EnableWindow(true);
 
         SetStatusText(IDS_READY, message.m_param.m_callCleared.m_reason);
+
+        if (m_hPowerRequirement != NULL) {
+          ReleasePowerRequirement(m_hPowerRequirement);
+          m_hPowerRequirement = NULL;
+        }
+        PowerPolicyNotify(PPN_UNATTENDEDMODE, TRUE);
       }
   }
 }
@@ -864,6 +953,7 @@ void CMobileOpalDlg::OnCallAnswer()
     else if (command.m_type == OpalCmdSetUpCall) {
       m_currentCallToken = response->m_param.m_callSetUp.m_callToken;
       AddRecentCall(m_callAddress);
+      EnableFullPower();
     }
     else if (command.m_type == OpalCmdAnswerCall) {
       m_currentCallToken = m_incomingCallToken;
@@ -894,14 +984,38 @@ void CMobileOpalDlg::OnMenuOptionsGeneral()
   dlg.m_strDisplayName = GetOptionString(DisplayNameKey);
   dlg.m_strStunServer = GetOptionString(STUNServerKey);
   dlg.m_interfaceAddress = GetOptionString(InterfaceAddressKey, L"*");
-  dlg.m_AutoStartTxVideo = GetOptionInt(AutoStartTxVideoKey, true) != 0;
 
   if (dlg.DoModal() == IDOK) {
     SetOptionString(UserNameKey, dlg.m_strUsername);
     SetOptionString(DisplayNameKey, dlg.m_strDisplayName);
     SetOptionString(STUNServerKey, dlg.m_strStunServer);
     SetOptionString(InterfaceAddressKey, dlg.m_interfaceAddress);
+    InitialiseOPAL();
+  }
+}
+
+
+void CMobileOpalDlg::OnMenuOptionsCodecs()
+{
+  COptionsCodecs dlg;
+  dlg.m_AutoStartTxVideo = GetOptionInt(AutoStartTxVideoKey, true) != 0;
+  dlg.m_MediaOrder = GetOptionString(MediaOrderKey, DefaultMediaOrder);
+  dlg.m_MediaMask = GetOptionString(MediaMaskKey, DefaultMediaMask);
+  dlg.m_MediaOptions = GetOptionString(MediaOptionsKey, DefaultMediaOptions);
+
+  OpalMessage command;
+  OpalMessage * response;
+  memset(&command, 0, sizeof(command));
+  command.m_type = OpalCmdSetGeneralParameters;
+  if ((response = OpalSendMessage(m_opal, &command)) != NULL && response->m_type != OpalIndCommandError)
+    dlg.m_AllMediaOptions = CString(response->m_param.m_general.m_mediaOptions);
+  OpalFreeMessage(response);
+
+  if (dlg.DoModal() == IDOK) {
     SetOptionInt(AutoStartTxVideoKey, dlg.m_AutoStartTxVideo);
+    SetOptionString(MediaOrderKey, dlg.m_MediaOrder);
+    SetOptionString(MediaMaskKey, dlg.m_MediaMask);
+    SetOptionString(MediaOptionsKey, dlg.m_MediaOptions);
     InitialiseOPAL();
   }
 }
@@ -952,6 +1066,6 @@ void CMobileOpalDlg::OnSpeakerphone()
   m_speakerphone = !m_speakerphone;
   SetSpeakerMode(m_speakerphone);
 
-  HMENU hMenu = (HMENU)::SendMessage(m_dlgCommandBar.m_hCommandBar, SHCMBM_GETSUBMENU, 0, IDS_OPTIONS);
+  HMENU hMenu = (HMENU)::SendMessageW(m_dlgCommandBar.m_hCommandBar, SHCMBM_GETSUBMENU, 0, IDS_OPTIONS);
   CheckMenuItem(hMenu, IDM_SPEAKERPHONE, m_speakerphone ? MF_CHECKED : MF_UNCHECKED);
 }
