@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21912 $
- * $Author: rjongbloed $
- * $Date: 2009-01-09 20:47:43 -0600 (Fri, 09 Jan 2009) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #pragma once
@@ -85,6 +85,7 @@ public:
   afx_msg void OnSelectedAddress();
   afx_msg void OnSpeakerphone();
   afx_msg void OnMenuOptionsGeneral();
+  afx_msg void OnMenuOptionsCodecs();
   afx_msg void OnMenuOptionsH323();
   afx_msg void OnMenuOptionsSIP();
   afx_msg void OnExit();
