@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21752 $
- * $Author: rjongbloed $
- * $Date: 2008-12-10 02:48:17 +0000 (Wed, 10 Dec 2008) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #pragma once
@@ -60,5 +60,4 @@ public:
   CString m_strStunServer;
   CString m_interfaceAddress;
   CComboBox m_interfaceAddressCombo;
-  BOOL m_AutoStartTxVideo;
 };
