@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24884 $
- * $Author: rjongbloed $
- * $Date: 2010-11-21 17:27:05 -0600 (Sun, 21 Nov 2010) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -611,7 +611,7 @@ void OpalCodecInfo::Main()
     needHelp = false;
   }
 
-  OpalPluginCodecManager & codecMgr = *PFactory<PPluginModuleManager>::CreateInstanceAs<OpalPluginCodecManager>("OpalPluginCodecManager");
+  OpalPluginCodecManager & codecMgr = OpalPluginCodecManager::GetInstance();
   PPluginModuleManager::PluginListType pluginList = codecMgr.GetPluginList();
 
   if (args.HasOption('p')) {
