@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26455 $
- * $Author: rjongbloed $
- * $Date: 2011-09-20 21:56:57 -0500 (Tue, 20 Sep 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include "precompile.h"
@@ -85,7 +85,7 @@ void ConfOPAL::Main()
     return;
   }
 
-  if (!m_manager->Initialise(args, true))
+  if (!m_manager->Initialise(args, true, "mcu:<du>"))
     return;
 
   // Set up PCSS to do speaker playback
@@ -120,10 +120,6 @@ void ConfOPAL::Main()
   MyMixerEndPoint * mixer = m_manager->m_mixer = new MyMixerEndPoint(*m_manager, info);
 
 
-  m_manager->AddRouteEntry("sip.*:.* = mcu:<du>");
-  m_manager->AddRouteEntry("h323.*:.* = mcu:<du>");
-
-
   // Wait for call to come in and finish
   PCLI * cli;
   if (args.HasOption('c')) {
