@@ -21,14 +21,14 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23014 $
- * $Author: rjongbloed $
- * $Date: 2009-06-29 02:30:11 -0500 (Mon, 29 Jun 2009) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
 #include <ptclib/cli.h>
-#include <opal/manager.h>
+#include <opal/console_mgr.h>
 #include <opal/opalmixer.h>
 #include <opal/ivr.h>
 #include <opal/pcss.h>
