@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26925 $
- * $Author: rjongbloed $
- * $Date: 2012-02-01 01:36:38 -0600 (Wed, 01 Feb 2012) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef _OpenPhone_MAIN_H
@@ -85,22 +85,13 @@ class MyPCSSEndPoint : public OpalPCSSEndPoint
   public:
     MyPCSSEndPoint(MyManager & manager);
 
-    virtual OpalMediaFormatList GetMediaFormats() const;
     virtual PSoundChannel * CreateSoundChannel(const OpalPCSSConnection & connection, const OpalMediaFormat & mediaFormat, PBoolean isSource);
 
-    MyManager & m_manager;
-
-/*
-    bool TransmitExternalIM(
-      OpalConnection & conn, 
-      const OpalMediaFormat & format, 
-      RTP_IMFrame & frame
-    );
-*/
-
   private:
     virtual PBoolean OnShowIncoming(const OpalPCSSConnection & connection);
     virtual PBoolean OnShowOutgoing(const OpalPCSSConnection & connection);
+
+    MyManager & m_manager;
 };
 
 class IMDialog;
@@ -200,6 +191,7 @@ class VideoControlDialog : public wxDialog
     MyManager & m_manager;
     wxSlider  * m_TargetBitRate;
     wxSlider  * m_FrameRate;
+    wxSlider  * m_TSTO;
 
     DECLARE_EVENT_TABLE()
 };
@@ -305,24 +297,24 @@ class CallPanelBase : public wxPanel
   protected:
     CallPanelBase(
       MyManager & manager,
-      const PwxString & token,
+      const PSafePtr<OpalCall> & call,
       wxWindow * parent,
       const wxChar * resource
     );
 
   public:
-    const PwxString & GetToken() const { return m_token; }
+    PwxString GetToken() const { return m_call->GetToken(); }
 
   protected:
     MyManager & m_manager;
-    PwxString   m_token;
+    PSafePtr<OpalCall> m_call;
 };
 
 
 class CallingPanel : public CallPanelBase
 {
   public:
-    CallingPanel(MyManager & manager, const PwxString & token, wxWindow * parent);
+    CallingPanel(MyManager & manager, const PSafePtr<OpalCall> & call, wxWindow * parent);
 
   private:
     void OnHangUp(wxCommandEvent & event);
@@ -334,7 +326,7 @@ class CallingPanel : public CallPanelBase
 class AnswerPanel : public CallPanelBase
 {
   public:
-    AnswerPanel(MyManager & manager, const PwxString & token, wxWindow * parent);
+    AnswerPanel(MyManager & manager, const PSafePtr<OpalCall> & call, wxWindow * parent);
 
     void SetPartyNames(const PwxString & calling, const PwxString & called);
 
@@ -416,7 +408,7 @@ class StatisticsPage
 class InCallPanel : public CallPanelBase
 {
   public:
-    InCallPanel(MyManager & manager, const PwxString & token, wxWindow * parent);
+    InCallPanel(MyManager & manager, const PSafePtr<OpalCall> & call, wxWindow * parent);
     virtual bool Show(bool show = true);
 
     void OnStreamsChanged();
