@@ -24,15 +24,15 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24051 $
- * $Author: rjongbloed $
- * $Date: 2010-02-10 17:59:12 -0600 (Wed, 10 Feb 2010) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifndef _OpenPhone_VERSION_H
 #define _OpenPhone_VERSION_H
 
-#include <opal/buildopts.h>
+#include "../../version.h"
 
 #define	PRODUCT_NAME_TEXT	"Open Phone"
 #define	EXE_NAME_TEXT	        "openphone"
@@ -40,12 +40,6 @@
 #define	COPYRIGHT_HOLDER	"Vox Lucida Pty. Ltd. (Australia)"
 
 
-#define MAJOR_VERSION OPAL_MAJOR
-#define MINOR_VERSION OPAL_MINOR
-#define BUILD_TYPE    BetaCode
-#define BUILD_NUMBER  OPAL_BUILD
-
-
 #endif  // _OpenPhone_VERSION_H
 
 
