@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26805 $
- * $Author: rjongbloed $
- * $Date: 2011-12-17 01:27:03 -0600 (Sat, 17 Dec 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -984,7 +984,7 @@ PBoolean MyManager::InitialiseH323EP(PArgList & args, PBoolean secure, H323EndPo
   }
 
   if (args.HasOption('b')) {
-    unsigned initialBandwidth = args.GetOptionString('b').AsUnsigned()*100;
+    unsigned initialBandwidth = args.GetOptionString('b').AsUnsigned()/100;
     if (initialBandwidth == 0) {
       cerr << "Illegal bandwidth specified." << endl;
       return PFalse;
