@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26521 $
- * $Author: rjongbloed $
- * $Date: 2011-10-04 23:57:06 -0500 (Tue, 04 Oct 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -334,6 +334,11 @@ class PWAVFileFormatPlugin : public PWAVFileFormat
       UpdateHeader(wavFmtChunk, extendedHeader);
     }
 
+    bool CanSetChannels(unsigned channels) const
+    {
+      return channels == 1;
+    }
+
     void UpdateHeader(PWAV::FMTChunk & wavFmtChunk, PBYTEArray & /*extendedHeader*/)
     {
       wavFmtChunk.bytesPerSample = (WORD)(wavFmtChunk.numChannels*m_mediaFormat.GetFrameSize());
