@@ -24,9 +24,9 @@
  *
  * Contributor(s): Post Increment
  *
- * $Revision: 23238 $
- * $Author: rjongbloed $
- * $Date: 2009-08-20 19:37:01 -0500 (Thu, 20 Aug 2009) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -44,9 +44,6 @@ using namespace std;
 
 #include <opal/mediafmt.h>
 
-namespace PWLibStupidLinkerHacks {
-  int rateControlKickerVal = 0;
-};
 
 //
 // 20 bytes for nominal TCP header
@@ -61,6 +58,7 @@ namespace PWLibStupidLinkerHacks {
 
 #define new PNEW
 
+
 /////////////////////////////////////////////////////////////////////////////
 //
 //
@@ -328,6 +326,9 @@ class OpalStandardVideoRateController : public OpalVideoRateController
 };
 
 
+PFACTORY_CREATE(PFactory<OpalVideoRateController>, OpalStandardVideoRateController, "Standard", false);
+
+
 OpalStandardVideoRateController::OpalStandardVideoRateController()
 {
 }
@@ -416,8 +417,7 @@ bool OpalStandardVideoRateController::CheckFrameRate(bool PTRACE_PARAM(reporting
   return false;
 }
 
-bool OpalStandardVideoRateController::CheckBitRate(bool PTRACE_PARAM(reporting),
-                                                   unsigned PTRACE_PARAM(currentBitRate))
+bool OpalStandardVideoRateController::CheckBitRate(bool PTRACE_PARAM(reporting), unsigned PTRACE_PARAM(currentBitRate))
 {
   if (m_bitRateCalc.GetHistoryCount() == 0)
     return false;
@@ -451,6 +451,5 @@ bool OpalStandardVideoRateController::Pop(RTP_DataFrameList & outputPackets, boo
 }
 
 
-static PFactory<OpalVideoRateController>::Worker<OpalStandardVideoRateController> opalStandardVideoRateController("Standard");
 
 #endif // OPAL_VIDEO
