@@ -22,9 +22,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 27023 $
- * $Author: rjongbloed $
- * $Date: 2012-02-21 16:43:42 -0600 (Tue, 21 Feb 2012) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -66,7 +66,7 @@ OpalRFC2833EventsMask & OpalRFC2833EventsMask::operator&=(const OpalRFC2833Event
 {
   iterator lhs;
   const_iterator rhs;
-  for (lhs = begin(), rhs = other.begin(); lhs != end() && rhs != end(); ++lhs,++rhs)
+  for (lhs = begin(), rhs = other.begin(); lhs != end() && rhs != other.end(); ++lhs,++rhs)
     *lhs = *lhs && *rhs;
   return *this;
 }
@@ -223,8 +223,8 @@ OpalRFC2833Proto::OpalRFC2833Proto(OpalRTPConnection & conn, const PNotifier & r
   , m_receiveState(ReceiveIdle)
   , m_receivedTone('\0')
   , m_tonesReceived(0)
+  , m_previousReceivedTimestamp(0)
   , m_transmitState(TransmitIdle)
-  , m_rtpSession(NULL)
   , m_transmitTimestamp(0)
   , m_rewriteTransmitTimestamp(false)
   , m_transmitCode('\0')
@@ -247,16 +247,7 @@ OpalRFC2833Proto::~OpalRFC2833Proto()
 
 PBoolean OpalRFC2833Proto::SendToneAsync(char tone, unsigned duration)
 {
-  PWaitAndSignal mutex(m_mutex);
-
-  // find an audio session in the current connection to send the packet on
-  if (m_rtpSession == NULL) {
-    OpalMediaStreamPtr stream = m_connection.GetMediaStream(OpalMediaType::Audio(), false);
-    if (stream == NULL || (m_rtpSession = m_connection.GetSession(stream->GetSessionID())) == NULL) {
-      PTRACE(2, "RFC2833\tNo RTP session suitable for RFC2833 for " << m_baseMediaFormat);
-      return false;
-    }
-  }
+  PWaitAndSignal mutex(m_sendMutex);
 
   // convert tone to correct code
   PINDEX code = ASCIIToRFC2833(tone, m_txEvents[NSECodeBase]);
@@ -299,11 +290,11 @@ PBoolean OpalRFC2833Proto::SendToneAsync(char tone, unsigned duration)
 
 void OpalRFC2833Proto::SendAsyncFrame()
 {
-  // be thread safe
-  PWaitAndSignal mutex(m_mutex);
-
-  if (m_rtpSession == NULL) {
-    PTRACE(2, "RFC2833\tCannot send as no RTP session attached to " << m_baseMediaFormat);
+  // find an audio session in the current connection to send the packet on
+  RTP_Session * rtpSession = NULL;
+  OpalMediaStreamPtr stream = m_connection.GetMediaStream(OpalMediaType::Audio(), false);
+  if (stream == NULL || (rtpSession = m_connection.GetSession(stream->GetSessionID())) == NULL) {
+    PTRACE(2, "RFC2833\tNo RTP session suitable for " << m_baseMediaFormat);
     m_transmitState = TransmitIdle;
   }
 
@@ -359,11 +350,12 @@ void OpalRFC2833Proto::SendAsyncFrame()
     case TransmitEnding3:
       payload[1] |= 0x80;
       m_transmitState = TransmitIdle;
-      m_asyncTransmitTimer.Stop();
+      m_asyncTransmitTimer.Stop(false);
       break;
 
     default:
       PAssertAlways("RFC2833\tUnknown transmit state.");
+      m_transmitState = TransmitIdle;
       return;
   }
 
@@ -374,7 +366,7 @@ void OpalRFC2833Proto::SendAsyncFrame()
   if (!m_rewriteTransmitTimestamp)
     frame.SetTimestamp(m_transmitTimestamp);
 
-  if (!m_rtpSession->WriteOOBData(frame, m_rewriteTransmitTimestamp)) {
+  if (!rtpSession->WriteOOBData(frame, m_rewriteTransmitTimestamp)) {
     PTRACE(3, "RFC2833\tRTP session transmission stopped for " << m_baseMediaFormat);
     // Abort further transmission
     m_transmitState = TransmitIdle;
@@ -486,7 +478,9 @@ char OpalRFC2833Proto::RFC2833ToASCII(PINDEX rfc2833, bool hasNSE)
 
 void OpalRFC2833Proto::AsyncTimeout(PTimer &, INT)
 {
+  m_sendMutex.Wait();
   SendAsyncFrame();
+  m_sendMutex.Signal();
 }
 
 
@@ -521,7 +515,7 @@ void OpalRFC2833Proto::ReceivedPacket(RTP_DataFrame & frame, RTP_Session::SendRe
 
   status = RTP_Session::e_IgnorePacket;
 
-  PWaitAndSignal mutex(m_mutex);
+  PWaitAndSignal mutex(m_receiveMutex);
 
   if (frame.GetPayloadSize() < 4) {
     PTRACE(2, "RFC2833\tIgnoring packet size " << frame.GetPayloadSize() << " - too small for " << m_baseMediaFormat);
@@ -559,10 +553,10 @@ void OpalRFC2833Proto::ReceivedPacket(RTP_DataFrame & frame, RTP_Session::SendRe
     if (m_receiveState == ReceiveActive)
       m_receiveTimer = 200;
     else
-      m_receiveTimer.Stop();
+      m_receiveTimer.Stop(false);
   }
   else {
-    m_receiveTimer.Stop();
+    m_receiveTimer.Stop(false);
 
     // finish any existing tone
     if (m_receiveState == ReceiveActive) 
@@ -586,16 +580,18 @@ void OpalRFC2833Proto::ReceivedPacket(RTP_DataFrame & frame, RTP_Session::SendRe
 
 void OpalRFC2833Proto::ReceiveTimeout(PTimer &, INT)
 {
+  m_receiveMutex.Wait();
+
   PTRACE(3, "RFC2833\tTimeout occurred while receiving " << (unsigned)m_receivedTone
          << " for " << m_baseMediaFormat);
 
-  PWaitAndSignal mutex(m_mutex);
-
   if (m_receiveState != ReceiveIdle) {
     m_receiveState = ReceiveIdle;
     //OnEndReceive(receivedTone, 0, 0);
   }
 
+  m_receiveMutex.Signal();
+
   m_receiveTimer.Stop(false);
 }
 
