@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24807 $
- * $Author: rjongbloed $
- * $Date: 2010-10-19 23:33:34 -0500 (Tue, 19 Oct 2010) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -96,7 +96,6 @@ OpalVideoTranscoder::OpalVideoTranscoder(const OpalMediaFormat & inputMediaForma
   : OpalTranscoder(inputMediaFormat, outputMediaFormat)
   , inDataSize(10*1024)
   , outDataSize(10*1024)
-  , videoEncoderMaxOutputSize(10*1024)
   , forceIFrame(false)
   , lastFrameWasIFrame(false)
   , m_totalFrames(0)
@@ -112,7 +111,7 @@ static void SetFrameBytes(const OpalMediaFormat & fmt, const PString & widthOpti
   int height = fmt.GetOptionInteger(heightOption, PVideoFrameInfo::CIFHeight);
   int newSize = PVideoDevice::CalculateFrameBytes(width, height, fmt);
   if (newSize > 0)
-    size = newSize;
+    size = sizeof(PluginCodec_Video_FrameHeader) + newSize;
 }
 
 
@@ -126,7 +125,10 @@ bool OpalVideoTranscoder::UpdateMediaFormats(const OpalMediaFormat & input, cons
   SetFrameBytes(inputMediaFormat,  OpalVideoFormat::MaxRxFrameWidthOption(), OpalVideoFormat::MaxRxFrameHeightOption(), inDataSize);
   SetFrameBytes(outputMediaFormat, OpalVideoFormat::FrameWidthOption(),      OpalVideoFormat::FrameHeightOption(),      outDataSize);
 
-  videoEncoderMaxOutputSize = outputMediaFormat.GetOptionInteger(OpalVideoFormat::MaxTxPacketSizeOption(), outDataSize);
+  if (outputMediaFormat.GetOptionInteger(OpalMediaFormat::MaxTxPacketSizeOption()) > maxOutputSize) {
+    PTRACE(4, "Media\tReducing \"" << OpalMediaFormat::MaxTxPacketSizeOption() << "\" to " << maxOutputSize);
+    outputMediaFormat.SetOptionInteger(OpalMediaFormat::MaxTxPacketSizeOption(), maxOutputSize);
+  }
 
   return PTrue;
 }
@@ -146,8 +148,8 @@ PINDEX OpalVideoTranscoder::GetOptimalDataFrameSize(PBoolean input) const
 
 PBoolean OpalVideoTranscoder::ExecuteCommand(const OpalMediaCommand & command)
 {
-  if (PIsDescendant(&command, OpalVideoUpdatePicture)) {
-    PTRACE_IF(3, !forceIFrame, "Media\tI-frame forced in video stream");
+  if (outputMediaFormat != OpalYUV420P && PIsDescendant(&command, OpalVideoUpdatePicture)) {
+    PTRACE_IF(3, !forceIFrame, "Media\tI-Frame forced in video stream");
     forceIFrame = true; // Reset when I-Frame is sent
     return PTrue;
   }
