@@ -18,9 +18,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21862 $
- * $Author: csoutheren $
- * $Date: 2008-12-22 21:24:53 -0600 (Mon, 22 Dec 2008) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 /*
@@ -52,7 +52,7 @@
 OPAL_INSTANTIATE_MEDIATYPE(h224, OpalH224MediaType);
 
 OpalH224MediaType::OpalH224MediaType()
-  : OpalRTPAVPMediaType("h224", "application", 4)
+  : OpalRTPAVPMediaType("h224", "application")
 {
 }
 
@@ -68,8 +68,20 @@ class SDPH224MediaDescription : public SDPRTPAVPMediaDescription
 {
   PCLASSINFO(SDPH224MediaDescription, SDPRTPAVPMediaDescription);
   public:
-    SDPH224MediaDescription(const OpalTransportAddress & address);
-    virtual PString GetSDPMediaType() const;
+    SDPH224MediaDescription(const OpalTransportAddress & address)
+      : SDPRTPAVPMediaDescription(address, OpalH224MediaType::MediaType())
+    {
+    }
+
+    virtual SDPMediaDescription * CreateEmpty() const
+    {
+      return new SDPH224MediaDescription(OpalTransportAddress());
+    }
+
+    virtual PString GetSDPMediaType() const
+    {
+      return "application";
+    }
 };
 
 SDPMediaDescription * OpalH224MediaType::CreateSDPMediaDescription(const OpalTransportAddress & localAddress)
@@ -77,16 +89,6 @@ SDPMediaDescription * OpalH224MediaType::CreateSDPMediaDescription(const OpalTra
   return new SDPH224MediaDescription(localAddress);
 }
 
-SDPH224MediaDescription::SDPH224MediaDescription(const OpalTransportAddress & address)
-: SDPRTPAVPMediaDescription(address)
-{
-}
-
-PString SDPH224MediaDescription::GetSDPMediaType() const
-{
-  return "application";
-}
-
 #endif
 
 /////////////////////////////////////////////////////////////////////////
@@ -96,13 +98,13 @@ const OpalMediaFormat & GetOpalH224_H323AnnexQ()
   static class H224_AnnexQ_MediaFormat : public OpalH224MediaFormat { 
     public: 
       H224_AnnexQ_MediaFormat() 
-        : OpalH224MediaFormat("H.224/H323AnnexQ", (RTP_DataFrame::PayloadTypes)100)  // Most other implementations seem to use this payload code
+        : OpalH224MediaFormat("H.224/H323AnnexQ", RTP_DataFrame::DynamicBase)
       { 
         OpalMediaOption * option = new OpalMediaOptionBoolean("HDLC Tunneling", true, OpalMediaOption::MinMerge, false);
         AddOption(option);
       } 
-  } const f; 
-  return f; 
+  } const h224q; 
+  return h224q; 
 };
 
 const OpalMediaFormat & GetOpalH224_HDLCTunneling()
@@ -115,8 +117,8 @@ const OpalMediaFormat & GetOpalH224_HDLCTunneling()
         OpalMediaOption * option = new OpalMediaOptionBoolean("HDLC Tunneling", true, OpalMediaOption::MinMerge, true);
         AddOption(option);
       } 
-  } const f; 
-  return f; 
+  } const h224h; 
+  return h224h; 
 }
 
 OpalH224MediaFormat::OpalH224MediaFormat(
@@ -1137,27 +1139,25 @@ OpalH224MediaStream::OpalH224MediaStream(OpalConnection & connection,
 
 OpalH224MediaStream::~OpalH224MediaStream()
 {
+  Close();
 }
 
-void OpalH224MediaStream::OnPatchStart()
+void OpalH224MediaStream::OnStartMediaPatch()
 {	
   h224Handler.StartTransmit();
+  OpalMediaStream::OnStartMediaPatch();
 }
 
-PBoolean OpalH224MediaStream::Close()
+
+void OpalH224MediaStream::InternalClose()
 {
-  if (OpalMediaStream::Close() == PFalse) {
-    return PFalse;
-  }
-	
   if (IsSource()) {
     h224Handler.StopTransmit();
     h224Handler.SetTransmitMediaStream(NULL);
   }
-	
-  return PTrue;
 }
 
+
 PBoolean OpalH224MediaStream::ReadPacket(RTP_DataFrame & /*packet*/)
 {
   return PFalse;
