@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26728 $
- * $Author: rjongbloed $
- * $Date: 2011-12-01 22:59:14 -0600 (Thu, 01 Dec 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -363,7 +363,12 @@ H323Channel::Directions H323UnidirectionalChannel::GetDirection() const
 
 PBoolean H323UnidirectionalChannel::SetInitialBandwidth()
 {
-  return SetBandwidthUsed(capability->GetMediaFormat().GetBandwidth()/100);
+  unsigned bandwidth = 0;
+  if (GetDirection() == IsTransmitter)
+    bandwidth = capability->GetMediaFormat().GetOptionInteger(OpalMediaFormat::TargetBitRateOption());
+  if (bandwidth == 0)
+    bandwidth = capability->GetMediaFormat().GetOptionInteger(OpalMediaFormat::MaxBitRateOption());
+  return SetBandwidthUsed(bandwidth/100);
 }
 
 
@@ -458,7 +463,6 @@ H323_RealTimeChannel::H323_RealTimeChannel(H323Connection & connection,
                                            Directions direction)
   : H323UnidirectionalChannel(connection, capability, direction)
 {
-  rtpPayloadType = capability.GetMediaFormat().GetPayloadType();
 }
 
 
@@ -519,9 +523,11 @@ void H323_RealTimeChannel::OnSendOpenAck(const H245_OpenLogicalChannel & open,
 void H323_RealTimeChannel::OnSendOpenAck(H245_H2250LogicalChannelAckParameters & param) const
 {
   // We are master, we use OUR session ID
-  if (connection.IsH245Master()) {
+  // Also include standard ones as the odd implementation (cough RMX cough) wants it
+  unsigned sessionID = GetSessionID();
+  if (connection.IsH245Master() || sessionID <= H323Capability::DefaultDataSessionID) {
     param.IncludeOptionalField(H245_H2250LogicalChannelAckParameters::e_sessionID);
-    param.m_sessionID = GetSessionID();
+    param.m_sessionID = sessionID;
   }
 
   PTRACE(3, "H323RTP\tSending open logical channel ACK: sessionID=" << GetSessionID());
@@ -627,33 +633,11 @@ PBoolean H323_RealTimeChannel::OnReceivedAckPDU(const H245_H2250LogicalChannelAc
 }
 
 
-PBoolean H323_RealTimeChannel::SetDynamicRTPPayloadType(int newType)
+RTP_DataFrame::PayloadTypes H323_RealTimeChannel::GetDynamicRTPPayloadType() const
 {
-  PTRACE(4, "H323RTP\tAttempting to set dynamic RTP payload type: " << newType);
-
-  // This is "no change"
-  if (newType == -1)
-    return PTrue;
-
-  // Check for illegal type
-  if (newType < RTP_DataFrame::DynamicBase || newType >= RTP_DataFrame::IllegalPayloadType)
-    return PFalse;
-
-  // Check for overwriting "known" type
-  if (rtpPayloadType < RTP_DataFrame::DynamicBase)
-    return PFalse;
-
-  rtpPayloadType = (RTP_DataFrame::PayloadTypes)newType;
-
-  OpalMediaStream * mediaStream = GetMediaStream();
-  if (mediaStream != NULL) {
-    OpalMediaFormat adjustedMediaFormat = mediaStream->GetMediaFormat();
-    adjustedMediaFormat.SetPayloadType(rtpPayloadType);
-    mediaStream->UpdateMediaFormat(adjustedMediaFormat);
-  }
-
-  PTRACE(3, "H323RTP\tSet dynamic payload type to " << rtpPayloadType);
-  return PTrue;
+  OpalMediaFormat mediaFormat = mediaStream != NULL ? mediaStream->GetMediaFormat()
+                                                    : capability->GetMediaFormat();
+  return mediaFormat.GetPayloadType();
 }
 
 
