@@ -40,15 +40,13 @@
 #include <ptclib/random.h>
 #include <ptclib/cypher.h>
 
+#include <opal/endpoint.h>
 #include <h323/h235auth.h>
 #include <h323/h323pdu.h>
 
 
 #define new PNEW
 
-namespace PWLibStupidLinkerHacks {
-  int h235AuthLoader;
-};
 
 /////////////////////////////////////////////////////////////////////////////
 
@@ -104,8 +102,11 @@ PBoolean H235Authenticator::PrepareTokens(PASN_Array & clearTokens,
       clearTokens.Append(clearToken);
   }
 
-  H225_CryptoH323Token * cryptoToken = CreateCryptoToken();
-  if (cryptoToken != NULL)
+  H225_CryptoH323Token * cryptoToken;
+  if ((cryptoToken = CreateCryptoToken(false)) != NULL)
+    cryptoTokens.Append(cryptoToken);
+
+  if ((cryptoToken = CreateCryptoToken(true)) != NULL)
     cryptoTokens.Append(cryptoToken);
 
   return PTrue;
@@ -118,7 +119,7 @@ H235_ClearToken * H235Authenticator::CreateClearToken()
 }
 
 
-H225_CryptoH323Token * H235Authenticator::CreateCryptoToken()
+H225_CryptoH323Token * H235Authenticator::CreateCryptoToken(bool)
 {
   return NULL;
 }
@@ -316,7 +317,7 @@ H235Authenticator::ValidationResult
 
 ///////////////////////////////////////////////////////////////////////////////
 
-static PFactory<H235Authenticator>::Worker<H235AuthSimpleMD5> factoryH235AuthSimpleMD5("SimpleMD5");
+PFACTORY_CREATE(PFactory<H235Authenticator>, H235AuthSimpleMD5, "SimpleMD5", false);
 
 static const char OID_MD5[] = "1.2.840.113549.2.5";
 
@@ -337,17 +338,7 @@ const char * H235AuthSimpleMD5::GetName() const
 }
 
 
-static PWCharArray GetUCS2plusNULL(const PString & str)
-{
-  PWCharArray ucs2 = str.AsUCS2();
-  PINDEX len = ucs2.GetSize();
-  if (len > 0 && ucs2[len-1] != 0)
-    ucs2.SetSize(len+1);
-  return ucs2;
-}
-
-
-H225_CryptoH323Token * H235AuthSimpleMD5::CreateCryptoToken()
+H225_CryptoH323Token * H235AuthSimpleMD5::CreateCryptoToken(bool digits)
 {
   if (!IsActive())
     return NULL;
@@ -357,20 +348,39 @@ H225_CryptoH323Token * H235AuthSimpleMD5::CreateCryptoToken()
     return NULL;
   }
 
+  if (digits && !OpalIsE164(localId, true))
+    return NULL;
+
   // Cisco compatible hash calculation
   H235_ClearToken clearToken;
 
   // fill the PwdCertToken to calculate the hash
   clearToken.m_tokenOID = "0.0";
 
+  // Create the H.225 crypto token
+  H225_CryptoH323Token * cryptoToken = new H225_CryptoH323Token;
+  cryptoToken->SetTag(H225_CryptoH323Token::e_cryptoEPPwdHash);
+  H225_CryptoH323Token_cryptoEPPwdHash & cryptoEPPwdHash = *cryptoToken;
+
+  // Set the alias
+  if (digits) {
+    cryptoEPPwdHash.m_alias.SetTag(H225_AliasAddress::e_dialedDigits);
+    (PASN_IA5String &)cryptoEPPwdHash.m_alias = localId;
+  }
+  else {
+    cryptoEPPwdHash.m_alias.SetTag(H225_AliasAddress::e_h323_ID);
+    (PASN_BMPString &)cryptoEPPwdHash.m_alias = localId;
+  }
+
+  // Use SetValueRaw to make sure trailing NULL is included
   clearToken.IncludeOptionalField(H235_ClearToken::e_generalID);
-  clearToken.m_generalID = GetUCS2plusNULL(localId);
+  clearToken.m_generalID.SetValueRaw(localId.AsUCS2());
 
   clearToken.IncludeOptionalField(H235_ClearToken::e_password);
-  clearToken.m_password = GetUCS2plusNULL(password);
+  clearToken.m_password.SetValueRaw(password.AsUCS2());
 
   clearToken.IncludeOptionalField(H235_ClearToken::e_timeStamp);
-  clearToken.m_timeStamp = (int)time(NULL);
+  clearToken.m_timeStamp = (int)PTime().GetTimeInSeconds();
 
   // Encode it into PER
   PPER_Stream strm;
@@ -383,14 +393,7 @@ H225_CryptoH323Token * H235AuthSimpleMD5::CreateCryptoToken()
   PMessageDigest5::Code digest;
   stomach.Complete(digest);
 
-  // Create the H.225 crypto token
-  H225_CryptoH323Token * cryptoToken = new H225_CryptoH323Token;
-  cryptoToken->SetTag(H225_CryptoH323Token::e_cryptoEPPwdHash);
-  H225_CryptoH323Token_cryptoEPPwdHash & cryptoEPPwdHash = *cryptoToken;
-
   // Set the token data that actually goes over the wire
-  H323SetAliasAddress(localId, cryptoEPPwdHash.m_alias);
-
   cryptoEPPwdHash.m_timeStamp = clearToken.m_timeStamp;
   cryptoEPPwdHash.m_token.m_algorithmOID = OID_MD5;
   cryptoEPPwdHash.m_token.m_hash.SetData(sizeof(digest)*8, (const BYTE *)&digest);
@@ -424,10 +427,10 @@ H235Authenticator::ValidationResult H235AuthSimpleMD5::ValidateCryptoToken(
   clearToken.m_tokenOID = "0.0";
 
   clearToken.IncludeOptionalField(H235_ClearToken::e_generalID);
-  clearToken.m_generalID = GetUCS2plusNULL(alias);
+  clearToken.m_generalID.SetValueRaw(alias.AsUCS2()); // Use SetValueRaw to make sure trailing NULL is included
 
   clearToken.IncludeOptionalField(H235_ClearToken::e_password);
-  clearToken.m_password = GetUCS2plusNULL(password);
+  clearToken.m_password.SetValueRaw(password.AsUCS2());
 
   clearToken.IncludeOptionalField(H235_ClearToken::e_timeStamp);
   clearToken.m_timeStamp = cryptoEPPwdHash.m_timeStamp;
@@ -486,7 +489,7 @@ PBoolean H235AuthSimpleMD5::IsSecuredPDU(unsigned rasPDU, PBoolean received) con
 
 ///////////////////////////////////////////////////////////////////////////////
 
-static PFactory<H235Authenticator>::Worker<H235AuthCAT> factoryH235AuthCAT("SimpleCAT");
+PFACTORY_CREATE(PFactory<H235Authenticator>, H235AuthCAT, "SimpleCAT", false);
 
 static const char OID_CAT[] = "1.2.840.113548.10.1.2.1";
 
@@ -523,10 +526,10 @@ H235_ClearToken * H235AuthCAT::CreateClearToken()
   clearToken->m_tokenOID = OID_CAT;
 
   clearToken->IncludeOptionalField(H235_ClearToken::e_generalID);
-  clearToken->m_generalID = GetUCS2plusNULL(localId);
+  clearToken->m_generalID.SetValueRaw(localId.AsUCS2()); // Use SetValueRaw to make sure trailing NULL is included
 
   clearToken->IncludeOptionalField(H235_ClearToken::e_timeStamp);
-  clearToken->m_timeStamp = (int)time(NULL);
+  clearToken->m_timeStamp = (int)PTime().GetTimeInSeconds();
   PUInt32b timeStamp = (DWORD)clearToken->m_timeStamp;
 
   clearToken->IncludeOptionalField(H235_ClearToken::e_random);
