@@ -26,9 +26,9 @@
  *
  * Contributor(s): Many thanks to Simon Horne.
  *
- * $Revision: 26654 $
- * $Author: rjongbloed $
- * $Date: 2011-11-14 20:49:07 -0600 (Mon, 14 Nov 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -352,11 +352,25 @@ PBoolean H323EndPoint::UseGatekeeper(const PString & address,
   }
 }
 
+static H323TransportAddress GetGatekeeperAddress(const PString & address)
+{
+  PIPSocketAddressAndPort addrPort(address, H225_RAS::DefaultRasUdpPort);
+
+#if OPAL_PTLIB_DNS
+  if (!addrPort.IsValid()) {
+    PIPSocketAddressAndPortVector addresses;
+    if (PDNS::LookupSRV(address, "_h323rs._udp", addrPort.GetPort(), addresses) && !addresses.empty())
+      addrPort = addresses[0];
+  }
+#endif //OPAL_PTLIB_DNS
+
+  return H323TransportAddress(addrPort.AsString(), H225_RAS::DefaultRasUdpPort, "udp");
+}
 
 PBoolean H323EndPoint::SetGatekeeper(const PString & address, H323Transport * transport)
 {
-  H323TransportAddress h323addr(address, H225_RAS::DefaultRasUdpPort, "udp");
-  return InternalCreateGatekeeper(transport) && gatekeeper->DiscoverByAddress(h323addr);
+  return InternalCreateGatekeeper(transport) &&
+         gatekeeper->DiscoverByAddress(GetGatekeeperAddress(address));
 }
 
 
@@ -364,8 +378,8 @@ PBoolean H323EndPoint::SetGatekeeperZone(const PString & address,
                                      const PString & identifier,
                                      H323Transport * transport)
 {
-  H323TransportAddress h323addr(address, H225_RAS::DefaultRasUdpPort, "udp");
-  return InternalCreateGatekeeper(transport) && gatekeeper->DiscoverByNameAndAddress(identifier, h323addr);
+  return InternalCreateGatekeeper(transport) &&
+         gatekeeper->DiscoverByNameAndAddress(identifier, GetGatekeeperAddress(address));
 }
 
 
@@ -707,14 +721,6 @@ void H323EndPoint::ConsultationTransfer(const PString & primaryCallToken,
 }
 
 
-void H323EndPoint::HoldCall(const PString & token, PBoolean localHold)
-{
-  PSafePtr<H323Connection> connection = FindConnectionWithLock(token);
-  if (connection != NULL)
-    connection->HoldCall(localHold);
-}
-
-
 PBoolean H323EndPoint::IntrudeCall(const PString & remoteParty,
                                unsigned capabilityLevel,
                                void * userData)
@@ -786,7 +792,7 @@ PBoolean H323EndPoint::ParsePartyName(const PString & remoteParty,
       PIPSocket::Address ip = hostname;
       if (!ip.IsValid()) {
         PIPSocketAddressAndPortVector addresses;
-        if (PDNS::LookupSRV(hostname, "_h323._tcp", url.GetPort(), addresses) && !addresses.empty()) {
+        if (PDNS::LookupSRV(hostname, "_h323cs._tcp", url.GetPort(), addresses) && !addresses.empty()) {
           // Only use first entry
           url.SetHostName(addresses[0].GetAddress().AsString());
           url.SetPort(addresses[0].GetPort());
@@ -951,18 +957,17 @@ PBoolean H323EndPoint::ParsePartyName(const PString & remoteParty,
 
 PSafePtr<H323Connection> H323EndPoint::FindConnectionWithLock(const PString & token, PSafetyMode mode)
 {
-  PSafePtr<H323Connection> connnection = PSafePtrCast<OpalConnection, H323Connection>(GetConnectionWithLock(token, mode));
-  if (connnection != NULL)
-    return connnection;
+  PSafePtr<H323Connection> connection = PSafePtrCast<OpalConnection, H323Connection>(GetConnectionWithLock(token, mode));
+  if (connection != NULL)
+    return connection;
 
-  for (PSafePtr<OpalConnection> conn(connectionsActive, mode); conn != NULL; ++conn) {
-    connnection = PSafePtrCast<OpalConnection, H323Connection>(conn);
-    if(connnection != NULL) {      //cast success
-      if (connnection->GetCallIdentifier().AsString() == token)
-        return connnection;
-      if (connnection->GetConferenceIdentifier().AsString() == token)
-        return connnection;
-    }
+  for (PSafePtr<OpalConnection> conn(connectionsActive, PSafeReference); conn != NULL; ++conn) {
+    connection = PSafePtrCast<OpalConnection, H323Connection>(conn);
+    if (  connection != NULL &&
+          connection.SetSafetyMode(PSafeReadOnly) &&
+         (connection->GetCallIdentifier().AsString() == token ||
+          connection->GetConferenceIdentifier().AsString() == token))
+      return connection.SetSafetyMode(mode) ? connection : NULL;
   }
 
   return NULL;
