@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26939 $
- * $Author: rjongbloed $
- * $Date: 2012-02-07 01:17:49 -0600 (Tue, 07 Feb 2012) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -77,7 +77,7 @@ void H323TraceDumpPDU(const char * proto,
   if (!PTrace::CanTrace(3))
     return;
 
-  ostream & trace = PTrace::Begin(3, __FILE__, __LINE__);
+  ostream & trace = PTrace::Begin(PTrace::CanTrace(4) ? 4 : 3, __FILE__, __LINE__);
   trace << proto << '\t' << (writing ? "Send" : "Receiv") << "ing PDU:";
 
   if (PTrace::CanTrace(4)) {
@@ -347,32 +347,17 @@ void H323SetRTPPacketization(H245_ArrayOf_RTPPayloadType & rtpPacketizations,
                              const OpalMediaFormat & mediaFormat,
                              RTP_DataFrame::PayloadTypes payloadType)
 {
-  PString mediaPacketization = mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationsOption(),
-                               mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationOption()));
-  if (mediaPacketization.IsEmpty())
-    return;
-  
   // Special case handling by Product Id
   PString h323ProductId = mediaFormat.GetOptionString("h323ProductId");
   if (h323ProductId == "NetMeeting")
     return;
 
-  PStringArray packetizationStrings = mediaPacketization.Tokenise(",");
-  for (PINDEX i = 0; i < packetizationStrings.GetSize(); i++) {
-    PString & packetizationString = packetizationStrings[i];
-    
+  PStringSet mediaPacketizations = mediaFormat.GetMediaPacketizations();
+  for (PINDEX i = 0; i < mediaPacketizations.GetSize(); i++) {
     rtpPacketizations.SetSize(rtpPacketizationCount+1);
-    if (H323SetRTPPacketization(rtpPacketizations[rtpPacketizationCount], packetizationString, mediaFormat, payloadType)) {
-      
-      // Check if already in list
-      PINDEX test;
-      for (test = 0; test < rtpPacketizationCount; test++) {
-        if (rtpPacketizations[test] == rtpPacketizations[rtpPacketizationCount])
-          break;
-      }
-      if (test == rtpPacketizationCount)
-        rtpPacketizationCount++;
-    }
+    if (H323SetRTPPacketization(rtpPacketizations[rtpPacketizationCount],
+                                mediaPacketizations.GetKeyAt(i), mediaFormat, payloadType))
+      rtpPacketizationCount++;
   }
 }
 
@@ -380,20 +365,14 @@ bool H323SetRTPPacketization(H245_RTPPayloadType & rtpPacketization,
                              const OpalMediaFormat & mediaFormat,
                              RTP_DataFrame::PayloadTypes payloadType)
 {
-  PString mediaPacketization = mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationsOption(),
-                               mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationOption()));
-  if (mediaPacketization.IsEmpty())
-    return PFalse;
-
   // Special case handling by Product Id
   PString h323ProductId = mediaFormat.GetOptionString("h323ProductId");
   if (h323ProductId == "NetMeeting")
     return PFalse;
-  
-  PStringArray packetizationStrings = mediaPacketization.Tokenise(",");
-  
-  // Only use the first packetization (= highest priority)
-  return H323SetRTPPacketization(rtpPacketization, packetizationStrings[0], mediaFormat, payloadType);
+
+  PStringSet mediaPacketizations = mediaFormat.GetMediaPacketizations();
+  return !mediaPacketizations.IsEmpty() &&
+         H323SetRTPPacketization(rtpPacketization, mediaPacketizations.GetKeyAt(0), mediaFormat, payloadType);
 }
 
 bool H323SetRTPPacketization(H245_RTPPayloadType & rtpPacketization,
@@ -441,8 +420,12 @@ PString H323GetRTPPacketization(const H245_RTPPayloadType & rtpPacketization)
 
     case H245_RTPPayloadType_payloadDescriptor::e_oid :
       mediaPacketization = ((const PASN_ObjectId &)rtpPacketization.m_payloadDescriptor).AsString();
-      if (mediaPacketization.IsEmpty()) {
-        PTRACE(1, "RTP_UDP\tInvalid OID in packetization type.");
+      PTRACE_IF(1, mediaPacketization.IsEmpty(), "RTP_UDP\tInvalid OID in packetization type.");
+
+      // Some stupid endpoints (e.g. Polycom) use this, one zero short!
+      if (mediaPacketization == "0.0.8.241.0.0.0") {
+        mediaPacketization    = "0.0.8.241.0.0.0.0";
+        PTRACE(4, "RTP_UDP\tDetected bogus H.264 OID in packetization type, compensating");
       }
       break;
     case H245_RTPPayloadType_payloadDescriptor::e_nonStandardIdentifier :
@@ -581,60 +564,47 @@ unsigned H323GetGenericParameterInteger(const H245_ArrayOf_GenericParameter & pa
 }
 
 
-H245_ParameterValue * H323AddGenericParameter(H245_ArrayOf_GenericParameter & params, unsigned ordinal, bool reorder)
+H245_ParameterValue * H323AddGenericParameter(H245_ArrayOf_GenericParameter & params, unsigned ordinal)
 {
   PINDEX size = params.GetSize();
   params.SetSize(size+1);
-
-  PINDEX pos = size;
-  if (reorder) {
-    for (pos = size; pos > 0; pos--) {
-      const H245_GenericParameter & param = params[pos-1];
-      if (param.m_parameterIdentifier.GetTag() == H245_ParameterIdentifier::e_standard &&
-                            ((const PASN_Integer &)param.m_parameterIdentifier) < ordinal)
-        break;
-      params[pos] = param;
-    }
-  }
-
-  H245_GenericParameter & param = params[pos];
+  H245_GenericParameter & param = params[size];
   param.m_parameterIdentifier.SetTag(H245_ParameterIdentifier::e_standard);
   (PASN_Integer &)param.m_parameterIdentifier = ordinal;
   return &param.m_parameterValue;
 }
 
 
-void H323AddGenericParameterBoolean(H245_ArrayOf_GenericParameter & params, unsigned ordinal, bool value, bool reorder)
+void H323AddGenericParameterBoolean(H245_ArrayOf_GenericParameter & params, unsigned ordinal, bool value)
 {
   // Do not include a logical at all if it is false
   if (value)
-    H323AddGenericParameter(params, ordinal, reorder)->SetTag(H245_ParameterValue::e_logical);
+    H323AddGenericParameter(params, ordinal)->SetTag(H245_ParameterValue::e_logical);
 }
 
 
 void H323AddGenericParameterInteger(H245_ArrayOf_GenericParameter & params,
                                     unsigned ordinal,
                                     unsigned value,
-                                    H245_ParameterValue::Choices subType,
-                                    bool reorder)
+                                    H245_ParameterValue::Choices subType)
 {
-  H245_ParameterValue * param = H323AddGenericParameter(params, ordinal, reorder);
+  H245_ParameterValue * param = H323AddGenericParameter(params, ordinal);
   param->SetTag(subType);
   (PASN_Integer &)*param = value;
 }
 
 
-void H323AddGenericParameterString(H245_ArrayOf_GenericParameter & params, unsigned ordinal, const PString & value, bool reorder)
+void H323AddGenericParameterString(H245_ArrayOf_GenericParameter & params, unsigned ordinal, const PString & value)
 {
-  H245_ParameterValue * param = H323AddGenericParameter(params, ordinal, reorder);
+  H245_ParameterValue * param = H323AddGenericParameter(params, ordinal);
   param->SetTag(H245_ParameterValue::e_octetString);
   (PASN_OctetString &)*param = value;
 }
 
 
-void H323AddGenericParameterOctets(H245_ArrayOf_GenericParameter & params, unsigned ordinal, const PBYTEArray & value, bool reorder)
+void H323AddGenericParameterOctets(H245_ArrayOf_GenericParameter & params, unsigned ordinal, const PBYTEArray & value)
 {
-  H245_ParameterValue * param = H323AddGenericParameter(params, ordinal, reorder);
+  H245_ParameterValue * param = H323AddGenericParameter(params, ordinal);
   param->SetTag(H245_ParameterValue::e_octetString);
   (PASN_OctetString &)*param = value;
 }
