@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23466 $
- * $Author: rjongbloed $
- * $Date: 2009-09-16 02:31:35 -0500 (Wed, 16 Sep 2009) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -410,7 +410,7 @@ PBoolean Q931::Encode(PBYTEArray & data) const
 
 void Q931::PrintOn(ostream & strm) const
 {
-  int indent = strm.precision() + 2;
+  std::streamsize indent = strm.precision() + 2;
   ios::fmtflags flags = strm.flags();
 
   strm << "{\n"
@@ -572,6 +572,46 @@ void Q931::RemoveIE(InformationElementCodes ie)
 }
 
 
+void Q931::SetBearerCapabilities(const PString & str)
+{
+  InformationTransferCapability capability = TransferSpeech;
+  unsigned transferRate = 1; // 64kbps
+  unsigned codingStandard = 0; // ITU-T standardized coding
+  unsigned userInfoLayer1 = 5; // Recommendations H.221 and H.242
+
+  if (str.FindSpan("0123456789ABCDEFabcdef") == P_MAX_INDEX) {
+    PINDEX byteCount = str.GetLength()/2;
+    if (byteCount >= 2) {
+      PBYTEArray data(byteCount);
+      for (PINDEX i = 0; i < byteCount; ++i)
+        data[i] = (BYTE)str.Mid(i*2, 2).AsUnsigned(16);
+      SetIE(BearerCapabilityIE, data);
+      return;
+    }
+  }
+
+  PStringArray caps = str.Tokenise(',');
+  switch (caps.GetSize()) {
+    default :
+      userInfoLayer1 = caps[3].AsUnsigned();
+    case 3 :
+      codingStandard = caps[2].AsUnsigned();
+    case 2 :
+      transferRate = caps[1].AsUnsigned();
+    case 1 :
+      if (caps[0] *= "speech")
+        capability = TransferSpeech;
+      else if (caps[0] *= "digital")
+        capability = TransferUnrestrictedDigital;
+      else
+        capability = (Q931::InformationTransferCapability)caps[0].AsUnsigned();
+    case 0 :
+      break;
+  }
+  SetBearerCapabilities(capability, transferRate, codingStandard, userInfoLayer1);
+}
+
+
 void Q931::SetBearerCapabilities(InformationTransferCapability capability,
                                  unsigned transferRate,
                                  unsigned codingStandard,
