@@ -25,9 +25,9 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 23294 $
- * $Author: rjongbloed $
- * $Date: 2009-08-28 01:26:26 -0500 (Fri, 28 Aug 2009) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -64,9 +64,10 @@ IAX2Connection::IAX2Connection(OpalCall & call,               /* Owner call for
 			     void * /*userData */,
 			     const PString & inRemoteParty,
            const PString & inRemotePartyName)
-  : OpalConnection(call, ep, token), 
-endpoint(ep),
-iax2Processor(*new IAX2CallProcessor(ep))
+  : OpalConnection(call, ep, token)
+  , endpoint(ep)
+  , iax2Processor(*new IAX2CallProcessor(ep))
+  , jitterBuffer(400, 2000)
 {  
   opalPayloadType = RTP_DataFrame::IllegalPayloadType;
 
@@ -79,11 +80,11 @@ iax2Processor(*new IAX2CallProcessor(ep))
   PStringArray res = IAX2EndPoint::DissectRemoteParty(inRemoteParty);
   remotePartyNumber = res[IAX2EndPoint::extensionIndex];
 
-   SetCallToken(token);
+  SetCallToken(token);
   originating = PFalse;
 
   ep.CopyLocalMediaFormats(localMediaFormats);
-  AdjustMediaFormats(localMediaFormats);
+  AdjustMediaFormats(true, NULL, localMediaFormats);
   PTRACE(5, "Local ordered codecs are " << localMediaFormats);
   
   local_hold = PFalse;
@@ -95,7 +96,6 @@ iax2Processor(*new IAX2CallProcessor(ep))
 
 IAX2Connection::~IAX2Connection()
 {
-  jitterBuffer.CloseDown();
   iax2Processor.Terminate();
   iax2Processor.WaitForTermination(1000);
   if (!iax2Processor.IsTerminated()) {
@@ -113,23 +113,11 @@ void IAX2Connection::StartOperation()
   SetPhase(SetUpPhase);
 }
 
-void IAX2Connection::Release( CallEndReason reason)		        
-{ 
-  PTRACE(4, "IAX2Con\tRelease( CallEndReason " << reason);
-
-  iax2Processor.Hangup(reason); ///Send hangup frame
-
-  iax2Processor.Release(reason); 
-
-  jitterBuffer.CloseDown();
-
-  OpalConnection::Release(reason);
-}
-
 void IAX2Connection::OnReleased()
 {
   PTRACE(4, "IAX2Con\tOnReleased()" << *this);
 
+  iax2Processor.Hangup(GetCallEndReasonText(GetCallEndReason())); ///Send hangup frame
   iax2Processor.OnReleased();
   OpalConnection::OnReleased();  
 }
@@ -167,7 +155,7 @@ void IAX2Connection::AnsweringCall(AnswerCallResponse response)
   PTRACE(3, "IAX2Con\tAnswering call: " << response);
 
   PSafeLockReadWrite safeLock(*this);
-  if (!safeLock.IsLocked() || GetPhase() >= ReleasingPhase)
+  if (!safeLock.IsLocked() || IsReleased())
     return;
 
   OpalConnection::AnsweringCall(response);
@@ -176,26 +164,24 @@ void IAX2Connection::AnsweringCall(AnswerCallResponse response)
 
 PBoolean IAX2Connection::SetConnected()
 {
-    PTRACE(3, "IAX2Con\t SET CONNECTED " 
+  PTRACE(3, "IAX2Con\t SET CONNECTED " 
 	 << PString(IsOriginating() ? " Originating" : "Receiving"));
   ////if no media streams, try and start them
-
+  
+  if (!IsOriginating())
+    iax2Processor.SendAnswerMessageToRemoteNode();
+  
   if (mediaStreams.IsEmpty()) {
-    if (!IsOriginating())
-      iax2Processor.SendAnswerMessageToRemoteNode();
-
+    
     ownerCall.OpenSourceMediaStreams(*this, OpalMediaType::Audio(), 1);
     PSafePtr<OpalConnection> otherParty = GetOtherPartyConnection();
     if (otherParty != NULL)
       ownerCall.OpenSourceMediaStreams(*otherParty, OpalMediaType::Audio(), 1);
-
-    
+        
     jitterBuffer.SetDelay(endpoint.GetManager().GetMinAudioJitterDelay() * 8, 
 			  endpoint.GetManager().GetMaxAudioJitterDelay() * 8);
     PTRACE(5, "Iax2Con\t Start jitter buffer");
-    jitterBuffer.Resume();
-  }
-
+  }  
   return OpalConnection::SetConnected();
 }
 
@@ -211,7 +197,6 @@ PBoolean IAX2Connection::SetAlerting(const PString & PTRACE_PARAM(calleeName), P
     if (GetPhase() == AlertingPhase)
       return PFalse;
     
-    alertingTime = PTime();
     SetPhase(AlertingPhase);
   }
 
@@ -232,11 +217,10 @@ void IAX2Connection::OnConnected()
     if (otherParty != NULL)
       ownerCall.OpenSourceMediaStreams(*otherParty, OpalMediaType::Audio(), 1);
 
-   
+
     jitterBuffer.SetDelay(endpoint.GetManager().GetMinAudioJitterDelay() * 8, 
 			  endpoint.GetManager().GetMaxAudioJitterDelay() * 8);
     PTRACE(5, "Iax2Con\t Start jitter buffer");
-    jitterBuffer.Resume();
   }
 
     // Let OPAL do it's thing with the OnConnected callback.
@@ -262,18 +246,6 @@ PBoolean IAX2Connection::SendUserInputString(const PString & value)
   return OpalConnection::SendUserInputString(value);
 }
 
-OpalConnection::SendUserInputModes IAX2Connection::GetRealSendUserInputMode() const
-{
-  switch (sendUserInputMode) {
-    case SendUserInputAsString:
-    case SendUserInputAsTone:
-      return sendUserInputMode;
-    default:
-      break;
-  }
-
-  return SendUserInputAsTone;
-}
   
 PBoolean IAX2Connection::SendUserInputTone(char tone, unsigned /*duration*/ ) 
 { 
@@ -366,12 +338,13 @@ void IAX2Connection::BuildRemoteCapabilityTable(unsigned int remoteCapability, u
     remoteMediaFormats.Reorder(PStringArray(wildcard));
   }
 
-  AdjustMediaFormats(remoteMediaFormats);
+  AdjustMediaFormats(false, NULL, remoteMediaFormats);
   PTRACE(4, "Connection\tREMOTE Codecs are " << remoteMediaFormats);
 }
 
 void IAX2Connection::EndCallNow(CallEndReason reason)
 { 
+  PTRACE(4, "IAX2Con\tEndCallNow() - reason is " << reason);
   OpalConnection::ClearCall(reason); 
 }
 
@@ -415,44 +388,38 @@ unsigned int IAX2Connection::ChooseCodec()
   return 0;
 }
 
-PBoolean IAX2Connection::IsConnectionOnHold()
+PBoolean IAX2Connection::IsOnHold(bool fromRemote)
 {
-  return (local_hold || remote_hold);
+  return fromRemote ? remote_hold : local_hold;
 }
 
-bool IAX2Connection::HoldConnection()
+bool IAX2Connection::Hold(bool fromRemote, bool placeOnHold)
 {
-  if (local_hold)
+  if (IsOnHold(fromRemote))
     return true;
-    
-  local_hold = PTrue;
-  PauseMediaStreams(PTrue);
-  endpoint.OnHold(*this);
-  
-  iax2Processor.SendHold();
+
+  if (fromRemote)
+    return false;
+
+  local_hold = placeOnHold;
+  PauseMediaStreams(placeOnHold);
+  OnHold(fromRemote, placeOnHold);
+
+  if (placeOnHold)
+    iax2Processor.SendHold();
+  else
+    iax2Processor.SendHoldRelease();
   return true;
 }
 
-bool IAX2Connection::RetrieveConnection()
-{
-  if (!local_hold)
-    return true;
-  
-  local_hold = PFalse;
-  PauseMediaStreams(PFalse);  
-  endpoint.OnHold(*this);
-  
-  iax2Processor.SendHoldRelease();
-  return true;
-}
 
 void IAX2Connection::RemoteHoldConnection()
 {
   if (remote_hold)
     return;
     
-  remote_hold = PTrue;
-  endpoint.OnHold(*this);
+  remote_hold = true;
+  OnHold(true, true);
 }
 
 void IAX2Connection::RemoteRetrieveConnection()
@@ -460,14 +427,14 @@ void IAX2Connection::RemoteRetrieveConnection()
   if (!remote_hold)
     return;
     
-  remote_hold = PFalse;    
-  endpoint.OnHold(*this);
+  remote_hold = false;    
+  endpoint.OnHold(*this, true, false);
 }
 
 bool IAX2Connection::TransferConnection(const PString & remoteParty)
 {
   //The call identity is not used because we do not handle supervised transfers yet.  
-  PTRACE(3, "Transfer call to " + remoteParty);
+  PTRACE(3, "IAX2\tTransfer call to \"" + remoteParty << '"');
   
   PStringArray rpList = IAX2EndPoint::DissectRemoteParty(remoteParty);
   PString remoteAddress = GetRemoteInfo().RemoteAddress();
@@ -505,21 +472,23 @@ void IAX2Connection::ReceivedSoundPacketFromNetwork(IAX2Frame *soundFrame)
       return;
     }
 
-    RTP_DataFrame *mediaFrame = new RTP_DataFrame(soundFrame->GetMediaDataSize());
-    mediaFrame->SetTimestamp(soundFrame->GetTimeStamp() * 8);
-    mediaFrame->SetMarker(PFalse);
-    mediaFrame->SetPayloadType(opalPayloadType);
+    RTP_DataFrame mediaFrame(soundFrame->GetMediaDataSize());
+    mediaFrame.SetTimestamp(soundFrame->GetTimeStamp() * 8);
+    mediaFrame.SetMarker(PFalse);
+    mediaFrame.SetPayloadType(opalPayloadType);
 
-    mediaFrame->SetSize(mediaFrame->GetPayloadSize() + mediaFrame->GetHeaderSize());
-    memcpy(mediaFrame->GetPayloadPtr(), soundFrame->GetMediaDataPointer(), 
-	   soundFrame->GetMediaDataSize());
-    jitterBuffer.NewFrameFromNetwork(mediaFrame);
+    mediaFrame.SetSize(mediaFrame.GetPayloadSize() + mediaFrame.GetHeaderSize());
+    memcpy(mediaFrame.GetPayloadPtr(), soundFrame->GetMediaDataPointer(), soundFrame->GetMediaDataSize());
+    jitterBuffer.WriteData(mediaFrame);
     PTRACE(5, "RTP\tIAX2 frame now on jitter buffer (As a RTP frame)");
     delete soundFrame;
 }
 
 PBoolean IAX2Connection::ReadSoundPacket(RTP_DataFrame & packet)
 { 
+  if (IsReleased()) /*We are hanging up. */
+    return false;   /* Sending sound now is meaningless */
+  
   PTRACE(6, "Iax2Con\t Start read from  jitter buffer"); 
   if (!jitterBuffer.ReadData(packet)) {
     PINDEX zeroBytes = packet.GetSize() - packet.GetHeaderSize();
@@ -535,11 +504,9 @@ PBoolean IAX2Connection::ReadSoundPacket(RTP_DataFrame & packet)
 
 #endif // OPAL_IAX2
 
-/* The comment below is magic for those who use emacs to edit this file. */
-/* With the comment below, the tab key does auto indent to 2 spaces.     */
-
-
-/*
+/* The comment below is magic for those who use emacs to edit this file.
+ * With the comment below, the tab key does auto indent to 2 spaces.    
+ *
  * Local Variables:
  * mode:c
  * c-basic-offset:2
