@@ -24,9 +24,9 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 21293 $
- * $Author: rjongbloed $
- * $Date: 2008-10-12 23:24:41 +0000 (Sun, 12 Oct 2008) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -121,19 +121,18 @@ void IAX2IeData::PrintOn(ostream & strm) const
        <<   "recevied frames (total frames received) u32          " << receivedPackets   << endl
        <<   "Max playout delay for received frame (in ms) u16     " << receivedDelay     << endl
        <<   "Dropped frames (presumably by jitterbuf) u32         " << receivedDropped   << endl
-       <<   "Frames received Out of Order u32                     " << receivedOoo       << endl;
+       <<   "Frames received Out of Order u32                     " << receivedOoo       << endl
+       <<   "Call Token element                                   " << callToken         << endl;
 }
 
 
 #endif // OPAL_IAX2
 
-/* The comment below is magic for those who use emacs to edit this file. */
-/* With the comment below, the tab key does auto indent to 4 spaces.     */
-
-/*
+/* The comment below is magic for those who use emacs to edit this file. 
+ * With the comment below, the tab key does auto indent to 2 spaces.     
+ *
  * Local Variables:
  * mode:c
- * c-file-style:linux
  * c-basic-offset:2
  * End:
  */
