@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26995 $
- * $Author: rjongbloed $
- * $Date: 2012-02-16 19:30:39 -0600 (Thu, 16 Feb 2012) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -411,10 +411,6 @@ PBoolean OpalMSRPMediaStream::WritePacket(RTP_DataFrame & frame)
   return m_msrpSession.WritePacket(frame);
 }
 
-PBoolean OpalMSRPMediaStream::Close()
-{
-  return OpalIMMediaStream::Close();
-}
 
 void OpalMSRPMediaStream::OnReceiveMSRP(OpalMSRPManager &, OpalMSRPManager::IncomingMSRP & incomingMSRP)
 {
