@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25076 $
- * $Author: csoutheren $
- * $Date: 2011-01-20 01:06:33 -0600 (Thu, 20 Jan 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -324,7 +324,7 @@ OpalIMContext::SentStatus OpalSIPIMContext::InternalSendOutsideCall(OpalIM * mes
   SIPMessage::Params params;
   PopulateParams(params, *message);
 
-  return ep->SendMESSAGE(params);
+  return ep->SendMESSAGE(params) ? SentPending : SentNoTransport;
 }
 
 
