@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22675 $
- * $Author: rjongbloed $
- * $Date: 2009-05-19 23:23:29 -0500 (Tue, 19 May 2009) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -42,11 +42,15 @@
 #include <sip/sdp.h>
 #endif
 
+
+const char T140[] = "t140";
+
+
 #if OPAL_SIP
 
 /////////////////////////////////////////////////////////
 //
-//  SDP media description for audio media
+//  SDP media description for text media
 //
 
 class SDPT140MediaDescription : public SDPRTPAVPMediaDescription
@@ -54,11 +58,19 @@ class SDPT140MediaDescription : public SDPRTPAVPMediaDescription
   PCLASSINFO(SDPT140MediaDescription, SDPRTPAVPMediaDescription);
   public:
     SDPT140MediaDescription(const OpalTransportAddress & address)
-      : SDPRTPAVPMediaDescription(address)
-    { }
+      : SDPRTPAVPMediaDescription(address, T140)
+    {
+    }
+
+    virtual SDPMediaDescription * CreateEmpty() const
+    {
+      return new SDPT140MediaDescription(OpalTransportAddress());
+    }
 
     virtual PString GetSDPMediaType() const
-    {  return "text";  }
+    {
+      return "text";
+    }
 };
 
 #endif
@@ -66,7 +78,7 @@ class SDPT140MediaDescription : public SDPRTPAVPMediaDescription
 /////////////////////////////////////////////////////////////////////////////
 
 OpalT140MediaType::OpalT140MediaType()
-  : OpalRTPAVPMediaType("t140", "text", 7)
+  : OpalRTPAVPMediaType(T140, "text")
 {
 }
 
@@ -84,7 +96,8 @@ SDPMediaDescription * OpalT140MediaType::CreateSDPMediaDescription(const OpalTra
 T140String::T140String()
   : length(0)
 { 
-  AppendUnicode16(ZERO_WIDTH_NO_BREAK); 
+  AppendUnicode16(ZERO_WIDTH_NO_BREAK);
+  SetAt(length, '\0');
 }
 
 T140String::T140String(const PBYTEArray & bytes)
@@ -96,6 +109,7 @@ T140String::T140String(const PBYTEArray & bytes)
       ch != ZERO_WIDTH_NO_BREAK)
     AppendUnicode16(ZERO_WIDTH_NO_BREAK);
   AppendUTF((const BYTE *)bytes, bytes.GetSize());
+  SetAt(length, '\0');
 }
 
 T140String::T140String(const BYTE * data, PINDEX len)
@@ -107,6 +121,7 @@ T140String::T140String(const BYTE * data, PINDEX len)
       ch != ZERO_WIDTH_NO_BREAK)
     AppendUnicode16(ZERO_WIDTH_NO_BREAK); 
   AppendUTF((const BYTE *)data, len);
+  SetAt(length, '\0');
 }
 
 T140String::T140String(const char * chars)
@@ -118,6 +133,7 @@ T140String::T140String(const char * chars)
       ch != ZERO_WIDTH_NO_BREAK)
     AppendUnicode16(ZERO_WIDTH_NO_BREAK); 
   AppendUTF((const BYTE *)chars, strlen(chars));
+  SetAt(length, '\0');
 }
 
 T140String::T140String(const PString & str)
@@ -129,17 +145,11 @@ T140String::T140String(const PString & str)
       ch != ZERO_WIDTH_NO_BREAK)
     AppendUnicode16(ZERO_WIDTH_NO_BREAK); 
   AppendUTF((const BYTE *)(const char *)str, str.GetLength());
+  SetAt(length, '\0');
 }
 
 PINDEX T140String::AppendUTF(const BYTE * utf, PINDEX utfLen)
 {
-  WORD ch;
-  if (utfLen >= 3 &&
-      GetUTF(utf, utfLen, ch) == 3 &&
-      ch == ZERO_WIDTH_NO_BREAK) {
-    utf += 3; 
-    utfLen -= 3;
-  }
   if (utfLen > 0) {
     memcpy(GetPointer(length+utfLen)+length, utf, utfLen);
     length += utfLen;
@@ -184,8 +194,8 @@ PINDEX T140String::SetUTF(BYTE * ptr, WORD c)
   }
 
   //if (c <= 0xffff) {
-  ptr[0] = 0xe0 | (ch >> 4) | (cl >> 6); 
-  ptr[1] = 0x80 | (ch << 2) | (cl >> 6); 
+  ptr[0] = 0xe0 | (ch >> 4); 
+  ptr[1] = 0x80 | ((ch & 0xf) << 2) | (cl >> 6); 
   ptr[2] = 0x80 | (cl & 0x3f);
   return 3;
 }
@@ -211,7 +221,7 @@ PINDEX T140String::GetUTF(const BYTE * ptr, PINDEX len, WORD & ch)
 
   // 0x80 .. 0x7ff
   if (ptr[0] <= 0xdf) {
-    ch = (ptr[0] << 6) | (ptr[1] & 0x3f);
+    ch = ((ptr[0] & 0x1f) << 6) | (ptr[1] & 0x3f);
     return 2;
   }
 
@@ -219,7 +229,29 @@ PINDEX T140String::GetUTF(const BYTE * ptr, PINDEX len, WORD & ch)
     return 0;
 
   // 0x800 .. 0xffff
-  ch = (ptr[0] << 12) | ((ptr[1] & 0x3f) << 6) || (ptr[2] & 0x3f);
+  ch = ((ptr[0] & 0xf) << 12) | ((ptr[1] & 0x3f) << 6) | (ptr[2] & 0x3f);
 
   return 3;
 }
+
+bool T140String::AsString(PString & str)
+{
+  PINDEX pos = 0;
+  while (pos < GetSize()) {
+    WORD ch;
+    PINDEX len = GetUTF(pos, ch);
+    if (len == 0)
+      return false;
+    if (len == 1)
+      str += (char)(ch & 0xff);
+    else if (ch == UTF_NEWLINE)
+      str += '\n';
+    else // if (ch == ZERO_WIDTH_NO_BREAK) 
+    {
+      ;
+    }
+    pos += len;
+  }
+
+  return true;
+}
