@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25368 $
- * $Author: rjongbloed $
- * $Date: 2011-03-21 01:34:07 -0500 (Mon, 21 Mar 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -40,6 +40,8 @@
 #endif
 
 
+PURL_LEGACY_SCHEME(isdn, true, false, true, true, false, true, true, false, false, false, 0)
+
 enum {
     MaxLineCount = 30,
     MaxBlockCount = 2,
@@ -566,15 +568,24 @@ PSafePtr<OpalConnection> OpalCapiEndPoint::MakeConnection(OpalCall & call,
                                                       unsigned int   options,
                                      OpalConnection::StringOptions * stringOptions)
 {
-  PINDEX colon = party.Find(':');
-  PString number = party.Mid(colon != P_MAX_INDEX ? colon+1 : 0);
+  PURL uri;
+  if (!uri.Parse(party, "isdn")) {
+    PTRACE(1, "CAPI\tNo available bearers for outgoing call.");
+    return NULL;
+  }
+  
+  OpalConnection::StringOptions localStringOptions;
+  if (stringOptions == NULL)
+    stringOptions = &localStringOptions;
+
+  stringOptions->ExtractFromURL(uri);
 
   unsigned controller, bearer;
   if (GetFreeLine(controller, bearer)) {
     OpalCapiConnection * connection = CreateConnection(call, userData, options, stringOptions, controller, bearer);
     if (AddConnection(connection) != NULL) {
       m_cbciToConnection[(controller<<8)|bearer] = connection;
-      connection->m_calledPartyNumber = number;
+      connection->m_calledPartyNumber = uri.GetUserName(); // Number part
       return connection;
     }
   }
@@ -586,7 +597,10 @@ PSafePtr<OpalConnection> OpalCapiEndPoint::MakeConnection(OpalCall & call,
 
 OpalMediaFormatList OpalCapiEndPoint::GetMediaFormats() const
 {
-  return OpalG711_ALAW_64K;
+  OpalMediaFormatList formats;
+  formats += OpalG711_ALAW_64K;;
+  //formats += OpalG711_ULAW_64K;  // Need to figure out how to detect/select this
+  return formats;
 }
 
 
@@ -601,6 +615,36 @@ OpalCapiConnection * OpalCapiEndPoint::CreateConnection(OpalCall & call,
 }
 
 
+PString OpalCapiEndPoint::GetDriverInfo() const
+{
+  OpalCapiProfile profile;
+  DWORD capiversion_min;
+  DWORD capiversion_maj;
+  DWORD manufacturerversion_min;
+  DWORD manufacturerversion_maj;
+  DWORD InstalledControllers;
+  char szBuffer[64] = "";
+  PStringStream str;
+
+  m_capi->GET_MANUFACTURER(szBuffer);
+  str << "CAPI_manufacturer=" << szBuffer << '\n';
+
+  m_capi->GET_VERSION(&capiversion_maj, &capiversion_min, &manufacturerversion_maj, &manufacturerversion_min);
+  str << "CAPI_version=" << capiversion_maj << '.' << capiversion_min << '(' << manufacturerversion_maj << '.' << manufacturerversion_min << ")\n";
+
+  m_capi->GET_PROFILE(&profile, 0);
+  InstalledControllers = profile.m_NumControllers;
+  if (InstalledControllers > 30)
+    InstalledControllers = 1;
+  str << "ISDN_controllers=" << InstalledControllers << '\n';
+  for (DWORD controller = 1; controller <= InstalledControllers; ++controller) {
+    m_capi->GET_PROFILE(&profile, controller);
+    str << "Controller#_" << controller << "_NumB=" << profile.m_NumBChannels << '\n';
+  }
+  return str;
+}
+
+
 unsigned OpalCapiEndPoint::OpenControllers()
 {
   {
@@ -699,9 +743,7 @@ void OpalCapiEndPoint::ProcessMessages(PThread &, INT)
 
       case CapiNoError :
         PTRACE(pMessage->header.m_Command == CAPI_DATA_B3 ? 6 : 4, "CAPI\tGot message " << *pMessage);
-        inUseFlag.Wait();
         ProcessMessage(*pMessage);
-        inUseFlag.Signal();
         break;
 
       default:
@@ -715,11 +757,20 @@ void OpalCapiEndPoint::ProcessMessages(PThread &, INT)
 
 bool OpalCapiEndPoint::IdToConnMap::Forward(const OpalCapiMessage & message, DWORD id)
 {
+  PSafePtr<OpalCapiConnection> connection;
+
+  m_mutex.Wait();
+
   iterator iter = find(id);
-  if (iter == end() || iter->second == NULL)
+  if (iter != end())
+    connection = iter->second;
+
+  m_mutex.Signal();
+
+  if (connection == NULL)
     return false;
 
-  iter->second->ProcessMessage(message);
+  connection->ProcessMessage(message);
   return true;
 }
 
@@ -744,15 +795,23 @@ void OpalCapiEndPoint::ProcessMessage(const OpalCapiMessage & message)
 
     case CAPI_INFO :
     case CAPI_CONNECT_ACTIVE :
-    case CAPI_DISCONNECT :
       m_plciToConnection.Forward(message, message.param.m_PLCI);
       break;
 
+    case CAPI_DISCONNECT :
+      if (!m_plciToConnection.Forward(message, message.param.m_PLCI) && message.header.m_Subcommand == CAPI_IND) {
+        // Is a DISCONNECT_IND and it was not forwarded to connection (already gone), then have to handle it here
+        OpalCapiMessage resp(CAPI_DISCONNECT, CAPI_RESP, sizeof(OpalCapiMessage::Params::DisconnectResp));
+        resp.param.disconnect_resp.m_PLCI = message.param.disconnect_ind.m_PLCI;
+        PutMessage(resp);
+      }
+      break;
+
     case CAPI_CONNECT_B3 :
     case CAPI_CONNECT_B3_ACTIVE :
     case CAPI_DATA_B3 :
     case CAPI_DISCONNECT_B3 :
-      m_ncciToConnection.Forward(message, message.param.m_NCCI);
+      m_plciToConnection.Forward(message, message.param.m_NCCI&0xffff);
       break;
   }
 }
@@ -812,7 +871,6 @@ OpalCapiConnection::OpalCapiConnection(OpalCall & call,
   , m_PLCI(0)
   , m_NCCI(0)
 {
-  synchronousOnRelease = false;
 }
 
 
@@ -827,11 +885,14 @@ PBoolean OpalCapiConnection::SetUpConnection()
   originating = true;
   SetPhase(SetUpPhase);
 
+  OnApplyStringOptions();
+
   OpalCapiMessage message(CAPI_CONNECT, CAPI_REQ, sizeof(OpalCapiMessage::Params::ConnectReq));
   message.param.connect_req.m_Controller = m_controller;
   message.param.connect_req.m_CIPValue = 16; // Telephony
   message.Add(0x80, m_calledPartyNumber); // Called party number
-  message.Add(0x00, 0x80, localPartyName.Left(localPartyName.FindSpan("0123456789#*"))); // Calling party number
+  PString number(m_stringOptions(OPAL_OPT_CALLING_PARTY_NUMBER, m_stringOptions(OPAL_OPT_CALLING_PARTY_NAME, localPartyName)));
+  message.Add(0x00, 0x80, number.Left(number.FindSpan("0123456789#*"))); // Calling party number
   message.AddEmpty(); // Called party subaddress
   message.AddEmpty(); // Calling party subaddress
 
@@ -877,7 +938,11 @@ PBoolean OpalCapiConnection::SetConnected()
   OpalCapiMessage message(CAPI_CONNECT, CAPI_RESP, sizeof(OpalCapiMessage::Params::ConnectResp));
   message.param.connect_resp.m_PLCI = m_PLCI;
   message.param.connect_resp.m_Reject = 0;
-  message.AddEmpty(); // B protocol
+  OpalCapiMessage::Bprotocol proto;
+  proto.m_B1protocol = 1; // Physical layer: 64 kbit/s bit-transparent operation with byte framing from the network
+  proto.m_B2protocol = 1; // Data link layer: Transparent
+  proto.m_B3protocol = 0; // Network layer: Transparent
+  message.Add(&proto, sizeof(proto)); // B protocol
   message.Add(localPartyName); // Connected party number
   message.AddEmpty(); // Connected party subaddress
   message.AddEmpty(); // Low Layer Compatibility
@@ -889,15 +954,13 @@ PBoolean OpalCapiConnection::SetConnected()
 void OpalCapiConnection::OnReleased()
 {
   if (m_NCCI != 0) {
-    m_endpoint.m_ncciToConnection.erase(m_NCCI);
-
     OpalCapiMessage message(CAPI_DISCONNECT_B3, CAPI_REQ, sizeof(OpalCapiMessage::Params::DisconnectB3Req));
     message.param.disconnect_b3_req.m_NCCI = m_NCCI;
     message.AddEmpty(); // Network Control Protocol Information
     PutMessage(message);
   }
 
-  if (m_NCCI != 0) {
+  if (m_PLCI != 0) {
     m_endpoint.m_plciToConnection.erase(m_PLCI);
 
     OpalCapiMessage message(CAPI_DISCONNECT, CAPI_REQ, sizeof(OpalCapiMessage::Params::DisconnectReq));
@@ -967,7 +1030,9 @@ void OpalCapiConnection::ProcessMessage(const OpalCapiMessage & message)
       PINDEX pos = sizeof(OpalCapiMessage::Header) + sizeof(message.param.connect_ind);
       message.Get(pos, NULL, 1, m_calledPartyNumber); // Called party address
       message.Get(pos, NULL, 2, remotePartyNumber); // Calling party address
+      remotePartyAddress = remotePartyNumber;
 
+      OnApplyStringOptions();
       if (OnIncomingConnection(0, NULL))
         ownerCall.OnSetUp(*this);
       break;
@@ -1010,7 +1075,6 @@ void OpalCapiConnection::ProcessMessage(const OpalCapiMessage & message)
       switch (message.param.connect_b3_conf.m_Info) {
         case 0 :
           m_NCCI = message.param.connect_b3_conf.m_NCCI;
-          m_endpoint.m_ncciToConnection[m_NCCI] = this;
           AutoStartMediaStreams();
           OnConnectedInternal();
           break;
@@ -1027,13 +1091,14 @@ void OpalCapiConnection::ProcessMessage(const OpalCapiMessage & message)
     case CAPICMD(CAPI_CONNECT_B3,CAPI_IND) :
     {
       m_NCCI = message.param.connect_b3_ind.m_NCCI;
-      m_endpoint.m_ncciToConnection[m_NCCI] = this;
 
       OpalCapiMessage resp(CAPI_CONNECT_B3, CAPI_RESP, sizeof(OpalCapiMessage::Params::ConnectB3Resp));
       resp.param.connect_b3_resp.m_NCCI = m_NCCI;
       resp.param.connect_b3_resp.m_Reject = 0; // Accept call
       resp.AddEmpty(); // Network Control Protocol Information
       PutMessage(resp);
+
+      AutoStartMediaStreams();
       break;
     }
 
@@ -1072,10 +1137,7 @@ void OpalCapiConnection::ProcessMessage(const OpalCapiMessage & message)
       resp.param.disconnect_b3_resp.m_NCCI = m_NCCI;
       PutMessage(resp);
 
-      if (m_NCCI != 0) {
-        m_endpoint.m_ncciToConnection.erase(m_NCCI);
-        m_NCCI = 0;
-      }
+      m_NCCI = 0;
       break;
     }
 
@@ -1182,12 +1244,10 @@ OpalCapiMediaStream::OpalCapiMediaStream(OpalCapiConnection & conn,
 }
 
 
-PBoolean OpalCapiMediaStream::Close()
+void OpalCapiMediaStream::InternalClose()
 {
   m_queue.Close();
-  bool ok = OpalMediaStream::Close();
   m_written.Signal();
-  return ok;
 }
 
 
