@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25368 $
- * $Author: rjongbloed $
- * $Date: 2011-03-21 01:34:07 -0500 (Mon, 21 Mar 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -964,7 +964,7 @@ PBoolean OpalLineMediaStream::Open()
 }
 
 
-PBoolean OpalLineMediaStream::Close()
+void OpalLineMediaStream::InternalClose()
 {
   if (directLineNumber != UINT_MAX)
     line.GetDevice().SetLineToLineDirect(line.GetLineNumber(), directLineNumber, false);
@@ -972,8 +972,6 @@ PBoolean OpalLineMediaStream::Close()
     line.StopReading();
   else
     line.StopWriting();
-
-  return OpalMediaStream::Close();
 }
 
 
