@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22444 $
- * $Author: rjongbloed $
- * $Date: 2009-04-20 18:49:06 -0500 (Mon, 20 Apr 2009) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifdef __GNUC__
@@ -41,7 +41,7 @@
 #include <ptclib/dtmf.h>
 
 
-PFACTORY_CREATE(PFactory<PPluginModuleManager>, OpalPluginLIDManager, "OpalPluginLIDManager", true);
+PFACTORY_CREATE_SINGLETON(PFactory<PPluginModuleManager>, OpalPluginLIDManager);
 
 ///////////////////////////////////////////////////////////////////////////////
 
