@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26649 $
- * $Author: rjongbloed $
- * $Date: 2011-11-13 19:59:25 -0600 (Sun, 13 Nov 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -60,7 +60,6 @@ OpalCall::OpalCall(OpalManager & mgr)
   , isEstablished(PFalse)
   , isClearing(PFalse)
   , callEndReason(OpalConnection::NumCallEndReasons)
-  , endCallSyncPoint(NULL)
 #if OPAL_HAS_MIXER
   , m_recordManager(NULL)
 #endif
@@ -103,7 +102,8 @@ void OpalCall::OnEstablishedCall()
 void OpalCall::SetCallEndReason(OpalConnection::CallEndReason reason)
 {
   // Only set reason if not already set to something
-  if (callEndReason == OpalConnection::NumCallEndReasons)
+  if (callEndReason == OpalConnection::NumCallEndReasons &&
+     (reason != OpalConnection::EndedByCallForwarded || connectionsActive.GetSize() <= 1))
     callEndReason = reason;
 }
 
@@ -112,49 +112,78 @@ void OpalCall::Clear(OpalConnection::CallEndReason reason, PSyncPoint * sync)
 {
   PTRACE(3, "Call\tClearing " << (sync != NULL ? "(sync) " : "") << *this << " reason=" << reason);
 
-  if (!LockReadWrite())
-    return;
+  {
+    PSafeLockReadWrite lock(*this);
+    if (!lock.IsLocked() || isClearing) {
+      if (sync != NULL)
+        sync->Signal();
+      return;
+    }
 
-  isClearing = PTrue;
+    isClearing = true;
 
-  SetCallEndReason(reason);
+    SetCallEndReason(reason);
 
-  if (sync != NULL && !connectionsActive.IsEmpty()) {
-    // only set the sync point if it is NULL
-    if (endCallSyncPoint == NULL)
-      endCallSyncPoint = sync;
-    else {
-      PAssertAlways("Can only have one thread doing ClearCallSynchronous");
+    if (sync != NULL)
+      m_endCallSyncPoint.push_back(sync);
+
+    switch (connectionsActive.GetSize()) {
+      case 0 :
+        break;
+
+      case 1 :
+        {
+          PSafePtr<OpalConnection> connection = connectionsActive.GetAt(0, PSafeReference);
+          if (connection != NULL)
+            connection->Release(reason);
+        }
+        break;
+
+      default :
+        // Release all but A-Party, it gets done in the B-Party OnReleased thread.
+        for (PINDEX i = 1; i < connectionsActive.GetSize(); ++i) {
+          PSafePtr<OpalConnection> connection = connectionsActive.GetAt(i, PSafeReference);
+          if (connection != NULL)
+            connection->Release(reason);
+        }
     }
   }
 
-  UnlockReadWrite();
-
+  // Outside of lock
   InternalOnClear();
+}
 
-  PSafePtr<OpalConnection> connection;
-  while (EnumerateConnections(connection, PSafeReadWrite))
-    connection->Release(reason);
+
+void OpalCall::InternalOnClear()
+{
+  if (!connectionsActive.IsEmpty())
+    return;
+
+  if (m_isCleared.TestAndSet(true))
+    return;
+
+  OnCleared();
+
+#if OPAL_HAS_MIXER
+  StopRecording();
+#endif
+
+  if (LockReadWrite()) {
+    while (!m_endCallSyncPoint.empty()) {
+      PTRACE(5, "Call\tSignalling end call.");
+      m_endCallSyncPoint.front()->Signal();
+      m_endCallSyncPoint.pop_front();
+    }
+    UnlockReadWrite();
+  }
+
+  manager.activeCalls.RemoveAt(GetToken());
 }
 
 
 void OpalCall::OnCleared()
 {
   manager.OnClearedCall(*this);
-
-#if OPAL_HAS_MIXER
-  StopRecording();
-#endif
-
-  if (!LockReadWrite())
-    return;
-
-  if (endCallSyncPoint != NULL) {
-    endCallSyncPoint->Signal();
-    endCallSyncPoint = NULL;
-  }
-
-  UnlockReadWrite();
 }
 
 
@@ -601,7 +630,6 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
 
     sinkStream = otherConnection->OpenMediaStream(sinkFormat, sessionID, false);
     if (sinkStream != NULL) {
-      startedOne = true;
       if (!sourceStream.SetSafetyMode(PSafeReadOnly))
         return false;
 
@@ -613,8 +641,12 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
           return false;
       }
 
-      patch->AddSink(sinkStream);
       sourceStream.SetSafetyMode(PSafeReference);
+
+      if (patch->AddSink(sinkStream))
+        startedOne = true;
+      else
+        sinkStream->Close();
     }
   }
 
@@ -736,22 +768,13 @@ void OpalCall::OnReleased(OpalConnection & connection)
   {
     PSafePtr<OpalConnection> last = connectionsActive.GetAt(0, PSafeReference);
     if (last != NULL)
-      last->Release(connection.GetCallEndReason());
+      last->Release(connection.GetCallEndReason(), true);
   }
 
   InternalOnClear();
 }
 
 
-void OpalCall::InternalOnClear()
-{
-  if (connectionsActive.IsEmpty() && manager.activeCalls.Contains(GetToken())) {
-    OnCleared();
-    manager.activeCalls.RemoveAt(GetToken());
-  }
-}
-
-
 void OpalCall::OnHold(OpalConnection & /*connection*/,
                       bool /*fromRemote*/, 
                       bool /*onHold*/)
