@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26939 $
- * $Author: rjongbloed $
- * $Date: 2012-02-07 01:17:49 -0600 (Tue, 07 Feb 2012) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -221,7 +221,6 @@ OpalConnection::OpalConnection(OpalCall & call,
   , displayName(ep.GetDefaultDisplayName())
   , remotePartyName(token)
   , callEndReason(NumCallEndReasons)
-  , synchronousOnRelease(true)
   , silenceDetector(NULL)
 #if OPAL_AEC
   , echoCanceler(NULL)
@@ -395,6 +394,8 @@ void OpalConnection::SetCallEndReasonText(CallEndReasonCodes reasonCode, const P
 
 void OpalConnection::SetCallEndReason(CallEndReason reason)
 {
+  PWaitAndSignal mutex(m_phaseMutex);
+
   // Only set reason if not already set to something
   if (callEndReason == NumCallEndReasons) {
     PTRACE(3, "OpalCon\tCall end reason for " << *this << " set to " << reason);
@@ -404,19 +405,25 @@ void OpalConnection::SetCallEndReason(CallEndReason reason)
 }
 
 
-void OpalConnection::ClearCall(CallEndReason reason)
+void OpalConnection::ClearCall(CallEndReason reason, PSyncPoint * sync)
 {
-  // Now set reason for the connection close
   SetCallEndReason(reason);
-  ownerCall.Clear(reason);
+  ownerCall.Clear(reason, sync);
 }
 
 
 void OpalConnection::ClearCallSynchronous(PSyncPoint * sync, CallEndReason reason)
 {
-  // Now set reason for the connection close
   SetCallEndReason(reason);
-  ownerCall.Clear(reason, sync);
+
+  PSyncPoint syncPoint;
+  if (sync == NULL)
+    sync = &syncPoint;
+
+  ClearCall(reason, sync);
+
+  PTRACE(5, "OpalCon\tSynchronous wait for " << *this);
+  sync->Wait();
 }
 
 
@@ -427,7 +434,7 @@ bool OpalConnection::TransferConnection(const PString & PTRACE_PARAM(remoteParty
 }
 
 
-void OpalConnection::Release(CallEndReason reason)
+void OpalConnection::Release(CallEndReason reason, bool synchronous)
 {
   {
     PWaitAndSignal mutex(m_phaseMutex);
@@ -436,21 +443,11 @@ void OpalConnection::Release(CallEndReason reason)
       return;
     }
     SetPhase(ReleasingPhase);
+    SetCallEndReason(reason);
   }
 
-  if (synchronousOnRelease) {
-    if (!LockReadWrite()) {
-      PTRACE(2, "OpalCon\tAlready released " << *this);
-      return;
-    }
-
-    PTRACE(3, "OpalCon\tReleasing " << *this);
-
-    // Now set reason for the connection close
-    SetCallEndReason(reason);
-
-    UnlockReadWrite();
-
+  if (synchronous) {
+    PTRACE(3, "OpalCon\tReleasing synchronously " << *this);
     OnReleased();
     return;
   }
@@ -459,21 +456,15 @@ void OpalConnection::Release(CallEndReason reason)
 
   // Add a reference for the thread we are about to start
   SafeReference();
-  PThread::Create(PCREATE_NOTIFIER(OnReleaseThreadMain), reason.AsInteger(),
+  PThread::Create(PCREATE_NOTIFIER(OnReleaseThreadMain), 0,
                   PThread::AutoDeleteThread,
                   PThread::NormalPriority,
                   "OnRelease");
 }
 
 
-void OpalConnection::OnReleaseThreadMain(PThread &, INT reason)
+void OpalConnection::OnReleaseThreadMain(PThread &, INT)
 {
-  if (LockReadWrite()) {
-    // Now set reason for the connection close
-    SetCallEndReason(CallEndReason(reason));
-    UnlockReadWrite();
-  }
-
   OnReleased();
 
   PTRACE(4, "OpalCon\tOnRelease thread completed for " << *this);
@@ -705,38 +696,45 @@ void OpalConnection::AutoStartMediaStreams(bool force)
 
 
 #if OPAL_FAX
-bool OpalConnection::SwitchFaxMediaStreams(bool enableFax)
+bool OpalConnection::SwitchFaxMediaStreams(bool toT38)
 {
   if (m_faxMediaStreamsSwitchState != e_NotSwitchingFaxMediaStreams) {
     PTRACE(2, "OpalCon\tNested call to SwitchFaxMediaStreams on " << *this);
     return false;
   }
 
-  PTRACE(3, "OpalCon\tSwitchFaxMediaStreams to " << (enableFax ? "fax" : "audio") << " on " << *this);
-  OpalMediaFormat format = enableFax ? OpalT38 : OpalG711uLaw;
+  PTRACE(3, "OpalCon\tSwitchFaxMediaStreams to " << (toT38 ? "T.38" : "audio") << " on " << *this);
+  OpalMediaFormat format = toT38 ? OpalT38 : OpalG711uLaw;
   if (!ownerCall.OpenSourceMediaStreams(*this, format.GetMediaType(), 1, format))
     return false;
 
-  m_faxMediaStreamsSwitchState = enableFax ? e_SwitchingToFaxMediaStreams : e_SwitchingFromFaxMediaStreams;
+  m_faxMediaStreamsSwitchState = toT38 ? e_SwitchingToFaxMediaStreams : e_SwitchingFromFaxMediaStreams;
   return true;
 }
 
 
-void OpalConnection::OnSwitchedFaxMediaStreams(bool enabledFax)
+void OpalConnection::OnSwitchedFaxMediaStreams(bool toT38, bool success)
 {
   if (m_faxMediaStreamsSwitchState != e_NotSwitchingFaxMediaStreams) {
     PTRACE(3, "OpalCon\tSwitch of media streams to "
-           << (m_faxMediaStreamsSwitchState == e_SwitchingToFaxMediaStreams ? "fax" : "audio") << ' '
-           << (enabledFax != (m_faxMediaStreamsSwitchState == e_SwitchingToFaxMediaStreams) ? "failed" : "succeeded")
+           << (toT38 ? "T.38" : "audio") << ' '
+           << (success ? "succeeded" : "failed")
            << " on " << *this);
 
     m_faxMediaStreamsSwitchState = e_NotSwitchingFaxMediaStreams;
 
     PSafePtr<OpalConnection> other = GetOtherPartyConnection();
     if (other != NULL)
-      other->OnSwitchedFaxMediaStreams(enabledFax);
+      other->OnSwitchedFaxMediaStreams(toT38, success);
   }
 }
+
+
+bool OpalConnection::OnSwitchingFaxMediaStreams(bool toT38)
+{
+  PTRACE(3, "OpalCon\tRemote switch of media streams to " << (toT38 ? "T.38" : "audio") << " on " << *this);
+  return !toT38;
+}
 #endif
 
 
@@ -1183,6 +1181,24 @@ PBoolean OpalConnection::SetAudioVolume(PBoolean /*source*/, unsigned /*percenta
 }
 
 
+PBoolean OpalConnection::GetAudioVolume(PBoolean /*source*/, unsigned & /*percentage*/)
+{
+  return PFalse;
+}
+
+
+bool OpalConnection::SetAudioMute(bool /*source*/, bool /*mute*/)
+{
+  return false;
+}
+
+
+bool OpalConnection::GetAudioMute(bool /*source*/, bool & /*mute*/)
+{
+  return false;
+}
+
+
 unsigned OpalConnection::GetAudioSignalLevel(PBoolean /*source*/)
 {
     return UINT_MAX;
