@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25448 $
- * $Author: rjongbloed $
- * $Date: 2011-03-29 19:56:36 -0500 (Tue, 29 Mar 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -515,13 +515,19 @@ PBoolean OpalEndPoint::ClearCall(const PString & token,
 
 
 PBoolean OpalEndPoint::ClearCallSynchronous(const PString & token,
-                                        OpalConnection::CallEndReason reason,
-                                        PSyncPoint * sync)
+                                            OpalConnection::CallEndReason reason,
+                                            PSyncPoint * sync)
 {
   PSyncPoint syncPoint;
   if (sync == NULL)
     sync = &syncPoint;
-  return manager.ClearCall(token, reason, sync);
+
+  if (!ClearCall(token, reason, sync))
+    return false;
+
+  PTRACE(5, "OpalCon\tSynchronous wait for " << token);
+  sync->Wait();
+  return true;
 }
 
 
