@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26930 $
- * $Author: rjongbloed $
- * $Date: 2012-02-01 18:50:09 -0600 (Wed, 01 Feb 2012) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -442,13 +442,10 @@ PBoolean OpalIVRMediaStream::Open()
 }
 
 
-PBoolean OpalIVRMediaStream::Close()
+void OpalIVRMediaStream::InternalClose()
 {
   if (connection.IsReleased())
-    return OpalRawMediaStream::Close();
-
-  // Pause the vxmlSession here ...
-  return OpalMediaStream::Close();
+    OpalRawMediaStream::InternalClose();
 }
 
 
