@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26939 $
- * $Author: rjongbloed $
- * $Date: 2012-02-07 01:17:49 -0600 (Tue, 07 Feb 2012) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -49,6 +49,8 @@ OpalLocalEndPoint::OpalLocalEndPoint(OpalManager & mgr, const char * prefix)
   : OpalEndPoint(mgr, prefix, CanTerminateCall)
   , m_deferredAlerting(false)
   , m_deferredAnswer(false)
+  , m_defaultAudioSynchronicity(e_Synchronous)
+  , m_defaultVideoSourceSynchronicity(e_Synchronous)
 {
   PTRACE(3, "LocalEP\tCreated endpoint.");
 }
@@ -85,6 +87,12 @@ OpalLocalConnection * OpalLocalEndPoint::CreateConnection(OpalCall & call,
 }
 
 
+bool OpalLocalEndPoint::OnOutgoingSetUp(const OpalLocalConnection & /*connection*/)
+{
+  return true;
+}
+
+
 bool OpalLocalEndPoint::OnOutgoingCall(const OpalLocalConnection & /*connection*/)
 {
   return true;
@@ -187,9 +195,17 @@ bool OpalLocalEndPoint::OnWriteMediaData(const OpalLocalConnection & /*connectio
 }
 
 
-bool OpalLocalEndPoint::IsSynchronous() const
+OpalLocalEndPoint::Synchronicity
+OpalLocalEndPoint::GetSynchronicity(const OpalMediaFormat & mediaFormat,
+                                    bool isSource) const
 {
-  return true;
+  if (mediaFormat.GetMediaType() == OpalMediaType::Audio())
+    return m_defaultAudioSynchronicity;
+
+  if (isSource && mediaFormat.GetMediaType() == OpalMediaType::Audio())
+    return m_defaultVideoSourceSynchronicity;
+
+  return e_Asynchronous;
 }
 
 
@@ -197,13 +213,13 @@ bool OpalLocalEndPoint::IsSynchronous() const
 
 OpalLocalConnection::OpalLocalConnection(OpalCall & call,
                                 OpalLocalEndPoint & ep,
-                                             void * /*userData*/,
+                                             void * userData,
                                            unsigned options,
                     OpalConnection::StringOptions * stringOptions,
                                                char tokenPrefix)
   : OpalConnection(call, ep, ep.GetManager().GetNextToken(tokenPrefix), options, stringOptions)
   , endpoint(ep)
-  , userData(NULL)
+  , m_userData(userData)
 {
 #if OPAL_PTLIB_DTMF
   m_sendInBandDTMF = m_detectInBandDTMF = false;
@@ -243,7 +259,7 @@ PBoolean OpalLocalConnection::SetUpConnection()
     }
 
     PTRACE(3, "LocalCon\tOutgoing call routed to " << ownerCall.GetPartyB() << " for " << *this);
-    if (!ownerCall.OnSetUp(*this)) {
+    if (!OnOutgoingSetUp() || !ownerCall.OnSetUp(*this)) {
       Release(EndedByNoAccept);
       return false;
     }
@@ -259,7 +275,7 @@ PBoolean OpalLocalConnection::SetUpConnection()
 
     OnApplyStringOptions();
 
-    if (!endpoint.OnIncomingCall(*this)) {
+    if (!OnIncoming()) {
       Release(EndedByLocalBusy);
       return false;
     }
@@ -295,7 +311,8 @@ OpalMediaStream * OpalLocalConnection::CreateMediaStream(const OpalMediaFormat &
                                                          unsigned sessionID,
                                                          PBoolean isSource)
 {
-  return new OpalLocalMediaStream(*this, mediaFormat, sessionID, isSource, endpoint.IsSynchronous());
+  return new OpalLocalMediaStream(*this, mediaFormat, sessionID, isSource,
+                                  endpoint.GetSynchronicity(mediaFormat, isSource));
 }
 
 
@@ -322,6 +339,24 @@ bool OpalLocalConnection::SendUserInputString(const PString & value)
 }
 
 
+bool OpalLocalConnection::OnOutgoingSetUp()
+{
+  return endpoint.OnOutgoingSetUp(*this);
+}
+
+
+bool OpalLocalConnection::OnOutgoing()
+{
+  return endpoint.OnOutgoingCall(*this);
+}
+
+
+bool OpalLocalConnection::OnIncoming()
+{
+  return endpoint.OnIncomingCall(*this);
+}
+
+
 void OpalLocalConnection::AlertingIncoming()
 {
   if (LockReadWrite()) {
@@ -351,10 +386,10 @@ OpalLocalMediaStream::OpalLocalMediaStream(OpalLocalConnection & connection,
                                            const OpalMediaFormat & mediaFormat,
                                            unsigned sessionID,
                                            bool isSource,
-                                           bool isSynchronous)
+                                           OpalLocalEndPoint::Synchronicity synchronicity)
   : OpalMediaStream(connection, mediaFormat, sessionID, isSource)
   , OpalMediaStreamPacing(mediaFormat)
-  , m_isSynchronous(isSynchronous)
+  , m_synchronicity(synchronicity)
 {
 }
 
@@ -394,7 +429,7 @@ PBoolean OpalLocalMediaStream::ReadData(BYTE * data, PINDEX size, PINDEX & lengt
   if (!ep.OnReadMediaData(conn, *this, data, size, length))
     return false;
 
-  if (!m_isSynchronous)
+  if (m_synchronicity == OpalLocalEndPoint::e_SimulateSyncronous)
     Pace(true, size, marker);
   return true;
 }
@@ -407,7 +442,7 @@ PBoolean OpalLocalMediaStream::WriteData(const BYTE * data, PINDEX length, PINDE
   if (!ep.OnWriteMediaData(conn, *this, data, length, written))
     return false;
 
-  if (!m_isSynchronous)
+  if (m_synchronicity == OpalLocalEndPoint::e_SimulateSyncronous)
     Pace(false, written, marker);
   return true;
 }
@@ -415,7 +450,7 @@ PBoolean OpalLocalMediaStream::WriteData(const BYTE * data, PINDEX length, PINDE
 
 PBoolean OpalLocalMediaStream::IsSynchronous() const
 {
-  return m_isSynchronous;
+  return m_synchronicity != OpalLocalEndPoint::e_Asynchronous;
 }
 
 
