@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26825 $
- * $Author: rjongbloed $
- * $Date: 2011-12-21 19:02:59 -0600 (Wed, 21 Dec 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -82,7 +82,7 @@ static const char * const DefaultMediaFormatOrder[] = {
   OPAL_H264_MODE1,  // SIP version, packetisation mode 1
   OPAL_H264_MODE0,  // SIP version, packetisation mode 0
   OPAL_MPEG4,
-  OPAL_H263,
+  OPAL_H263 "*",
   OPAL_H261
 };
 
@@ -207,7 +207,7 @@ OpalManager::OpalManager()
   , defaultUserName(PProcess::Current().GetUserName())
   , defaultDisplayName(defaultUserName)
   , m_defaultMediaTypeOfService(0xb8)  // New DiffServ value for Expidited Forwarding as per RFC3246
-  , rtpPayloadSizeMax(576-20-16-12) // Max safe MTU size (576 bytes as per RFC879) minus IP, UDP an RTP headers
+  , rtpPayloadSizeMax(1400) // RFC879 recommends 576 bytes, but that is ancient history, 99.999% of the time 1400+ bytes is used.
   , rtpPacketSizeMax(2048)
   , minAudioJitterDelay(50)  // milliseconds
   , maxAudioJitterDelay(250) // milliseconds
@@ -494,20 +494,14 @@ PBoolean OpalManager::ClearCall(const PString & token,
     The real work is done in the OpalGarbageCollector thread.
    */
 
-  {
-    // Find the call by token, callid or conferenceid
-    PSafePtr<OpalCall> call = activeCalls.FindWithLock(token, PSafeReference);
-    if (call == NULL) {
-      PTRACE(2, "OpalMan\tCould not find/lock call token \"" << token << '"');
-      return false;
-    }
-
-    call->Clear(reason, sync);
+  // Find the call by token, callid or conferenceid
+  PSafePtr<OpalCall> call = activeCalls.FindWithLock(token, PSafeReference);
+  if (call == NULL) {
+    PTRACE(2, "OpalMan\tCould not find/lock call token \"" << token << '"');
+    return false;
   }
 
-  if (sync != NULL)
-    sync->Wait();
-
+  call->Clear(reason, sync);
   return true;
 }
 
@@ -516,7 +510,11 @@ PBoolean OpalManager::ClearCallSynchronous(const PString & token,
                                        OpalConnection::CallEndReason reason)
 {
   PSyncPoint wait;
-  return ClearCall(token, reason, &wait);
+  if (!ClearCall(token, reason, &wait))
+    return false;
+
+  wait.Wait();
+  return false;
 }
 
 
@@ -1009,7 +1007,8 @@ PBoolean OpalManager::CreateVideoOutputDevice(const OpalConnection & connection,
   PINDEX start = args.deviceName.Find("TITLE=\"");
   if (start != P_MAX_INDEX) {
     start += 7;
-    args.deviceName.Splice(preview ? "Local Preview" : connection.GetRemotePartyName(), start, args.deviceName.Find('"', start)-start);
+    static PConstString const LocalPreview("Local Preview");
+    args.deviceName.Splice(preview ? LocalPreview : connection.GetRemotePartyName(), start, args.deviceName.Find('"', start)-start);
   }
 
   autoDelete = true;
@@ -1023,17 +1022,10 @@ PBoolean OpalManager::CreateVideoOutputDevice(const OpalConnection & connection,
 OpalMediaPatch * OpalManager::CreateMediaPatch(OpalMediaStream & source,
                                                PBoolean requiresPatchThread)
 {
-  if (requiresPatchThread) {
+  if (requiresPatchThread)
     return new OpalMediaPatch(source);
-  } else {
+  else
     return new OpalPassiveMediaPatch(source);
-  }
-}
-
-
-void OpalManager::DestroyMediaPatch(OpalMediaPatch * patch)
-{
-  delete patch;
 }
 
 
