@@ -23,9 +23,9 @@
  *
  * Contributor(s): ________________________________________.
  *
- * $Revision: 26818 $
- * $Author: rjongbloed $
- * $Date: 2011-12-20 17:29:16 -0600 (Tue, 20 Dec 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -72,7 +72,6 @@ OpalMediaStream::OpalMediaStream(OpalConnection & conn, const OpalMediaFormat &
   , timestamp(0)
   , marker(true)
   , mismatchedPayloadTypes(0)
-  , mediaPatch(NULL)
   , m_payloadType(mediaFormat.GetPayloadType())
   , m_frameTime(mediaFormat.GetFrameTime())
   , m_frameSize(mediaFormat.GetFrameSize())
@@ -92,12 +91,9 @@ OpalMediaStream::~OpalMediaStream()
 
 void OpalMediaStream::PrintOn(ostream & strm) const
 {
-  strm << GetClass() << '-';
-  if (isSource)
-    strm << "Source";
-  else
-    strm << "Sink";
-  strm << '-' << mediaFormat;
+  strm << GetClass() << '[' << this << "]-"
+       << (isSource ? "Source" : "Sink")
+       << '-' << mediaFormat;
 }
 
 
@@ -109,12 +105,11 @@ OpalMediaFormat OpalMediaStream::GetMediaFormat() const
 
 bool OpalMediaStream::UpdateMediaFormat(const OpalMediaFormat & newMediaFormat)
 {
-  PSafeLockReadWrite safeLock(*this);
-  if (!safeLock.IsLocked())
-    return false;
+  // We make referenced copy of pointer so can't be deleted out from under us
+  PatchPtr mediaPatch = m_mediaPatch;
 
-  // If we are source, then update the sink side, and vice versa
-  return mediaPatch != NULL && mediaPatch->UpdateMediaFormat(newMediaFormat);
+  return mediaPatch != NULL ? mediaPatch->UpdateMediaFormat(newMediaFormat)
+                            : InternalUpdateMediaFormat(newMediaFormat);
 }
 
 
@@ -133,9 +128,8 @@ bool OpalMediaStream::InternalUpdateMediaFormat(const OpalMediaFormat & newMedia
 
 PBoolean OpalMediaStream::ExecuteCommand(const OpalMediaCommand & command)
 {
-  PSafeLockReadOnly safeLock(*this);
-  if (!safeLock.IsLocked())
-    return false;
+  // We make referenced copy of pointer so can't be deleted out from under us
+  PatchPtr mediaPatch = m_mediaPatch;
 
   if (mediaPatch == NULL)
     return false;
@@ -164,9 +158,8 @@ PBoolean OpalMediaStream::Start()
   if (!Open())
     return false;
 
-  PSafeLockReadOnly safeLock(*this);
-  if (!safeLock.IsLocked())
-    return false;
+  // We make referenced copy of pointer so can't be deleted out from under us
+  PatchPtr mediaPatch = m_mediaPatch;
 
   if (mediaPatch == NULL)
     return false;
@@ -194,33 +187,22 @@ PBoolean OpalMediaStream::Close()
 
   // Allow for race condition where it is closed in another thread during the above wait
   if (!isOpen) {
+    PTRACE(4, "Media\tAlready closed stream " << *this);
     UnlockReadWrite();
     return false;
   }
 
   isOpen = false;
 
+  InternalClose();
+
   UnlockReadWrite();
 
   connection.OnClosedMediaStream(*this);
-
-  if (mediaPatch != NULL && LockReadWrite()) {
-    PTRACE(4, "Media\tDisconnecting " << *this << " from patch thread " << *mediaPatch);
-    OpalMediaPatch * patch = mediaPatch;
-    mediaPatch = NULL;
-
-    if (IsSink())
-      patch->RemoveSink(this);
-	
-    UnlockReadWrite();
-
-    if (IsSource()) {
-      patch->Close();
-      connection.GetEndPoint().GetManager().DestroyMediaPatch(patch);
-    }
-  }
-
+  SetPatch(NULL);
   connection.RemoveMediaStream(*this);
+
+  PTRACE(5, "Media\tClosed stream " << *this);
   return true;
 }
 
@@ -393,14 +375,11 @@ PBoolean OpalMediaStream::WriteData(const BYTE * buffer, PINDEX length, PINDEX &
 
 PBoolean OpalMediaStream::PushPacket(RTP_DataFrame & packet)
 {
-  OpalMediaPatch * patch = NULL;
-  if (LockReadOnly()) {
-    patch = mediaPatch;
-    UnlockReadOnly();
-  }
+  // We make referenced copy of pointer so can't be deleted out from under us
+  PatchPtr mediaPatch = m_mediaPatch;
 
   // OpalMediaPatch::PushFrame() might block, do outside of mutex
-  return patch != NULL && patch->PushFrame(packet);
+  return mediaPatch != NULL && mediaPatch->PushFrame(packet);
 }
 
 
@@ -435,14 +414,17 @@ bool OpalMediaStream::EnableJitterBuffer(bool) const
 
 bool OpalMediaStream::SetPaused(bool pause, bool fromPatch)
 {
-  PSafeLockReadWrite safeLock(*this);
-  if (!safeLock.IsLocked())
-    return false;
+  // We make referenced copy of pointer so can't be deleted out from under us
+  PatchPtr mediaPatch = m_mediaPatch;
 
   // If we are source, then update the sink side, and vice versa
   if (!fromPatch && mediaPatch != NULL)
     return mediaPatch->SetPaused(pause);
 
+  PSafeLockReadWrite mutex(*this);
+  if (!mutex.IsLocked())
+    return false;
+
   if (m_paused == pause)
     return false;
 
@@ -456,6 +438,8 @@ bool OpalMediaStream::SetPaused(bool pause, bool fromPatch)
 
 PBoolean OpalMediaStream::SetPatch(OpalMediaPatch * patch)
 {
+  PatchPtr mediaPatch = m_mediaPatch.Set(patch);
+
 #if PTRACING
   if (PTrace::CanTrace(4) && (patch != NULL || mediaPatch != NULL)) {
     ostream & trace = PTrace::Begin(4, __FILE__, __LINE__);
@@ -469,23 +453,11 @@ PBoolean OpalMediaStream::SetPatch(OpalMediaPatch * patch)
   }
 #endif
 
-  PSafeLockReadWrite safeLock(*this);
-  if (!safeLock.IsLocked())
-    return false;
-
-  if (mediaPatch == NULL) {
-    mediaPatch = patch;
-    return true;
-  }
-
-  OpalMediaPatch * oldPatch = mediaPatch;
-  mediaPatch = patch;
-
-  if (IsSink())
-    oldPatch->RemoveSink(this);
-  else {
-    oldPatch->Close();
-    connection.GetEndPoint().GetManager().DestroyMediaPatch(oldPatch);
+  if (mediaPatch != NULL) {
+    if (IsSink())
+      mediaPatch->RemoveSink(this);
+    else
+      mediaPatch->Close();
   }
 
   return true;
@@ -494,24 +466,30 @@ PBoolean OpalMediaStream::SetPatch(OpalMediaPatch * patch)
 
 void OpalMediaStream::AddFilter(const PNotifier & filter, const OpalMediaFormat & stage) const
 {
-  PSafeLockReadOnly safeLock(*this);
-  if (safeLock.IsLocked() && mediaPatch != NULL)
+  // We make referenced copy of pointer so can't be deleted out from under us
+  PatchPtr mediaPatch = m_mediaPatch;
+
+  if (mediaPatch != NULL)
     mediaPatch->AddFilter(filter, stage);
 }
 
 
 PBoolean OpalMediaStream::RemoveFilter(const PNotifier & filter, const OpalMediaFormat & stage) const
 {
-  PSafeLockReadOnly safeLock(*this);
-  return safeLock.IsLocked() && mediaPatch != NULL && mediaPatch->RemoveFilter(filter, stage);
+  // We make referenced copy of pointer so can't be deleted out from under us
+  PatchPtr mediaPatch = m_mediaPatch;
+
+  return mediaPatch != NULL && mediaPatch->RemoveFilter(filter, stage);
 }
 
 
 #if OPAL_STATISTICS
 void OpalMediaStream::GetStatistics(OpalMediaStatistics & statistics, bool fromPatch) const
 {
-  PSafeLockReadOnly safeLock(*this);
-  if (safeLock.IsLocked() && mediaPatch != NULL && !fromPatch)
+  // We make referenced copy of pointer so can't be deleted out from under us
+  PatchPtr mediaPatch = m_mediaPatch;
+
+  if (mediaPatch != NULL && !fromPatch)
     mediaPatch->GetStatistics(statistics, IsSink());
 }
 #endif
@@ -519,7 +497,11 @@ void OpalMediaStream::GetStatistics(OpalMediaStatistics & statistics, bool fromP
 
 void OpalMediaStream::OnStartMediaPatch() 
 { 
-  connection.OnStartMediaPatch(*mediaPatch);
+  // We make referenced copy of pointer so can't be deleted out from under us
+  PatchPtr mediaPatch = m_mediaPatch;
+
+  if (mediaPatch != NULL)
+    connection.OnStartMediaPatch(*mediaPatch);
 }
 
 
@@ -536,6 +518,10 @@ OpalMediaStreamPacing::OpalMediaStreamPacing(const OpalMediaFormat & mediaFormat
   , m_frameTime(mediaFormat.GetFrameTime())
   , m_frameSize(mediaFormat.GetFrameSize())
   , m_timeUnits(mediaFormat.GetTimeUnits())
+  /* No more than 1 second of "slip", reset timing if thread does not execute
+     the timeout for a whole second. Prevents too much bunching up of data
+     in "bad" conditions, that really should not happen. */
+  , m_delay(1000)
 {
   PAssert(!(m_isAudio && m_frameSize == 0), PInvalidParameter);
 }
@@ -614,6 +600,19 @@ PBoolean OpalNullMediaStream::WriteData(const BYTE * /*buffer*/, PINDEX length,
 }
 
 
+bool OpalNullMediaStream::SetPaused(bool pause, bool fromPatch)
+{
+  if (!OpalMediaStream::SetPaused(pause, fromPatch))
+    return false;
+
+  // If coming out of pause, restart pacing delay
+  if (!pause)
+    m_delay.Restart();
+
+  return true;
+}
+
+
 PBoolean OpalNullMediaStream::RequiresPatchThread() const
 {
   return m_requiresPatchThread;
@@ -670,17 +669,11 @@ PBoolean OpalRTPMediaStream::Open()
 }
 
 
-PBoolean OpalRTPMediaStream::Close()
+void OpalRTPMediaStream::InternalClose()
 {
-  if (!isOpen)
-    return false;
-    
-  PTRACE(3, "Media\tClosing RTP for " << *this);
-
   // Break any I/O blocks and wait for the thread that uses this object to
   // terminate before we allow it to be deleted.
   rtpSession.Close(IsSource());
-  return OpalMediaStream::Close();
 }
 
 
@@ -703,6 +696,9 @@ bool OpalRTPMediaStream::SetPaused(bool pause, bool fromPatch)
 
 PBoolean OpalRTPMediaStream::ReadPacket(RTP_DataFrame & packet)
 {
+  if (!IsOpen())
+    return false;
+
   if (IsSink()) {
     PTRACE(1, "Media\tTried to read from sink media stream");
     return false;
@@ -718,6 +714,9 @@ PBoolean OpalRTPMediaStream::ReadPacket(RTP_DataFrame & packet)
 
 PBoolean OpalRTPMediaStream::WritePacket(RTP_DataFrame & packet)
 {
+  if (!IsOpen())
+    return false;
+
   if (IsSource()) {
     PTRACE(1, "Media\tTried to write to source media stream");
     return false;
@@ -755,7 +754,7 @@ PBoolean OpalRTPMediaStream::RequiresPatchThread() const
 
 bool OpalRTPMediaStream::EnableJitterBuffer(bool enab) const
 {
-  if (IsSink() || !RequiresPatchThread())
+  if (!IsOpen() || IsSink() || !RequiresPatchThread())
     return false;
 
   unsigned minJitter, maxJitter;
@@ -775,7 +774,7 @@ bool OpalRTPMediaStream::EnableJitterBuffer(bool enab) const
 
 PBoolean OpalRTPMediaStream::SetPatch(OpalMediaPatch * patch)
 {
-  if (!isOpen || IsSink() || mediaPatch == NULL)
+  if (!isOpen || IsSink())
     return OpalMediaStream::SetPatch(patch);
 
   rtpSession.Close(true);
@@ -938,15 +937,10 @@ bool OpalRawMediaStream::SetChannel(PChannel * chan, bool autoDelete)
 }
 
 
-PBoolean OpalRawMediaStream::Close()
+void OpalRawMediaStream::InternalClose()
 {
-  if (!isOpen)
-    return false;
-
   if (m_channel != NULL)
     m_channel->Close();
-
-  return OpalMediaStream::Close();
 }
 
 
@@ -1243,18 +1237,21 @@ PBoolean OpalVideoMediaStream::Open()
 }
 
 
-PBoolean OpalVideoMediaStream::Close()
+void OpalVideoMediaStream::InternalClose()
 {
-  if (!OpalMediaStream::Close())
-    return false;
+  if (m_inputDevice != NULL) {
+    if (m_autoDeleteInput)
+      m_inputDevice->Close();
+    else
+      m_inputDevice->Stop();
+  }
 
-  if (m_inputDevice != NULL)
-    m_inputDevice->Stop();
-
-  if (m_outputDevice != NULL)
-    m_outputDevice->Stop();
-
-  return true;
+  if (m_outputDevice != NULL) {
+    if (m_autoDeleteOutput)
+      m_outputDevice->Close();
+    else
+      m_outputDevice->Stop();
+  }
 }
 
 
@@ -1435,13 +1432,9 @@ PBoolean OpalUDPMediaStream::IsSynchronous() const
 }
 
 
-PBoolean OpalUDPMediaStream::Close()
+void OpalUDPMediaStream::InternalClose()
 {
-  if (!isOpen)
-    return false;
-    
   udpTransport.Close();
-  return OpalMediaStream::Close();
 }
 
 
