@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21833 $
- * $Author: rjongbloed $
- * $Date: 2008-12-17 23:15:31 -0600 (Wed, 17 Dec 2008) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -39,15 +39,13 @@
 #include <opal/mediatype.h>
 #include <opal/rtpconn.h>
 #include <opal/call.h>
+#include <opal/endpoint.h>
+#include <opal/manager.h>
 
 #if OPAL_H323
 #include <h323/channels.h>
 #endif
 
-namespace PWLibStupidLinkerHacks {
-  int mediaTypeLoader;
-}; // namespace PWLibStupidLinkerHacks
-
 
 OPAL_INSTANTIATE_MEDIATYPE(audio, OpalAudioMediaType);
 
@@ -57,16 +55,30 @@ OPAL_INSTANTIATE_MEDIATYPE(video, OpalVideoMediaType);
 
 OPAL_INSTANTIATE_SIMPLE_MEDIATYPE_NO_SDP(userinput); 
 
-///////////////////////////////////////////////////////////////////////////////
 
-ostream & operator << (ostream & strm, const OpalMediaType & mediaType)
-{ mediaType.PrintOn(strm); return strm; }
+typedef std::map<unsigned, OpalMediaTypeDefinition *> SessionIDToMediaTypeMap_T;
+static SessionIDToMediaTypeMap_T & GetSessionIDToMediaTypeMap()
+{
+  static SessionIDToMediaTypeMap_T map;
+  return map;
+}
+
+static PMutex & GetMapMutex()
+{
+  static PMutex mutex;
+  return mutex;
+}
+
+
+
+///////////////////////////////////////////////////////////////////////////////
 
 const OpalMediaType & OpalMediaType::Audio()     { static const OpalMediaType type = "audio";     return type; }
 const OpalMediaType & OpalMediaType::Video()     { static const OpalMediaType type = "video";     return type; }
 const OpalMediaType & OpalMediaType::Fax()       { static const OpalMediaType type = "fax";       return type; };
 const OpalMediaType & OpalMediaType::UserInput() { static const OpalMediaType type = "userinput"; return type; };
 
+
 ///////////////////////////////////////////////////////////////////////////////
 
 OpalMediaTypeDefinition * OpalMediaType::GetDefinition() const
@@ -80,6 +92,16 @@ OpalMediaTypeDefinition * OpalMediaType::GetDefinition(const OpalMediaType & key
   return OpalMediaTypeFactory::CreateInstance(key);
 }
 
+
+OpalMediaTypeDefinition * OpalMediaType::GetDefinition(unsigned sessionId)
+{
+  PWaitAndSignal mutex(GetMapMutex());
+  SessionIDToMediaTypeMap_T & typeMap = GetSessionIDToMediaTypeMap();
+  SessionIDToMediaTypeMap_T::iterator iter = typeMap.find(sessionId);
+  return iter != typeMap.end() ? iter->second : NULL;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 
 OpalMediaTypeDefinition::OpalMediaTypeDefinition(const char * mediaType,
@@ -88,7 +110,7 @@ OpalMediaTypeDefinition::OpalMediaTypeDefinition(const char * mediaType,
 #else
                                                  const char *,
 #endif
-                                                 unsigned preferredSessionId,
+                                                 unsigned requiredSessionId,
                                                  OpalMediaType::AutoStartMode autoStart)
   : m_mediaType(mediaType)
   , m_autoStart(autoStart)
@@ -96,23 +118,28 @@ OpalMediaTypeDefinition::OpalMediaTypeDefinition(const char * mediaType,
   , m_sdpType(sdpType != NULL ? sdpType : "")
 #endif
 {
-  PWaitAndSignal m(GetMapMutex());
+  PWaitAndSignal mutex(GetMapMutex());
 
   SessionIDToMediaTypeMap_T & typeMap = GetSessionIDToMediaTypeMap();
 
-  // if the preferred session ID is already taken, then find a new one
-  if (typeMap.find(preferredSessionId) != typeMap.end())
-    preferredSessionId = 0;
-
-  // if no preferred session Id 
-  if (preferredSessionId == 0) {
-    preferredSessionId = 1;
-    while (typeMap.find(preferredSessionId) != typeMap.end())
-      ++preferredSessionId;
+  if (requiredSessionId != 0 &&
+      PAssert(typeMap.find(requiredSessionId) == typeMap.end(),
+              "Cannot have multiple media types with same session ID"))
+    m_defaultSessionId = requiredSessionId;
+  else {
+    // Allocate session ID
+    m_defaultSessionId = 4; // Start after audio, video and data that are "special"
+    while (typeMap.find(m_defaultSessionId) != typeMap.end())
+      ++m_defaultSessionId;
   }
 
-  typeMap.insert(SessionIDToMediaTypeMap_T::value_type(preferredSessionId, mediaType));
-  GetMediaTypeToSessionIDMap().insert(MediaTypeToSessionIDMap_T::value_type(mediaType, preferredSessionId));
+  typeMap[m_defaultSessionId] = this;
+}
+
+
+OpalMediaTypeDefinition::~OpalMediaTypeDefinition()
+{
+  GetSessionIDToMediaTypeMap().erase(m_defaultSessionId);
 }
 
 
@@ -122,54 +149,7 @@ OpalMediaSession * OpalMediaTypeDefinition::CreateMediaSession(OpalConnection &
 }
 
 
-unsigned OpalMediaTypeDefinition::GetDefaultSessionId(const OpalMediaType & mediaType)
-{
-  PWaitAndSignal m(GetMapMutex());
-
-  MediaTypeToSessionIDMap_T::iterator r = GetMediaTypeToSessionIDMap().find(mediaType);
-  if (r != GetMediaTypeToSessionIDMap().end())
-    return r->second;
-
-  return 0;
-}
-
-
-OpalMediaType OpalMediaTypeDefinition::GetMediaTypeForSessionId(unsigned sessionId)
-{
-  PWaitAndSignal m(GetMapMutex());
-
-  SessionIDToMediaTypeMap_T::iterator r = GetSessionIDToMediaTypeMap().find(sessionId);
-  if (r != GetSessionIDToMediaTypeMap().end())
-    return r->second;
-
-  return OpalMediaType();
-}
-
-
-OpalMediaTypeDefinition::SessionIDToMediaTypeMap_T & OpalMediaTypeDefinition::GetSessionIDToMediaTypeMap()
-{
-  PWaitAndSignal m(GetMapMutex());
-  static SessionIDToMediaTypeMap_T map;
-  return map;
-}
-
-
-OpalMediaTypeDefinition::MediaTypeToSessionIDMap_T & OpalMediaTypeDefinition::GetMediaTypeToSessionIDMap()
-{
-  PWaitAndSignal m(GetMapMutex());
-  static MediaTypeToSessionIDMap_T map;
-  return map;
-}
-
-
-PMutex & OpalMediaTypeDefinition::GetMapMutex()
-{
-  static PMutex mutex;
-  return mutex;
-}
-
-
-RTP_UDP * OpalMediaTypeDefinition::CreateRTPSession(OpalRTPConnection & /*connection*/,
+RTP_UDP * OpalMediaTypeDefinition::CreateRTPSession(OpalRTPConnection & connection,
                                                     unsigned sessionID,
                                                     bool remoteIsNAT)
 {
@@ -179,7 +159,7 @@ RTP_UDP * OpalMediaTypeDefinition::CreateRTPSession(OpalRTPConnection & /*connec
   params.isAudio = m_mediaType == OpalMediaType::Audio();
   params.remoteIsNAT = remoteIsNAT;
 
-  return new RTP_UDP(params);
+  return connection.GetEndPoint().GetManager().CreateRTPSession(params);
 }
 
 
@@ -187,9 +167,9 @@ RTP_UDP * OpalMediaTypeDefinition::CreateRTPSession(OpalRTPConnection & /*connec
 
 OpalRTPAVPMediaType::OpalRTPAVPMediaType(const char * mediaType,
                                          const char * sdpType,
-                                         unsigned preferredSessionId,
+                                         unsigned requiredSessionId,
                                          OpalMediaType::AutoStartMode autoStart)
-  : OpalMediaTypeDefinition(mediaType, sdpType, preferredSessionId, autoStart)
+  : OpalMediaTypeDefinition(mediaType, sdpType, requiredSessionId, autoStart)
 {
 }
 
