@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26995 $
- * $Author: rjongbloed $
- * $Date: 2012-02-16 19:30:39 -0600 (Thu, 16 Feb 2012) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -58,7 +58,7 @@ class OpalManager_C;
 
 ostream & operator<<(ostream & strm, OpalMessageType type)
 {
-  static const char * Types[] = {
+  static const char * const Types[] = {
     "IndCommandError",
     "CmdSetGeneralParameters",
     "CmdSetProtocolParameters",
@@ -90,7 +90,11 @@ ostream & operator<<(ostream & strm, OpalMessageType type)
     "IndTransferCall",
     "IndCompletedIVR"
   };
-  return strm << Types[type];
+  if (type >= 0 && type < PARRAYSIZE(Types))
+    strm << Types[type];
+  else
+    strm << '<' << (unsigned)type << '>';
+  return strm;
 }
 
 
@@ -159,12 +163,10 @@ class OpalLocalEndPoint_C : public OpalLocalEndPoint
     virtual bool OnWriteMediaFrame(const OpalLocalConnection &, const OpalMediaStream &, RTP_DataFrame & frame);
     virtual bool OnReadMediaData(const OpalLocalConnection &, const OpalMediaStream &, void *, PINDEX, PINDEX &);
     virtual bool OnWriteMediaData(const OpalLocalConnection &, const OpalMediaStream &, const void *, PINDEX, PINDEX &);
-    virtual bool IsSynchronous() const;
 
     OpalMediaDataFunction m_mediaReadData;
     OpalMediaDataFunction m_mediaWriteData;
     OpalMediaDataType     m_mediaDataHeader;
-    OpalMediaTiming       m_mediaTiming;
 
   private:
     OpalManager_C & m_manager;
@@ -178,6 +180,7 @@ class OpalIVREndPoint_C : public OpalIVREndPoint
   public:
     OpalIVREndPoint_C(OpalManager_C & manager);
 
+    virtual bool OnOutgoingCall(const OpalLocalConnection &);
     virtual bool OnIncomingCall(OpalLocalConnection & connection);
     virtual void OnEndDialog(OpalIVRConnection & connection);
 
@@ -434,7 +437,6 @@ OpalLocalEndPoint_C::OpalLocalEndPoint_C(OpalManager_C & mgr)
   , m_mediaReadData(NULL)
   , m_mediaWriteData(NULL)
   , m_mediaDataHeader(OpalMediaDataPayloadOnly)
-  , m_mediaTiming(OpalMediaTimingSynchronous)
   , m_manager(mgr)
 {
 }
@@ -617,12 +619,6 @@ bool OpalLocalEndPoint_C::OnWriteMediaData(const OpalLocalConnection & connectio
 }
 
 
-bool OpalLocalEndPoint_C::IsSynchronous() const
-{
-  return m_mediaTiming == OpalMediaTimingSynchronous;
-}
-
-
 ///////////////////////////////////////
 
 #if OPAL_HAS_PCSS
@@ -663,6 +659,15 @@ OpalIVREndPoint_C::OpalIVREndPoint_C(OpalManager_C & manager)
 }
 
 
+bool OpalIVREndPoint_C::OnOutgoingCall(const OpalLocalConnection & connection)
+{
+  OpalMessageBuffer message(OpalIndAlerting);
+  SetOutgoingCallInfo(message, connection);
+  m_manager.PostMessage(message);
+  return true;
+}
+
+
 bool OpalIVREndPoint_C::OnIncomingCall(OpalLocalConnection & connection)
 {
   m_manager.SendIncomingCallInfo(connection);
@@ -1236,9 +1241,14 @@ void OpalManager_C::HandleSetGeneral(const OpalMessage & command, OpalMessageBuf
       m_localEP->m_mediaDataHeader = command.m_param.m_general.m_mediaDataHeader;
 
     if (m_apiVersion >= 20) {
-      response->m_param.m_general.m_mediaTiming = m_localEP->m_mediaTiming;
+      response->m_param.m_general.m_mediaTiming = (OpalMediaTiming)(m_localEP->GetDefaultAudioSynchronicity()+1);
       if (command.m_param.m_general.m_mediaTiming != 0)
-        m_localEP->m_mediaTiming = command.m_param.m_general.m_mediaTiming;
+        m_localEP->SetDefaultAudioSynchronicity((OpalLocalEndPoint::Synchronicity)(command.m_param.m_general.m_mediaTiming-1));
+      if (m_apiVersion >= 27) {
+        response->m_param.m_general.m_videoSourceTiming = (OpalMediaTiming)(m_localEP->GetDefaultVideoSourceSynchronicity()+1);
+        if (command.m_param.m_general.m_mediaTiming != 0)
+          m_localEP->SetDefaultVideoSourceSynchronicity((OpalLocalEndPoint::Synchronicity)(command.m_param.m_general.m_videoSourceTiming-1));
+      }
     }
   }
 
@@ -1933,7 +1943,7 @@ void OpalManager_C::HandleMediaStream(const OpalMessage & command, OpalMessageBu
 void OpalManager_C::HandleStartRecording(const OpalMessage & command, OpalMessageBuffer & response)
 {
   PSafePtr<OpalCall> call;
-  if (!FindCall(command.m_param.m_userInput.m_callToken, response, call))
+  if (!FindCall(command.m_param.m_recording.m_callToken, response, call))
     return;
 
 #if OPAL_HAS_MIXER
