@@ -25,9 +25,9 @@
  * Contributor(s): Craig Southeren (craigs@postincrement.com)
  *                 Robert Jongbloed (robertj@voxlucida.com.au)
  *
- * $Revision: 26969 $
- * $Author: rjongbloed $
- * $Date: 2012-02-10 18:45:16 -0600 (Fri, 10 Feb 2012) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -109,9 +109,9 @@ void OpalBaseMixer::RemoveStream(const Key_T & key)
   }
 
   if (m_inputStreams.empty())
-    StopPushThread(false);
-
-  m_mutex.Signal();
+    StopPushThread(false); // Will unlock mutex
+  else
+    m_mutex.Signal();
 }
 
 
@@ -125,9 +125,7 @@ void OpalBaseMixer::RemoveAllStreams()
     delete iter->second;
   m_inputStreams.clear();
 
-  StopPushThread(false);
-
-  m_mutex.Signal();
+  StopPushThread(false); // Will unlock mutex
 }
 
 
@@ -218,21 +216,17 @@ void OpalBaseMixer::StopPushThread(bool lock)
   if (lock)
     m_mutex.Wait();
 
+  PThread * thread = m_workerThread;
+  m_workerThread = NULL;
   m_threadRunning = false;
-  if (m_workerThread != NULL) {
-    m_mutex.Signal();
 
+  m_mutex.Signal();
+
+  if (thread != NULL) {
     PTRACE(4, "Mixer\tWaiting for push thread to terminate");
-    m_workerThread->WaitForTermination();
-
-    m_mutex.Wait();
-
-    delete m_workerThread;
-    m_workerThread = NULL;
+    PAssert(thread->WaitForTermination(5000), "Mixer worker thread took too long to terminate.");
+    delete thread;
   }
-
-  if (lock)
-    m_mutex.Signal();
 }
 
 
@@ -658,6 +652,11 @@ bool OpalVideoMixer::MixStreams(RTP_DataFrame & frame)
       return false;
   }
 
+  // This makes sure subimage are on 32 bit boundary, some parts of the
+  // system can get mightily upset if this is not the case.
+  w &= 0xfffffffc;
+  h &= 0xfffffffc;
+
   for (StreamMap_T::iterator iter = m_inputStreams.begin(); iter != m_inputStreams.end(); ++iter) {
     ((VideoStream *)iter->second)->InsertVideoFrame(x, y, w, h);
 
@@ -714,7 +713,12 @@ void OpalVideoMixer::VideoStream::InsertVideoFrame(unsigned x, unsigned y, unsig
                                 m_mixer.m_width, m_mixer.m_height, m_mixer.m_frameStore.GetPointer(),
                                 PVideoFrameInfo::eScale);
 
-  m_queue.pop();
+  /* To avoid continual build up of frames in queue if input frame rate
+     greater than mixer frame, we flush the queue, but keep one to allow for
+     slight mismatches in timing when frame rates are identical. */
+  do {
+    m_queue.pop();
+  } while (m_queue.size() > 1);
 }
 
 
@@ -935,14 +939,14 @@ void OpalMixerConnection::OnApplyStringOptions()
 
 bool OpalMixerConnection::SendUserInputString(const PString & value)
 {
-  m_node->BroadcastUserInput(this, value);
+  m_endpoint.GetNodeManager().QueueUserInput(m_node, this, value);
   return true;
 }
 
 
 PBoolean OpalMixerConnection::SendUserInputTone(char tone, unsigned /*duration*/)
 {
-  m_node->BroadcastUserInput(this, tone);
+  m_endpoint.GetNodeManager().QueueUserInput(m_node, this, tone);
   return true;
 }
 
@@ -1028,14 +1032,9 @@ PBoolean OpalMixerMediaStream::Open()
 }
 
 
-PBoolean OpalMixerMediaStream::Close()
+void OpalMixerMediaStream::InternalClose()
 {
-  if (!isOpen)
-    return false;
-
   m_node->DetachStream(this);
-
-  return OpalMediaStream::Close();
 }
 
 
@@ -1564,6 +1563,7 @@ bool OpalMixerNode::VideoMixer::OnMixed(RTP_DataFrame * & output)
 
 
 OpalMixerNodeManager::OpalMixerNodeManager()
+  : m_userInputPool(1)
 {
   m_nodesByName.DisallowDeleteObjects();
 }
@@ -1656,6 +1656,18 @@ void OpalMixerNodeManager::RemoveNodeNames(PStringList names)
 }
 
 
+void OpalMixerNodeManager::QueueUserInput(const PSafePtr<OpalMixerNode> & node,
+                                          const OpalMixerConnection * connection,
+                                          const PString & value)
+{
+  m_userInputPool.AddWork(new UserInput(node, connection, value));
+}
+
+
+void OpalMixerNodeManager::UserInput::Work()
+{
+  m_node->BroadcastUserInput(m_connection, m_value);
+}
+
+
 //////////////////////////////////////////////////////////////////////////////
-
-
