@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21004 $
- * $Author: rjongbloed $
- * $Date: 2008-09-16 02:08:56 -0500 (Tue, 16 Sep 2008) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #ifdef P_USE_PRAGMA
@@ -36,59 +36,60 @@
 
 #include <opal/buildopts.h>
 
-#include <opal/opalvxml.h>
+#include <opal/ivr.h>
 
-#include <opal/connection.h>
+#include <opal/call.h>
 #include <codec/opalwavfile.h>
 
 
 ///////////////////////////////////////////////////////////////
 
-#if OPAL_PTLIB_VXML
+#if OPAL_IVR
 
-OpalVXMLSession::OpalVXMLSession(OpalConnection * _conn, PTextToSpeech * tts, PBoolean autoDelete)
+OpalVXMLSession::OpalVXMLSession(OpalIVRConnection & conn, PTextToSpeech * tts, PBoolean autoDelete)
   : PVXMLSession(tts, autoDelete),
-    conn(_conn)
+    m_connection(conn)
 {
-  if (tts == NULL) {
-    PFactory<PTextToSpeech>::KeyList_T engines = PFactory<PTextToSpeech>::GetKeyList();
-    if (engines.size() != 0) {
-      PFactory<PTextToSpeech>::Key_T name;
-#ifdef _WIN32
-      name = "Microsoft SAPI";
-      if (std::find(engines.begin(), engines.end(), name) == engines.end())
-#endif
-        name = engines[0];
-      SetTextToSpeech(name);
-    }
-  }
+  if (tts == NULL)
+    SetTextToSpeech(PString::Empty());
 }
 
 
-PBoolean OpalVXMLSession::Close()
+void OpalVXMLSession::OnEndDialog()
 {
-  if (!IsOpen())
-    return PTrue;
-
-  PBoolean ok = PVXMLSession::Close();
-  conn->Release();
-  return ok;
+  m_connection.OnEndDialog();
+  PVXMLSession::OnEndDialog();
 }
 
 
 void OpalVXMLSession::OnEndSession()
 {
-  conn->Release();
+  PTRACE(3, "IVR\tEnd of session, releasing connection.");
+  m_connection.Release();
 }
 
 
-PWAVFile * OpalVXMLSession::CreateWAVFile(const PFilePath & fn, PFile::OpenMode mode, int opts, unsigned fmt)
-{ 
-  return new OpalWAVFile(fn, mode, opts, fmt); 
+bool OpalVXMLSession::OnTransfer(const PString & destination, TransferType type)
+{
+  switch (type) {
+    case BridgedTransfer :
+      // We do not make a distinction between bridged and blind transfers
+
+    case BlindTransfer :
+      if (m_connection.GetCall().Transfer(destination))
+        return true;
+      return m_connection.GetCall().Transfer(destination, &m_connection);
+
+    case ConsultationTransfer :
+      break;
+  }
+
+  // Unsupported
+  return false;
 }
 
 
-#endif // #if OPAL_PTLIB_EXPAT
+#endif // OPAL_IVR
 
 
 // End of File /////////////////////////////////////////////////////////////
