@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26944 $
- * $Author: rjongbloed $
- * $Date: 2012-02-07 17:00:36 -0600 (Tue, 07 Feb 2012) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -64,13 +64,7 @@ OpalMediaPatch::OpalMediaPatch(OpalMediaStream & src)
 
 OpalMediaPatch::~OpalMediaPatch()
 {
-  PWaitAndSignal m(patchThreadMutex);
-  inUse.StartWrite();
-  if (patchThread != NULL) {
-    PAssert(patchThread->WaitForTermination(10000), "Media patch thread not terminated.");
-    delete patchThread;
-    patchThread = NULL;
-  }
+  StopThread();
   PTRACE(5, "Patch\tDestroyed media patch " << this);
 }
 
@@ -79,7 +73,8 @@ void OpalMediaPatch::PrintOn(ostream & strm) const
 {
   strm << "Patch[" << this << "] " << source;
 
-  inUse.StartRead();
+  if (!LockReadOnly())
+    return;
 
   if (sinks.GetSize() > 0) {
     strm << " -> ";
@@ -95,7 +90,7 @@ void OpalMediaPatch::PrintOn(ostream & strm) const
     }
   }
 
-  inUse.EndRead();
+  UnlockReadOnly();
 }
 
 void OpalMediaPatch::Start()
@@ -112,11 +107,31 @@ void OpalMediaPatch::Start()
 }
 
 
+void OpalMediaPatch::StopThread()
+{
+  patchThreadMutex.Wait();
+  PThread * thread = patchThread;
+  patchThread = NULL;
+  patchThreadMutex.Signal();
+
+  if (thread == NULL)
+    return;
+
+  if (!thread->IsSuspended()) {
+    PTRACE(4, "Patch\tWaiting for media patch thread to stop " << *this);
+    PAssert(thread->WaitForTermination(10000), "Media patch thread not terminated.");
+  }
+
+  delete thread;
+}
+
+
 void OpalMediaPatch::Close()
 {
   PTRACE(3, "Patch\tClosing media patch " << *this);
 
-  inUse.StartWrite();
+  if (!LockReadWrite())
+    return;
 
   if (m_bypassFromPatch != NULL)
     m_bypassFromPatch->SetBypassPatch(NULL);
@@ -129,33 +144,25 @@ void OpalMediaPatch::Close()
 
   while (sinks.GetSize() > 0) {
     OpalMediaStreamPtr stream = sinks.front().stream;
-    inUse.EndWrite();
+    UnlockReadWrite();
     if (!stream->Close()) {
       // The only way we can get here is if the sink is in the proccess of being closed
-      // but is blocked on the inUse mutex waiting to remove the sink from this patch.
+      // but is blocked on the mutex waiting to remove the sink from this patch.
       // Se we unlock it, and wait for it to do it in the other thread.
       PThread::Sleep(10);
     }
-    inUse.StartWrite();
-  }
-
-  PTRACE(4, "Patch\tWaiting for media patch thread to stop " << *this);
-  {
-    PWaitAndSignal m(patchThreadMutex);
-    if (patchThread != NULL && !patchThread->IsSuspended()) {
-      inUse.EndWrite();
-      PAssert(patchThread->WaitForTermination(10000), "Media patch thread not terminated.");
+    if (!LockReadWrite())
       return;
-    }
   }
+  UnlockReadWrite();
 
-  inUse.EndWrite();
+  StopThread();
 }
 
 
 PBoolean OpalMediaPatch::AddSink(const OpalMediaStreamPtr & sinkStream)
 {
-  PWriteWaitAndSignal mutex(inUse);
+  PSafeLockReadWrite mutex(*this);
 
   if (PAssertNULL(sinkStream) == NULL)
     return false;
@@ -272,16 +279,17 @@ void OpalMediaPatch::RemoveSink(const OpalMediaStreamPtr & stream)
   if (PAssertNULL(stream) == NULL)
     return;
 
-  PTRACE(3, "Patch\tRemoving media stream sink " << *stream);
+  PTRACE(3, "Patch\tRemoving sink " << *stream << " from " << *this);
 
   bool closeSource = false;
 
-  inUse.StartWrite();
+  if (!LockReadWrite())
+    return;
 
   for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s) {
     if (s->stream == stream) {
       sinks.erase(s);
-      PTRACE(5, "Patch\tRemoved media stream sink " << *stream);
+      PTRACE(5, "Patch\tRemoved sink " << *stream << " from " << *this);
       break;
     }
   }
@@ -292,7 +300,7 @@ void OpalMediaPatch::RemoveSink(const OpalMediaStreamPtr & stream)
       m_bypassFromPatch->SetBypassPatch(NULL);
   }
 
-  inUse.EndWrite();
+  UnlockReadWrite();
 
   if (closeSource  && source.GetPatch() == this)
     source.Close();
@@ -301,7 +309,7 @@ void OpalMediaPatch::RemoveSink(const OpalMediaStreamPtr & stream)
 
 OpalMediaStreamPtr OpalMediaPatch::GetSink(PINDEX i) const
 {
-  PReadWaitAndSignal mutex(inUse);
+  PSafeLockReadOnly mutex(*this);
   return i < sinks.GetSize() ? sinks[i].stream : OpalMediaStreamPtr();
 }
 
@@ -322,7 +330,8 @@ OpalMediaFormat OpalMediaPatch::GetSinkFormat(PINDEX i) const
 
 OpalTranscoder * OpalMediaPatch::GetAndLockSinkTranscoder(PINDEX i) const
 {
-  inUse.StartRead();
+  if (!LockReadOnly())
+    return NULL;
 
   if (i >= sinks.GetSize()) {
     UnLockSinkTranscoder();
@@ -344,14 +353,15 @@ OpalTranscoder * OpalMediaPatch::GetAndLockSinkTranscoder(PINDEX i) const
 
 void OpalMediaPatch::UnLockSinkTranscoder() const
 {
-  inUse.EndRead();
+  UnlockReadOnly();
 }
 
 
 #if OPAL_STATISTICS
 void OpalMediaPatch::GetStatistics(OpalMediaStatistics & statistics, bool fromSink) const
 {
-  inUse.StartRead();
+  if (!LockReadOnly())
+    return;
 
   if (fromSink)
     source.GetStatistics(statistics, true);
@@ -359,7 +369,7 @@ void OpalMediaPatch::GetStatistics(OpalMediaStatistics & statistics, bool fromSi
   if (!sinks.IsEmpty())
     sinks.front().GetStatistics(statistics, !fromSink);
 
-  inUse.EndRead();
+  UnlockReadOnly();
 }
 
 
@@ -409,7 +419,7 @@ OpalMediaPatch::Sink::~Sink()
 
 void OpalMediaPatch::AddFilter(const PNotifier & filter, const OpalMediaFormat & stage)
 {
-  PWriteWaitAndSignal mutex(inUse);
+  PSafeLockReadWrite mutex(*this);
 
   if (source.GetMediaFormat().GetMediaType() != stage.GetMediaType())
     return;
@@ -427,7 +437,7 @@ void OpalMediaPatch::AddFilter(const PNotifier & filter, const OpalMediaFormat &
 
 PBoolean OpalMediaPatch::RemoveFilter(const PNotifier & filter, const OpalMediaFormat & stage)
 {
-  PWriteWaitAndSignal mutex(inUse);
+  PSafeLockReadWrite mutex(*this);
 
   for (PList<Filter>::iterator f = filters.begin(); f != filters.end(); ++f) {
     if (f->notifier == filter && f->stage == stage) {
@@ -444,20 +454,21 @@ PBoolean OpalMediaPatch::RemoveFilter(const PNotifier & filter, const OpalMediaF
 void OpalMediaPatch::FilterFrame(RTP_DataFrame & frame,
                                  const OpalMediaFormat & mediaFormat)
 {
-  inUse.StartRead();
+  if (!LockReadOnly())
+    return;
 
   for (PList<Filter>::iterator f = filters.begin(); f != filters.end(); ++f) {
     if (f->stage.IsEmpty() || f->stage == mediaFormat)
       f->notifier(frame, (INT)this);
   }
 
-  inUse.EndRead();
+  UnlockReadOnly();
 }
 
 
 bool OpalMediaPatch::UpdateMediaFormat(const OpalMediaFormat & mediaFormat)
 {
-  PWriteWaitAndSignal mutex(inUse);
+  PSafeLockReadWrite mutex(*this);
 
   bool atLeastOne = source.InternalUpdateMediaFormat(mediaFormat);
 
@@ -471,7 +482,7 @@ bool OpalMediaPatch::UpdateMediaFormat(const OpalMediaFormat & mediaFormat)
 
 PBoolean OpalMediaPatch::ExecuteCommand(const OpalMediaCommand & command, PBoolean fromSink)
 {
-  PReadWaitAndSignal mutex(inUse);
+  PSafeLockReadOnly mutex(*this);
 
   if (fromSink) {
     OpalMediaPatch * patch = m_bypassToPatch != NULL ? m_bypassToPatch : this;
@@ -499,7 +510,7 @@ void OpalMediaPatch::OnMediaCommand(OpalMediaCommand & command, INT)
 
 bool OpalMediaPatch::SetPaused(bool pause)
 {
-  PReadWaitAndSignal mutex(inUse);
+  PSafeLockReadOnly mutex(*this);
 
   bool atLeastOne = source.SetPaused(pause, true);
 
@@ -528,7 +539,7 @@ bool OpalMediaPatch::OnStartMediaPatch()
 
 bool OpalMediaPatch::EnableJitterBuffer()
 {
-  PReadWaitAndSignal mutex(inUse);
+  PSafeLockReadOnly mutex(*this);
 
   bool enab = m_bypassToPatch == NULL;
 
@@ -586,11 +597,7 @@ void OpalMediaPatch::Main()
       break;
     }
  
-    inUse.StartRead();
-    bool written = DispatchFrame(sourceFrame);
-    inUse.EndRead();
-
-    if (!written) {
+    if (!DispatchFrame(sourceFrame)) {
       PTRACE(4, "Patch\tThread ended because all sink writes failed");
       break;
     }
@@ -635,7 +642,7 @@ void OpalMediaPatch::Main()
 
 bool OpalMediaPatch::SetBypassPatch(OpalMediaPatch * patch)
 {
-  PWriteWaitAndSignal mutex(inUse);
+  PSafeLockReadWrite mutex(*this);
 
   if (!PAssert(m_bypassFromPatch == NULL, PLogicError))
     return false; // Can't be both!
@@ -676,19 +683,20 @@ bool OpalMediaPatch::SetBypassPatch(OpalMediaPatch * patch)
 
 PBoolean OpalMediaPatch::PushFrame(RTP_DataFrame & frame)
 {
-  PReadWaitAndSignal mutex(inUse);
   return DispatchFrame(frame);
 }
 
 
 bool OpalMediaPatch::DispatchFrame(RTP_DataFrame & frame)
 {
+  if (!LockReadOnly())
+    return false;
+
   if (m_bypassFromPatch != NULL) {
     PTRACE(3, "Patch\tMedia patch bypass started by " << *m_bypassFromPatch << " on " << *this);
-    inUse.EndRead();
+    UnlockReadOnly();
     m_bypassEnded.Wait();
     PTRACE(4, "Patch\tMedia patch bypass ended on " << *this);
-    inUse.StartRead();
     return true;
   }
 
@@ -703,12 +711,14 @@ bool OpalMediaPatch::DispatchFrame(RTP_DataFrame & frame)
     }
   }
   else {
+    PSafeLockReadOnly guard(*m_bypassToPatch);
     for (PList<Sink>::iterator s = m_bypassToPatch->sinks.begin(); s != m_bypassToPatch->sinks.end(); ++s) {
       if (s->stream->WritePacket(frame))
         written = true;
     }
   }
 
+  UnlockReadOnly();
   return written;
 }
 
@@ -799,8 +809,6 @@ void OpalMediaPatch::Sink::SetRateControlParameters(const OpalMediaFormat & medi
   if ((mediaFormat.GetMediaType() == OpalMediaType::Video()) && mediaFormat != OpalYUV420P) {
     rateController = NULL;
     PString rc = mediaFormat.GetOptionString(OpalVideoFormat::RateControllerOption());
-    if (rc.IsEmpty() && mediaFormat.GetOptionBoolean(OpalVideoFormat::RateControlEnableOption()))
-      rc = "Standard";
     if (!rc.IsEmpty()) {
       rateController = PFactory<OpalVideoRateController>::CreateInstance(rc);
       if (rateController != NULL) {   
@@ -946,12 +954,17 @@ OpalMediaPatch::Thread::Thread(OpalMediaPatch & p)
 
 OpalPassiveMediaPatch::OpalPassiveMediaPatch(OpalMediaStream & source)
   : OpalMediaPatch(source)
+  , m_started(false)
 {
 }
 
 
 void OpalPassiveMediaPatch::Start()
 {
+  if (m_started)
+    return;
+
+  m_started = true;
   OnStartMediaPatch();
 }
 
