@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26995 $
- * $Author: rjongbloed $
- * $Date: 2012-02-16 19:30:39 -0600 (Thu, 16 Feb 2012) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 
@@ -41,6 +41,8 @@
 
 ///////////////////////////////////////////////////////////////////////
 
+PURL_LEGACY_SCHEME(pres, true, false, true, true, false, true, true, false, false, false, 0)
+
 const PCaselessString & OpalPresentity::AuthNameKey()     { static const PConstCaselessString s("Auth ID");       return s; }
 const PCaselessString & OpalPresentity::AuthPasswordKey() { static const PConstCaselessString s("Auth Password"); return s; }
 const PCaselessString & OpalPresentity::TimeToLiveKey()   { static const PConstCaselessString s("Time to Live");  return s; }
@@ -170,12 +172,8 @@ PObject::Comparison OpalPresenceInfo::Compare(const PObject & obj) const
 
 ///////////////////////////////////////////////////////////////////////
 
-static PAtomicInteger::IntegerType g_idNumber = 1;
-
-
 OpalPresentity::OpalPresentity()
   : m_manager(NULL)
-  , m_idNumber(g_idNumber++)
   , m_temporarilyUnavailable(false)
   , m_localState(OpalPresenceInfo::NoPresence)
 {
@@ -186,7 +184,6 @@ OpalPresentity::OpalPresentity(const OpalPresentity & other)
   : PSafeObject(other)
   , m_manager(other.m_manager)
   , m_attributes(other.m_attributes)
-  , m_idNumber(g_idNumber++)
   , m_temporarilyUnavailable(false)
   , m_localState(OpalPresenceInfo::NoPresence)
 {
@@ -217,6 +214,26 @@ OpalPresentity * OpalPresentity::Create(OpalManager & manager, const PURL & url,
 }
 
 
+bool OpalPresentity::Open()
+{
+  if (m_open.TestAndSet(true))
+    return false; // Already open
+
+  PTRACE(3, "OpalPres\t'" << m_aor << "' opening.");
+  return true;
+}
+
+
+bool OpalPresentity::Close()
+{
+  if (!m_open.TestAndSet(false))
+    return false; // Aleady closed
+
+  PTRACE(3, "OpalPres\t'" << m_aor << "' closing.");
+  return true;
+}
+
+
 bool OpalPresentity::SubscribeToPresence(const PURL & presentity, bool subscribe, const PString & note)
 {
   if (!IsOpen())
@@ -521,14 +538,6 @@ OpalPresentityCommand * OpalPresentity::InternalCreateCommand(const char * cmdNa
 }
 
 
-PString OpalPresentity::GetID() const 
-{ 
-  PStringStream strm;
-  strm << "id" << (unsigned)m_idNumber;
-  return strm;
-}
-
-
 void OpalPresentity::Internal_SendMessageToCommand(const OpalSendMessageToCommand & cmd)
 {
   OpalEndPoint * endpoint = m_manager->FindEndPoint(m_aor.GetScheme());
