@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26320 $
- * $Author: rjongbloed $
- * $Date: 2011-08-17 06:58:09 -0500 (Wed, 17 Aug 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -103,7 +103,7 @@ bool OpalTranscoder::UpdateMediaFormats(const OpalMediaFormat & input, const Opa
 
 PBoolean OpalTranscoder::ExecuteCommand(const OpalMediaCommand & /*command*/)
 {
-  return PFalse;
+  return false;
 }
 
 
@@ -283,7 +283,7 @@ static bool MergeFormats(const OpalMediaFormatList & masterFormats,
   else {
     srcFormat = *masterFormat;
     PTRACE(5, "Opal\tInitial source format from master:\n" << setw(-1) << srcFormat
-                        << "Merging with capability\n" << setw(-1) << srcCapability);
+                            << "Merging with capability\n" << setw(-1) << srcCapability);
     if (!srcFormat.Merge(srcCapability))
       return false;
   }
@@ -296,13 +296,9 @@ static bool MergeFormats(const OpalMediaFormatList & masterFormats,
   else {
     dstFormat = *masterFormat;
     PTRACE(5, "Opal\tInitial destination format from master:\n" << setw(-1) << dstFormat
-                             << "Merging with capability\n" << setw(-1) << dstCapability);
+                                 << "Merging with capability\n" << setw(-1) << dstCapability);
 
-    OpalMediaFormat tmpCapability = dstCapability;
-    if (!tmpCapability.Merge(dstFormat))
-      return false;
-
-    if (!dstFormat.Update(tmpCapability))
+    if (!dstFormat.Update(dstCapability)) // This includes the PayloadType, a normal Merge does not.
       return false;
   }
 
@@ -426,9 +422,8 @@ OpalMediaFormatList OpalTranscoder::GetPossibleFormats(const OpalMediaFormatList
   // Run through the formats connection can do directly and calculate all of
   // the possible formats, including ones via a transcoder
   for (OpalMediaFormatList::const_iterator f = formats.begin(); f != formats.end(); ++f) {
-    OpalMediaFormat format = *f;
-    possibleFormats += format;
-    OpalMediaFormatList srcFormats = GetSourceFormats(format);
+    possibleFormats += *f;
+    OpalMediaFormatList srcFormats = GetSourceFormats(*f);
     for (OpalMediaFormatList::iterator s = srcFormats.begin(); s != srcFormats.end(); ++s) {
       OpalMediaFormatList dstFormats = GetDestinationFormats(*s);
       if (dstFormats.GetSize() > 0) {
@@ -476,13 +471,15 @@ void OpalFramedTranscoder::CalculateSizes()
 {
   unsigned framesPerPacket = outputMediaFormat.GetOptionInteger(OpalAudioFormat::TxFramesPerPacketOption(),
                               inputMediaFormat.GetOptionInteger(OpalAudioFormat::TxFramesPerPacketOption(), 1));
+  unsigned channels = outputMediaFormat.GetOptionInteger(OpalAudioFormat::ChannelsOption(),
+                       inputMediaFormat.GetOptionInteger(OpalAudioFormat::ChannelsOption(), 1));
   unsigned inFrameSize = inputMediaFormat.GetFrameSize();
   unsigned outFrameSize = outputMediaFormat.GetFrameSize();
   unsigned inFrameTime = inputMediaFormat.GetFrameTime();
   unsigned outFrameTime = outputMediaFormat.GetFrameTime();
   unsigned leastCommonMultiple = inFrameTime*outFrameTime/GreatestCommonDivisor(inFrameTime, outFrameTime);
-  inputBytesPerFrame = leastCommonMultiple/inFrameTime*inFrameSize*framesPerPacket;
-  outputBytesPerFrame = leastCommonMultiple/outFrameTime*outFrameSize*framesPerPacket;
+  inputBytesPerFrame = leastCommonMultiple/inFrameTime*inFrameSize*framesPerPacket*channels;
+  outputBytesPerFrame = leastCommonMultiple/outFrameTime*outFrameSize*framesPerPacket*channels;
 
   PINDEX inMaxTimePerFrame  = inputMediaFormat.GetOptionInteger(OpalAudioFormat::MaxFramesPerPacketOption()) * 
                               inputMediaFormat.GetOptionInteger(OpalAudioFormat::FrameTimeOption());
