@@ -28,9 +28,9 @@
  *     US Joint Forces Command Joint Concept Development & Experimentation (J9)
  *     http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision: 26760 $
- * $Author: rjongbloed $
- * $Date: 2011-12-06 19:02:40 -0600 (Tue, 06 Dec 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -101,12 +101,12 @@ OpalTransportAddress::OpalTransportAddress(const PIPSocket::Address & addr, WORD
 }
 
 
-PString OpalTransportAddress::GetHostName() const
+PString OpalTransportAddress::GetHostName(bool includeService) const
 {
   if (transport == NULL)
     return *this;
 
-  return transport->GetHostName(*this);
+  return transport->GetHostName(*this, includeService);
 }
 
 
@@ -271,7 +271,7 @@ void OpalTransportAddressArray::AppendStringCollection(const PCollection & coll)
 
 /////////////////////////////////////////////////////////////////
 
-PString OpalInternalTransport::GetHostName(const OpalTransportAddress & address) const
+PString OpalInternalTransport::GetHostName(const OpalTransportAddress & address, bool) const
 {
   // skip transport identifier
   PINDEX pos = address.Find('$');
@@ -366,20 +366,23 @@ static PBoolean SplitAddress(const PString & addr, PString & host, PString & dev
 }
 
 
-PString OpalInternalIPTransport::GetHostName(const OpalTransportAddress & address) const
+PString OpalInternalIPTransport::GetHostName(const OpalTransportAddress & address, bool includeService) const
 {
   PString host, device, service;
   if (!SplitAddress(address, host, device, service))
     return address;
 
-  if (!device.IsEmpty())
-    return host+device;
+  PString hostname = host+device;
 
-  PIPSocket::Address ip;
-  if (ip.FromString(host))
-    return ip.AsString(true);
+  if (device.IsEmpty()) {
+    PIPSocket::Address ip;
+    if (ip.FromString(host))
+      hostname = ip.AsString(true);
+  }
 
-  return host;
+  if (includeService)
+    hostname += ':' + service;
+  return hostname;
 }
 
 
@@ -430,6 +433,14 @@ PBoolean OpalInternalIPTransport::GetIpAndPort(const OpalTransportAddress & addr
     if (PIPSocket::GetHostAddress(host, ip))
       return true;
     PTRACE(1, "Opal\tCould not find host \"" << host << '"');
+    // In this case the Version may be 0 which will cause problems later on
+    // so here we fix it by calling GetAny(4)
+    if (ip.GetVersion() == 0) {
+      PTRACE(3, "OpalIP\tOpalInternalIPTransport IP Address version is 0, so fixing");
+      ip = PIPSocket::Address::GetAny(4);
+      PTRACE(3, "OpalIP\tOpalInternalIPTransport new IP Address and version: "
+             << ip.AsString(true) << " - " << ip.GetVersion());
+    }
   }
   else {
     if (ip.FromString(device))
@@ -1333,7 +1344,9 @@ OpalTransportAddress OpalTransportUDP::GetLocalAddress(bool allowNAT) const
   PMonitoredSocketChannel * socket = (PMonitoredSocketChannel *)readChannel;
   if (socket != NULL) {
     OpalTransportUDP * thisWritable = const_cast<OpalTransportUDP *>(this);
-    socket->GetLocal(thisWritable->localAddress, thisWritable->localPort, allowNAT && !manager.IsLocalAddress(remoteAddress));
+    if (!socket->GetLocal(thisWritable->localAddress, thisWritable->localPort,
+                          allowNAT && !manager.IsLocalAddress(remoteAddress)))
+      return OpalTransportAddress();
   }
 
   return OpalTransportIP::GetLocalAddress(allowNAT);
