@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26163 $
- * $Author: rjongbloed $
- * $Date: 2011-07-11 00:36:48 -0500 (Mon, 11 Jul 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -41,6 +41,8 @@
 
 #include <rtp/jitter.h>
 
+#include <rtp/metrics.h>
+
 
 const unsigned JitterRoundingGuardBits = 4;
 
@@ -259,6 +261,7 @@ void OpalJitterBuffer::SetDelay(unsigned minJitterDelay, unsigned maxJitterDelay
   m_packetsTooLate        = 0;
   m_bufferOverruns        = 0;
   m_consecutiveMarkerBits = 0;
+  m_lastSyncSource        = 0;
 
   Reset();
 
@@ -356,6 +359,14 @@ PBoolean OpalJitterBuffer::WriteData(const RTP_DataFrame & frame, const PTimeInt
   m_lastTimestamp = timestamp;
 
 
+
+  if (frame.GetSyncSource() != m_lastSyncSource) {
+    Reset();
+    m_lastSyncSource = frame.GetSyncSource();
+    PTRACE(4, "Jitter\tBuffer reset due to SSRC change.");
+  }
+
+
   // Add to buffer
   pair<FrameMap::iterator,bool> result = m_frames.insert(FrameMap::value_type(timestamp, frame));
   if (result.second) {
@@ -637,7 +648,7 @@ void OpalJitterBufferThread::JitterThreadMain(PThread &, INT)
   PTRACE(4, "Jitter\tReceive thread started: " << *this);
 
   while (m_running) {
-    RTP_DataFrame frame(0, m_packetSize);
+    RTP_DataFrame frame(m_packetSize);
 
     // Keep reading from the RTP transport frames
     if (!OnReadPacket(frame) || !WriteData(frame))
@@ -658,7 +669,6 @@ RTP_JitterBuffer::RTP_JitterBuffer(RTP_Session & sess,
   : OpalJitterBufferThread(minJitterDelay, maxJitterDelay, time, packetSize),
     m_session(sess)
 {
-  StartThread();
 }
 
 
@@ -678,9 +688,17 @@ RTP_JitterBuffer::~RTP_JitterBuffer()
 
 PBoolean RTP_JitterBuffer::OnReadPacket(RTP_DataFrame & frame)
 {
-  bool success = m_session.ReadData(frame);
-  PTRACE_IF(6, success, "Jitter\tOnReadPacket: Frame from network, timestamp " << frame.GetTimestamp());
-  return success;
+  if (!m_session.ReadData(frame))
+    return false;
+
+#if OPAL_RTCP_XR
+  RTCP_XR_Metrics * metrics = m_session.GetExtendedMetrics();
+  if (metrics != NULL)
+    metrics->SetJitterDelay(GetCurrentJitterDelay()/GetTimeUnits());
+#endif
+
+  PTRACE(6, "Jitter\tOnReadPacket: Frame from network, timestamp " << frame.GetTimestamp());
+  return true;
 }
 
 
