@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26862 $
- * $Author: rjongbloed $
- * $Date: 2012-01-15 06:30:18 -0600 (Sun, 15 Jan 2012) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -584,13 +584,15 @@ RTP_Session::RTP_Session(const Params & params)
   : m_timeUnits(params.isAudio ? 8 : 90)
   , canonicalName(PProcess::Current().GetUserName())
   , toolName(PProcess::Current().GetName())
-  , reportTimeInterval(0, 12)  // Seconds
   , lastSRTimestamp(0)
   , lastSRReceiveTime(0)
   , outOfOrderWaitTime(GetDefaultOutOfOrderWaitTime())
   , firstPacketSent(0)
   , firstPacketReceived(0)
-  , reportTimer(reportTimeInterval)
+#if OPAL_RTCP_XR
+  , m_metrics(NULL)
+#endif
+  , m_reportTimer(0, 12)  // Seconds
   , failed(false)
 {
   PAssert(params.id > 0, PInvalidParameter);
@@ -632,10 +634,19 @@ RTP_Session::RTP_Session(const Params & params)
 
   m_encodingHandler = NULL;
   SetEncoding(params.encoding);
+
+  m_reportTimer.SetNotifier(PCREATE_NOTIFIER(SendReport));
 }
 
+
 RTP_Session::~RTP_Session()
 {
+  m_reportTimer.Stop(true);
+
+#if OPAL_RTCP_XR
+  delete m_metrics;
+#endif
+
 #if PTRACING
   PTime now;
   int sentDuration = (now-firstPacketSent).GetSeconds();
@@ -750,7 +761,7 @@ void RTP_Session::SendBYE()
 
 PString RTP_Session::GetCanonicalName() const
 {
-  PWaitAndSignal mutex(reportMutex);
+  PWaitAndSignal mutex(m_reportMutex);
   PString s = canonicalName;
   s.MakeUnique();
   return s;
@@ -759,14 +770,15 @@ PString RTP_Session::GetCanonicalName() const
 
 void RTP_Session::SetCanonicalName(const PString & name)
 {
-  PWaitAndSignal mutex(reportMutex);
+  PWaitAndSignal mutex(m_reportMutex);
   canonicalName = name;
+  canonicalName.MakeUnique();
 }
 
 
 PString RTP_Session::GetToolName() const
 {
-  PWaitAndSignal mutex(reportMutex);
+  PWaitAndSignal mutex(m_reportMutex);
   PString s = toolName;
   s.MakeUnique();
   return s;
@@ -775,8 +787,9 @@ PString RTP_Session::GetToolName() const
 
 void RTP_Session::SetToolName(const PString & name)
 {
-  PWaitAndSignal mutex(reportMutex);
+  PWaitAndSignal mutex(m_reportMutex);
   toolName = name;
+  toolName.MakeUnique();
 }
 
 
@@ -794,15 +807,21 @@ void RTP_Session::SetJitterBufferSize(unsigned minJitterDelay,
                                       unsigned timeUnits,
                                         PINDEX packetSize)
 {
+  PWaitAndSignal mutex(dataMutex);
+
   if (timeUnits > 0)
     m_timeUnits = timeUnits;
 
   if (minJitterDelay == 0 && maxJitterDelay == 0) {
     PTRACE_IF(4, m_jitterBuffer != NULL, "RTP\tSwitching off jitter buffer " << *m_jitterBuffer);
+    // This can block waiting for JB thread to end, signal mutex to avoid deadlock
+    dataMutex.Signal();
     m_jitterBuffer.SetNULL();
+    dataMutex.Wait();
   }
   else {
     resequenceOutOfOrderPackets = false;
+    FlushData();
     if (m_jitterBuffer != NULL) {
       PTRACE(4, "RTP\tSetting jitter buffer time from " << minJitterDelay << " to " << maxJitterDelay);
       m_jitterBuffer->SetDelay(minJitterDelay, maxJitterDelay, packetSize);
@@ -810,6 +829,7 @@ void RTP_Session::SetJitterBufferSize(unsigned minJitterDelay,
     else {
       m_jitterBuffer = new RTP_JitterBuffer(*this, minJitterDelay, maxJitterDelay, m_timeUnits, packetSize);
       PTRACE(4, "RTP\tCreated RTP jitter buffer " << *m_jitterBuffer);
+      m_jitterBuffer->StartThread();
     }
   }
 }
@@ -843,12 +863,18 @@ PBoolean RTP_Session::ReadBufferedData(RTP_DataFrame & frame)
   m_outOfOrderPackets.pop_back();
   expectedSequenceNumber = (WORD)(sequenceNumber + 1);
 
-  PTRACE(5, "RTP\tSession " << sessionID << ", ssrc=" << syncSourceIn << ", resequenced "
+  PTRACE(m_outOfOrderPackets.empty() ? 2 : 5,
+         "RTP\tSession " << sessionID << ", ssrc=" << syncSourceIn << ", resequenced "
          << (m_outOfOrderPackets.empty() ? "last" : "next") << " out of order packet " << sequenceNumber);
   return true;
 }
 
 
+void RTP_Session::FlushData()
+{
+}
+
+
 void RTP_Session::SetTxStatisticsInterval(unsigned packets)
 {
   txStatisticsInterval = PMAX(packets, 2);
@@ -912,142 +938,6 @@ void RTP_Session::AddReceiverReport(RTP_ControlFrame::ReceiverReport & receiver)
 } 
 
 
-#if OPAL_RTCP_XR
-
-void RTP_Session::InsertExtendedReportPacket(RTP_ControlFrame & report)
-{
-  report.StartNewPacket();
-  report.SetPayloadType(RTP_ControlFrame::e_ExtendedReport);
-  report.SetPayloadSize(sizeof(PUInt32b) + sizeof(RTP_ControlFrame::ExtendedReport));  // length is SSRC of packet sender plus XR
-  report.SetCount(1);
-  BYTE * payload = report.GetPayloadPtr();
-
-  // add the SSRC to the start of the payload
-  *(PUInt32b *)payload = syncSourceOut;
-  
-  RTP_ControlFrame::ExtendedReport & xr = *(RTP_ControlFrame::ExtendedReport *)(payload+4);
-  
-  xr.bt = 0x07;
-  xr.type_specific = 0x00;
-  xr.length = 0x08;
-  xr.ssrc = syncSourceOut;
-  
-  xr.loss_rate = m_metrics.GetLossRate();
-  xr.discard_rate = m_metrics.GetDiscardRate();
-  xr.burst_density = m_metrics.GetBurstDensity();
-  xr.gap_density = m_metrics.GetGapDensity();
-  xr.burst_duration = m_metrics.GetBurstDuration();
-  xr.gap_duration = m_metrics.GetGapDuration();
-  xr.round_trip_delay = m_metrics.GetRoundTripDelay();
-  xr.end_system_delay = m_metrics.GetEndSystemDelay();
-  xr.signal_level = 0x7F;
-  xr.noise_level = 0x7F;
-  xr.rerl = 0x7F;	
-  xr.gmin = 16;
-  xr.r_factor = m_metrics.RFactor();
-  xr.ext_r_factor = 0x7F;
-  xr.mos_lq = m_metrics.MOS_LQ();
-  xr.mos_cq = m_metrics.MOS_CQ();
-  xr.rx_config = 0x00;
-  xr.reserved = 0x00;
-
-  JitterBufferPtr jitter = m_jitterBuffer; // Increase reference count in case gets deleted out from under us
-  if (jitter != NULL) {
-    xr.jb_nominal = (WORD)(jitter->GetMinJitterDelay()/jitter->GetTimeUnits());
-    xr.jb_maximum = (WORD)(jitter->GetCurrentJitterDelay()/jitter->GetTimeUnits());
-    xr.jb_absolute = (WORD)(jitter->GetMaxJitterDelay()/jitter->GetTimeUnits());
-  }
-  
-  report.EndPacket();
-  
-  PTRACE(3, "RTP\tSession " << sessionID << ", SentExtendedReport:"
-            " ssrc=" << xr.ssrc
-         << " loss_rate=" << (PUInt32b) xr.loss_rate
-         << " discard_rate=" << (PUInt32b) xr.discard_rate
-         << " burst_density=" << (PUInt32b) xr.burst_density
-         << " gap_density=" << (PUInt32b) xr.gap_density
-         << " burst_duration=" << xr.burst_duration
-         << " gap_duration=" << xr.gap_duration
-         << " round_trip_delay="<< xr.round_trip_delay
-         << " end_system_delay="<< xr.end_system_delay
-         << " gmin="<< (PUInt32b) xr.gmin
-         << " r_factor="<< (PUInt32b) xr.r_factor
-         << " mos_lq="<< (PUInt32b) xr.mos_lq
-         << " mos_cq="<< (PUInt32b) xr.mos_cq
-         << " jb_nominal_delay="<< xr.jb_nominal
-         << " jb_maximum_delay="<< xr.jb_maximum
-         << " jb_absolute_delay="<< xr.jb_absolute);
-
-}
-
-static RTP_Session::ExtendedReportArray
-BuildExtendedReportArray(const RTP_ControlFrame & frame, PINDEX offset)
-{
-  RTP_Session::ExtendedReportArray reports;
-
-  const RTP_ControlFrame::ExtendedReport * rr = (const RTP_ControlFrame::ExtendedReport *)(frame.GetPayloadPtr()+offset);
-  for (PINDEX repIdx = 0; repIdx < (PINDEX)frame.GetCount(); repIdx++) {
-    RTP_Session::ExtendedReport * report = new RTP_Session::ExtendedReport;
-    report->sourceIdentifier = rr->ssrc;
-    report->lossRate = rr->loss_rate;
-    report->discardRate = rr->discard_rate;
-    report->burstDensity = rr->burst_density;
-    report->gapDensity = rr->gap_density;
-    report->roundTripDelay = rr->round_trip_delay;
-    report->RFactor = rr->r_factor;
-    report->mosLQ = rr->mos_lq;
-    report->mosCQ = rr->mos_cq;
-    report->jbNominal = rr->jb_nominal;
-    report->jbMaximum = rr->jb_maximum;
-    report->jbAbsolute = rr->jb_absolute;
-    reports.SetAt(repIdx, report);
-    rr++;
-  }
-  return reports;
-}
-
-
-void RTP_Session::OnRxSenderReportToMetrics(const RTP_ControlFrame & frame, PINDEX offset)
-{
-  const RTP_ControlFrame::ReceiverReport * rr = (const RTP_ControlFrame::ReceiverReport *)(frame.GetPayloadPtr()+offset);
-  for (unsigned repIdx = 0; repIdx < frame.GetCount(); repIdx++, rr++)
-    m_metrics.OnRxSenderReport(rr->lsr, rr->dlsr);
-}
-
-
-void RTP_Session::OnRxExtendedReport(DWORD PTRACE_PARAM(src), const ExtendedReportArray & PTRACE_PARAM(reports))
-{
-#if PTRACING
-  if (PTrace::CanTrace(3)) {
-    ostream & strm = PTrace::Begin(2, __FILE__, __LINE__);
-    strm << "RTP\tSession " << sessionID << ", OnExtendedReport: ssrc=" << src << '\n';
-    for (PINDEX i = 0; i < reports.GetSize(); i++)
-      strm << "  XR: " << reports[i] << '\n';
-    strm << PTrace::End;
-  }
-#endif
-}
-
-
-void RTP_Session::ExtendedReport::PrintOn(ostream & strm) const
-{
-  strm << "ssrc=" << sourceIdentifier
-       << " loss_rate=" << lossRate
-       << " discard_rate=" << discardRate
-       << " burst_density=" << burstDensity
-       << " gap_density=" << gapDensity
-       << " round_trip_delay=" << roundTripDelay
-       << " r_factor=" << RFactor
-       << " mos_lq=" << mosLQ
-       << " mos_cq=" << mosCQ
-       << " jb_nominal=" << jbNominal
-       << " jb_maximum=" << jbMaximum
-       << " jb_absolute=" << jbAbsolute;
-}
-
-#endif
-
-
 RTP_Session::SendReceiveStatus RTP_Session::OnSendData(RTP_DataFrame & frame)
 {
   return EncodingLock(*this)->OnSendData(frame);
@@ -1063,18 +953,20 @@ RTP_Session::SendReceiveStatus RTP_Session::Internal_OnSendData(RTP_DataFrame &
   frame.SetSequenceNumber(++lastSentSequenceNumber);
   frame.SetSyncSource(syncSourceOut);
 
+  DWORD frameTimestamp = frame.GetTimestamp();
+
   // special handling for first packet
   if (packetsSent == 0) {
 
     // establish timestamp offset
     if (oobTimeStampBaseEstablished)  {
-      timeStampOffs = oobTimeStampOutBase - frame.GetTimestamp() + ((PTimer::Tick() - oobTimeStampBase).GetInterval() * 8);
-      frame.SetTimestamp(frame.GetTimestamp() + timeStampOffs);
+      timeStampOffs = oobTimeStampOutBase - frameTimestamp + ((PTimer::Tick() - oobTimeStampBase).GetInterval() * m_timeUnits);
+      frameTimestamp += timeStampOffs;
     }
     else {
       oobTimeStampBaseEstablished = true;
       timeStampOffs               = 0;
-      oobTimeStampOutBase         = frame.GetTimestamp();
+      oobTimeStampOutBase         = frameTimestamp;
       oobTimeStampBase            = PTimer::Tick();
     }
 
@@ -1088,19 +980,18 @@ RTP_Session::SendReceiveStatus RTP_Session::Internal_OnSendData(RTP_DataFrame &
            << " m=" << frame.GetMarker()
            << " x=" << frame.GetExtension()
            << " seq=" << frame.GetSequenceNumber()
-           << " ts=" << frame.GetTimestamp()
+           << " ts=" << frameTimestamp
            << " src=" << hex << frame.GetSyncSource()
            << " ccnt=" << frame.GetContribSrcCount() << dec);
   }
 
   else {
     // set timestamp
-    DWORD ts = frame.GetTimestamp() + timeStampOffs;
-    frame.SetTimestamp(ts);
+    frameTimestamp += timeStampOffs;
 
     // reset OOB timestamp every marker bit
     if (frame.GetMarker()) {
-      oobTimeStampOutBase = ts;
+      oobTimeStampOutBase = frameTimestamp;
       oobTimeStampBase    = PTimer::Tick();
     }
 
@@ -1117,6 +1008,8 @@ RTP_Session::SendReceiveStatus RTP_Session::Internal_OnSendData(RTP_DataFrame &
     }
   }
 
+  frame.SetTimestamp(frameTimestamp);
+  lastSentTimestamp = frameTimestamp;
   lastSentPacketTime = tick;
 
   octetsSent += frame.GetPayloadSize();
@@ -1129,9 +1022,6 @@ RTP_Session::SendReceiveStatus RTP_Session::Internal_OnSendData(RTP_DataFrame &
   if (packetsSent == 1 && userData != NULL)
     userData->OnTxStatistics(*this);
 
-  if (!SendReport())
-    return e_AbortTransport;
-
   if (txStatisticsCount < txStatisticsInterval)
     return e_ProcessPacket;
 
@@ -1230,7 +1120,8 @@ RTP_Session::SendReceiveStatus RTP_Session::Internal_OnReceiveData(RTP_DataFrame
            << " ccnt=" << frame.GetContribSrcCount() << dec);
 
 #if OPAL_RTCP_XR
-    m_metrics.SetPayloadInfo(frame);
+    delete m_metrics; // Should be NULL, but just in case ...
+    m_metrics = RTCP_XR_Metrics::Create(frame);
 #endif
 
     if ((frame.GetPayloadType() == RTP_DataFrame::T38) &&
@@ -1259,12 +1150,6 @@ RTP_Session::SendReceiveStatus RTP_Session::Internal_OnReceiveData(RTP_DataFrame
         PTRACE(2, "RTP\tSession " << sessionID << ", packet from SSRC=" << hex << frame.GetSyncSource() << " ignored, expecting SSRC=" << syncSourceIn << dec);
         return e_IgnorePacket; // Non fatal error, just ignore
       }
-
-      JitterBufferPtr jitter = m_jitterBuffer; // Increase reference count
-      if (jitter != NULL) {
-        jitter->Reset();
-        PTRACE(4, "RTP\tSession " << sessionID << ", jitter buffer reset due to SSRC change.");
-      }
     }
 
     WORD sequenceNumber = frame.GetSequenceNumber();
@@ -1279,8 +1164,11 @@ RTP_Session::SendReceiveStatus RTP_Session::Internal_OnReceiveData(RTP_DataFrame
         packetsOutOfOrder++;
       }
 
-      // Only do statistics on packets after first received in talk burst
-      if ( ! (isAudio && frame.GetMarker()) ) {
+      /* For audio we do not do statistics on start of talk burst (marker bit)
+         as that could be a substantial time and is not really "jitter".
+         For video we measure jitter between whoile frames which is indicated
+         by the marker bit being on. */
+      if (frame.GetMarker() != isAudio) {
         DWORD diff = (tick - lastReceivedPacketTime).GetInterval();
 
         averageReceiveTimeAccum += diff;
@@ -1292,18 +1180,11 @@ RTP_Session::SendReceiveStatus RTP_Session::Internal_OnReceiveData(RTP_DataFrame
 
         // As per RFC3550 Appendix 8
         diff *= GetJitterTimeUnits(); // Convert to timestamp units
-        long variance = diff - lastTransitTime;
+        long variance = diff > lastTransitTime ? (diff - lastTransitTime) : (lastTransitTime - diff);
         lastTransitTime = diff;
-        if (variance < 0)
-          variance = -variance;
         jitterLevel += variance - ((jitterLevel+(1<<(JitterRoundingGuardBits-1))) >> JitterRoundingGuardBits);
         if (jitterLevel > maximumJitterLevel)
           maximumJitterLevel = jitterLevel;
-#if OPAL_RTCP_XR
-        JitterBufferPtr jitter = m_jitterBuffer; // Increase reference count in case gets deleted out from under us
-        if (jitter != NULL)
-          m_metrics.SetJitterDelay(jitter->GetCurrentJitterDelay()/jitter->GetTimeUnits());
-#endif
       }
 
       if (frame.GetMarker())
@@ -1318,7 +1199,7 @@ RTP_Session::SendReceiveStatus RTP_Session::Internal_OnReceiveData(RTP_DataFrame
     }
     else if (sequenceNumber < expectedSequenceNumber) {
 #if OPAL_RTCP_XR
-      m_metrics.OnPacketDiscarded();
+      if (m_metrics != NULL) m_metrics->OnPacketDiscarded();
 #endif
 
       // Check for Cisco bug where sequence numbers suddenly start incrementing
@@ -1352,9 +1233,22 @@ RTP_Session::SendReceiveStatus RTP_Session::Internal_OnReceiveData(RTP_DataFrame
         // the lowest numbered packet.
         SaveOutOfOrderPacket(frame);
 
-        frame = m_outOfOrderPackets.back();
-        m_outOfOrderPackets.pop_back();
-        sequenceNumber = frame.GetSequenceNumber();
+        for (;;) {
+          if (m_outOfOrderPackets.empty())
+            return e_IgnorePacket;
+
+          frame = m_outOfOrderPackets.back();
+          m_outOfOrderPackets.pop_back();
+
+          sequenceNumber = frame.GetSequenceNumber();
+          if (sequenceNumber >= expectedSequenceNumber)
+            break;
+
+          PTRACE(2, "RTP\tSession " << sessionID << ", ssrc=" << syncSourceIn
+                 << ", incorrect sequence after re-ordering, got "
+                 << sequenceNumber << " expected " << expectedSequenceNumber);
+        }
+
         outOfOrderPacketTime = tick;
       }
 
@@ -1366,7 +1260,7 @@ RTP_Session::SendReceiveStatus RTP_Session::Internal_OnReceiveData(RTP_DataFrame
       expectedSequenceNumber = (WORD)(sequenceNumber + 1);
       consecutiveOutOfOrderPackets = 0;
 #if OPAL_RTCP_XR
-      m_metrics.OnPacketLost(dropped);
+      if (m_metrics != NULL) m_metrics->OnPacketLost(dropped);
 #endif
     }
   }
@@ -1377,16 +1271,13 @@ RTP_Session::SendReceiveStatus RTP_Session::Internal_OnReceiveData(RTP_DataFrame
   packetsReceived++;
   
 #if OPAL_RTCP_XR
-  m_metrics.OnPacketReceived();
+  if (m_metrics != NULL) m_metrics->OnPacketReceived();
 #endif
 
   // Call the statistics call-back on the first PDU with total count == 1
   if (packetsReceived == 1 && userData != NULL)
     userData->OnRxStatistics(*this);
 
-  if (!SendReport())
-    return e_AbortTransport;
-
   if (rxStatisticsCount >= rxStatisticsInterval) {
 
     rxStatisticsCount = 0;
@@ -1507,30 +1398,17 @@ PBoolean RTP_Session::InsertReportPacket(RTP_ControlFrame & report)
   }
 
   report.EndPacket();
-
-  // Wait a fuzzy amount of time so things don't get into lock step
-  int interval = (int)reportTimeInterval.GetMilliSeconds();
-  int third = interval/3;
-  interval += PRandom::Number()%(2*third);
-  interval -= third;
-  reportTimer = interval;
-
   return true;
 }
 
 
-PBoolean RTP_Session::SendReport()
+void RTP_Session::SendReport(PTimer&, INT)
 {
-  PWaitAndSignal mutex(reportMutex);
-
-  if (reportTimer.IsRunning())
-    return true;
+  PWaitAndSignal mutex(m_reportMutex);
 
   // Have not got anything yet, do nothing
-  if (packetsSent == 0 && packetsReceived == 0) {
-    reportTimer = reportTimeInterval;
-    return true;
-  }
+  if (packetsSent == 0 && packetsReceived == 0)
+    return;
 
   RTP_ControlFrame report;
   InsertReportPacket(report);
@@ -1547,10 +1425,10 @@ PBoolean RTP_Session::SendReport()
   
 #if OPAL_RTCP_XR
   //Generate and send RTCP-XR packet
-  InsertExtendedReportPacket(report);
+  if (m_metrics != NULL) m_metrics->InsertExtendedReportPacket(sessionID, syncSourceOut, m_jitterBuffer, report);
 #endif
 
-  return WriteControl(report);
+  WriteControl(report);
 }
 
 
@@ -1573,8 +1451,8 @@ void RTP_Session::GetStatistics(OpalMediaStatistics & statistics, bool receiver)
 #endif
 
 
-static RTP_Session::ReceiverReportArray
-BuildReceiverReportArray(const RTP_ControlFrame & frame, PINDEX offset)
+RTP_Session::ReceiverReportArray
+RTP_Session::BuildReceiverReportArray(const RTP_ControlFrame & frame, PINDEX offset)
 {
   RTP_Session::ReceiverReportArray reports;
 
@@ -1589,6 +1467,9 @@ BuildReceiverReportArray(const RTP_ControlFrame & frame, PINDEX offset)
     report->lastTimestamp = (PInt64)(DWORD)rr->lsr;
     report->delay = ((PInt64)rr->dlsr << 16)/1000;
     reports.SetAt(repIdx, report);
+#if OPAL_RTCP_XR
+    if (m_metrics != NULL) m_metrics->OnRxSenderReport(rr->lsr, rr->dlsr);
+#endif
     rr++;
   }
 
@@ -1624,9 +1505,7 @@ RTP_Session::SendReceiveStatus RTP_Session::OnReceiveControl(RTP_ControlFrame &
           sender.rtpTimestamp = sr.rtp_ts;
           sender.packetsSent = sr.psent;
           sender.octetsSent = sr.osent;
-#if OPAL_RTCP_XR
-          OnRxSenderReportToMetrics(frame, sizeof(PUInt32b)+sizeof(RTP_ControlFrame::SenderReport));
-#endif
+
           OnRxSenderReport(sender, BuildReceiverReportArray(frame, sizeof(PUInt32b)+sizeof(RTP_ControlFrame::SenderReport)));
         }
         else {
@@ -1722,7 +1601,7 @@ RTP_Session::SendReceiveStatus RTP_Session::OnReceiveControl(RTP_ControlFrame &
 #if OPAL_RTCP_XR
       case RTP_ControlFrame::e_ExtendedReport :
         if (size >= (PINDEX)(sizeof(PUInt32b)+frame.GetCount()*sizeof(RTP_ControlFrame::ExtendedReport)))
-          OnRxExtendedReport(*(const PUInt32b *)payload, BuildExtendedReportArray(frame, sizeof(PUInt32b)));
+          OnRxExtendedReport(*(const PUInt32b *)payload, RTCP_XR_Metrics::BuildExtendedReportArray(frame, sizeof(PUInt32b)));
         else {
           PTRACE(2, "RTP\tSession " << sessionID << ", ReceiverReport packet truncated");
         }
@@ -2016,7 +1895,6 @@ PBoolean RTP_UDP::Open(PIPSocket::Address transportLocalAddress,
 {
   PWaitAndSignal mutex(dataMutex);
 
-  m_firstData = true;
   m_firstControl = true;
 
   // save local address 
@@ -2129,6 +2007,7 @@ PBoolean RTP_UDP::Open(PIPSocket::Address transportLocalAddress,
          << localAddress << ':' << localDataPort << '-' << localControlPort
          << " ssrc=" << syncSourceOut);
 
+  m_reportTimer.RunContinuous(m_reportTimer.GetResetTime());
   return true;
 }
 
@@ -2137,12 +2016,19 @@ void RTP_UDP::Reopen(PBoolean reading)
 {
   PWaitAndSignal mutex(dataMutex);
 
-  if (reading)
+  if (reading) {
+    if (!shutdownRead)
+      return;
     shutdownRead = false;
-  else
+  }
+  else {
+    if (!shutdownWrite)
+      return;
     shutdownWrite = false;
+  }
 
   badTransmitCounter = 0;
+  m_reportTimer.RunContinuous(m_reportTimer.GetResetTime());
 
   PTRACE(3, "RTP_UDP\tSession " << sessionID << " reopened for " << (reading ? "reading" : "writing"));
 }
@@ -2166,10 +2052,11 @@ bool RTP_UDP::Close(PBoolean reading)
 
       if (dataSocket != NULL && controlSocket != NULL) {
         PIPSocket::Address addr;
-        controlSocket->GetLocalAddress(addr);
+        WORD port;
+        controlSocket->PUDPSocket::GetLocalAddress(addr, port);
         if (addr.IsAny())
           PIPSocket::GetHostAddress(addr);
-        dataSocket->WriteTo("", 1, addr, controlSocket->GetPort());
+        dataSocket->WriteTo("", 1, addr, port);
       }
     }
 
@@ -2185,6 +2072,9 @@ bool RTP_UDP::Close(PBoolean reading)
     shutdownWrite = true;
   }
 
+  if (shutdownRead && shutdownWrite)
+    m_reportTimer.Stop(false);
+
   return true;
 }
 
@@ -2254,72 +2144,57 @@ PBoolean RTP_UDP::ReadData(RTP_DataFrame & frame)
 
 PBoolean RTP_UDP::Internal_ReadData(RTP_DataFrame & frame)
 {
-  for (;;) {
-    int selectStatus = WaitForPDU(*dataSocket, *controlSocket, reportTimer);
+  SendReceiveStatus receiveStatus = e_IgnorePacket;
+  while (receiveStatus == e_IgnorePacket) {
+    if (shutdownRead || PAssertNULL(dataSocket) == NULL || PAssertNULL(controlSocket) == NULL)
+      return false;
 
-    {
-      PWaitAndSignal mutex(dataMutex);
-      if (shutdownRead) {
-        PTRACE(3, "RTP_UDP\tSession " << sessionID << ", Read shutdown.");
-        return false;
-      }
+    int selectStatus = WaitForPDU(*dataSocket, *controlSocket, PMaxTimeInterval);
+    if (shutdownRead)
+      return false;
+
+    if (selectStatus > 0) {
+      PTRACE(1, "RTP_UDP\tSession " << sessionID << ", Select error: "
+              << PChannel::GetErrorText((PChannel::Errors)selectStatus));
+      return false;
     }
 
-    switch (selectStatus) {
-      case -2 :
-        if (ReadControlPDU() == e_AbortTransport)
-          return false;
-        break;
+    if (selectStatus == 0)
+      receiveStatus = OnReadTimeout(frame);
 
-      case -3 :
-        if (ReadControlPDU() == e_AbortTransport)
-          return false;
-        // Then do -1 case
-
-      case -1 :
-        switch (ReadDataPDU(frame)) {
-          case e_ProcessPacket :
-            if (!shutdownRead) {
-              switch (OnReceiveData(frame)) {
-                case e_ProcessPacket :
-                  return true;
-                case e_IgnorePacket :
-                  break;
-                case e_AbortTransport :
-                  return false;
-              }
-            }
-          case e_IgnorePacket :
-            break;
-          case e_AbortTransport :
-            return false;
-        }
-        break;
-
-      case 0 :
-        switch (OnReadTimeout(frame)) {
-          case e_ProcessPacket :
-            if (!shutdownRead)
-              return true;
-          case e_IgnorePacket :
-            break;
-          case e_AbortTransport :
-            return false;
-        }
-        break;
-
-      case PSocket::Interrupted:
-        PTRACE(2, "RTP_UDP\tSession " << sessionID << ", Interrupted.");
-        return false;
-
-      default :
-        PTRACE(1, "RTP_UDP\tSession " << sessionID << ", Select error: "
-                << PChannel::GetErrorText((PChannel::Errors)selectStatus));
+    if ((-selectStatus & 2) != 0) {
+      if (ReadControlPDU() == e_AbortTransport)
         return false;
     }
+
+    if ((-selectStatus & 1) != 0)
+      receiveStatus = ReadDataPDU(frame);
   }
+
+  return receiveStatus == e_ProcessPacket;
 }
 
+
+void RTP_UDP::FlushData()
+{
+  if (shutdownRead || dataSocket == NULL)
+    return;
+
+  PTimeInterval oldTimeout = dataSocket->GetReadTimeout();
+  dataSocket->SetReadTimeout(0);
+
+  PINDEX count = 0;
+  BYTE buffer[2000];
+  while (dataSocket->Read(buffer, sizeof(buffer)))
+    ++count;
+
+  dataSocket->SetReadTimeout(oldTimeout);
+
+  PTRACE_IF(3, count > 0, "RTP_UDP\tSession " << sessionID << ", flushed "
+            << count << " RTP data packets before activating jitter buffer");
+}
+
+
 int RTP_UDP::WaitForPDU(PUDPSocket & dataSocket, PUDPSocket & controlSocket, const PTimeInterval & timeout)
 {
   return EncodingLock(*this)->WaitForPDU(dataSocket, controlSocket, timeout);
@@ -2327,21 +2202,6 @@ int RTP_UDP::WaitForPDU(PUDPSocket & dataSocket, PUDPSocket & controlSocket, con
 
 int RTP_UDP::Internal_WaitForPDU(PUDPSocket & dataSocket, PUDPSocket & controlSocket, const PTimeInterval & timeout)
 {
-  if (m_firstData && isAudio) {
-    PTimeInterval oldTimeout = dataSocket.GetReadTimeout();
-    dataSocket.SetReadTimeout(0);
-
-    BYTE buffer[2000];
-    PINDEX count = 0;
-    while (dataSocket.Read(buffer, sizeof(buffer)))
-      ++count;
-
-    PTRACE_IF(3, count > 0, "RTP_UDP\tSession " << sessionID << ", flushed " << count << " RTP data packets on startup");
-
-    dataSocket.SetReadTimeout(oldTimeout);
-    m_firstData = false;
-  }
-
   return PSocket::Select(dataSocket, controlSocket, timeout);
 }
 
@@ -2419,6 +2279,11 @@ RTP_Session::SendReceiveStatus RTP_UDP::ReadDataOrControlPDU(BYTE * framePtr,
       // Shouldn't happen, but it does.
       return RTP_Session::e_IgnorePacket;
 
+    case 0 :
+      PTRACE(4, "RTP_UDP\tSession " << sessionID << ", " << channelName
+             << " received UDP packet with no payload.");
+      return e_IgnorePacket;
+
     default:
       PTRACE(1, "RTP_UDP\tSession " << sessionID << ", " << channelName
              << " read error (" << socket.GetErrorNumber(PChannel::LastReadError) << "): "
@@ -2440,7 +2305,9 @@ RTP_Session::SendReceiveStatus RTP_UDP::Internal_ReadDataPDU(RTP_DataFrame & fra
     return status;
 
   // Check received PDU is big enough
-  return frame.SetPacketSize(dataSocket->GetLastReadCount()) ? e_ProcessPacket : e_IgnorePacket;
+  if (frame.SetPacketSize(dataSocket->GetLastReadCount()))
+    return OnReceiveData(frame);
+  return e_IgnorePacket;
 }
 
 
@@ -2463,7 +2330,7 @@ RTP_Session::SendReceiveStatus RTP_UDP::OnReadTimeout(RTP_DataFrame & frame)
 
 RTP_Session::SendReceiveStatus RTP_UDP::Internal_OnReadTimeout(RTP_DataFrame & /*frame*/)
 {
-  return SendReport() ? e_IgnorePacket : e_AbortTransport;
+  return e_IgnorePacket;
 }
 
 
@@ -2519,12 +2386,9 @@ PBoolean RTP_UDP::WriteData(RTP_DataFrame & frame)
 
 PBoolean RTP_UDP::Internal_WriteData(RTP_DataFrame & frame)
 {
-  {
-    PWaitAndSignal mutex(dataMutex);
-    if (shutdownWrite) {
-      PTRACE(3, "RTP_UDP\tSession " << sessionID << ", write shutdown.");
-      return false;
-    }
+  if (shutdownWrite || dataSocket == NULL) {
+    PTRACE(3, "RTP_UDP\tSession " << sessionID << ", write shutdown.");
+    return false;
   }
 
   // Trying to send a PDU before we are set up!
