@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 27008 $
- * $Author: rjongbloed $
- * $Date: 2012-02-19 20:47:08 -0600 (Sun, 19 Feb 2012) $
+ * $Revision$
+ * $Author$
+ * $Date$
  *
  * This code implements all or part of the following RFCs
  *
@@ -75,7 +75,8 @@ ostream & operator<<(ostream & strm, SIPHandler::State state)
 
 SIPHandler::SIPHandler(SIP_PDU::Methods method, SIPEndPoint & ep, const SIPParameters & params)
   : endpoint(ep)
-  , authentication(NULL)
+  , m_authentication(NULL)
+  , m_authenticateErrors(0)
   , m_username(params.m_authID)
   , m_password(params.m_password)
   , m_realm(params.m_realm)
@@ -91,6 +92,7 @@ SIPHandler::SIPHandler(SIP_PDU::Methods method, SIPEndPoint & ep, const SIPParam
   , m_state(Unavailable)
   , m_receivedResponse(false)
   , m_proxy(params.m_proxyAddress)
+  , m_retry403(params.m_retry403)
 {
   m_transactions.DisallowDeleteObjects();
   m_expireTimer.SetNotifier(PCREATE_NOTIFIER(OnExpireTimeout));
@@ -111,7 +113,7 @@ SIPHandler::~SIPHandler()
     delete m_transport;
   }
 
-  delete authentication;
+  delete m_authentication;
 
   PTRACE(4, "SIP\tDestroyed " << m_method << " handler for " << m_addressOfRecord);
 }
@@ -303,25 +305,29 @@ OpalTransport * SIPHandler::GetTransport()
     m_transport = NULL;
   }
 
-  if (m_proxy.IsEmpty()) {
-    // Look for a "proxy" parameter to override default proxy
-    const PStringToString & params = m_remoteAddress.GetParamVars();
-    if (params.Contains(OPAL_PROXY_PARAM)) {
-      m_proxy.Parse(params(OPAL_PROXY_PARAM));
-      m_remoteAddress.SetParamVar(OPAL_PROXY_PARAM, PString::Empty());
-    }
+  const PStringToString & remoteParams = m_remoteAddress.GetParamVars();
+
+  // Look for a "proxy" parameter to override default proxy
+  if (m_proxy.IsEmpty() && remoteParams.Contains(OPAL_PROXY_PARAM)) {
+    m_proxy.Parse(remoteParams(OPAL_PROXY_PARAM));
+    m_remoteAddress.SetParamVar(OPAL_PROXY_PARAM, PString::Empty());
   }
 
   SIPURL url;
-  if (!m_proxy.IsEmpty())
+  if (!m_proxy.IsEmpty()) {
     url = m_proxy;
+    url.AdjustToDNS(endpoint);
+  }
   else {
     url = m_remoteAddress;
-    url.AdjustToDNS();
+    url.AdjustToDNS(endpoint);
   }
 
-  // Must specify a network interface or get infinite recursion
-  m_transport = endpoint.CreateTransport(url, "*");
+  PString localInterface = remoteParams(OPAL_INTERFACE_PARAM);
+  if (localInterface.IsEmpty())
+    localInterface = "*"; // Must specify a network interface or get infinite recursion
+
+  m_transport = endpoint.CreateTransport(url, localInterface);
   return m_transport;
 }
 
@@ -375,11 +381,12 @@ bool SIPHandler::WriteSIPHandler(OpalTransport & transport, bool forked)
   if (GetState() == Unsubscribing)
     mime.SetExpires(0);
 
-  if (authentication != NULL) {
+  if (m_authentication != NULL) {
     SIPAuthenticator auth(*transaction);
-    authentication->Authorise(auth); // If already have info from last time, use it!
+    m_authentication->Authorise(auth); // If already have info from last time, use it!
   }
 
+  PTRACE(2, "SIP\tStarting transaction on " << transport);
   if (transaction->Start()) {
     m_transactions.Append(transaction);
     return true;
@@ -407,21 +414,32 @@ void SIPHandler::OnReceivedResponse(SIPTransaction & transaction, SIP_PDU & resp
 
   m_transactions.Remove(&transaction); // Take this transaction out of list
 
-  // And kill all the rest
-  PSafePtr<SIPTransaction> transToGo;
-  while ((transToGo = m_transactions.GetAt(0)) != NULL) {
-    m_transactions.Remove(transToGo);
-    transToGo->Abort();
-  }
+  switch (response.GetStatusCode()) {
+    default :
+      if (responseClass != 2)
+        break;
 
-  // Finally end connect mode on the transport
-  m_transport->SetInterface(transaction.GetInterface());
+    case SIP_PDU::Failure_UnAuthorised :
+    case SIP_PDU::Failure_ProxyAuthenticationRequired :
+    case SIP_PDU::Failure_IntervalTooBrief :
+    case SIP_PDU::Failure_TemporarilyUnavailable:
+
+      // And kill all the rest
+      PSafePtr<SIPTransaction> transToGo;
+      while ((transToGo = m_transactions.GetAt(0)) != NULL) {
+        m_transactions.Remove(transToGo);
+        transToGo->Abort();
+      }
+
+      // Finally end connect mode on the transport
+      m_transport->SetInterface(transaction.GetInterface());
+  }
 
   switch (response.GetStatusCode()) {
     case SIP_PDU::Failure_UnAuthorised :
     case SIP_PDU::Failure_ProxyAuthenticationRequired :
       OnReceivedAuthenticationRequired(transaction, response);
-      break;
+      return;
 
     case SIP_PDU::Failure_IntervalTooBrief :
       OnReceivedIntervalTooBrief(transaction, response);
@@ -431,12 +449,18 @@ void SIPHandler::OnReceivedResponse(SIPTransaction & transaction, SIP_PDU & resp
       OnReceivedTemporarilyUnavailable(transaction, response);
       break;
 
+    case SIP_PDU::Failure_Forbidden:
+      OnReceivedForbidden(transaction, response);
+      break;
+
     default :
       if (responseClass == 2)
         OnReceivedOK(transaction, response);
       else
         OnFailed(response);
   }
+
+  m_authenticateErrors = 0;
 }
 
 
@@ -449,6 +473,16 @@ void SIPHandler::OnReceivedIntervalTooBrief(SIPTransaction & /*transaction*/, SI
 }
 
 
+void SIPHandler::OnReceivedForbidden(SIPTransaction & /*transaction*/, SIP_PDU & response)
+{
+  OnFailed(SIP_PDU::Failure_Forbidden);
+  if (m_retry403) {
+    PTRACE(2, "SIP\tRetrying on 403 Forbidden (in violation of RFC 3261) on " << m_transport);
+    RetryLater(response.GetMIME().GetInteger("Retry-After", m_offlineExpireTime));
+  }
+}
+
+
 void SIPHandler::OnReceivedTemporarilyUnavailable(SIPTransaction & /*transaction*/, SIP_PDU & response)
 {
   OnFailed(SIP_PDU::Failure_TemporarilyUnavailable);
@@ -458,70 +492,24 @@ void SIPHandler::OnReceivedTemporarilyUnavailable(SIPTransaction & /*transaction
 
 void SIPHandler::OnReceivedAuthenticationRequired(SIPTransaction & transaction, SIP_PDU & response)
 {
-  bool isProxy = response.GetStatusCode() == SIP_PDU::Failure_ProxyAuthenticationRequired;
-
-#if PTRACING
-  const char * proxyTrace = isProxy ? "Proxy " : "";
-#endif
-  PTRACE(3, "SIP\tReceived " << proxyTrace << "Authentication Required response");
-  
-  // authenticate 
-  PString errorMsg;
-  SIPAuthentication * newAuth = PHTTPClientAuthentication::ParseAuthenticationRequired(isProxy, response.GetMIME(), errorMsg);
-  if (newAuth == NULL) {
-    PTRACE(2, "SIP\t" << errorMsg);
-    OnFailed(SIP_PDU::Failure_Forbidden);
-    return;
-  }
-
   // If either username or password blank, try and fine values from other
   // handlers which might be logged into the realm, or the proxy, if one.
-  PString username = m_username;
-  PString password = m_password;
-  if (username.IsEmpty() || password.IsEmpty()) {
-    // Try to find authentication parameters for the given realm,
-    // if not, use the proxy authentication parameters (if any)
-    if (endpoint.GetAuthentication(newAuth->GetAuthRealm(), username, password)) {
-      PTRACE (3, "SIP\tFound auth info for realm " << newAuth->GetAuthRealm());
-    }
-    else if (username.IsEmpty()) {
-      if (m_proxy.IsEmpty()) {
-        delete newAuth;
-        PTRACE(2, "SIP\tAuthentication not possible yet, no credentials available.");
-        OnFailed(SIP_PDU::Failure_TemporarilyUnavailable);
-        if (!transaction.IsCanceled())
-          RetryLater(m_offlineExpireTime);
-        return;
-      }
-
-      PTRACE (3, "SIP\tNo auth info for realm " << newAuth->GetAuthRealm() << ", using proxy auth");
-      username = m_proxy.GetUserName();
-      password = m_proxy.GetPassword();
-    }
-  }
-
-  newAuth->SetUsername(username);
-  newAuth->SetPassword(password);
-
-  // check to see if this is a follow-on from the last authentication scheme used
-  if (GetState() == Subscribing && authentication != NULL && *newAuth == *authentication) {
-    delete newAuth;
-    PTRACE(1, "SIP\tAuthentication already performed using current credentials, not trying again.");
-    OnFailed(SIP_PDU::Failure_UnAuthorised);
+  SIP_PDU::StatusCodes status = endpoint.HandleAuthentication(m_authentication,
+                                                              m_authenticateErrors,
+                                                              response,
+                                                              GetProxy(),
+                                                              m_username,
+                                                              m_password);
+  if (status != SIP_PDU::Successful_OK) {
+    OnFailed(status);
+    if (GetState() != Unsubscribing && !transaction.IsCanceled())
+      RetryLater(m_offlineExpireTime);
     return;
   }
 
-  PTRACE(4, "SIP\tUpdating authentication credentials.");
-
-  // switch authentication schemes
-  delete authentication;
-  authentication = newAuth;
-  m_username = username;
-  m_password = password;
-
   // If we changed realm (or hadn't got one yet) update the handler database
-  if (m_realm != newAuth->GetAuthRealm()) {
-    m_realm = newAuth->GetAuthRealm();
+  if (m_realm != m_authentication->GetAuthRealm()) {
+    m_realm = m_authentication->GetAuthRealm();
     PTRACE(3, "SIP\tAuth realm set to " << m_realm);
     endpoint.UpdateHandlerIndexes(this);
   }
@@ -593,6 +581,14 @@ void SIPHandler::OnFailed(SIP_PDU::StatusCodes code)
         SetState(Unavailable);
         break;
       }
+    case SIP_PDU::Failure_Forbidden:
+      if (m_retry403) {
+        if (GetState() != Unsubscribing) {
+          SetState(Unavailable);
+          break;
+        }
+      }
+
       // Do next case to finalise Unsubscribe even though there was an error
 
     default :
@@ -619,6 +615,7 @@ void SIPHandler::OnExpireTimeout(PTimer &, INT)
 
     case Unavailable :
       PTRACE(2, "SIP\tStarting " << GetMethod() << " for offline retry");
+      m_addressOfRecord.TryNextSRV(endpoint);
       if (SendRequest(Restoring))
         return;
       break;
@@ -759,6 +756,7 @@ void SIPRegisterHandler::OnReceivedOK(SIPTransaction & transaction, SIP_PDU & re
   for (SIPURLList::iterator reply = replyContacts.begin(); reply != replyContacts.end(); ) {
     if (reply->GetHostAddress() == externalAddress) {
       externalAddress.MakeEmpty(); // Clear this so no further action taken
+      m_externalAddress.MakeEmpty();
       ++reply;
     }
     else if (std::find(requestedContacts.begin(), requestedContacts.end(), *reply) != requestedContacts.end())
@@ -792,27 +790,29 @@ void SIPRegisterHandler::OnReceivedOK(SIPTransaction & transaction, SIP_PDU & re
     return;
   }
 
-  // Remember (possibly new) NAT address
-  m_externalAddress == externalAddress;
-
   if (GetExpire() == 0) {
     // If we had discovered we are behind NAT and had unregistered, re-REGISTER with new addresses
-    PTRACE(2, "SIP\tRe-registering with NAT address " << externalAddress);
+    PTRACE(2, "SIP\tRe-registering NAT address change (" << m_contactAddresses << ") to " << externalAddress);
     for (SIPURLList::iterator contact = m_contactAddresses.begin(); contact != m_contactAddresses.end(); ++contact)
       contact->SetHostAddress(externalAddress);
+    m_contactAddresses.unique();
     SetExpire(m_originalExpireTime);
   }
   else {
     /* If we got here then we have done a successful register, but registrar indicated
        that we are behind firewall. Unregister what we just registered */
-    PTRACE(2, "SIP\tRemote indicated change of REGISTER Contact header required due to NAT");
     for (SIPURLList::iterator contact = replyContacts.begin(); contact != replyContacts.end(); ++contact)
       contact->GetFieldParameters().Remove("expires");
+    PTRACE(2, "SIP\tRemote indicated change of REGISTER Contact address(s) (" << replyContacts
+           << ") required due to NAT address " << externalAddress << ", previous=" << m_externalAddress);
     m_contactAddresses = replyContacts;
     SetExpire(0);
   }
 
-  SendRequest(previousState);
+  // Remember (possibly new) NAT address
+  m_externalAddress == externalAddress;
+
+  SendRequest(Refreshing);
   SendStatus(SIP_PDU::Information_Trying, previousState);
 }
 
@@ -885,8 +885,11 @@ void SIPRegisterHandler::UpdateParameters(const SIPRegister::Params & params)
   if (params.m_expire > 0)
     SetExpire(m_parameters.m_expire = params.m_expire);
 
+  m_parameters.m_compatibility = params.m_compatibility;
   m_parameters.m_contactAddress = params.m_contactAddress;
   m_contactAddresses.clear();
+
+  PTRACE(4, "SIP\tREGISTER parameters updated.");
 }
 
 
@@ -1068,6 +1071,8 @@ PBoolean SIPSubscribeHandler::OnReceivedNOTIFY(SIP_PDU & request)
 
   SIPMIMEInfo & requestMIME = request.GetMIME();
 
+  requestMIME.GetProductInfo(m_productInfo);
+
   // If we received this NOTIFY before, send the previous response
   if (m_dialog.IsDuplicateCSeq(requestMIME.GetCSeqIndex())) {
 
@@ -1089,9 +1094,8 @@ PBoolean SIPSubscribeHandler::OnReceivedNOTIFY(SIP_PDU & request)
   PStringToString subscriptionStateInfo;
   PCaselessString subscriptionState = requestMIME.GetSubscriptionState(subscriptionStateInfo);
   if (subscriptionState.IsEmpty()) {
-    PTRACE(2, "SIP\tNOTIFY received without Subscription-State");
-    m_previousResponse->SetInfo("No Subscription-State field");
-    return request.SendResponse(*m_transport, *m_previousResponse, &endpoint);
+    PTRACE(2, "SIP\tNOTIFY received without Subscription-State, assuming 'active'");
+    subscriptionState = "active";
   }
 
   // Check the susbscription state
@@ -1302,7 +1306,12 @@ class SIPMwiEventPackageHandler : public SIPEventPackageHandler
 
     PMIMEInfo info(request.GetEntityBody());
 
-    PString account = info.Get("Message-Account", handler.GetAddressOfRecord().AsString());
+    const SIPURL & aor = handler.GetAddressOfRecord();
+    PString account = info.Get("Message-Account");
+    SIPURL accountURI(account);
+    if (account.IsEmpty() || aor.GetUserName() == account ||
+            (accountURI.GetUserName() == "asterisk" && accountURI.GetHostName() == aor.GetHostName()))
+      account = aor.AsString();
 
     bool nothingSent = true;
     for (PINDEX z = 0 ; z < PARRAYSIZE(validMessageClasses); z++) {
@@ -1316,7 +1325,7 @@ class SIPMwiEventPackageHandler : public SIPEventPackageHandler
     // lower case yes/no to call back function, regardless of what comes down the wire.
     if (nothingSent)
       handler.GetEndPoint().OnMWIReceived(account, OpalManager::NumMessageWaitingTypes,
-                                 (info.Get("Message-Waiting") *= "yes") ? "yes" : "no");
+                                 (info.Get("Messages-Waiting") *= "yes") ? "yes" : "no");
 
     return true;
   }
@@ -1357,8 +1366,11 @@ class SIPPresenceEventPackageHandler : public SIPEventPackageHandler
     else {
       PString error;
       PString body = request.GetEntityBody();
-      if (handler.GetProductInfo().name.Find("Asterisk") != P_MAX_INDEX)
+      if (handler.GetProductInfo().name.Find("Asterisk") != P_MAX_INDEX) {
+        PTRACE(4, "SIP\tCompensating for " << handler.GetProductInfo().name << ","
+                  " replacing " << to.AsString() << " with " << from.AsString());
         body.Replace(to.AsString(), from.AsString());
+      }
       if (!SIPPresenceInfo::ParseXML(body, infoList, error))
         return false;
     }
@@ -2125,15 +2137,16 @@ SIPOptionsHandler::SIPOptionsHandler(SIPEndPoint & endpoint, const SIPOptions::P
 
   m_offlineExpireTime = 0; // No retries for offline, just give up
 
+  /* The "singleton" operation of the OPTION command is executed as though we
+     are unsubscribing from an existing subscription, so when completed the
+     handler is disposed of. So, we need to fake the subscribe state. */
   SetState(Subscribed);
+  m_receivedResponse = true;
 }
 
 
 SIPTransaction * SIPOptionsHandler::CreateTransaction(OpalTransport & transport)
 { 
-  if (GetState() == Unsubscribing)
-    return NULL;
-
   return new SIPOptions(endpoint, transport, GetCallID(), m_parameters);
 }
 
