@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26878 $
- * $Author: rjongbloed $
- * $Date: 2012-01-20 23:06:22 -0600 (Fri, 20 Jan 2012) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -41,6 +41,7 @@
 
 #include <ptlib/socket.h>
 #include <opal/transports.h>
+#include <codec/opalplugin.h>
 
 
 #define SIP_DEFAULT_SESSION_NAME    "Opal SIP Session"
@@ -386,7 +387,7 @@ void SDPMediaFormat::SetMediaFormatOptions(OpalMediaFormat & mediaFormat) const
 
 bool SDPMediaFormat::PreEncode()
 {
-  m_mediaFormat.SetOptionString(OpalMediaFormat::ProtocolOption(), "SIP");
+  m_mediaFormat.SetOptionString(OpalMediaFormat::ProtocolOption(), PLUGINCODEC_OPTION_PROTOCOL_SIP);
   return m_mediaFormat.ToCustomisedOptions();
 }
 
@@ -440,7 +441,7 @@ bool SDPMediaFormat::PostDecode(const OpalMediaFormatList & mediaFormats, unsign
     m_mediaFormat.SetOptionInteger(OpalMediaFormat::MaxBitRateOption(), bandwidth);
   }
 
-  m_mediaFormat.SetOptionString(OpalMediaFormat::ProtocolOption(), "SIP");
+  m_mediaFormat.SetOptionString(OpalMediaFormat::ProtocolOption(), PLUGINCODEC_OPTION_PROTOCOL_SIP);
   if (m_mediaFormat.ToNormalisedOptions())
     return true;
 
@@ -1090,12 +1091,16 @@ bool SDPAudioMediaDescription::PrintOn(ostream & str, const PString & connectStr
      what should be rare cases.
     */
 
+  PTRACE(2, "SDP\tm_offerPTime: " << m_offerPTime);
   if (m_offerPTime) {
     unsigned ptime = 0;
     for (SDPMediaFormatList::const_iterator format = formats.begin(); format != formats.end(); ++format) {
+      // PTRACE(2, "SDP\tformat: " << format);
       const OpalMediaFormat & mediaFormat = format->GetMediaFormat();
+      PTRACE(2, "SDP\tmedia format: " << mediaFormat);
       if (mediaFormat.HasOption(OpalAudioFormat::TxFramesPerPacketOption())) {
         unsigned ptime1 = mediaFormat.GetOptionInteger(OpalAudioFormat::TxFramesPerPacketOption()) * mediaFormat.GetFrameTime() / mediaFormat.GetTimeUnits();
+        PTRACE(2, "SDP\tptime1: " << ptime1);
         if (ptime < ptime1)
           ptime = ptime1;
       }
@@ -1103,6 +1108,8 @@ bool SDPAudioMediaDescription::PrintOn(ostream & str, const PString & connectStr
     if (ptime > 0)
       str << "a=ptime:" << ptime << "\r\n";
   }
+  else
+    str << "a=ptime: 20\r\n";   // LXK hard code ptime to 20 msecs for Verizon Certification
 
   unsigned largestFrameTime = 0;
   unsigned maxptime = UINT_MAX;
