@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26951 $
- * $Author: rjongbloed $
- * $Date: 2012-02-07 21:15:47 -0600 (Tue, 07 Feb 2012) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -215,18 +215,23 @@ void SIPEndPoint::NATBindingRefresh(PTimer &, INT)
     for (PSafePtr<SIPHandler> handler = activeSIPHandlers.GetFirstHandler(PSafeReadOnly); handler != NULL; ++handler) {
 
       OpalTransport * transport = NULL;
-      if (handler->GetState () != SIPHandler::Subscribed ||
+      if (  handler->GetState () != SIPHandler::Subscribed ||
+           (handler->GetMethod() != SIP_PDU::Method_REGISTER &&
+            handler->GetMethod() != SIP_PDU::Method_SUBSCRIBE) ||
            (transport = handler->GetTransport()) == NULL ||
-           transport->IsReliable() ||
+            transport->IsReliable() ||
            GetManager().GetNatMethod(transport->GetRemoteAddress().GetHostName()) == NULL)
         continue;
 
+      PTRACE(4, "SIP\tNAT Binding refresh for " << handler->GetMethod()
+             << " to " << handler->GetAddressOfRecord() << ", id=" << handler->GetCallID());
       switch (natMethod) {
-
         case Options:
           {
             SIPOptions::Params params;
+            params.m_addressOfRecord = handler->GetAddressOfRecord();
             params.m_remoteAddress = transport->GetRemoteAddress().GetHostName();
+            params.m_interface = transport->GetInterface();
             SendOPTIONS(params);
           }
           break;
@@ -251,7 +256,7 @@ OpalTransport * SIPEndPoint::CreateTransport(const SIPURL & remoteURL, const PSt
   if (remoteAddress.IsEmpty()) {
     if (GetRegistrationsCount() == 0) {
       PTRACE(1, "SIP\tCannot use tel URI with phone-context or existing registration.");
-      return false;
+      return NULL;
     }
     remoteAddress = SIPURL(GetRegistrations()[0]).GetHostAddress();
   }
@@ -559,7 +564,7 @@ PBoolean SIPEndPoint::OnReceivedPDU(OpalTransport & transport, SIP_PDU * pdu)
                 break;
 
               case SIPConnection::IsDuplicateINVITE : // Completely ignore duplicate INVITE
-                return true;
+                return false;
 
               case SIPConnection::IsReINVITE : // Pass on to worker thread if re-INVITE
                 m_connectionThreadPool.AddWork(new SIP_Work(*this, pdu, token), token);
@@ -569,7 +574,7 @@ PBoolean SIPEndPoint::OnReceivedPDU(OpalTransport & transport, SIP_PDU * pdu)
                 SIP_PDU response(*pdu, SIP_PDU::Failure_LoopDetected);
                 response.GetMIME().SetProductInfo(GetUserAgent(), connection->GetProductInfo());
                 pdu->SendResponse(transport, response);
-                return true;
+                return false;
             }
           }
         }
@@ -884,7 +889,7 @@ PBoolean SIPEndPoint::OnReceivedINVITE(OpalTransport & transport, SIP_PDU * requ
   SIPConnection *connection = CreateConnection(*call,
                                                SIPURL::GenerateTag(),
                                                NULL,
-                                               SIPURL(),
+                                               mime.GetFrom(),
                                                newTransport,
                                                request);
   if (!AddConnection(connection)) {
@@ -917,19 +922,18 @@ void SIPEndPoint::OnTransactionFailed(SIPTransaction & transaction)
 
 PBoolean SIPEndPoint::OnReceivedNOTIFY(OpalTransport & transport, SIP_PDU & pdu)
 {
-  SIPEventPackage eventPackage(pdu.GetMIME().GetEvent());
+  const SIPMIMEInfo & mime = pdu.GetMIME();
+  SIPEventPackage eventPackage(mime.GetEvent());
 
   PTRACE(3, "SIP\tReceived NOTIFY " << eventPackage);
   
   // A NOTIFY will have the same CallID than the SUBSCRIBE request it corresponds to
   // Technically should check for whole dialog, but call-id will do.
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(pdu.GetMIME().GetCallID(), PSafeReadWrite);
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(mime.GetCallID(), PSafeReadWrite);
 
   if (handler == NULL && eventPackage == SIPSubscribe::MessageSummary) {
     PTRACE(4, "SIP\tWork around Asterisk bug in message-summary event package.");
-    SIPURL url_from (pdu.GetMIME().GetFrom());
-    SIPURL url_to (pdu.GetMIME().GetTo());
-    PString to = url_to.GetUserName() + "@" + url_from.GetHostName();
+    SIPURL to(mime.GetTo().GetUserName() + "@" + mime.GetFrom().GetHostName());
     handler = activeSIPHandlers.FindSIPHandlerByUrl(to, SIP_PDU::Method_SUBSCRIBE, eventPackage, PSafeReadWrite);
   }
 
@@ -1073,6 +1077,7 @@ bool SIPEndPoint::Register(const SIPRegister::Params & newParams, PString & aor,
 
   SIPRegister::Params params(newParams);
   params.Normalise(GetDefaultLocalPartyName(), GetRegistrarTimeToLive());
+  PTRACE(5, "SIP\tNormalised REGISTER\n" << params);
   PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(params.m_addressOfRecord, SIP_PDU::Method_REGISTER, PSafeReadWrite);
 
   // If there is already a request with this URL and method, 
@@ -1212,6 +1217,7 @@ bool SIPEndPoint::Subscribe(const SIPSubscribe::Params & newParams, PString & to
 
   SIPSubscribe::Params params(newParams);
   params.Normalise(GetDefaultLocalPartyName(), GetNotifierTimeToLive());
+  PTRACE(5, "SIP\tNormalised SUBSCRIBE\n" << params);
   PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(params.m_addressOfRecord, SIP_PDU::Method_SUBSCRIBE, params.m_eventPackage, PSafeReadWrite);
 
   // If there is already a request with this URL and method, 
@@ -1273,9 +1279,11 @@ bool SIPEndPoint::Unsubscribe(SIPSubscribe::PredefinedPackages eventPackage,
 
 bool SIPEndPoint::Unsubscribe(const PString & eventPackage, const PString & token, bool invalidateNotifiers)
 {
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(token, PSafeReference);
+  PSafePtr<SIPSubscribeHandler> handler = PSafePtrCast<SIPHandler, SIPSubscribeHandler>(
+                                                activeSIPHandlers.FindSIPHandlerByCallID(token, PSafeReference));
   if (handler == NULL)
-    handler = activeSIPHandlers.FindSIPHandlerByUrl(token, SIP_PDU::Method_SUBSCRIBE, eventPackage, PSafeReference);
+    handler = PSafePtrCast<SIPHandler, SIPSubscribeHandler>(
+          activeSIPHandlers.FindSIPHandlerByUrl(token, SIP_PDU::Method_SUBSCRIBE, eventPackage, PSafeReference));
   else {
     if (!eventPackage.IsEmpty() && handler->GetEventPackage() != eventPackage)
       handler.SetNULL();
@@ -1287,13 +1295,10 @@ bool SIPEndPoint::Unsubscribe(const PString & eventPackage, const PString & toke
   }
 
   if (invalidateNotifiers) {
-    PSafePtr<SIPSubscribeHandler> subscription = PSafePtrCast<SIPHandler, SIPSubscribeHandler>(handler);
-    if (subscription != NULL) {
-      SIPSubscribe::Params params(subscription->GetParams());
-      params.m_onNotify = NULL;
-      params.m_onSubcribeStatus = NULL;
-      subscription->UpdateParameters(params);
-    }
+    SIPSubscribe::Params params(handler->GetParams());
+    params.m_onNotify = NULL;
+    params.m_onSubcribeStatus = NULL;
+    handler->UpdateParameters(params);
   }
 
   return handler->ActivateState(SIPHandler::Unsubscribing);
@@ -1500,10 +1505,11 @@ bool SIPEndPoint::SendOPTIONS(const SIPOptions::Params & newParams)
 {
   SIPOptions::Params params(newParams);
   params.Normalise(GetDefaultLocalPartyName(), GetNotifierTimeToLive());
+  PTRACE(5, "SIP\tNormalised OPTIONS\n" << params);
 
   PSafePtr<SIPHandler> handler = new SIPOptionsHandler(*this, params);
   activeSIPHandlers.Append(handler);
-  return handler->ActivateState(SIPHandler::Subscribing);
+  return handler->ActivateState(SIPHandler::Unsubscribing);
 }
 
 
@@ -1532,6 +1538,7 @@ bool SIPEndPoint::Publish(const SIPSubscribe::Params & newParams, const PString
 
   SIPSubscribe::Params params(newParams);
   params.Normalise(GetDefaultLocalPartyName(), GetNotifierTimeToLive());
+  PTRACE(5, "SIP\tNormalised PUBLISH\n" << params);
   PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(params.m_addressOfRecord, SIP_PDU::Method_PUBLISH, params.m_eventPackage, PSafeReadWrite);
   if (handler != NULL)
     handler->SetBody(body);
@@ -1658,26 +1665,80 @@ unsigned SIPEndPoint::GetAllowedMethods() const
 }
 
 
-bool SIPEndPoint::GetAuthentication(const PString & realm, PString & user, PString & password) 
+SIP_PDU::StatusCodes SIPEndPoint::HandleAuthentication(SIPAuthentication * & authentication,
+                                                       unsigned & authenticateErrors,
+                                                       const SIP_PDU & response,
+                                                       const SIPURL & proxyOverride,
+                                                       const PString & user,
+                                                       const PString & pass)
 {
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByAuthRealm(realm, user, PSafeReadOnly);
-  if (handler == NULL) {
-    if (m_registeredUserMode)
-      return false;
+  bool isProxy = response.GetStatusCode() == SIP_PDU::Failure_ProxyAuthenticationRequired;
 
-    handler = activeSIPHandlers.FindSIPHandlerByAuthRealm(realm, PSafeReadOnly);
-    if (handler == NULL)
-      return false;
+#if PTRACING
+  const char * proxyTrace = isProxy ? "Proxy " : "";
+#endif
+
+  PTRACE(3, "SIP\tReceived " << proxyTrace << "Authentication Required response for " << response.GetTransactionID());
+
+  // authenticate 
+  PString errorMsg;
+  SIPAuthentication * newAuthentication = PHTTPClientAuthentication::ParseAuthenticationRequired(isProxy, response.GetMIME(), errorMsg);
+  if (newAuthentication == NULL) {
+    PTRACE(2, "SIP\t" << proxyTrace << " Authentication error: " << errorMsg);
+    return SIP_PDU::Failure_Forbidden;
   }
 
-  if (handler->GetPassword().IsEmpty())
-    return false;
+  PString realm = newAuthentication->GetAuthRealm();
+  PString username = user;
+  PString password = pass;
 
-  // really just after password, but username MAY change too.
-  user     = handler->GetUsername();
-  password = handler->GetPassword();
+  if (username.IsEmpty() || password.IsEmpty()) {
+    // Try to find authentication parameters for the given realm
+    PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByAuthRealm(realm, username, PSafeReadOnly);
+    if (handler != NULL ||
+          (!m_registeredUserMode &&
+            (handler = activeSIPHandlers.FindSIPHandlerByAuthRealm(realm, PSafeReadOnly)) != NULL)) {
+      // really just after password, but username MAY change too.
+      username = handler->GetUsername();
+      password = handler->GetPassword();
+      PTRACE (3, "SIP\tUsing auth info for realm \"" << realm << '"');
+    }
+  }
 
-  return true;
+  // Still need stuff, check proxy
+  if (username.IsEmpty() || password.IsEmpty()) {
+    SIPURL proxy = proxyOverride;
+    if (proxy.IsEmpty())
+      proxy = GetProxy();
+    if (!proxy.IsEmpty()) {
+      PTRACE (3, "SIP\tNo auth info for realm \"" << realm << "\", using proxy auth");
+      username = proxy.GetUserName();
+      password = proxy.GetPassword();
+    }
+  }
+
+  if (username.IsEmpty() || password.IsEmpty()) {
+    PTRACE(2, "SIP\tAuthentication not possible yet, no credentials available.");
+    return SIP_PDU::Failure_UnAuthorised;
+  }
+
+  newAuthentication->SetUsername(username);
+  newAuthentication->SetPassword(password);
+
+  // Only check for same credentials if same request (via CSeq)
+  if (authenticateErrors > 1 && authentication != NULL && *newAuthentication == *authentication) {
+    PTRACE(1, "SIP\tAuthentication already performed using current credentials, not trying again.");
+    return SIP_PDU::Failure_UnAuthorised;
+  }
+
+  PTRACE(4, "SIP\t" << (authentication != NULL ? "Upd" : "Cre") << "ating"
+            " authentication credentials of user \"" << username << "\" for realm \"" << realm << '"');
+
+  // switch authentication schemes
+  delete authentication;
+  authentication = newAuthentication;
+  ++authenticateErrors;
+  return SIP_PDU::Successful_OK;
 }
 
 
