@@ -23,9 +23,9 @@
  *
  * Contributor(s): Vyacheslav Frolov.
  *
- * $Revision: 22029 $
- * $Author: csoutheren $
- * $Date: 2009-02-09 23:25:00 -0600 (Mon, 09 Feb 2009) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 /*
@@ -72,7 +72,7 @@ SDPMediaDescription * OpalFaxMediaType::CreateSDPMediaDescription(const OpalTran
 /////////////////////////////////////////////////////////////////////////////
 
 SDPFaxMediaDescription::SDPFaxMediaDescription(const OpalTransportAddress & address)
-  : SDPMediaDescription(address)
+  : SDPMediaDescription(address, OpalMediaType::Fax())
 {
   t38Attributes.SetAt("T38FaxRateManagement", "transferredTCF");
   t38Attributes.SetAt("T38FaxVersion",        "0");
@@ -85,6 +85,12 @@ PCaselessString SDPFaxMediaDescription::GetSDPTransportType() const
   return "udptl";
 }
 
+SDPMediaDescription * SDPFaxMediaDescription::CreateEmpty() const
+{
+  return new SDPFaxMediaDescription(OpalTransportAddress());
+}
+
+
 PString SDPFaxMediaDescription::GetSDPMediaType() const 
 { 
   return "image"; 
@@ -92,12 +98,23 @@ PString SDPFaxMediaDescription::GetSDPMediaType() const
 
 SDPMediaFormat * SDPFaxMediaDescription::CreateSDPMediaFormat(const PString & portString)
 {
-  return new SDPMediaFormat(*this, RTP_DataFrame::DynamicBase, portString);
+  const OpalMediaFormat mediaFormat(RTP_DataFrame::DynamicBase, 0, portString, "sip");
+  if (mediaFormat.IsEmpty()) {
+    PTRACE(2, "SDPFax\tCould not find media format for " << portString);
+    return NULL;
+  }
+
+  PTRACE(3, "SDPFax\tUsing RTP payload " << mediaFormat.GetPayloadType() << " for " << portString);
+
+  return new SDPMediaFormat(*this, mediaFormat);
 }
 
 
 PString SDPFaxMediaDescription::GetSDPPortList() const
 {
+  if (formats.IsEmpty())
+    return " t38"; // Have to have SOMETHING
+
   PStringStream str;
 
   // output encoding names for non RTP
@@ -136,13 +153,38 @@ void SDPFaxMediaDescription::ProcessMediaOptions(SDPMediaFormat & /*sdpFormat*/,
   if (mediaFormat.GetMediaType() == OpalMediaType::Fax()) {
     for (PINDEX i = 0; i < mediaFormat.GetOptionCount(); ++i) {
       const OpalMediaOption & option = mediaFormat.GetOption(i);
-      if (option.GetName().Left(3) *= "t38") 
-        t38Attributes.SetAt(option.GetName(), option.AsString());
+      if (option.GetName().Left(3) *= "t38") {
+        PString value = option.AsString();
+        if (value != option.GetFMTPDefault())
+          t38Attributes.SetAt(option.GetName(), option.AsString());
+      }
     }
   }
 }
 
-/////////////////////////////////////////////////////////////////////////////
+
+bool SDPFaxMediaDescription::PostDecode(const OpalMediaFormatList & mediaFormats)
+{
+  if (!SDPMediaDescription::PostDecode(mediaFormats))
+    return false;
+
+  for (SDPMediaFormatList::iterator format = formats.begin(); format != formats.end(); ++format) {
+    OpalMediaFormat & mediaFormat = format->GetWritableMediaFormat();
+    if (mediaFormat.GetMediaType() == OpalMediaType::Fax()) {
+      for (PINDEX i = 0; i < t38Attributes.GetSize(); ++i) {
+        PString key = t38Attributes.GetKeyAt(i);
+        PString data = t38Attributes.GetDataAt(i);
+        if (!mediaFormat.SetOptionValue(key, data)) {
+          PTRACE(2, "T38\tCould not set option \"" << key << "\" to \"" << data << '"');
+        }
+      }
+      PTRACE(5, "T38\tMedia format set from SDP:\n" << setw(-1) << mediaFormat);
+    }
+  }
+
+  return true;
+}
+
 
 #endif // OPAL_T38_CAPABILITY
 #endif // OPAL_SIP
