@@ -23,9 +23,9 @@
  *
  * Contributor(s): Vyacheslav Frolov.
  *
- * $Revision: 26159 $
- * $Author: rjongbloed $
- * $Date: 2011-07-10 22:02:39 -0500 (Sun, 10 Jul 2011) $
+ * $Revision$
+ * $Author$
+ * $Date$
  */
 
 #include <ptlib.h>
@@ -70,18 +70,19 @@ class OpalFaxMediaStream : public OpalNullMediaStream
       m_isAudio = true; // Even though we are not REALLY audio, act like we are
     }
 
-    bool Close()
+    virtual void InternalClose()
     {
-      if (isOpen &&
-          m_connection.m_state == OpalFaxConnection::e_CompletedSwitch &&
+      if (m_connection.m_state == OpalFaxConnection::e_CompletedSwitch &&
           m_connection.m_finalStatistics.m_fax.m_result < 0) {
-        if (mediaPatch != NULL)
-          mediaPatch->ExecuteCommand(OpalFaxTerminate(), false);
+        // make a referenced copy so can't be deleted out from under us
+        PatchPtr patch = m_mediaPatch;
+        if (patch != NULL)
+          patch->ExecuteCommand(OpalFaxTerminate(), false);
         GetStatistics(m_connection.m_finalStatistics);
         PTRACE(4, "FAX\tGot final statistics: result=" << m_connection.m_finalStatistics.m_fax.m_result);
       }
 
-      return OpalNullMediaStream::Close();
+      OpalNullMediaStream::InternalClose();
     }
 
   private:
@@ -113,7 +114,7 @@ class T38PseudoRTP_Handler : public RTP_Encoding
 
       rtpUDP->SetJitterBufferSize(0, 0);
       m_consecutiveBadPackets  = 0;
-      m_oneGoodPacket          = false;
+      m_awaitingGoodPacket     = true;
       m_expectedSequenceNumber = 0;
       m_secondaryPacket        = -1;
 
@@ -398,27 +399,43 @@ class T38PseudoRTP_Handler : public RTP_Encoding
 
       PPER_Stream rawData(thisUDPTL, pduSize);
 
-      // Decode the PDU
-      if (!m_receivedPacket.Decode(rawData)) {
-  #if PTRACING
-        if (m_oneGoodPacket)
-          PTRACE(2, "T38_UDPTL\tRaw data decode failure:\n  "
-                 << setprecision(2) << rawData << "\n  UDPTL = "
-                 << setprecision(2) << m_receivedPacket);
-        else
-          PTRACE(2, "T38_UDPTL\tRaw data decode failure: " << rawData.GetSize() << " bytes.");
+      // Decode the PDU, but not if still receiving RTP
+      bool decodeBad = !m_receivedPacket.Decode(rawData);
+      // LXK Change to solve the Cisco CUBE bug of starting with UDPTL Sequence Number of 32768.
+      // The T.38 Recommendation requires the first UDPTL packet to have a Sequence Number of 0.
+      // Previously we ignored packets with sequence numbers >= 32768. Now we only ignore packets 
+      // with sequence numbers of 32768 (0x8000 PCMU identifier) or 32776 (0x8008 PCMA identifier). 
+      // (We only support the PCMU and PCMA codecs to start a call.) All others are accepted, so 
+      // once the CUBE  moves to 32769, we have only missed the first UDPTL packet and we are OK.
+      if (decodeBad || 
+             (m_awaitingGoodPacket && 
+                 (m_receivedPacket.m_seq_number == 0x8000 ||    // PCMU first two bytes 
+                  m_receivedPacket.m_seq_number == 0x8008))) {  // PCMA first two bytes
+        if (++m_consecutiveBadPackets > 1000) {
+          PTRACE(1, "T38_UDPTL\tRaw data decode failed 1000 times, remote probably not switched from audio, aborting!");
+          return RTP_Session::e_AbortTransport;
+        }
+
+ #if PTRACING
+        static const unsigned Level = 2;
+        if (PTrace::CanTrace(Level)) {
+          ostream & trace = PTrace::Begin(Level, __FILE__, __LINE__);
+          trace << "T38_UDPTL\t";
+          if (m_awaitingGoodPacket)
+            trace << "Probable RTP packet: " << rawData.GetSize() << " bytes.";
+          else
+            trace << "Raw data decode failure:\n  "
+                  << setprecision(2) << rawData << "\n  UDPTL = "
+                  << setprecision(2) << m_receivedPacket;
+          trace << PTrace::End;
+        }
   #endif
 
-        m_consecutiveBadPackets++;
-        if (m_consecutiveBadPackets < 1000)
-          return RTP_Session::e_IgnorePacket;
-
-        PTRACE(1, "T38_UDPTL\tRaw data decode failed 1000 times, remote probably not switched from audio, aborting!");
-        return RTP_Session::e_AbortTransport;
+        return RTP_Session::e_IgnorePacket;
       }
 
-      PTRACE_IF(3, !m_oneGoodPacket, "T38_UDPTL\tFirst decoded UDPTL packet");
-      m_oneGoodPacket = true;
+      PTRACE_IF(3, m_awaitingGoodPacket, "T38_UDPTL\tFirst decoded UDPTL packet");
+      m_awaitingGoodPacket = false;
       m_consecutiveBadPackets = 0;
 
       PTRACE(5, "T38_UDPTL\tDecoded UDPTL packet:\n  " << setprecision(2) << m_receivedPacket);
@@ -447,7 +464,7 @@ class T38PseudoRTP_Handler : public RTP_Encoding
 
   protected:
     int             m_consecutiveBadPackets;
-    bool            m_oneGoodPacket;
+    bool            m_awaitingGoodPacket;
     T38_UDPTLPacket m_receivedPacket;
     unsigned        m_expectedSequenceNumber;
     int             m_secondaryPacket;
@@ -558,6 +575,7 @@ OpalMediaFormatList OpalFaxEndPoint::GetMediaFormats() const
   OpalMediaFormatList formats;
   formats += OpalT38;
   formats += TIFF_File_FormatName;
+PTRACE(4, "OpalFaxEndPoint\tGetMediaFormats for " << *this << "\n    " << setfill(',') << formats << setfill(' '));
   return formats;
 }
 
@@ -589,7 +607,7 @@ OpalFaxConnection::OpalFaxConnection(OpalCall        & call,
 
   m_switchTimer.SetNotifier(PCREATE_NOTIFIER(OnSwitchTimeout));
 
-  PTRACE(3, "FAX\tCreated FAX connection with token \"" << callToken << "\","
+  PTRACE(3, "FAX\tCreated fax connection with token \"" << callToken << "\","
             " receiving=" << receiving << ","
             " disabledT38=" << disableT38 << ","
             " filename=\"" << filename << '"');
@@ -721,7 +739,6 @@ void OpalFaxConnection::OnStopMediaPatch(OpalMediaPatch & patch)
 
     // Not an explicit switch, so fax plug in indicated end of fax
     if (m_state == e_CompletedSwitch && m_faxMediaStreamsSwitchState == e_NotSwitchingFaxMediaStreams) {
-      synchronousOnRelease = false;
 #if OPAL_STATISTICS
       InternalGetStatistics(m_finalStatistics, true);
       PTRACE(3, "FAX\tGot final statistics: result=" << m_finalStatistics.m_fax.m_result);
@@ -814,24 +831,29 @@ void OpalFaxConnection::OnSwitchTimeout(PTimer &, INT)
 }
 
 
-bool OpalFaxConnection::SwitchFaxMediaStreams(bool enableFax)
+bool OpalFaxConnection::SwitchFaxMediaStreams(bool toT38)
 {
   PSafePtr<OpalConnection> other = GetOtherPartyConnection();
-  if (other != NULL && other->SwitchFaxMediaStreams(enableFax))
+  if (other != NULL && other->SwitchFaxMediaStreams(toT38))
     return true;
 
-  PTRACE(1, "FAX\tMode change request to " << (enableFax ? "fax" : "audio") << " failed");
+  PTRACE(1, "FAX\tMode change request to " << (toT38 ? "T.38" : "audio") << " failed");
   return false;
 }
 
 
-void OpalFaxConnection::OnSwitchedFaxMediaStreams(bool enabledFax)
+void OpalFaxConnection::OnSwitchedFaxMediaStreams(bool toT38, bool success)
 {
-  if (enabledFax) {
-    PTRACE(3, "FAX\tMode change request to fax succeeded");
+  if (!toT38) {
+    PTRACE(3, "FAX\tMode change request to audio");
+    return;
+  }
+
+  if (success) {
+    PTRACE(3, "FAX\tMode change request to T.38 succeeded");
   }
   else {
-    PTRACE(4, "FAX\tMode change request to fax failed, falling back to G.711");
+    PTRACE(4, "FAX\tMode change request to T.38 failed, falling back to G.711");
     if (m_stringOptions.GetBoolean(OPAL_NO_G111_FAX))
       OnFaxCompleted(true);
     else {
@@ -844,6 +866,12 @@ void OpalFaxConnection::OnSwitchedFaxMediaStreams(bool enabledFax)
 }
 
 
+bool OpalFaxConnection::OnSwitchingFaxMediaStreams(bool toT38)
+{
+  return !(toT38 && m_disableT38);
+}
+
+
 void OpalFaxConnection::OpenFaxStreams(PThread &, INT)
 {
   if (LockReadWrite()) {
